// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright 2016 Freescale Semiconductor, Inc.
 * Copyright 2018-2022 NXP
 */

#include <common.h>
#include <command.h>
#include <asm/arch/sys_proto.h>
#include <linux/errno.h>
#include <asm/io.h>
#include <stdbool.h>
#include <mmc.h>
#include <env.h>

#ifdef CONFIG_SYS_MMC_ENV_PART
uint mmc_get_env_part(struct mmc *mmc)
{
	uint part = EXT_CSD_EXTRACT_BOOT_PART(mmc->part_config);

	if (part > 2)
		/* Fallback to user partition if not boot0/boot1 */
		part = 0;

	return part;
}
#endif

static int check_mmc_autodetect(void)
{
	char *autodetect_str = env_get("mmcautodetect");

	if (autodetect_str && !strcmp(autodetect_str, "yes"))
		return 1;

	return 0;
}

/* This should be defined for each board */
__weak int mmc_map_to_kernel_blk(int dev_no)
{
	return dev_no;
}

static void at_board_late_mmc_env_init(void) {
	char buf[100];
	int part = 1;
	u32 dev_no = mmc_get_env_dev();

	if (!check_mmc_autodetect())
		return;

	/* evk board:
	 * emmc    -> dev_no = 0, ext csd bootpart = 1/2
	 * sd card -> dev_no = 1, ext csd bootpart = 7
	 *
	 * atmark image has kernel/dtb in p1/2 and rootfs in p1/2 as well,
	 * so use part as is directly for both
	 */
	if (dev_no == 0) {
		/* emmc, get boot partition from extcsd */
		struct mmc *mmc;

		mmc = find_mmc_device(dev_no);

		if (!mmc) {
			printf("booting on emmc but no mmc device found, skipping mmcautodetect\n");
			return;
		}
		if (!mmc->has_init) {
			printf("mmc dev found but hasn't been init, skipping mmcautodetect\n");
			return;
		}

		part = EXT_CSD_EXTRACT_BOOT_PART(mmc->part_config);
	} else {
		/* sd card, get boot partition from env */
		part = env_get_ulong("mmcpart", 10, 1);
	}

	/* device where to load linux from */
	snprintf(buf, sizeof(buf), "%d", dev_no);
	env_set("mmcdev", buf);
	/* partition index where linux/dtb are stored */
	snprintf(buf, sizeof(buf), "%d", part);
	env_set("mmcpart", buf);
	/* kernel boot command */
	snprintf(buf, sizeof(buf), "/dev/mmcblk%dp%d rootwait ro",
		 mmc_map_to_kernel_blk(dev_no), part);
	env_set("mmcroot", buf);

	snprintf(buf, sizeof(buf), "mmc dev %d", dev_no);
	run_command(buf, 0);
}

void board_late_mmc_env_init(void)
{
	at_board_late_mmc_env_init();
}
