// SPDX-License-Identifier: GPL-2.0+
/*
 * NXP i.MX BBNSM RTC driver
 *
 * Copyright (c) 2024 Atmark Techno, Inc
 */

/*
 *FIXME : Not support Time Alarm & Time Alarm Interrupt.
 */
#include <common.h>
#include <div64.h>
#include <dm.h>
#include <rtc.h>
#include <timestamp.h>
#include <syscon.h>
#include <regmap.h>
#include <linux/err.h>

#define BBNSM_RTC_LS 0x40
#define BBNSM_RTC_MS 0x44

struct bbnsm_rtc_priv {
	struct regmap *regmap;
};

static int bbnsm_rtc_get(struct udevice *dev, struct rtc_time *time)
{
	struct bbnsm_rtc_priv *priv = dev_get_priv(dev);
	u32 lsb, msb;
	u64 now;

	regmap_read(priv->regmap, BBNSM_RTC_LS, &lsb);
	regmap_read(priv->regmap, BBNSM_RTC_MS, &msb);

	/* convert to seconds */
	now = (msb << 17) | (lsb >> 15);

	rtc_to_tm(now, time);

	return 0;
}

static int bbnsm_rtc_probe(struct udevice *dev)
{
	struct bbnsm_rtc_priv *priv = dev_get_priv(dev);

	priv->regmap = syscon_node_to_regmap(dev_ofnode(dev_get_parent(dev)));
	if (IS_ERR(priv->regmap)) {
		pr_err("unable to find regmap\n");
		return -ENODEV;
	}

	return 0;
}

static const struct rtc_ops bbnsm_rtc_ops = {
	.get = bbnsm_rtc_get,
};

static const struct udevice_id bbnsm_rtc_ids[] = {
	{ .compatible = "nxp,bbnsm-rtc" },
	{ }
};

U_BOOT_DRIVER(rtc_bbnsm) = {
	.name	= "rtc_bbnsm",
	.id	= UCLASS_RTC,
	.of_match	= bbnsm_rtc_ids,
	.ops	= &bbnsm_rtc_ops,
	.probe	= bbnsm_rtc_probe,
	.priv_auto	= sizeof(struct bbnsm_rtc_priv),
};
