/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2020 NXP
 */

#ifndef __ARMADILLO_900_H
#define __ARMADILLO_900_H

#include <linux/sizes.h>
#include <asm/arch/imx-regs.h>
#include "imx_env.h"

#define CFG_POST CFG_SYS_POST_MEMORY
/*
 * Use last 4 Bytes of SRAM0 (64KB).
 *
 * There is a possibility that the state cannot be saved (ROM code
 * could use any bit (in advance)), but so far there have been no
 * problems.
 *
 * After POST memtest, it is recommended to power off -> on to enable
 * ROM API. Note that ROM API will not be restored by rebooting
 * (resetting).
 *
 * For information on SRAM0 and ROM APIs, please refer to the imx8ulp
 * reference manual (rev.1).
 */
#define CFG_SYS_POST_WORD_ADDR 0x2201fffc

#define CFG_SYS_UBOOT_BASE	(QSPI0_AMBA_BASE + CONFIG_SYS_MMCSD_RAW_MODE_U_BOOT_SECTOR * 512)

#ifdef CONFIG_SPL_BUILD
#define CFG_MALLOC_F_ADDR		0x22048000


#endif

/* ENET Config */
#if defined(CONFIG_FEC_MXC)
#define PHY_ANEG_TIMEOUT		20000

#define CFG_FEC_MXC_PHYADDR		0
#endif

/* Initial environment variables */
#define CFG_EXTRA_ENV_SETTINGS		\
	"console=ttyLP0,115200 earlycon\0" \
	"m33_console=none\0" \
	"scriptaddr=0x83500000\0" \
	"kernel_addr_r=" __stringify(CONFIG_SYS_LOAD_ADDR) "\0" \
	/* "loadaddr=0x80400000\0" set from CONFIG_SYS_LOAD_ADDR */ \
	"fdt_addr_r=0x83000000\0" \
	"fdt_addr=0x83000000\0" \
	"fdt_size=0x40000\0" \
	"fdt_high=0xffffffffffffffff\0" \
	"fdtoverlay_addr_r=0x83040000\0" \
	"cntr_addr=0x98000000\0" \
	"boot_fit=try\0" \
	"bootm_size=0x10000000\0" \
	"mmcdev="__stringify(CONFIG_SYS_MMC_ENV_DEV)"\0" \
	"mmcpart=1\0" \
	"mmcroot=/dev/mmcblk"__stringify(CONFIG_SYS_MMC_ENV_DEV)"p1 rootwait rw\0" \
	"mmcautodetect=yes\0" \
	"mmcargs=setenv bootargs console=${console} root=${mmcroot} ${optargs}\0" \
	"script=boot/boot.scr\0" \
	"loadbootscript=size mmc ${mmcdev}:${mmcpart} ${script} " \
		"&& load mmc ${mmcdev}:${mmcpart} ${loadaddr} ${script};\0" \
	"bootscript=echo Running bootscript from mmc ...; " \
		"source;" \
		"log rec none alert \"\" 0 bootscript \"boot script did not boot\"\0" \
	"image=boot/Image\0" \
	"loadimage=load mmc ${mmcdev}:${mmcpart} ${loadaddr} ${image}\0" \
	"loadimage_bootsec=" \
		/* load at mmcblkXp10 for p1 */ \
		"setexp mmcpart_boot $mmcpart + 9; " \
		"load mmc ${mmcdev}:${mmcpart_boot} ${loadaddr} ${image}\0" \
	"fdtfile=boot/" CONFIG_DEFAULT_FDT_FILE "\0" \
	"loadfdt=" \
		"if load mmc ${mmcdev}:${mmcpart} ${fdt_addr_r} ${fdtfile}; then " \
			"echo \"Loading fdt ${fdtfile}\" " \
			"&& fdt addr ${fdt_addr_r} && fdt resize ${fdt_size}; " \
		"else " \
			"log rec none alert \"\" 0 loadfdt \"Could not load fdt file!\"; " \
			"false; " /* false required to fail 'run loadfdt' */ \
		"fi\0" \
	"overlays_list=boot/overlays.txt\0" \
	"load_overlays_list=" \
		"load mmc ${mmcdev}:${mmcpart} ${fdtoverlay_addr_r} ${overlays_list} " \
			"&& env import -t ${fdtoverlay_addr_r} ${filesize}\0" \
	"apply_overlays=" \
		"for overlay in ${fdt_overlays}; do " \
			"if load mmc ${mmcdev}:${mmcpart} ${fdtoverlay_addr_r} /boot/${overlay}; then " \
				"echo \"Applying fdt overlay: ${overlay}\"; " \
				"fdt apply ${fdtoverlay_addr_r}; " \
			"else " \
				"echo \"Warning: fdt overlay ${overlay} not found!\"; " \
			"fi; " \
		"done; setenv fdt_overlays\0" \
	"fitconf_base=#armadillo\0" \
	"apply_fit_overlays=" \
		"for overlay in ${fdt_overlays}; do " \
			"echo \"Appending overlay ${overlay}\"; " \
			"fitconf=\"${fitconf}#${overlay}\"; " \
		"done; setenv fdt_overlays\0" \
	"mmcboot=echo Booting from mmc ...; " \
		"run mmcargs; " \
		"if test ${boot_fit} = yes || test ${boot_fit} = try && iminfo quiet ${loadaddr}; then " \
			"run apply_fit_overlays; " \
			"run load_overlays_list apply_fit_overlays; " \
			"bootm ${loadaddr}\"${fitconf_base}${fitconf}\"; " \
			"log rec none alert \"\" 0 mmcboot \"Could not boot fit image\"; " \
		"else " \
			"if run loadfdt; then " \
				"run apply_overlays; " /* once first, if fdt_overlays is manually set */ \
				"run load_overlays_list apply_overlays; " \
				"booti ${loadaddr} - ${fdt_addr_r}; " \
				"log rec none alert \"\" 0 mmcboot \"Could not boot image\"; " \
			"else " \
				"log rec none alert \"\" 0 mmcboot \"Could not load fdt\"; " \
			"fi; " \
		"fi;\0" \
	"upgrade_available=1\0" \
	"rollback_mmc=" \
		/* refuse to rollback if dek offset is set, would not be bootable */ \
		/* this check to be superseded by common upgrade_available check */ \
		"test -z \"${dek_spl_offset}\" && saveenv && " \
		"mmc partconf " __stringify(CONFIG_SYS_MMC_ENV_DEV) " 0 ${fallback} 0 && reset\0" \
	"rollback_other=setenv mmcpart \"${fallback}\"; " \
		"setenv mmcroot \"mmcblk${mmcdev}p${fallback} rootwait ro\"; " \
		"saveenv; reset\0" \
	"setfallback=" \
		"setexpr fallback 3 - ${mmcpart}; " /* 2 -> 1, 1 -> 2. And recheck. */ \
		"test \"${fallback}\" = 1 || test \"${fallback}\" = 2\0" \
	"rollback=" \
		"setenv bootcount; " \
		"if test \"${upgrade_available}\" -gt 0 && run setfallback; then " \
			"log rec none alert \"\" 0 rollback " \
				"\"Rolling back to mmcblk${mmcdev}p${fallback}\"; " \
			"if test \"${mmcdev}\" = " __stringify(CONFIG_SYS_MMC_ENV_DEV) "; then " \
				"run rollback_mmc; " \
			"else " \
				"run rollback_other; " \
			"fi; " \
		"elif test \"${upgrade_available}\" -gt 0; then " \
			"log rec none alert \"\" 0 rollback " \
				"\"Cannot compute rollback target from mmcpart ${mmcpart}\"; " \
		"else " \
			"log rec none alert \"\" 0 rollback " \
				"\"Rollback target is not ready, refusing to rollback.\"; " \
		"fi; " \
		/* reset if rollback failed: we're not booting either way... */ \
		"saveenv; reset\0" \
	"altbootcmd=" \
		"log rec none alert \"\" 0 altbootcmd " \
			"\"Counted $bootlimit consecutive unfinished boots\"; " \
		"run rollback\0" \
	"sec_boot=" \
		/* set to 'true' to test */ \
		"setexpr.b lifecycle *0x2701041c; " \
		/* 0x8 for Open, 0x20 for closed, anything else we don't support */ \
		/* so only consider 0x8 as open and anything else secure */ \
		"test ${lifecycle} != 8\0" \
	"sec_boot_image=" \
		/* magic for secureboot container */ \
		"setexpr magic *$loadaddr; " \
		"test $magic = 8702c000\0" \
	/* doc should suggest setting it to true to test */ \
	"mmcboot_sec=" \
		/* We don't want to load the image to another address if secure boot */ \
		/* is enabled; instead we unpack from loadaddr to sec_boot_loadaddr: */ \
		/* move loadaddr to that and hand off to regular boot */ \
		"if auth_cntr ${loadaddr}; then " \
			"setenv loadaddr ${cntr_addr};" \
			"setenv boot_fit yes;" \
			"run mmcboot; " \
		"else " \
			"log rec none alert \"\" 0 mmcboot_sec " \
				"\"Could not authenticate os\"; " \
		"fi\0" \
	"update_encrypted_boot_inner=" \
		"for offset in ${dek_offsets}; do" \
			"setexpr offset ${loadaddr} + ${offset};" \
			"cp ${offset} ${fdt_addr}; " /* cannot update in place */ \
			"dek_blob ${fdt_addr} ${offset} 128 || exit 1; " \
		"done\0" \
	"update_encrypted_boot=" \
		"test \"${encrypted_update_available}\" = 1 && " \
		/* there can be only one attempt: */ \
		/* we unset encrypted_update_available first so userspace */ \
		/* can assume rolled back boot is broken if offset env is set */ \
		/* conversely, userspace should setenv offsets, write non-bootable */ \
		/* boot image and only then set encrypted_update_available. */ \
		"setenv encrypted_update_available && saveenv && " \
		"test -n \"${dek_offsets}\" && " \
		"run setfallback && " \
		"log rec none alert \"\" 0 update_encrypted_boot " \
			"\"Writing encrypted boot to mmcblk2boot${fallback}\" && " \
		"if mmc dev 2 ${fallback} && " \
		"mmc read ${loadaddr} 0 0x2000 && " \
		"run update_encrypted_boot_inner && " \
		"mmc write ${loadaddr} 0 0x2000 && " \
		/* clear other env vars then boot into new image */ \
		"setenv upgrade_available 1 && " \
		"setenv dek_offsets; then " \
			"run rollback_mmc; " \
		"else " \
			"log rec none alert \"\" 0 update_encrypted_boot " \
				"\"Flashing new boot image failed!!\"; " \
		"fi\0" \
	"distro_bootcmd=" \
		"mmc dev ${mmcdev}; " \
		"if mmc rescan; then " \
			"if run sec_boot; then " \
				"if run loadimage sec_boot_image " \
				    "|| run loadimage_bootsec; then " \
					"run mmcboot_sec; " \
				"fi; " \
			"else " \
				"run update_encrypted_boot; " \
				"if run loadbootscript; then " \
					"run bootscript; " \
				"elif run loadimage; then " \
					"if run sec_boot_image; then " \
						"run mmcboot_sec; " \
					"else " \
						"run mmcboot; " \
					"fi; " \
				"elif run loadimage_bootsec; then " \
					"run mmcboot_sec; " \
				"else " \
					"log rec none alert \"\" 0 distro_bootcmd \"Could not load image!\"; " \
				"fi; " \
			"fi; " \
		"else " \
			"log rec none alert \"\" 0 distro_bootcmd \"mmc not found!\"; " \
		"fi; " \
		"run rollback\0" \
	"optargs=quiet\0" \
	"timeoffset=32400\0"


/* Link Definitions */

#define CFG_SYS_INIT_RAM_ADDR	0x80000000
#define CFG_SYS_INIT_RAM_SIZE	0x80000

#define CFG_SYS_SDRAM_BASE		0x80000000
#define PHYS_SDRAM			0x80000000
#define PHYS_SDRAM_SIZE			0x40000000 /* 1GB DDR */

/* Using ULP WDOG for reset */
#define WDOG_BASE_ADDR			WDG3_RBASE

#ifdef CONFIG_ENV_WRITEABLE_LIST
/* In this mode we define envs we want to allow overriding at run time
 * Possible flags are as follow:
 * type flags:
 *  - s: string
 *  - d: decimal
 *  - x: hexadecimal
 *  - b: boolean
 *  - i: IP address
 *  - m: MAC address
 * access flags:
 *  - a: any
 *  - r: read-only
 *  - o: write-once
 *  - c: change-default
 *  - w: writeable
 *
 *  The following example value allows:
 *  - rollback related variables
 *  - network settings (mac addresses, interface selection for uboot net commands)
 */
#define CFG_ENV_FLAGS_LIST_STATIC \
	"bootcount:dw,upgrade_available:bw" \
	",ethaddr:mw,ethact:sw,ethprime:sw"
#endif

#endif
