/* Header file generated from BLE_1.8.9.xml */
/*
 * Copyright 2018-2022 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#ifndef _BLE_CMD_H
#define _BLE_CMD_H
/*==================================================================================================
Include Files
==================================================================================================*/
#include "os_abstraction.h"

/*==================================================================================================
Public macros
==================================================================================================*/
#define GATT_ENABLE TRUE
#define L2CAPCB_ENABLE TRUE
#define GATTDB_ATT_ENABLE TRUE
#define GAP_ENABLE TRUE
#define L2CAP_ENABLE TRUE
#define NVM_ENABLE TRUE
#define FSCI_ENABLE TRUE
#define GATTDB_APP_ENABLE TRUE

/*==================================================================================================
Public type definitions
==================================================================================================*/
#if L2CAP_ENABLE
typedef struct L2CAPSendAttDataRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint16_t PacketLength;  // Length of the ATT data packet to be sent
	uint8_t *Packet;  // The ATT data packet to be transmitted
} L2CAPSendAttDataRequest_t;

typedef struct L2CAPSendSmpDataRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint16_t PacketLength;  // Length of the SM data packet to be sent
	uint8_t *Packet;  // The SM data packet to be transmitted
} L2CAPSendSmpDataRequest_t;

#endif  /* L2CAP_ENABLE */

#if L2CAPCB_ENABLE
typedef PACKED_STRUCT L2CAPCBRegisterLePsmRequest_tag {
	uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
	uint16_t LePsmMtu;  // MTU value
} L2CAPCBRegisterLePsmRequest_t;

typedef PACKED_STRUCT L2CAPCBDeregisterLePsmRequest_tag {
	uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
} L2CAPCBDeregisterLePsmRequest_t;

typedef PACKED_STRUCT L2CAPCBConnectLePsmRequest_tag {
	uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint16_t InitialCredits;  // Initial credits
} L2CAPCBConnectLePsmRequest_t;

typedef PACKED_STRUCT L2CAPCBDisconnectLeCbChannelRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint16_t ChannelId;  // The L2CAP Channel Id assigned on the initiator
} L2CAPCBDisconnectLeCbChannelRequest_t;

/* Refuse reason */
typedef enum L2CAPCBCancelConnectionRequest_RefuseReason_tag {
	L2CAPCBCancelConnectionRequest_RefuseReason_gSuccessful_c = 0x0000,
	L2CAPCBCancelConnectionRequest_RefuseReason_gLePsmNotSupported_c = 0x0002,
	L2CAPCBCancelConnectionRequest_RefuseReason_gNoResourcesAvailable_c = 0x0004,
	L2CAPCBCancelConnectionRequest_RefuseReason_gInsufficientAuthentication_c = 0x0005,
	L2CAPCBCancelConnectionRequest_RefuseReason_gInsufficientAuthorization_c = 0x0006,
	L2CAPCBCancelConnectionRequest_RefuseReason_gInsufficientEncryptionKeySize_c = 0x0007,
	L2CAPCBCancelConnectionRequest_RefuseReason_gInsufficientEncryption_c = 0x0008,
	L2CAPCBCancelConnectionRequest_RefuseReason_gInvalidSourceCid_c = 0x0009,
	L2CAPCBCancelConnectionRequest_RefuseReason_gSourceCidAlreadyAllocated_c = 0x000A,
	L2CAPCBCancelConnectionRequest_RefuseReason_gTimerError_c = 0x000B,
	L2CAPCBCancelConnectionRequest_RefuseReason_gCommandRejected_c = 0x0100,
	L2CAPCBCancelConnectionRequest_RefuseReason_gResponseTimeout_c = 0xFFFE
} L2CAPCBCancelConnectionRequest_RefuseReason_t;

typedef PACKED_STRUCT L2CAPCBCancelConnectionRequest_tag {
	uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	L2CAPCBCancelConnectionRequest_RefuseReason_t RefuseReason;  // Refuse reason
} L2CAPCBCancelConnectionRequest_t;

typedef struct L2CAPCBSendLeCbDataRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint16_t ChannelId;  // The L2CAP Channel Id assigned on the initiator
	uint16_t PacketLength;  // Length of the data packet to be sent
	uint8_t *Packet;  // The data packet to be transmitted
} L2CAPCBSendLeCbDataRequest_t;

typedef PACKED_STRUCT L2CAPCBSendLeCreditRequest_tag {
	uint8_t DeviceId;  // The DeviceID to which credits are given
	uint16_t ChannelId;  // The L2CAP Channel Id assigned on the initiator
	uint16_t Credits;  // Number of credits to be given
} L2CAPCBSendLeCreditRequest_t;

typedef struct L2CAPCBEnhancedConnectLePsmRequest_tag {
	uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
	uint8_t DeviceId;  // The DeviceID to which credits are given
	uint16_t Mtu;  // Maximum Transmit Unit
	uint16_t InitialCredits;  // Initial Credits
	uint8_t NoOfChannels;  // Number of channels to request for opening(max 5)
	uint8_t *Cids;  // List of cids if an Enhanced Connection Request has been previously received
} L2CAPCBEnhancedConnectLePsmRequest_t;

typedef struct L2CAPCBEnhancedChannelReconfigureRequest_tag {
	uint8_t DeviceId;  // The DeviceID to which credits are given
	uint16_t NewMtu;  // New MTU value to be configured
	uint16_t NewMps;  // New MPS value to be configured
	uint8_t NoOfChannels;  // Number of channels to reconfigure(max 5)
	uint8_t *Cids;  // The list of CIDs
} L2CAPCBEnhancedChannelReconfigureRequest_t;

/* Reason to refuse the channel creation */
typedef enum L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_tag {
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gSuccessful_c = 0x0000,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gLePsmNotSupported_c = 0x0002,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gNoResourcesAvailable_c = 0x0004,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gInsufficientAuthentication_c = 0x0005,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gInsufficientAuthorization_c = 0x0006,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gInsufficientEncryptionKeySize_c = 0x0007,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gInsufficientEncryption_c = 0x0008,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gInvalidSourceCid_c = 0x0009,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gSourceCidAlreadyAllocated_c = 0x000A,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gTimerError_c = 0x000B,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gCommandRejected_c = 0x0100,
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_gResponseTimeout_c = 0xFFFE
} L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_t;

typedef struct L2CAPCBEnhancedCancelConnectionRequest_tag {
	uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
	uint8_t DeviceId;  // The DeviceID to which credits are given
	L2CAPCBEnhancedCancelConnectionRequest_RefuseReason_t RefuseReason;  // Reason to refuse the channel creation
	uint8_t NoOfChannels;  // Number of channels to disconnect(max 5)
	uint8_t *Cids;  // The list of CIDs
} L2CAPCBEnhancedCancelConnectionRequest_t;

#endif  /* L2CAPCB_ENABLE */

#if GATT_ENABLE
typedef PACKED_STRUCT GATTGetMtuRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
} GATTGetMtuRequest_t;

typedef PACKED_STRUCT GATTClientExchangeMtuRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint16_t Mtu;  // Desired MTU size for the connected peer
} GATTClientExchangeMtuRequest_t;

typedef PACKED_STRUCT GATTClientDiscoverAllPrimaryServicesRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t MaxNbOfServices;  // Maximum number of services to be filled
} GATTClientDiscoverAllPrimaryServicesRequest_t;

/* UUID type */
typedef enum UuidType_tag {
	Uuid16Bits = 0x01,
	Uuid128Bits = 0x02,
	Uuid32Bits = 0x03
} UuidType_t;

typedef struct GATTClientDiscoverPrimaryServicesByUuidRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	uint8_t MaxNbOfServices;  // Maximum number of services to be filled
} GATTClientDiscoverPrimaryServicesByUuidRequest_t;

/* Characteristic Properties as defined by GATT */
typedef enum Properties_tag {
	gNone_c = 0x00,
	gBroadcast_c = 0x01,
	gRead_c = 0x02,
	gWriteWithoutRsp_c = 0x04,
	gWrite_c = 0x08,
	gNotify_c = 0x10,
	gIndicate_c = 0x20,
	gAuthSignedWrites_c = 0x40,
	gExtendedProperties_c = 0x80
} Properties_t;

typedef struct GATTClientFindIncludedServicesRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which inclusions should be searched
	uint8_t MaxNbOfIncludedServices;  // Maximum number of included services to be filled
} GATTClientFindIncludedServicesRequest_t;

typedef struct GATTClientDiscoverAllCharacteristicsOfServiceRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which characteristics should be searched
	uint8_t MaxNbOfCharacteristics;  // Maximum number of characteristics to be filled
} GATTClientDiscoverAllCharacteristicsOfServiceRequest_t;

typedef struct GATTClientDiscoverCharacteristicOfServiceByUuidRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which characteristics should be searched
	uint8_t MaxNbOfCharacteristics;  // Maximum number of characteristics to be filled
} GATTClientDiscoverCharacteristicOfServiceByUuidRequest_t;

typedef struct GATTClientDiscoverAllCharacteristicDescriptorsRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic within which descriptors should be searched
	uint16_t EndingHandle;  // The last handle of the Characteristic
	uint8_t MaxNbOfDescriptors;  // Maximum number of descriptors to be filled
} GATTClientDiscoverAllCharacteristicDescriptorsRequest_t;

typedef struct GATTClientReadCharacteristicValueRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic whose value must be read
	uint16_t MaxReadBytes;  // Maximum number of bytes to be read
} GATTClientReadCharacteristicValueRequest_t;

typedef struct GATTClientReadUsingCharacteristicUuidRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	bool_t HandleRangeIncluded;  // Boolean value which indicates if handle range is included or not
	struct {
		uint16_t StartHandle;  // Start handle
		uint16_t EndHandle;  // End handle
	} HandleRange;  // Handle range for the search
	uint16_t MaxReadBytes;  // Maximum number of bytes to be read
} GATTClientReadUsingCharacteristicUuidRequest_t;

typedef struct GATTClientReadMultipleCharacteristicValuesRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t NbOfCharacteristics;  // Number of characteristics in the array
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Array of the characteristics whose values are to be read
} GATTClientReadMultipleCharacteristicValuesRequest_t;

typedef struct GATTClientWriteCharacteristicValueRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic whose value must be written
	uint16_t ValueLength;  // Number of bytes to be written
	uint8_t *Value;  // Array of bytes to be written
	bool_t WithoutResponse;  // Indicates if a Write Command will be used
	bool_t SignedWrite;  // Indicates if a Signed Write will be performed
	bool_t ReliableLongCharWrites;  // Indicates Reliable Long Writes
	uint8_t Csrk[16];  // The CSRK if signedWrite is TRUE, ignored otherwise
} GATTClientWriteCharacteristicValueRequest_t;

typedef struct GATTClientReadCharacteristicDescriptorRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	struct {
		uint16_t Handle;  // Attribute handle
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint16_t ValueLength;  // Length of the attribute value array
		uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
		uint8_t Value[0];  // Attribute value array
	} Descriptor;  // The characteristic descriptor whose value must be read
	uint16_t MaxReadBytes;  // Maximum number of bytes to be read
} GATTClientReadCharacteristicDescriptorRequest_t;

typedef struct GATTClientWriteCharacteristicDescriptorRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	struct {
		uint16_t Handle;  // Attribute handle
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint16_t ValueLength;  // Length of the attribute value array
		uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
		uint8_t Value[0];  // Attribute value array
	} Descriptor;  // The characteristic descriptor whose value must be written
	uint16_t ValueLength;  // Number of bytes to be written
	uint8_t *Value;  // Array of bytes to be written
} GATTClientWriteCharacteristicDescriptorRequest_t;

typedef struct GATTServerRegisterHandlesForWriteNotificationsRequest_tag {
	uint8_t HandleCount;  // Number of handles in array
	uint16_t *AttributeHandles;

} GATTServerRegisterHandlesForWriteNotificationsRequest_t;

typedef PACKED_STRUCT GATTServerSendAttributeWrittenStatusRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t AttributeHandle;  // The attribute handle that was written
	uint8_t Status;  // The status of the write operation
} GATTServerSendAttributeWrittenStatusRequest_t;

typedef PACKED_STRUCT GATTServerSendNotificationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t Handle;  // Handle of the Value of the Characteristic to be notified
} GATTServerSendNotificationRequest_t;

typedef PACKED_STRUCT GATTServerSendIndicationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t Handle;  // Handle of the Value of the Characteristic to be indicated
} GATTServerSendIndicationRequest_t;

typedef struct GATTServerSendInstantValueNotificationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t Handle;  // Handle of the Value of the Characteristic to be notified
	uint16_t ValueLength;  // Length of data to be notified
	uint8_t *Value;  // Data to be notified
} GATTServerSendInstantValueNotificationRequest_t;

typedef struct GATTServerSendInstantValueIndicationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t Handle;  // Handle of the Value of the Characteristic to be indicated
	uint16_t ValueLength;  // Length of data to be indicated
	uint8_t *Value;  // Data to be indicated
} GATTServerSendInstantValueIndicationRequest_t;

typedef struct GATTServerSendMultipleHandleValueNotificationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t HandleCount;  // Number of handles to be notified
	struct {
		uint16_t Handle;  // Handle of the Value of the Characteristic to be indicated
		uint16_t ValueLength;  // Length of data to be indicated
		uint8_t *Value;  // Data to be indicated
	} *HandleLengthValueList;  // List of handle, length, value tuples
} GATTServerSendMultipleHandleValueNotificationRequest_t;

typedef struct GATTServerRegisterHandlesForReadNotificationsRequest_tag {
	uint8_t HandleCount;  // Number of handles in array
	uint16_t *AttributeHandles;

} GATTServerRegisterHandlesForReadNotificationsRequest_t;

typedef PACKED_STRUCT GATTServerSendAttributeReadStatusRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t AttributeHandle;  // The attribute handle that was read
	uint8_t Status;  // The status of the read operation
} GATTServerSendAttributeReadStatusRequest_t;

typedef PACKED_STRUCT GATTServerRegisterUniqueHandlesForNotificationsRequest_tag {
	bool_t bWrite;  // Boolean value which indicates unique handles are notified for write or not
	bool_t bRead;  // Boolean value which indicates unique handles are notified for read or not
} GATTServerRegisterUniqueHandlesForNotificationsRequest_t;

typedef struct GATTServerUnregisterHandlesForWriteNotificationsRequest_tag {
	uint8_t HandleCount;  // Number of handles in array
	uint16_t *AttributeHandles;

} GATTServerUnregisterHandlesForWriteNotificationsRequest_t;

typedef struct GATTServerUnregisterHandlesForReadNotificationsRequest_tag {
	uint8_t HandleCount;  // Number of handles in array
	uint16_t *AttributeHandles;

} GATTServerUnregisterHandlesForReadNotificationsRequest_t;

typedef struct GATTClientReadMultipleVariableCharacteristicValuesRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t NbOfCharacteristics;  // Number of characteristics in the array
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Array of the characteristics whose values are to be read
} GATTClientReadMultipleVariableCharacteristicValuesRequest_t;

typedef PACKED_STRUCT GATTClientEnhancedDiscoverAllPrimaryServicesRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint8_t MaxNbOfServices;  // Maximum number of services to be filled
} GATTClientEnhancedDiscoverAllPrimaryServicesRequest_t;

typedef struct GATTClientEnhancedDiscoverPrimaryServicesByUuidRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	uint8_t MaxNbOfServices;  // Maximum number of services to be filled
} GATTClientEnhancedDiscoverPrimaryServicesByUuidRequest_t;

typedef struct GATTClientEnhancedFindIncludedServicesRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which inclusions should be searched
	uint8_t MaxNbOfIncludedServices;  // Maximum number of included services to be filled
} GATTClientEnhancedFindIncludedServicesRequest_t;

typedef struct GATTClientEnhancedDiscoverAllCharacteristicsOfServiceRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which characteristics should be searched
	uint8_t MaxNbOfCharacteristics;  // Maximum number of characteristics to be filled
} GATTClientEnhancedDiscoverAllCharacteristicsOfServiceRequest_t;

typedef struct GATTClientEnhancedDiscoverCharacteristicOfServiceByUuidRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t Value[0];  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which characteristics should be searched
	uint8_t MaxNbOfCharacteristics;  // Maximum number of characteristics to be filled
} GATTClientEnhancedDiscoverCharacteristicOfServiceByUuidRequest_t;

typedef struct GATTClientEnhancedDiscoverAllCharacteristicDescriptorsRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic within which descriptors should be searched
	uint16_t EndingHandle;  // The last handle of the Characteristic
	uint8_t MaxNbOfDescriptors;  // Maximum number of descriptors to be filled
} GATTClientEnhancedDiscoverAllCharacteristicDescriptorsRequest_t;

typedef struct GATTClientEnhancedReadCharacteristicValueRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic whose value must be read
	uint16_t MaxReadBytes;  // Maximum number of bytes to be read
} GATTClientEnhancedReadCharacteristicValueRequest_t;

typedef struct GATTClientEnhancedReadUsingCharacteristicUuidRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	bool_t HandleRangeIncluded;  // Boolean value which indicates if handle range is included or not
	struct {
		uint16_t StartHandle;  // Start handle
		uint16_t EndHandle;  // End handle
	} HandleRange;  // Handle range for the search
	uint16_t MaxReadBytes;  // Maximum number of bytes to be read
} GATTClientEnhancedReadUsingCharacteristicUuidRequest_t;

typedef struct GATTClientEnhancedReadMultipleCharacteristicValuesRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint8_t NbOfCharacteristics;  // Number of characteristics in the array
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Array of the characteristics whose values are to be read
} GATTClientEnhancedReadMultipleCharacteristicValuesRequest_t;

typedef struct GATTClientEnhancedWriteCharacteristicValueRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic whose value must be written
	uint16_t ValueLength;  // Number of bytes to be written
	uint8_t *Value;  // Array of bytes to be written
	bool_t WithoutResponse;  // Indicates if a Write Command will be used
	bool_t SignedWrite;  // Indicates if a Signed Write will be performed
	bool_t ReliableLongCharWrites;  // Indicates Reliable Long Writes
	uint8_t Csrk[16];  // The CSRK if signedWrite is TRUE, ignored otherwise
} GATTClientEnhancedWriteCharacteristicValueRequest_t;

typedef struct GATTClientEnhancedReadCharacteristicDescriptorRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	struct {
		uint16_t Handle;  // Attribute handle
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint16_t ValueLength;  // Length of the attribute value array
		uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
		uint8_t Value[0];  // Attribute value array
	} Descriptor;  // The characteristic descriptor whose value must be read
	uint16_t MaxReadBytes;  // Maximum number of bytes to be read
} GATTClientEnhancedReadCharacteristicDescriptorRequest_t;

typedef struct GATTClientEnhancedWriteCharacteristicDescriptorRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	struct {
		uint16_t Handle;  // Attribute handle
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint16_t ValueLength;  // Length of the attribute value array
		uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
		uint8_t Value[0];  // Attribute value array
	} Descriptor;  // The characteristic descriptor whose value must be written
	uint16_t ValueLength;  // Number of bytes to be written
	uint8_t *Value;  // Array of bytes to be written
} GATTClientEnhancedWriteCharacteristicDescriptorRequest_t;

typedef struct GATTClientEnhancedReadMultipleVariableCharacteristicValuesRequest_tag {
	uint8_t DeviceId;  // Device ID of the connected peer
	uint8_t BearerId;  // Enhanced ATT bearer ID of the connected peer
	uint8_t NbOfCharacteristics;  // Number of characteristics in the array
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t Value[0];  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Array of the characteristics whose values are to be read
} GATTClientEnhancedReadMultipleVariableCharacteristicValuesRequest_t;

typedef PACKED_STRUCT GATTServerEnhancedSendAttributeWrittenStatusRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint16_t AttributeHandle;  // The attribute handle that was written
	uint8_t Status;  // The status of the write operation
} GATTServerEnhancedSendAttributeWrittenStatusRequest_t;

typedef PACKED_STRUCT GATTServerEnhancedSendNotificationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint16_t Handle;  // Handle of the Value of the Characteristic to be notified
} GATTServerEnhancedSendNotificationRequest_t;

typedef PACKED_STRUCT GATTServerEnhancedSendIndicationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint16_t Handle;  // Handle of the Value of the Characteristic to be indicated
} GATTServerEnhancedSendIndicationRequest_t;

typedef struct GATTServerEnhancedSendInstantValueNotificationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint16_t Handle;  // Handle of the Value of the Characteristic to be notified
	uint16_t ValueLength;  // Length of data to be notified
	uint8_t *Value;  // Data to be notified
} GATTServerEnhancedSendInstantValueNotificationRequest_t;

typedef struct GATTServerEnhancedSendInstantValueIndicationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint16_t Handle;  // Handle of the Value of the Characteristic to be indicated
	uint16_t ValueLength;  // Length of data to be indicated
	uint8_t *Value;  // Data to be indicated
} GATTServerEnhancedSendInstantValueIndicationRequest_t;

typedef PACKED_STRUCT GATTServerEnhancedSendAttributeReadStatusRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint16_t AttributeHandle;  // The attribute handle that was read
	uint8_t Status;  // The status of the read operation
} GATTServerEnhancedSendAttributeReadStatusRequest_t;

typedef struct GATTServerEnhancedSendMultipleHandleValueNotificationRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BearerId;  // The Bearer ID of the ATT Bearer used
	uint8_t TotalLength[3];  // Length of the handle, value, value length tuples
	uint8_t *HandleLengthValueList;  // List of handle, value, value length tuples
} GATTServerEnhancedSendMultipleHandleValueNotificationRequest_t;

typedef PACKED_STRUCT GATTClientGetDatabaseHashRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint8_t BondIdx;  // Index of the bond in NVM
} GATTClientGetDatabaseHashRequest_t;

#endif  /* GATT_ENABLE */

#if GATTDB_APP_ENABLE
typedef struct GATTDBWriteAttributeRequest_tag {
	uint16_t Handle;  // The handle of the attribute to be written
	uint16_t ValueLength;  // The number of bytes to be written
	uint8_t *Value;  // The value to be written
} GATTDBWriteAttributeRequest_t;

typedef PACKED_STRUCT GATTDBReadAttributeRequest_tag {
	uint16_t Handle;  // The handle of the attribute to be read
	uint16_t MaxBytes;  // The maximum number of bytes to be read
} GATTDBReadAttributeRequest_t;

typedef struct GATTDBFindServiceHandleRequest_tag {
	uint16_t StartHandle;  // The handle to start the search. Should be 0x0001 on the first call
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
} GATTDBFindServiceHandleRequest_t;

typedef struct GATTDBFindCharValueHandleInServiceRequest_tag {
	uint16_t ServiceHandle;  // The handle of the Service declaration
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
} GATTDBFindCharValueHandleInServiceRequest_t;

typedef PACKED_STRUCT GATTDBFindCccdHandleForCharValueHandleRequest_tag {
	uint16_t CharValueHandle;  // The handle of the characteristic value
} GATTDBFindCccdHandleForCharValueHandleRequest_t;

typedef struct GATTDBFindDescriptorHandleForCharValueHandleRequest_tag {
	uint16_t CharValueHandle;  // The handle of the characteristic value
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
} GATTDBFindDescriptorHandleForCharValueHandleRequest_t;

typedef struct GATTDBDynamicAddPrimaryServiceDeclarationRequest_tag {
	uint16_t DesiredHandle;  // The desired handle for the Primary Service declaration
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
} GATTDBDynamicAddPrimaryServiceDeclarationRequest_t;

typedef struct GATTDBDynamicAddSecondaryServiceDeclarationRequest_tag {
	uint16_t DesiredHandle;  // The desired handle for the Secondary Service declaration
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
} GATTDBDynamicAddSecondaryServiceDeclarationRequest_t;

typedef struct GATTDBDynamicAddIncludeDeclarationRequest_tag {
	uint16_t IncludedServiceHandle;  // Handle of the included Service declaration
	uint16_t EndGroupHandle;  // Last handle of the included Service
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
} GATTDBDynamicAddIncludeDeclarationRequest_t;

/* Characteristic properties */
typedef enum GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_tag {
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gNone_c = 0x00,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gBroadcast_c = 0x01,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gRead_c = 0x02,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gWriteWithoutRsp_c = 0x04,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gWrite_c = 0x08,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gNotify_c = 0x10,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gIndicate_c = 0x20,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gAuthSignedWrites_c = 0x40,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_gExtendedProperties_c = 0x80
} GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_t;

/* Access permissions for the value attribute */
typedef enum GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_tag {
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionNone_c = 0x00,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagReadable_c = 0x01,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagReadWithEncryption_c = 0x02,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagReadWithAuthentication_c = 0x04,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagReadWithAuthorization_c = 0x08,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagWritable_c = 0x10,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagWriteWithEncryption_c = 0x20,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagWriteWithAuthentication_c = 0x40,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_gPermissionFlagWriteWithAuthorization_c = 0x80
} GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_t;

typedef struct GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_tag {
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_CharacteristicProperties_t CharacteristicProperties;  // Characteristic properties
	uint16_t MaxValueLength;  // If the Characteristic Value length is variable, this is the maximum length; for fixed lengths this must be set to 0
	uint16_t InitialValueLength;  // Value length at initialization; remains fixed if maxValueLength is set to 0, otherwise cannot be greater than maxValueLength
	uint8_t *InitialValue;  // Contains the initial value of the Characteristic
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_ValueAccessPermissions_t ValueAccessPermissions;  // Access permissions for the value attribute
} GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_t;

/* Access permissions for the descriptor attribute */
typedef enum GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_tag {
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionNone_c = 0x00,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagReadable_c = 0x01,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagReadWithEncryption_c = 0x02,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagReadWithAuthentication_c = 0x04,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagReadWithAuthorization_c = 0x08,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagWritable_c = 0x10,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagWriteWithEncryption_c = 0x20,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagWriteWithAuthentication_c = 0x40,
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_gPermissionFlagWriteWithAuthorization_c = 0x80
} GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_t;

typedef struct GATTDBDynamicAddCharacteristicDescriptorRequest_tag {
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	uint16_t DescriptorValueLength;  // Fixed length of the descriptor attribute value
	uint8_t *Value;  // Initial value of the descriptor attribute
	GATTDBDynamicAddCharacteristicDescriptorRequest_DescriptorAccessPermissions_t DescriptorAccessPermissions;  // Access permissions for the descriptor attribute
} GATTDBDynamicAddCharacteristicDescriptorRequest_t;

/* Characteristic properties */
typedef enum GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_tag {
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gNone_c = 0x00,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gBroadcast_c = 0x01,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gRead_c = 0x02,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gWriteWithoutRsp_c = 0x04,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gWrite_c = 0x08,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gNotify_c = 0x10,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gIndicate_c = 0x20,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gAuthSignedWrites_c = 0x40,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_gExtendedProperties_c = 0x80
} GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_t;

/* Access permissions for the value attribute */
typedef enum GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_tag {
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionNone_c = 0x00,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagReadable_c = 0x01,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagReadWithEncryption_c = 0x02,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagReadWithAuthentication_c = 0x04,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagReadWithAuthorization_c = 0x08,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagWritable_c = 0x10,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagWriteWithEncryption_c = 0x20,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagWriteWithAuthentication_c = 0x40,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_gPermissionFlagWriteWithAuthorization_c = 0x80
} GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_t;

typedef struct GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_tag {
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_CharacteristicProperties_t CharacteristicProperties;  // Characteristic properties
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_ValueAccessPermissions_t ValueAccessPermissions;  // Access permissions for the value attribute
} GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_t;

typedef PACKED_STRUCT GATTDBDynamicRemoveServiceRequest_tag {
	uint16_t ServiceHandle;  // Attribute handle of the Service declaration
} GATTDBDynamicRemoveServiceRequest_t;

typedef PACKED_STRUCT GATTDBDynamicRemoveCharacteristicRequest_tag {
	uint16_t CharacteristicHandle;  // Attribute handle of the Characteristic declaration
} GATTDBDynamicRemoveCharacteristicRequest_t;

/* Access permissions for the descriptor attribute */
typedef enum GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_tag {
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionNone_c = 0x00,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagReadable_c = 0x01,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagReadWithEncryption_c = 0x02,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagReadWithAuthentication_c = 0x04,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagReadWithAuthorization_c = 0x08,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagWritable_c = 0x10,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagWriteWithEncryption_c = 0x20,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagWriteWithAuthentication_c = 0x40,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_gPermissionFlagWriteWithAuthorization_c = 0x80
} GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_t;

typedef struct GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_tag {
	UuidType_t UuidType;  // UUID type
	union {
		uint8_t Uuid16Bits[2];  // UUID 16 bits
		uint8_t Uuid128Bits[16];  // UUID 128 bits
		uint8_t Uuid32Bits[4];  // UUID 32 bits
	} Uuid;  // UUID value
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_DescriptorAccessPermissions_t DescriptorAccessPermissions;  // Access permissions for the descriptor attribute
} GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_t;

#endif  /* GATTDB_APP_ENABLE */

#if GATTDB_ATT_ENABLE
typedef PACKED_STRUCT GATTDBAttFindInformationRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t StartingHandle;  // Starting handle
		uint16_t EndingHandle;  // Ending handle
	} Params;  // ATT find information request parameters
} GATTDBAttFindInformationRequest_t;

typedef struct GATTDBAttFindByTypeValueRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t StartingHandle;  // Starting handle
		uint16_t EndingHandle;  // Ending handle
		uint16_t AttributeType;  // Attribute type
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT find by type value request parameters
} GATTDBAttFindByTypeValueRequest_t;

/* ATT Uuid format */
typedef enum GATTDBAttReadByTypeRequest_Params_Format_tag {
	GATTDBAttReadByTypeRequest_Params_Format_Uuid16BitFormat = 0x01,
	GATTDBAttReadByTypeRequest_Params_Format_Uuid128BitFormat = 0x02
} GATTDBAttReadByTypeRequest_Params_Format_t;

typedef struct GATTDBAttReadByTypeRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t StartingHandle;  // Starting handle
		uint16_t EndingHandle;  // Ending handle
		GATTDBAttReadByTypeRequest_Params_Format_t Format;  // ATT Uuid format
		union {
			uint16_t Uuid16BitFormat;  // Uuid 16 bits format
			uint8_t Uuid128BitFormat[16];  // Uuid 128 bits format
		} AttributeType;  // Attribute type
	} Params;  // ATT read by type request parameters
} GATTDBAttReadByTypeRequest_t;

typedef PACKED_STRUCT GATTDBAttReadRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint16_t Params_AttributeHandle;

} GATTDBAttReadRequest_t;

typedef PACKED_STRUCT GATTDBAttReadBlobRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t ValueOffset;  // Value offset
	} Params;  // ATT read blob request parameters
} GATTDBAttReadBlobRequest_t;

typedef struct GATTDBAttReadMultipleRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t HandleCount;  // Number of handles
		uint16_t *ListOfHandles;

	} Params;  // ATT read multiple request parameters
} GATTDBAttReadMultipleRequest_t;

/* ATT Uuid format */
typedef enum GATTDBAttReadByGroupTypeRequest_Params_Format_tag {
	GATTDBAttReadByGroupTypeRequest_Params_Format_Uuid16BitFormat = 0x01,
	GATTDBAttReadByGroupTypeRequest_Params_Format_Uuid128BitFormat = 0x02
} GATTDBAttReadByGroupTypeRequest_Params_Format_t;

typedef struct GATTDBAttReadByGroupTypeRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t StartingHandle;  // Starting handle
		uint16_t EndingHandle;  // Ending handle
		GATTDBAttReadByGroupTypeRequest_Params_Format_t Format;  // ATT Uuid format
		union {
			uint16_t Uuid16BitFormat;  // Uuid 16 bits format
			uint8_t Uuid128BitFormat[16];  // Uuid 128 bits format
		} AttributeType;  // Attribute type
	} Params;  // ATT read by group type request parameters
} GATTDBAttReadByGroupTypeRequest_t;

typedef struct GATTDBAttWriteRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT write request and write command parameters
} GATTDBAttWriteRequest_t;

typedef struct GATTDBAttWriteCommandRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT write request and write command parameters
} GATTDBAttWriteCommandRequest_t;

typedef struct GATTDBAttSignedWriteCommandRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
		uint8_t AuthenticationSignature[12];  // Authentication signature
	} Params;  // ATT signed write command parameters
} GATTDBAttSignedWriteCommandRequest_t;

typedef struct GATTDBAttPrepareWriteRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t ValueOffset;  // Value offset
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT prepare write request parameters
} GATTDBAttPrepareWriteRequest_t;

/* Flags */
typedef enum GATTDBAttExecuteWriteRequest_Params_Flags_tag {
	GATTDBAttExecuteWriteRequest_Params_Flags_gCancel_c = 0x00,
	GATTDBAttExecuteWriteRequest_Params_Flags_gImmediately_c = 0x01
} GATTDBAttExecuteWriteRequest_Params_Flags_t;

typedef PACKED_STRUCT GATTDBAttExecuteWriteRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	GATTDBAttExecuteWriteRequest_Params_Flags_t Params_Flags;

} GATTDBAttExecuteWriteRequest_t;

typedef struct GATTDBAttExecuteWriteFromQueueRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t ValueOffset;  // Value offset
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT prepare write request parameters
} GATTDBAttExecuteWriteFromQueueRequest_t;

typedef struct GATTDBAttPrepareNotificationIndicationRequest_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT handle value notification/indication parameters
} GATTDBAttPrepareNotificationIndicationRequest_t;

#endif  /* GATTDB_ATT_ENABLE */

#if GAP_ENABLE
/* Security mode and level */
typedef enum GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_tag {
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_gMode1Level1_c = 0x10,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_gMode1Level2_c = 0x11,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_gMode1Level3_c = 0x12,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_gMode1Level4_c = 0x13,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_gMode2Level1_c = 0x21,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_gMode2Level2_c = 0x22
} GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_t;

/* Security mode and level */
typedef enum GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_tag {
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_gMode1Level1_c = 0x10,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_gMode1Level2_c = 0x11,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_gMode1Level3_c = 0x12,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_gMode1Level4_c = 0x13,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_gMode2Level1_c = 0x21,
	GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_gMode2Level2_c = 0x22
} GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_t;

typedef struct GAPRegisterDeviceSecurityRequirementsRequest_tag {
	bool_t SecurityRequirementsIncluded;  // Boolean value which indicates if security requirements are included or not
	struct {
		struct {
			GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_CentralSecurityRequirements_SecurityModeLevel_t SecurityModeLevel;  // Security mode and level
			bool_t Authorization;  // Authorization required
			uint16_t MinimumEncryptionKeySize;  // Minimum encryption key (LTK) size
		} CentralSecurityRequirements;  // Security requirements added to all services
		uint8_t NbOfServices;  // Number of service-specific requirements
		struct {
			uint16_t ServiceHandle;  // Handle of the Service declaration in the GATT Database
			struct {
				GAPRegisterDeviceSecurityRequirementsRequest_SecurityRequirements_GapServiceSecurityRequirements_Requirements_SecurityModeLevel_t SecurityModeLevel;  // Security mode and level
				bool_t Authorization;  // Authorization required
				uint16_t MinimumEncryptionKeySize;  // Minimum encryption key (LTK) size
			} Requirements;  // Requirements for all attributes in this service
		} *GapServiceSecurityRequirements;  // Array of service-specific requirements
	} SecurityRequirements;  // Device security requirements
} GAPRegisterDeviceSecurityRequirementsRequest_t;

/* Advertising type - default connectable undirected */
typedef enum GAPSetAdvertisingParametersRequest_AdvertisingType_tag {
	GAPSetAdvertisingParametersRequest_AdvertisingType_gConnectableUndirected_c = 0x00,
	GAPSetAdvertisingParametersRequest_AdvertisingType_gDirectedHighDutyCycle_c = 0x01,
	GAPSetAdvertisingParametersRequest_AdvertisingType_gScannable_c = 0x02,
	GAPSetAdvertisingParametersRequest_AdvertisingType_gNonConnectable_c = 0x03,
	GAPSetAdvertisingParametersRequest_AdvertisingType_gDirectedLowDutyCycle_c = 0x04
} GAPSetAdvertisingParametersRequest_AdvertisingType_t;

/* Indicates whether the advertising address will be the public address or the random address */
typedef enum GAPSetAdvertisingParametersRequest_OwnAddressType_tag {
	GAPSetAdvertisingParametersRequest_OwnAddressType_gPublic_c = 0x00,
	GAPSetAdvertisingParametersRequest_OwnAddressType_gRandom_c = 0x01
} GAPSetAdvertisingParametersRequest_OwnAddressType_t;

/* Address type of the peer; only used in directed advertising and Enhanced Privacy */
typedef enum GAPSetAdvertisingParametersRequest_PeerAddressType_tag {
	GAPSetAdvertisingParametersRequest_PeerAddressType_gPublic_c = 0x00,
	GAPSetAdvertisingParametersRequest_PeerAddressType_gRandom_c = 0x01
} GAPSetAdvertisingParametersRequest_PeerAddressType_t;

/* Indicates whether the connect and scan requests are filtered using the Filter Accept List - default does not use Filter Accept List */
typedef enum GAPSetAdvertisingParametersRequest_FilterPolicy_tag {
	GAPSetAdvertisingParametersRequest_FilterPolicy_gProcessAll_c = 0x00,
	GAPSetAdvertisingParametersRequest_FilterPolicy_gProcessConnAllScanWL_c = 0x01,
	GAPSetAdvertisingParametersRequest_FilterPolicy_gProcessScanAllConnWL_c = 0x02,
	GAPSetAdvertisingParametersRequest_FilterPolicy_gProcessFilterAcceptListOnly_c = 0x03
} GAPSetAdvertisingParametersRequest_FilterPolicy_t;

typedef PACKED_STRUCT GAPSetAdvertisingParametersRequest_tag {
	uint16_t MinInterval;  // Minimum desired advertising interval - default 1.28s
	uint16_t MaxInterval;  // Maximum desired advertising interval - default 1.28s
	GAPSetAdvertisingParametersRequest_AdvertisingType_t AdvertisingType;  // Advertising type - default connectable undirected
	GAPSetAdvertisingParametersRequest_OwnAddressType_t OwnAddressType;  // Indicates whether the advertising address will be the public address or the random address
	GAPSetAdvertisingParametersRequest_PeerAddressType_t PeerAddressType;  // Address type of the peer; only used in directed advertising and Enhanced Privacy
	uint8_t PeerAddress[6];  // Address of the peer
	uint8_t ChannelMap;  // 0x04|0x02|0x01  // Bit mask indicating which of the three advertising channels will be used - default all three
	GAPSetAdvertisingParametersRequest_FilterPolicy_t FilterPolicy;  // Indicates whether the connect and scan requests are filtered using the Filter Accept List - default does not use Filter Accept List
} GAPSetAdvertisingParametersRequest_t;

/* Advertising data type */
typedef enum GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_tag {
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdFlags_c = 0x01,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdIncomplete16bitServiceList_c = 0x02,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdComplete16bitServiceList_c = 0x03,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdIncomplete32bitServiceList_c = 0x04,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdComplete32bitServiceList_c = 0x05,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdIncomplete128bitServiceList_c = 0x06,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdComplete128bitServiceList_c = 0x07,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdShortenedLocalName_c = 0x08,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdCompleteLocalName_c = 0x09,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdTxPowerLevel_c = 0x0A,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdClassOfDevice_c = 0x0D,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdSimplePairingHashC192_c = 0x0E,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdSimplePairingRandomizerR192_c = 0x0F,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdSecurityManagerTkValue_c = 0x10,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdSecurityManagerOobFlags_c = 0x11,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdPeripheralConnectionIntervalRange_c = 0x12,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdServiceSolicitationList16bit_c = 0x14,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdServiceSolicitationList128bit_c = 0x15,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdServiceData16bit_c = 0x16,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdPublicTargetAddress_c = 0x17,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdRandomTargetAddress_c = 0x18,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdAppearance_c = 0x19,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdAdvertisingInterval_c = 0x1A,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdLeDeviceAddress_c = 0x1B,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdLeRole_c = 0x1C,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdSimplePairingHashC256_c = 0x1D,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdSimplePairingRandomizerR256_c = 0x1E,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdServiceSolicitationList32bit_c = 0x1F,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdServiceData32bit_c = 0x20,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdServiceData128bit_c = 0x21,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdUniformResourceIdentifier_c = 0x24,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAd3dInformationData_c = 0x3D,
	GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_gAdManufacturerSpecificData_c = 0xFF
} GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_t;

/* Advertising data type */
typedef enum GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_tag {
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdFlags_c = 0x01,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdIncomplete16bitServiceList_c = 0x02,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdComplete16bitServiceList_c = 0x03,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdIncomplete32bitServiceList_c = 0x04,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdComplete32bitServiceList_c = 0x05,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdIncomplete128bitServiceList_c = 0x06,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdComplete128bitServiceList_c = 0x07,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdShortenedLocalName_c = 0x08,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdCompleteLocalName_c = 0x09,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdTxPowerLevel_c = 0x0A,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdClassOfDevice_c = 0x0D,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdSimplePairingHashC192_c = 0x0E,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdSimplePairingRandomizerR192_c = 0x0F,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdSecurityManagerTkValue_c = 0x10,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdSecurityManagerOobFlags_c = 0x11,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdPeripheralConnectionIntervalRange_c = 0x12,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdServiceSolicitationList16bit_c = 0x14,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdServiceSolicitationList128bit_c = 0x15,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdServiceData16bit_c = 0x16,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdPublicTargetAddress_c = 0x17,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdRandomTargetAddress_c = 0x18,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdAppearance_c = 0x19,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdAdvertisingInterval_c = 0x1A,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdLeDeviceAddress_c = 0x1B,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdLeRole_c = 0x1C,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdSimplePairingHashC256_c = 0x1D,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdSimplePairingRandomizerR256_c = 0x1E,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdServiceSolicitationList32bit_c = 0x1F,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdServiceData32bit_c = 0x20,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdServiceData128bit_c = 0x21,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdUniformResourceIdentifier_c = 0x24,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAd3dInformationData_c = 0x3D,
	GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_gAdManufacturerSpecificData_c = 0xFF
} GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_t;

typedef struct GAPSetAdvertisingDataRequest_tag {
	bool_t AdvertisingDataIncluded;  // Boolean value which indicates if advertising data is included or not
	struct {
		uint8_t NbOfAdStructures;  // Number of advertising data structures
		struct {
			uint8_t Length;  // Advertising data length
			GAPSetAdvertisingDataRequest_AdvertisingData_AdStructures_Type_t Type;  // Advertising data type
			uint8_t *Data;  // Advertising data
		} *AdStructures;  // Advertising data structures
	} AdvertisingData;  // Advertising data
	bool_t ScanResponseDataIncluded;  // Boolean value which indicates if scan response data is included or not
	struct {
		uint8_t NbOfAdStructures;  // Number of advertising data structures
		struct {
			uint8_t Length;  // Advertising data length
			GAPSetAdvertisingDataRequest_ScanResponseData_AdStructures_Type_t Type;  // Advertising data type
			uint8_t *Data;  // Advertising data
		} *AdStructures;  // Advertising data structures
	} ScanResponseData;  // Scan response data
} GAPSetAdvertisingDataRequest_t;

/* GATT Database access type (the type of access granted - read or write) */
typedef enum GAPAuthorizeRequest_Access_tag {
	GAPAuthorizeRequest_Access_gRead_c = 0x00,
	GAPAuthorizeRequest_Access_gWrite_c = 0x01
} GAPAuthorizeRequest_Access_t;

typedef PACKED_STRUCT GAPAuthorizeRequest_tag {
	uint8_t DeviceId;  // The peer being authorized
	uint16_t Handle;  // The attribute handle
	GAPAuthorizeRequest_Access_t Access;  // GATT Database access type (the type of access granted - read or write)
} GAPAuthorizeRequest_t;

/* The bit mask representing the CCCD value to be saved */
typedef enum GAPSaveCccdRequest_Cccd_tag {
	GAPSaveCccdRequest_Cccd_gEmpty_c = 0x00,
	GAPSaveCccdRequest_Cccd_gNotification_c = 0x01,
	GAPSaveCccdRequest_Cccd_gIndication_c = 0x02
} GAPSaveCccdRequest_Cccd_t;

typedef PACKED_STRUCT GAPSaveCccdRequest_tag {
	uint8_t DeviceId;  // The peer GATT client
	uint16_t Handle;  // The handle of the CCCD as defined in the GATT Database
	GAPSaveCccdRequest_Cccd_t Cccd;  // The bit mask representing the CCCD value to be saved
} GAPSaveCccdRequest_t;

typedef PACKED_STRUCT GAPCheckNotificationStatusRequest_tag {
	uint8_t DeviceId;  // The peer GATT client
	uint16_t Handle;  // The handle of the CCCD
} GAPCheckNotificationStatusRequest_t;

typedef PACKED_STRUCT GAPCheckIndicationStatusRequest_tag {
	uint8_t DeviceId;  // The peer GATT client
	uint16_t Handle;  // The handle of the CCCD
} GAPCheckIndicationStatusRequest_t;

/* The desired security mode-level */
typedef enum GAPPairRequest_PairingParameters_SecurityModeAndLevel_tag {
	GAPPairRequest_PairingParameters_SecurityModeAndLevel_gMode1Level1_c = 0x10,
	GAPPairRequest_PairingParameters_SecurityModeAndLevel_gMode1Level2_c = 0x11,
	GAPPairRequest_PairingParameters_SecurityModeAndLevel_gMode1Level3_c = 0x12,
	GAPPairRequest_PairingParameters_SecurityModeAndLevel_gMode1Level4_c = 0x13,
	GAPPairRequest_PairingParameters_SecurityModeAndLevel_gMode2Level1_c = 0x21,
	GAPPairRequest_PairingParameters_SecurityModeAndLevel_gMode2Level2_c = 0x22
} GAPPairRequest_PairingParameters_SecurityModeAndLevel_t;

/* I/O capabilities used to determine the pairing method */
typedef enum GAPPairRequest_PairingParameters_LocalIoCapabilities_tag {
	GAPPairRequest_PairingParameters_LocalIoCapabilities_gIoDisplayOnly_c = 0x00,
	GAPPairRequest_PairingParameters_LocalIoCapabilities_gIoDisplayYesNo_c = 0x01,
	GAPPairRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardOnly_c = 0x02,
	GAPPairRequest_PairingParameters_LocalIoCapabilities_gIoNone_c = 0x03,
	GAPPairRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardDisplay_c = 0x04
} GAPPairRequest_PairingParameters_LocalIoCapabilities_t;

typedef PACKED_STRUCT GAPPairRequest_tag {
	uint8_t DeviceId;  // The peer to pair with
	struct {
		bool_t WithBonding;  // TRUE if this device is able to and wants to bond after pairing, FALSE otherwise
		GAPPairRequest_PairingParameters_SecurityModeAndLevel_t SecurityModeAndLevel;  // The desired security mode-level
		uint8_t MaxEncryptionKeySize;  // Maximum LTK size supported by the device
		GAPPairRequest_PairingParameters_LocalIoCapabilities_t LocalIoCapabilities;  // I/O capabilities used to determine the pairing method
		bool_t OobAvailable;  // TRUE if this device has Out-of-Band data that can be used for authenticated pairing, FALSE otherwise
		uint8_t CentralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Central
		uint8_t PeripheralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Peripheral
		bool_t LeSecureConnectionSupported;  // indicates if device supports LE Secure Connections pairing
		bool_t UseKeypressNotifications;  // Indicates if device supports Keypress Notification PDUs during Passkey Entry pairing
	} PairingParameters;  // Pairing parameters as required by the SMP
} GAPPairRequest_t;

/* The desired security mode-level */
typedef enum GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_tag {
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level1_c = 0x10,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level2_c = 0x11,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level3_c = 0x12,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level4_c = 0x13,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_gMode2Level1_c = 0x21,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_gMode2Level2_c = 0x22
} GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_t;

/* I/O capabilities used to determine the pairing method */
typedef enum GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_tag {
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_gIoDisplayOnly_c = 0x00,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_gIoDisplayYesNo_c = 0x01,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardOnly_c = 0x02,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_gIoNone_c = 0x03,
	GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardDisplay_c = 0x04
} GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_t;

typedef PACKED_STRUCT GAPSendPeripheralSecurityRequestRequest_tag {
	uint8_t DeviceId;  // The GAP peer to pair with
	struct {
		bool_t WithBonding;  // TRUE if this device is able to and wants to bond after pairing, FALSE otherwise
		GAPSendPeripheralSecurityRequestRequest_PairingParameters_SecurityModeAndLevel_t SecurityModeAndLevel;  // The desired security mode-level
		uint8_t MaxEncryptionKeySize;  // Maximum LTK size supported by the device
		GAPSendPeripheralSecurityRequestRequest_PairingParameters_LocalIoCapabilities_t LocalIoCapabilities;  // I/O capabilities used to determine the pairing method
		bool_t OobAvailable;  // TRUE if this device has Out-of-Band data that can be used for authenticated pairing, FALSE otherwise
		uint8_t CentralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Central
		uint8_t PeripheralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Peripheral
		bool_t LeSecureConnectionSupported;  // indicates if device supports LE Secure Connections pairing
		bool_t UseKeypressNotifications;  // Indicates if device supports Keypress Notification PDUs during Passkey Entry pairing
	} PairingParameters;  // Pairing parameters as required by the SMP (MaxEncryptionKeySize, CentralKeys and PeripheralKeys are not used by this command)
} GAPSendPeripheralSecurityRequestRequest_t;

typedef PACKED_STRUCT GAPEncryptLinkRequest_tag {
	uint8_t DeviceId;  // Device ID of the peer
} GAPEncryptLinkRequest_t;

/* The desired security mode-level */
typedef enum GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_tag {
	GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level1_c = 0x10,
	GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level2_c = 0x11,
	GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level3_c = 0x12,
	GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_gMode1Level4_c = 0x13,
	GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_gMode2Level1_c = 0x21,
	GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_gMode2Level2_c = 0x22
} GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_t;

/* I/O capabilities used to determine the pairing method */
typedef enum GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_tag {
	GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_gIoDisplayOnly_c = 0x00,
	GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_gIoDisplayYesNo_c = 0x01,
	GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardOnly_c = 0x02,
	GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_gIoNone_c = 0x03,
	GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardDisplay_c = 0x04
} GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_t;

typedef PACKED_STRUCT GAPAcceptPairingRequestRequest_tag {
	uint8_t DeviceId;  // The peer requesting authentication
	struct {
		bool_t WithBonding;  // TRUE if this device is able to and wants to bond after pairing, FALSE otherwise
		GAPAcceptPairingRequestRequest_PairingParameters_SecurityModeAndLevel_t SecurityModeAndLevel;  // The desired security mode-level
		uint8_t MaxEncryptionKeySize;  // Maximum LTK size supported by the device
		GAPAcceptPairingRequestRequest_PairingParameters_LocalIoCapabilities_t LocalIoCapabilities;  // I/O capabilities used to determine the pairing method
		bool_t OobAvailable;  // TRUE if this device has Out-of-Band data that can be used for authenticated pairing, FALSE otherwise
		uint8_t CentralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Central
		uint8_t PeripheralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Peripheral
		bool_t LeSecureConnectionSupported;  // indicates if device supports LE Secure Connections pairing
		bool_t UseKeypressNotifications;  // Indicates if device supports Keypress Notification PDUs during Passkey Entry pairing
	} PairingParameters;  // Pairing parameters as required by the SMP
} GAPAcceptPairingRequestRequest_t;

/* Pairing reject reason */
typedef enum GAPRejectPairingRequest_Reason_tag {
	GAPRejectPairingRequest_Reason_gOobNotAvailable_c = 0x02,
	GAPRejectPairingRequest_Reason_gIncompatibleIoCapabilities_c = 0x03,
	GAPRejectPairingRequest_Reason_gPairingNotSupported_c = 0x05,
	GAPRejectPairingRequest_Reason_gLowEncryptionKeySize_c = 0x06,
	GAPRejectPairingRequest_Reason_gUnspecifiedReason_c = 0x08,
	GAPRejectPairingRequest_Reason_gRepeatedAttempts_c = 0x09,
	GAPRejectPairingRequest_Reason_gLinkEncryptionFailed_c = 0xF0
} GAPRejectPairingRequest_Reason_t;

typedef PACKED_STRUCT GAPRejectPairingRequest_tag {
	uint8_t DeviceId;  // The GAP peer who requested authentication
	GAPRejectPairingRequest_Reason_t Reason;  // Pairing reject reason
} GAPRejectPairingRequest_t;

typedef PACKED_STRUCT GAPEnterPasskeyRequest_tag {
	uint8_t DeviceId;  // The GAP peer that requested a passkey entry
	uint32_t Passkey;  // The peer's secret passkey
} GAPEnterPasskeyRequest_t;

typedef PACKED_STRUCT GAPProvideOobRequest_tag {
	uint8_t DeviceId;  // The pairing device
	uint8_t Oob[16];  // OOB data
} GAPProvideOobRequest_t;

typedef PACKED_STRUCT GAPRejectPasskeyRequestRequest_tag {
	uint8_t DeviceId;  // The GAP peer that requested a passkey entry
} GAPRejectPasskeyRequestRequest_t;

/* Public or Random address */
typedef enum GAPSendSmpKeysRequest_Keys_AddressInfo_DeviceAddressType_tag {
	GAPSendSmpKeysRequest_Keys_AddressInfo_DeviceAddressType_gPublic_c = 0x00,
	GAPSendSmpKeysRequest_Keys_AddressInfo_DeviceAddressType_gRandom_c = 0x01
} GAPSendSmpKeysRequest_Keys_AddressInfo_DeviceAddressType_t;

typedef struct GAPSendSmpKeysRequest_tag {
	uint8_t DeviceId;  // The GAP peer who initiated the procedure
	struct {
		bool_t LtkIncluded;  // Boolean value which indicates if LTK size and LTK are included or not
		struct {
			uint8_t LtkSize;  // Encryption Key Size
			uint8_t *Ltk;  // Long Term (Encryption) Key
		} LtkInfo;  // LTK information (size and value)
		bool_t IrkIncluded;  // Boolean value which indicates if IRK is included or not
		uint8_t Irk[16];  // Identity Resolving Key
		bool_t CsrkIncluded;  // Boolean value which indicates if CSRK is included or not
		uint8_t Csrk[16];  // Connection Signature Resolving Key
		struct {
			uint8_t RandSize;  // Random value size
			uint8_t *Rand;  // Random value used to identify the LTK
			uint16_t Ediv;  // EDIV value used to identify the LTK
		} RandEdivInfo;  // Rand information (size and value) and Ediv
		bool_t AddressIncluded;  // Boolean value which indicates if Address is included or not
		struct {
			GAPSendSmpKeysRequest_Keys_AddressInfo_DeviceAddressType_t DeviceAddressType;  // Public or Random address
			uint8_t DeviceAddress[6];  // Device Address
		} AddressInfo;  // Address information (type and value)
	} Keys;  // The SMP keys of the local device
} GAPSendSmpKeysRequest_t;

typedef PACKED_STRUCT GAPRejectKeyExchangeRequestRequest_tag {
	uint8_t DeviceId;  // The GAP peer who requested the Key Exchange procedure
} GAPRejectKeyExchangeRequestRequest_t;

typedef struct GAPProvideLongTermKeyRequest_tag {
	uint8_t DeviceId;  // The GAP peer who requested encryption
	uint8_t LtkSize;  // The Long Term Key size
	uint8_t *Ltk;  // The Long Term Key
} GAPProvideLongTermKeyRequest_t;

typedef PACKED_STRUCT GAPDenyLongTermKeyRequest_tag {
	uint8_t DeviceId;  // The GAP peer who requested encryption
} GAPDenyLongTermKeyRequest_t;

typedef PACKED_STRUCT GAPLoadEncryptionInformationRequest_tag {
	uint8_t DeviceId;  // Device ID of the peer
} GAPLoadEncryptionInformationRequest_t;

typedef PACKED_STRUCT GAPSetLocalPasskeyRequest_tag {
	uint32_t Passkey;  // The SMP passkey
} GAPSetLocalPasskeyRequest_t;

/* Scanning type - default passive */
typedef enum GAPStartScanningRequest_ScanningParameters_Type_tag {
	GAPStartScanningRequest_ScanningParameters_Type_gPassive_c = 0x00,
	GAPStartScanningRequest_ScanningParameters_Type_gActive_c = 0x01
} GAPStartScanningRequest_ScanningParameters_Type_t;

/* Indicates whether the address used in scan requests will be the public address or the random address - default public address */
typedef enum GAPStartScanningRequest_ScanningParameters_OwnAddressType_tag {
	GAPStartScanningRequest_ScanningParameters_OwnAddressType_gPublic_c = 0x00,
	GAPStartScanningRequest_ScanningParameters_OwnAddressType_gRandom_c = 0x01
} GAPStartScanningRequest_ScanningParameters_OwnAddressType_t;

/* Indicates whether the advertising packets are filtered using the Filter Accept List - default does not use Filter Accept List */
typedef enum GAPStartScanningRequest_ScanningParameters_FilterPolicy_tag {
	GAPStartScanningRequest_ScanningParameters_FilterPolicy_gScanAll_c = 0x00,
	GAPStartScanningRequest_ScanningParameters_FilterPolicy_gScanWithFilterAcceptList_c = 0x01
} GAPStartScanningRequest_ScanningParameters_FilterPolicy_t;

/* Indicates whether duplicate advertising report filtering is enabled. For BLE 5.0 adds the possibility to reset filtering for each scan period */
typedef enum GAPStartScanningRequest_FilterDuplicates_tag {
	GAPStartScanningRequest_FilterDuplicates_Disable = 0x00,
	GAPStartScanningRequest_FilterDuplicates_Enable = 0x01,
	GAPStartScanningRequest_FilterDuplicates_PeriodicEnable = 0x02
} GAPStartScanningRequest_FilterDuplicates_t;

typedef struct GAPStartScanningRequest_tag {
	bool_t ScanningParametersIncluded;  // Boolean value which indicates if scanning parameters are included or not
	struct {
		GAPStartScanningRequest_ScanningParameters_Type_t Type;  // Scanning type - default passive
		uint16_t Interval;  // Scanning interval - default 10ms
		uint16_t Window;  // Scanning window - default 10ms
		GAPStartScanningRequest_ScanningParameters_OwnAddressType_t OwnAddressType;  // Indicates whether the address used in scan requests will be the public address or the random address - default public address
		GAPStartScanningRequest_ScanningParameters_FilterPolicy_t FilterPolicy;  // Indicates whether the advertising packets are filtered using the Filter Accept List - default does not use Filter Accept List
	} ScanningParameters;  // The scanning parameters
	GAPStartScanningRequest_FilterDuplicates_t FilterDuplicates;  // Indicates whether duplicate advertising report filtering is enabled. For BLE 5.0 adds the possibility to reset filtering for each scan period
	uint8_t ScanningPHYs;  // gLePhyCoded_c|gLePhy2M_c|gLePhy1M_c  // ScanningPHYs
	uint16_t Duration;  // Scan duration - Time = N * 10ms
	uint16_t Period;  // Periodic scanning - Time = N * 1.28s
} GAPStartScanningRequest_t;

/* Indicates whether the connection request is issued for a specific device or for all the devices in the Filter Accept List - default specific device */
typedef enum GAPConnectRequest_FilterPolicy_tag {
	GAPConnectRequest_FilterPolicy_gUseDeviceAddress_c = 0x00,
	GAPConnectRequest_FilterPolicy_gUseFilterAcceptList_c = 0x01
} GAPConnectRequest_FilterPolicy_t;

/* Indicates whether the address used in connection requests will be the public address or the random address - default public address */
typedef enum GAPConnectRequest_OwnAddressType_tag {
	GAPConnectRequest_OwnAddressType_gPublic_c = 0x00,
	GAPConnectRequest_OwnAddressType_gRandom_c = 0x01
} GAPConnectRequest_OwnAddressType_t;

/* When connecting to a specific device, this indicates that device's address type - default public address */
typedef enum GAPConnectRequest_PeerAddressType_tag {
	GAPConnectRequest_PeerAddressType_gPublic_c = 0x00,
	GAPConnectRequest_PeerAddressType_gRandom_c = 0x01
} GAPConnectRequest_PeerAddressType_t;

typedef PACKED_STRUCT GAPConnectRequest_tag {
	uint16_t ScanInterval;  // Scanning interval - default 10ms
	uint16_t ScanWindow;  // Scanning window - default 10ms
	GAPConnectRequest_FilterPolicy_t FilterPolicy;  // Indicates whether the connection request is issued for a specific device or for all the devices in the Filter Accept List - default specific device
	GAPConnectRequest_OwnAddressType_t OwnAddressType;  // Indicates whether the address used in connection requests will be the public address or the random address - default public address
	GAPConnectRequest_PeerAddressType_t PeerAddressType;  // When connecting to a specific device, this indicates that device's address type - default public address
	uint8_t PeerAddress[6];  // When connecting to a specific device, this indicates that device's address
	uint16_t ConnIntervalMin;  // The minimum desired connection interval - default 100ms
	uint16_t ConnIntervalMax;  // The maximum desired connection interval - default 200ms
	uint16_t ConnLatency;  // The desired connection latency (the maximum number of consecutive connection events the Peripheral is allowed to ignore) - default 0
	uint16_t SupervisionTimeout;  // The maximum time interval between consecutive over-the-air packets; if this timer expires, the connection is dropped - default 10s
	uint16_t ConnEventLengthMin;  // The minimum desired connection event length - default 0ms
	uint16_t ConnEventLengthMax;  // The maximum desired connection event length - default maximum possible, ~41 s
	bool_t usePeerIdentityAddress;  // TRUE if the address defined in the peerAddressType and peerAddress is an identity address
	uint8_t Initiating_PHYs;  // gLePhyCoded_c|gLePhy2M_c|gLePhy1M_c  // Phy Flags
} GAPConnectRequest_t;

typedef PACKED_STRUCT GAPDisconnectRequest_tag {
	uint8_t DeviceId;  // The connected peer to disconnect from
} GAPDisconnectRequest_t;

typedef struct GAPSaveCustomPeerInformationRequest_tag {
	uint8_t DeviceId;  // Device ID of the GAP peer
	uint16_t Offset;  // Data offset (offset from the beginning)
	uint16_t InfoSize;  // Data size
	uint8_t *Info;  // Data
} GAPSaveCustomPeerInformationRequest_t;

typedef PACKED_STRUCT GAPLoadCustomPeerInformationRequest_tag {
	uint8_t DeviceId;  // Device ID of the GAP peer
	uint16_t Offset;  // Data offset (offset from the beginning)
	uint16_t InfoSize;  // Data size
} GAPLoadCustomPeerInformationRequest_t;

typedef PACKED_STRUCT GAPCheckIfBondedRequest_tag {
	uint8_t DeviceId;  // Device ID of the GAP peer
} GAPCheckIfBondedRequest_t;

/* The address type (public or random) */
typedef enum GAPAddDeviceToFilterAcceptListRequest_AddressType_tag {
	GAPAddDeviceToFilterAcceptListRequest_AddressType_gPublic_c = 0x00,
	GAPAddDeviceToFilterAcceptListRequest_AddressType_gRandom_c = 0x01
} GAPAddDeviceToFilterAcceptListRequest_AddressType_t;

typedef PACKED_STRUCT GAPAddDeviceToFilterAcceptListRequest_tag {
	GAPAddDeviceToFilterAcceptListRequest_AddressType_t AddressType;  // The address type (public or random)
	uint8_t Address[6];  // The address of the filter accept listed device
} GAPAddDeviceToFilterAcceptListRequest_t;

/* The address type (public or random) */
typedef enum GAPRemoveDeviceFromFilterAcceptListRequest_AddressType_tag {
	GAPRemoveDeviceFromFilterAcceptListRequest_AddressType_gPublic_c = 0x00,
	GAPRemoveDeviceFromFilterAcceptListRequest_AddressType_gRandom_c = 0x01
} GAPRemoveDeviceFromFilterAcceptListRequest_AddressType_t;

typedef PACKED_STRUCT GAPRemoveDeviceFromFilterAcceptListRequest_tag {
	GAPRemoveDeviceFromFilterAcceptListRequest_AddressType_t AddressType;  // The address type (public or random)
	uint8_t Address[6];  // The address of the filter accept listed device
} GAPRemoveDeviceFromFilterAcceptListRequest_t;

typedef struct GAPCreateRandomDeviceAddressRequest_tag {
	bool_t IrkIncluded;  // Boolean value which indicates if IRK is included or not
	uint8_t Irk[16];  // The Identity Resolving Key to be used for a private resolvable address
	bool_t RandomPartIncluded;  // Boolean value which indicates if random part is included or not
	uint8_t RandomPart[3];  // Array containing the random part of a Private Resolvable Address
} GAPCreateRandomDeviceAddressRequest_t;

typedef struct GAPSaveDeviceNameRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer which name is saved
	uint8_t NameSize;  // Number of characters to be saved
	uint8_t *Name;  // Array of characters holding the name
} GAPSaveDeviceNameRequest_t;

typedef PACKED_STRUCT GAPGetBondedDeviceNameRequest_tag {
	uint8_t NvmIndex;  // Index of the device in NVM bonding area
	uint8_t MaxNameSize;  // Maximum number of characters to be copied, including the terminating NULL character
} GAPGetBondedDeviceNameRequest_t;

typedef PACKED_STRUCT GAPRemoveBondRequest_tag {
	uint8_t NvmIndex;  // Index of the device in NVM bonding area
} GAPRemoveBondRequest_t;

/* Transmission read type */
typedef enum GAPReadRadioPowerLevelRequest_TxReadType_tag {
	GAPReadRadioPowerLevelRequest_TxReadType_gTxPowerCurrentLevelInConnection_c = 0x00,
	GAPReadRadioPowerLevelRequest_TxReadType_gTxPowerMaximumLevelInConnection_c = 0x01,
	GAPReadRadioPowerLevelRequest_TxReadType_gTxPowerLevelForAdvertising_c = 0x02,
	GAPReadRadioPowerLevelRequest_TxReadType_gRssi_c = 0x03
} GAPReadRadioPowerLevelRequest_TxReadType_t;

typedef PACKED_STRUCT GAPReadRadioPowerLevelRequest_tag {
	GAPReadRadioPowerLevelRequest_TxReadType_t TxReadType;  // Transmission read type
	uint8_t DeviceId;  // The DeviceId for which the command is intended
} GAPReadRadioPowerLevelRequest_t;

typedef PACKED_STRUCT GAPVerifyPrivateResolvableAddressRequest_tag {
	uint8_t NvmIndex;  // Index of the device in NVM bonding area whose IRK must be checked
	uint8_t Address[6];  // The Private Resolvable Address to be verified
} GAPVerifyPrivateResolvableAddressRequest_t;

typedef PACKED_STRUCT GAPSetRandomAddressRequest_tag {
	uint8_t Address[6];  // The Private Resolvable, Private Non-Resolvable, or Static Random Address
} GAPSetRandomAddressRequest_t;

/* The scan mode to be activated. */
typedef enum GAPSetScanModeRequest_ScanMode_tag {
	GAPSetScanModeRequest_ScanMode_gDefaultScan_c = 0x00,
	GAPSetScanModeRequest_ScanMode_gLimitedDiscovery_c = 0x01,
	GAPSetScanModeRequest_ScanMode_gGeneralDiscovery_c = 0x02,
	GAPSetScanModeRequest_ScanMode_gAutoConnect_c = 0x03
} GAPSetScanModeRequest_ScanMode_t;

/* Indicates whether the connection request is issued for a specific device or for all the devices in the Filter Accept List - default specific device */
typedef enum GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_FilterPolicy_tag {
	GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_FilterPolicy_gUseDeviceAddress_c = 0x00,
	GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_FilterPolicy_gUseFilterAcceptList_c = 0x01
} GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_FilterPolicy_t;

/* Indicates whether the address used in connection requests will be the public address or the random address - default public address */
typedef enum GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_OwnAddressType_tag {
	GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_OwnAddressType_gPublic_c = 0x00,
	GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_OwnAddressType_gRandom_c = 0x01
} GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_OwnAddressType_t;

/* When connecting to a specific device, this indicates that device's address type - default public address */
typedef enum GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_PeerAddressType_tag {
	GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_PeerAddressType_gPublic_c = 0x00,
	GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_PeerAddressType_gRandom_c = 0x01
} GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_PeerAddressType_t;

typedef struct GAPSetScanModeRequest_tag {
	GAPSetScanModeRequest_ScanMode_t ScanMode;  // The scan mode to be activated.
	struct {
		uint8_t NbOfAddresses;  // Number of device addresses to automatically connect to
		bool_t WriteInFilterAcceptList;  // If set to TRUE, the device addresses will be written in the Filter Accept List before scanning is enabled
		struct {
			uint16_t ScanInterval;  // Scanning interval - default 10ms
			uint16_t ScanWindow;  // Scanning window - default 10ms
			GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_FilterPolicy_t FilterPolicy;  // Indicates whether the connection request is issued for a specific device or for all the devices in the Filter Accept List - default specific device
			GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_OwnAddressType_t OwnAddressType;  // Indicates whether the address used in connection requests will be the public address or the random address - default public address
			GAPSetScanModeRequest_AutoConnectParams_AutoConnectData_PeerAddressType_t PeerAddressType;  // When connecting to a specific device, this indicates that device's address type - default public address
			uint8_t PeerAddress[6];  // When connecting to a specific device, this indicates that device's address
			uint16_t ConnIntervalMin;  // The minimum desired connection interval - default 100ms
			uint16_t ConnIntervalMax;  // The maximum desired connection interval - default 200ms
			uint16_t ConnLatency;  // The desired connection latency (the maximum number of consecutive connection events the Peripheral is allowed to ignore) - default 0
			uint16_t SupervisionTimeout;  // The maximum time interval between consecutive over-the-air packets; if this timer expires, the connection is dropped - default 10s
			uint16_t ConnEventLengthMin;  // The minimum desired connection event length - default 0ms
			uint16_t ConnEventLengthMax;  // The maximum desired connection event length - default maximum possible, ~41 s
			bool_t usePeerIdentityAddress;  // TRUE if the address defined in the peerAddressType and peerAddress is an identity address
			uint8_t Initiating_PHYs;  // gLePhyCoded_c|gLePhy2M_c|gLePhy1M_c  // Phy Flags
		} *AutoConnectData;  // The connection request parameters
	} AutoConnectParams;  // Parameters for the Auto Connect Scan Mode
} GAPSetScanModeRequest_t;

/* The desired security mode-level */
typedef enum GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_tag {
	GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_gMode1Level1_c = 0x10,
	GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_gMode1Level2_c = 0x11,
	GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_gMode1Level3_c = 0x12,
	GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_gMode1Level4_c = 0x13,
	GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_gMode2Level1_c = 0x21,
	GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_gMode2Level2_c = 0x22
} GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_t;

/* I/O capabilities used to determine the pairing method */
typedef enum GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_tag {
	GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_gIoDisplayOnly_c = 0x00,
	GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_gIoDisplayYesNo_c = 0x01,
	GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardOnly_c = 0x02,
	GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_gIoNone_c = 0x03,
	GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_gIoKeyboardDisplay_c = 0x04
} GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_t;

typedef struct GAPSetDefaultPairingParametersRequest_tag {
	bool_t PairingParametersIncluded;  // Boolean value which indicates if pairing parameters are included or not
	struct {
		bool_t WithBonding;  // TRUE if this device is able to and wants to bond after pairing, FALSE otherwise
		GAPSetDefaultPairingParametersRequest_PairingParameters_SecurityModeAndLevel_t SecurityModeAndLevel;  // The desired security mode-level
		uint8_t MaxEncryptionKeySize;  // Maximum LTK size supported by the device
		GAPSetDefaultPairingParametersRequest_PairingParameters_LocalIoCapabilities_t LocalIoCapabilities;  // I/O capabilities used to determine the pairing method
		bool_t OobAvailable;  // TRUE if this device has Out-of-Band data that can be used for authenticated pairing, FALSE otherwise
		uint8_t CentralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Central
		uint8_t PeripheralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Peripheral
		bool_t LeSecureConnectionSupported;  // indicates if device supports LE Secure Connections pairing
		bool_t UseKeypressNotifications;  // Indicates if device supports Keypress Notification PDUs during Passkey Entry pairing
	} PairingParameters;  // Pairing parameters as required by the SMP
} GAPSetDefaultPairingParametersRequest_t;

typedef PACKED_STRUCT GAPUpdateConnectionParametersRequest_tag {
	uint8_t DeviceId;  // The DeviceID for which the command is intended
	uint16_t IntervalMin;  // The minimum value for the connection event interval
	uint16_t IntervalMax;  // The maximum value for the connection event interval
	uint16_t PeripheralLatency;  // The peripheral latency parameter
	uint16_t TimeoutMultiplier;  // The connection timeout parameter
	uint16_t MinCeLength;  // The minimum connection event length
	uint16_t MaxCeLength;  // The maximum connection event length
} GAPUpdateConnectionParametersRequest_t;

typedef PACKED_STRUCT GAPEnableUpdateConnectionParametersRequest_tag {
	uint8_t DeviceId;  // The DeviceID for which the command is intended
	bool_t Enable;  // Allow/dissalow the parameters update
} GAPEnableUpdateConnectionParametersRequest_t;

typedef PACKED_STRUCT GapUpdateLeDataLengthRequest_tag {
	uint8_t DeviceId;  // The DeviceID for which the command is intended
	uint16_t TxOctets;  // Maximum transmission number of payload octets
	uint16_t TxTime;  // Maximum transmission time
} GapUpdateLeDataLengthRequest_t;

typedef struct GAPEnableHostPrivacyRequest_tag {
	bool_t Enable;  // Enables or disables Host Privacy
	uint8_t Irk[16];  // Local IRK to be used for Resolvable Private Address generation
} GAPEnableHostPrivacyRequest_t;

/* Identity Address Type */
typedef enum GAPEnableControllerPrivacyRequest_PeerIdentities_IdentityAddressType_tag {
	GAPEnableControllerPrivacyRequest_PeerIdentities_IdentityAddressType_gPublic_c = 0x00,
	GAPEnableControllerPrivacyRequest_PeerIdentities_IdentityAddressType_gRandom_c = 0x01
} GAPEnableControllerPrivacyRequest_PeerIdentities_IdentityAddressType_t;

/* Privacy Mode */
typedef enum GAPEnableControllerPrivacyRequest_PeerIdentities_PrivacyMode_tag {
	GAPEnableControllerPrivacyRequest_PeerIdentities_PrivacyMode_gNetworkPrivacy_c = 0x00,
	GAPEnableControllerPrivacyRequest_PeerIdentities_PrivacyMode_gDevicePrivacy_c = 0x01
} GAPEnableControllerPrivacyRequest_PeerIdentities_PrivacyMode_t;

typedef struct GAPEnableControllerPrivacyRequest_tag {
	bool_t Enable;  // Enables or disables Controller Privacy
	uint8_t OwnIrk[16];  // Local IRK
	uint8_t PeerIdCount;  // Size of PeerIdentities array
	struct {
		GAPEnableControllerPrivacyRequest_PeerIdentities_IdentityAddressType_t IdentityAddressType;  // Identity Address Type
		uint8_t IdentityAddress[6];  // Identity Address
		uint8_t Irk[16];  // Identity Resolving Key
		GAPEnableControllerPrivacyRequest_PeerIdentities_PrivacyMode_t PrivacyMode;  // Privacy Mode
	} *PeerIdentities;  // Array of peer identity addresses and IRKs
} GAPEnableControllerPrivacyRequest_t;

typedef PACKED_STRUCT GAPLeScValidateNumericValueRequest_tag {
	uint8_t DeviceId;  // The DeviceID for which the command is intended
	bool_t Valid;  // TRUE if user has indicated that numeric values are matched, FALSE otherwise
} GAPLeScValidateNumericValueRequest_t;

typedef PACKED_STRUCT GAPLeScSetPeerOobDataRequest_tag {
	uint8_t DeviceId;  // The DeviceID for which the command is intended
	uint8_t RandomValue[16];  // LE SC OOB r (Random value)
	uint8_t ConfirmValue[16];  // LE SC OOB Cr (Random Confirm value)
} GAPLeScSetPeerOobDataRequest_t;

/* keypressNotifType */
typedef enum GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_tag {
	GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyEntryStarted_c = 0x00,
	GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyDigitStarted_c = 0x01,
	GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyDigitErased_c = 0x02,
	GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyCleared_c = 0x03,
	GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyEntryCompleted_c = 0x04
} GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_t;

typedef PACKED_STRUCT GAPLeScSendKeypressNotificationPrivacyRequest_tag {
	uint8_t DeviceId;  // The DeviceID for which the command is intended
	GAPLeScSendKeypressNotificationPrivacyRequest_GapLeScKeypressNotificationParams_keypressNotifType_t GapLeScKeypressNotificationParams_keypressNotifType;

} GAPLeScSendKeypressNotificationPrivacyRequest_t;

typedef PACKED_STRUCT GAPGetBondedDevicesIdentityInformationRequest_tag {
	uint8_t maxDevices;  // Maximum number of addresses to be obtained
} GAPGetBondedDevicesIdentityInformationRequest_t;

/* Advertising or Connection channel type for the Tx power level */
typedef enum GAPSetTxPowerLevelRequest_channelType_tag {
	GAPSetTxPowerLevelRequest_channelType_Advertising = 0x00,
	GAPSetTxPowerLevelRequest_channelType_Connection = 0x01
} GAPSetTxPowerLevelRequest_channelType_t;

typedef PACKED_STRUCT GAPSetTxPowerLevelRequest_tag {
	uint8_t powerLevel;  // power level value as described by the controller interface
	GAPSetTxPowerLevelRequest_channelType_t channelType;  // Advertising or Connection channel type for the Tx power level
} GAPSetTxPowerLevelRequest_t;

typedef PACKED_STRUCT GAPLeReadPhyRequest_tag {
	uint8_t DeviceId;  // Device ID of the peer.
} GAPLeReadPhyRequest_t;

/* Host preferences on Coded Phy */
typedef enum GAPLeSetPhyRequest_phyOptions_tag {
	GAPLeSetPhyRequest_phyOptions_gLeCodingNoPreference_c = 0x00,
	GAPLeSetPhyRequest_phyOptions_gLeCodingS2_c = 0x01,
	GAPLeSetPhyRequest_phyOptions_gLeCodingS8_c = 0x02
} GAPLeSetPhyRequest_phyOptions_t;

typedef PACKED_STRUCT GAPLeSetPhyRequest_tag {
	bool_t defaultMode;  // Use the provided values for all subsequent connections or provide a device ID for a single connection
	uint8_t DeviceId;  // Device ID of the peer.
	uint8_t allPhys;  // 0x02|0x01  // Bit mask indicating Host preferences on Tx and Rx Phy
	uint8_t txPhys;  // 0x04|0x02|0x01  // Bit mask indicating Host preferences on Tx Phy
	uint8_t rxPhys;  // 0x04|0x02|0x01  // Bit mask indicating Host preferences on Rx Phy
	GAPLeSetPhyRequest_phyOptions_t phyOptions;  // Host preferences on Coded Phy
} GAPLeSetPhyRequest_t;

typedef PACKED_STRUCT GAPControllerEnhancedNotificationRequest_tag {
	uint16_t EventType;  // 0x8000|0x4000|0x2000|0x1000|0x0800|0x0400|0x0200|0x0100|0x80|0x40|0x20|0x10|0x08|0x04|0x02|0x01  // Events to enable.
	uint8_t DeviceId;  // Device ID of the peer.
} GAPControllerEnhancedNotificationRequest_t;

typedef PACKED_STRUCT GAPLoadKeysRequest_tag {
	uint8_t NvmIndex;  // Index of the device in NVM bonding area
} GAPLoadKeysRequest_t;

/* Public or Random address */
typedef enum GAPSaveKeysRequest_Keys_AddressInfo_DeviceAddressType_tag {
	GAPSaveKeysRequest_Keys_AddressInfo_DeviceAddressType_gPublic_c = 0x00,
	GAPSaveKeysRequest_Keys_AddressInfo_DeviceAddressType_gRandom_c = 0x01
} GAPSaveKeysRequest_Keys_AddressInfo_DeviceAddressType_t;

typedef struct GAPSaveKeysRequest_tag {
	uint8_t NvmIndex;  // Index of the device in NVM bonding area
	struct {
		bool_t LtkIncluded;  // Boolean value which indicates if LTK size and LTK are included or not
		struct {
			uint8_t LtkSize;  // Encryption Key Size
			uint8_t *Ltk;  // Long Term (Encryption) Key
		} LtkInfo;  // LTK information (size and value)
		bool_t IrkIncluded;  // Boolean value which indicates if IRK is included or not
		uint8_t Irk[16];  // Identity Resolving Key
		bool_t CsrkIncluded;  // Boolean value which indicates if CSRK is included or not
		uint8_t Csrk[16];  // Connection Signature Resolving Key
		struct {
			uint8_t RandSize;  // Random value size
			uint8_t *Rand;  // Random value used to identify the LTK
			uint16_t Ediv;  // EDIV value used to identify the LTK
		} RandEdivInfo;  // Rand information (size and value) and Ediv
		bool_t AddressIncluded;  // Boolean value which indicates if Address is included or not
		struct {
			GAPSaveKeysRequest_Keys_AddressInfo_DeviceAddressType_t DeviceAddressType;  // Public or Random address
			uint8_t DeviceAddress[6];  // Device Address
		} AddressInfo;  // Address information (type and value)
	} Keys;  // The SMP keys distributed by the peer
	bool_t LeSc;  // LE SC connections pairing
	bool_t Authenticated;  // Peer was authenticated during pairing
} GAPSaveKeysRequest_t;

typedef PACKED_STRUCT GAPSetChannelMapRequest_tag {
	uint8_t ChannelMap[5];  // Channel Map
} GAPSetChannelMapRequest_t;

typedef PACKED_STRUCT GAPReadChannelMapRequest_tag {
	uint8_t DeviceId;  // Device ID of the peer.
} GAPReadChannelMapRequest_t;

/* Privacy Mode */
typedef enum GAPSetPrivacyModeRequest_PrivacyMode_tag {
	GAPSetPrivacyModeRequest_PrivacyMode_gNetworkPrivacy_c = 0x00,
	GAPSetPrivacyModeRequest_PrivacyMode_gDevicePrivacy_c = 0x01
} GAPSetPrivacyModeRequest_PrivacyMode_t;

typedef PACKED_STRUCT GAPSetPrivacyModeRequest_tag {
	uint8_t NvmIndex;  // Index of the device in NVM bonding area
	GAPSetPrivacyModeRequest_PrivacyMode_t PrivacyMode;  // Privacy Mode
} GAPSetPrivacyModeRequest_t;

typedef PACKED_STRUCT ControllerSetScanDupFiltModeRequest_tag {
	uint8_t Mode;  // Mode: 0: default behavior; 1: allow adv report on RPA update
} ControllerSetScanDupFiltModeRequest_t;

/* The address type (public or random) */
typedef enum GAPReadControllerLocalRPARequest_PeerIdentityAddressType_tag {
	GAPReadControllerLocalRPARequest_PeerIdentityAddressType_gPublic_c = 0x00,
	GAPReadControllerLocalRPARequest_PeerIdentityAddressType_gRandom_c = 0x01
} GAPReadControllerLocalRPARequest_PeerIdentityAddressType_t;

typedef PACKED_STRUCT GAPReadControllerLocalRPARequest_tag {
	GAPReadControllerLocalRPARequest_PeerIdentityAddressType_t PeerIdentityAddressType;  // The address type (public or random)
	uint8_t PeerIdentityAddress[6];  // The Identity Address of the Peer Device
} GAPReadControllerLocalRPARequest_t;

typedef PACKED_STRUCT GAPCheckNvmIndexRequest_tag {
	uint8_t NvmIndex;  // Index in NVM bonding area
} GAPCheckNvmIndexRequest_t;

typedef PACKED_STRUCT GAPGetDeviceIdFromConnHandleRequest_tag {
	uint16_t ConnHandle;  // Connection Handle
} GAPGetDeviceIdFromConnHandleRequest_t;

typedef PACKED_STRUCT GAPGetConnectionHandleFromDeviceId_tag {
	uint8_t DeviceId;  // Device ID of the peer.
} GAPGetConnectionHandleFromDeviceId_t;

/* Advertising index type */
typedef enum GAPAdvIndexChangeRequest_AdvIndexType_tag {
	GAPAdvIndexChangeRequest_AdvIndexType_gAdvIndexAscend_c = 0x00,
	GAPAdvIndexChangeRequest_AdvIndexType_gAdvIndexDescend_c = 0x01,
	GAPAdvIndexChangeRequest_AdvIndexType_gAdvIndexUser_c = 0x02,
	GAPAdvIndexChangeRequest_AdvIndexType_gAdvIndexRandom_c = 0x03
} GAPAdvIndexChangeRequest_AdvIndexType_t;

typedef PACKED_STRUCT GAPAdvIndexChangeRequest_tag {
	GAPAdvIndexChangeRequest_AdvIndexType_t AdvIndexType;  // Advertising index type
	uint8_t AUserDefinedChannels[3];  // User defined channels array
} GAPAdvIndexChangeRequest_t;

/* Bluetooth device address types */
typedef enum GAPSetExtAdvertisingParametersRequest_OwnAddressType_tag {
	GAPSetExtAdvertisingParametersRequest_OwnAddressType_gPublic_c = 0x00,
	GAPSetExtAdvertisingParametersRequest_OwnAddressType_gRandom_c = 0x01
} GAPSetExtAdvertisingParametersRequest_OwnAddressType_t;

/* Bluetooth device address types */
typedef enum GAPSetExtAdvertisingParametersRequest_PeerAddressType_tag {
	GAPSetExtAdvertisingParametersRequest_PeerAddressType_gPublic_c = 0x00,
	GAPSetExtAdvertisingParametersRequest_PeerAddressType_gRandom_c = 0x01
} GAPSetExtAdvertisingParametersRequest_PeerAddressType_t;

/* Indicates whether the connect and scan requests are filtered using the Filter Accept List - default does not use Filter Accept List */
typedef enum GAPSetExtAdvertisingParametersRequest_FilterPolicy_tag {
	GAPSetExtAdvertisingParametersRequest_FilterPolicy_gProcessAll_c = 0x00,
	GAPSetExtAdvertisingParametersRequest_FilterPolicy_gProcessConnAllScanWL_c = 0x01,
	GAPSetExtAdvertisingParametersRequest_FilterPolicy_gProcessScanAllConnWL_c = 0x02,
	GAPSetExtAdvertisingParametersRequest_FilterPolicy_gProcessFilterAcceptListOnly_c = 0x03
} GAPSetExtAdvertisingParametersRequest_FilterPolicy_t;

/* Phy Mode */
typedef enum GAPSetExtAdvertisingParametersRequest_PrimaryAdvPHY_tag {
	GAPSetExtAdvertisingParametersRequest_PrimaryAdvPHY_gLePhy1M_c = 0x01,
	GAPSetExtAdvertisingParametersRequest_PrimaryAdvPHY_gLePhy2M_c = 0x02,
	GAPSetExtAdvertisingParametersRequest_PrimaryAdvPHY_gLePhyCoded_c = 0x03
} GAPSetExtAdvertisingParametersRequest_PrimaryAdvPHY_t;

/* Phy Mode */
typedef enum GAPSetExtAdvertisingParametersRequest_SecondaryAdvPHY_tag {
	GAPSetExtAdvertisingParametersRequest_SecondaryAdvPHY_gLePhy1M_c = 0x01,
	GAPSetExtAdvertisingParametersRequest_SecondaryAdvPHY_gLePhy2M_c = 0x02,
	GAPSetExtAdvertisingParametersRequest_SecondaryAdvPHY_gLePhyCoded_c = 0x03
} GAPSetExtAdvertisingParametersRequest_SecondaryAdvPHY_t;

typedef PACKED_STRUCT GAPSetExtAdvertisingParametersRequest_tag {
	uint8_t SID;  // Advertising SID
	uint8_t Handle;  // Advertising set identifier
	uint32_t MinInterval;  // Minimum desired advertising interval - default 1.28s
	uint32_t MaxInterval;  // Maximum desired advertising interval - default 1.28s
	GAPSetExtAdvertisingParametersRequest_OwnAddressType_t OwnAddressType;  // Bluetooth device address types
	uint8_t OwnRandomAddress[6];  // Bluetooth device address
	GAPSetExtAdvertisingParametersRequest_PeerAddressType_t PeerAddressType;  // Bluetooth device address types
	uint8_t PeerAddress[6];  // Bluetooth device address
	uint8_t ChannelMap;  // 0x04|0x02|0x01  // Bit mask indicating which of the three advertising channels will be used - default all three
	GAPSetExtAdvertisingParametersRequest_FilterPolicy_t FilterPolicy;  // Indicates whether the connect and scan requests are filtered using the Filter Accept List - default does not use Filter Accept List
	uint16_t AdvProperties;  // Reserved|gExtAdvIncludeTxPower_c|gExtAdvAnonymous_c|gExtAdvLegacy_c|gExtAdvFastDirected_c|gExtAdvDirected_c|gExtAdvScannable_c|gExtAdvConnectable_c  // Extended Advertising Properties
	uint8_t AdvTxPowerLevel;  // Advertising transmission power level (in dBm)
	GAPSetExtAdvertisingParametersRequest_PrimaryAdvPHY_t PrimaryAdvPHY;  // Phy Mode
	GAPSetExtAdvertisingParametersRequest_SecondaryAdvPHY_t SecondaryAdvPHY;  // Phy Mode
	uint8_t SecondaryAdvMaxSkip;  // Maximum advertising events the Controller can skip before sending the AUX_ADV_IND packets on the secondary advertising channel. If 0, AUX_ADV_IND shall be sent prior to the next advertising event
	bool_t EnableScanReqNotification;  // Host is notified on scan requests
} GAPSetExtAdvertisingParametersRequest_t;

typedef PACKED_STRUCT GAPStartExtAdvertisingRequest_tag {
	uint8_t Handle;  // Advertising set ID
	uint16_t Duration;  // The duration of the advertising - Time = N * 10 ms
	uint8_t MaxExtAdvEvents;  // The maximum number of advertising events
} GAPStartExtAdvertisingRequest_t;

typedef PACKED_STRUCT GAPRemoveAdvertisingSetRequest_tag {
	uint8_t Handle;  // Advertising handle to remove or 0xFF to remove all
} GAPRemoveAdvertisingSetRequest_t;

typedef PACKED_STRUCT GAPStopExtAdvertisingRequest_tag {
	uint8_t Handle;  // Advertising set to be disabled or 0xFF to disable all
} GAPStopExtAdvertisingRequest_t;

/* The operation to be executed on the list */
typedef enum GAPUpdatePeriodicAdvListRequest_Operation_tag {
	GAPUpdatePeriodicAdvListRequest_Operation_gAddDevice_c = 0x00,
	GAPUpdatePeriodicAdvListRequest_Operation_gRemoveDevice_c = 0x01,
	GAPUpdatePeriodicAdvListRequest_Operation_gRemoveAllDevices_c = 0x02
} GAPUpdatePeriodicAdvListRequest_Operation_t;

/* Bluetooth device address types */
typedef enum GAPUpdatePeriodicAdvListRequest_OperationValue_gAddDevice_c_DeviceAddressType_tag {
	GAPUpdatePeriodicAdvListRequest_OperationValue_gAddDevice_c_DeviceAddressType_gPublic_c = 0x00,
	GAPUpdatePeriodicAdvListRequest_OperationValue_gAddDevice_c_DeviceAddressType_gRandom_c = 0x01
} GAPUpdatePeriodicAdvListRequest_OperationValue_gAddDevice_c_DeviceAddressType_t;

/* Bluetooth device address types */
typedef enum GAPUpdatePeriodicAdvListRequest_OperationValue_gRemoveDevice_c_DeviceAddressType_tag {
	GAPUpdatePeriodicAdvListRequest_OperationValue_gRemoveDevice_c_DeviceAddressType_gPublic_c = 0x00,
	GAPUpdatePeriodicAdvListRequest_OperationValue_gRemoveDevice_c_DeviceAddressType_gRandom_c = 0x01
} GAPUpdatePeriodicAdvListRequest_OperationValue_gRemoveDevice_c_DeviceAddressType_t;

typedef struct GAPUpdatePeriodicAdvListRequest_tag {
	GAPUpdatePeriodicAdvListRequest_Operation_t Operation;  // The operation to be executed on the list
	union {
		struct {
			GAPUpdatePeriodicAdvListRequest_OperationValue_gAddDevice_c_DeviceAddressType_t DeviceAddressType;  // Bluetooth device address types
			uint8_t DeviceAddress[6];  // Bluetooth device address
			uint8_t SID;  // Advertising set ID
		} gAddDevice_c;  // gAddDevice_c Parameter
		struct {
			GAPUpdatePeriodicAdvListRequest_OperationValue_gRemoveDevice_c_DeviceAddressType_t DeviceAddressType;  // Bluetooth device address types
			uint8_t DeviceAddress[6];  // Bluetooth device address
			uint8_t SID;  // Advertising set ID
		} gRemoveDevice_c;  // gRemoveDevice_c Parameter
  // gRemoveAllDevices_c Parameter
	} OperationValue;  // Depending on the Operation Value, the other parameters will be displayed in Test Tool
} GAPUpdatePeriodicAdvListRequest_t;

typedef PACKED_STRUCT GAPSetPeriodicAdvParametersRequest_tag {
	uint8_t Handle;  // Advertising set ID
	bool_t IncludeTxPower;  // Include TxPower in the advertising PDU
	uint16_t MinInterval;  // Minimum advertising interval for periodic advertising
	uint16_t MaxInterval;  // Maximum advertising interval for periodic advertising
} GAPSetPeriodicAdvParametersRequest_t;

typedef PACKED_STRUCT GAPStartPeriodicAdvertisingRequest_tag {
	uint8_t Handle;  // Advertising handle
} GAPStartPeriodicAdvertisingRequest_t;

typedef PACKED_STRUCT GAPStopPeriodicAdvertisingRequest_tag {
	uint8_t Handle;  // Advertising handle
} GAPStopPeriodicAdvertisingRequest_t;

typedef PACKED_STRUCT GAPSetExtAdvertisingDataRequest_tag {
	uint8_t Handle;  // Advertising handle
} GAPSetExtAdvertisingDataRequest_t;

/* Advertising data type */
typedef enum GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_tag {
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdFlags_c = 0x01,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdIncomplete16bitServiceList_c = 0x02,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdComplete16bitServiceList_c = 0x03,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdIncomplete32bitServiceList_c = 0x04,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdComplete32bitServiceList_c = 0x05,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdIncomplete128bitServiceList_c = 0x06,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdComplete128bitServiceList_c = 0x07,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdShortenedLocalName_c = 0x08,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdCompleteLocalName_c = 0x09,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdTxPowerLevel_c = 0x0A,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdClassOfDevice_c = 0x0D,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdSimplePairingHashC192_c = 0x0E,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdSimplePairingRandomizerR192_c = 0x0F,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdSecurityManagerTkValue_c = 0x10,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdSecurityManagerOobFlags_c = 0x11,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdPeripheralConnectionIntervalRange_c = 0x12,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdServiceSolicitationList16bit_c = 0x14,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdServiceSolicitationList128bit_c = 0x15,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdServiceData16bit_c = 0x16,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdPublicTargetAddress_c = 0x17,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdRandomTargetAddress_c = 0x18,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdAppearance_c = 0x19,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdAdvertisingInterval_c = 0x1A,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdLeDeviceAddress_c = 0x1B,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdLeRole_c = 0x1C,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdSimplePairingHashC256_c = 0x1D,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdSimplePairingRandomizerR256_c = 0x1E,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdServiceSolicitationList32bit_c = 0x1F,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdServiceData32bit_c = 0x20,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdServiceData128bit_c = 0x21,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdUniformResourceIdentifier_c = 0x24,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAd3dInformationData_c = 0x3D,
	GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_gAdManufacturerSpecificData_c = 0xFF
} GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_t;

typedef struct GAPSetPeriodicAdvertisingDataRequest_tag {
	uint8_t Handle;  // Advertising handle
	struct {
		uint8_t NbOfAdStructures;  // Number of advertising data structures
		struct {
			uint8_t Length;  // Advertising data length
			GAPSetPeriodicAdvertisingDataRequest_PeriodicAdvertisingData_AdStructures_Type_t Type;  // Advertising data type
			uint8_t *Data;  // Advertising data
		} *AdStructures;  // Advertising data structures
	} PeriodicAdvertisingData;  // Extended Advertising data
} GAPSetPeriodicAdvertisingDataRequest_t;

/* Periodic advertising create sync request parameters */
typedef enum GAPPeriodicAdvCreateSyncRequest_DeviceAddressType_tag {
	GAPPeriodicAdvCreateSyncRequest_DeviceAddressType_gPublic_c = 0x00,
	GAPPeriodicAdvCreateSyncRequest_DeviceAddressType_gRandom_c = 0x01
} GAPPeriodicAdvCreateSyncRequest_DeviceAddressType_t;

typedef PACKED_STRUCT GAPPeriodicAdvCreateSyncRequest_tag {
	uint8_t Options;  // Periodic advertising create sync request parameters
	uint8_t SID;  // Periodic advertising create sync request parameters
	GAPPeriodicAdvCreateSyncRequest_DeviceAddressType_t DeviceAddressType;  // Periodic advertising create sync request parameters
	uint8_t DeviceAddress[6];  // Periodic advertising create sync request parameters
	uint16_t SkipCount;  // Periodic advertising create sync request parameters
	uint16_t Timeout;  // Periodic advertising create sync request parameters
	uint8_t SyncCteType;  // 0x10|0x08|0x04|0x02|0x01  // Periodic advertising create sync request parameters
} GAPPeriodicAdvCreateSyncRequest_t;

typedef PACKED_STRUCT GAPPeriodicAdvTerminateSyncRequest_tag {
	uint16_t SyncHandle;  // Sync_Handle to be used to identify the periodic advertiser
} GAPPeriodicAdvTerminateSyncRequest_t;

/* Private key type */
typedef enum GAPGenerateDHKeyV2Request_PrivateKeyType_tag {
	GAPGenerateDHKeyV2Request_PrivateKeyType_UseGeneratedKey = 0x00,
	GAPGenerateDHKeyV2Request_PrivateKeyType_DebugKey = 0x01
} GAPGenerateDHKeyV2Request_PrivateKeyType_t;

typedef PACKED_STRUCT GAPGenerateDHKeyV2Request_tag {
	uint8_t RemoteP256PublicKey[64];  // Remote p-256 Public Key
	GAPGenerateDHKeyV2Request_PrivateKeyType_t PrivateKeyType;  // Private key type
} GAPGenerateDHKeyV2Request_t;

/* Action */
typedef enum GAPModifySleepClockAccuracyRequest_Action_tag {
	GAPModifySleepClockAccuracyRequest_Action_SwitchToMoreAccurateClock = 0x00,
	GAPModifySleepClockAccuracyRequest_Action_SwitchToLessAccurateClock = 0x01
} GAPModifySleepClockAccuracyRequest_Action_t;

typedef PACKED_STRUCT GAPModifySleepClockAccuracyRequest_tag {
	GAPModifySleepClockAccuracyRequest_Action_t Action;  // Action
} GAPModifySleepClockAccuracyRequest_t;

typedef PACKED_STRUCT ControllerConfigureAdvCodingSchemeRequest_tag {
	uint8_t CodingScheme;  // Mode: 0: primary adv coding scheme S8, secondary adv coding scheme S8;
                           //             1: primary adv coding scheme S8, secondary adv coding scheme S2;
                           //             2: primary adv coding scheme S2, secondary adv coding scheme S8;
                           //             3: primary adv coding scheme S2, secondary adv coding scheme S2
} ControllerConfigureAdvCodingSchemeRequest_t;

/* CTE Type */
typedef enum GAPSetConnectionlessCteTransmitParametersRequest_CteType_tag {
	GAPSetConnectionlessCteTransmitParametersRequest_CteType_gCteTypeAoA_c = 0x01,
	GAPSetConnectionlessCteTransmitParametersRequest_CteType_gCteTypeAoD1us_c = 0x02,
	GAPSetConnectionlessCteTransmitParametersRequest_CteType_gCteTypeAoD2us_c = 0x03,
	GAPSetConnectionlessCteTransmitParametersRequest_CteType_gCteTypeNoCte_c = 0xFF
} GAPSetConnectionlessCteTransmitParametersRequest_CteType_t;

typedef struct GAPSetConnectionlessCteTransmitParametersRequest_tag {
	uint8_t Handle;  // Advertising set handle
	uint8_t CteLength;  // Constant Tone Extension length in 8 us
	GAPSetConnectionlessCteTransmitParametersRequest_CteType_t CteType;  // CTE Type
	uint8_t CteCount;  // Number of CTEs to transmit in each periodic advertising interval
	uint8_t SwitchingPatternLength;  // Number of Antenna IDs in pattern
	uint8_t *AntennaIds;  // List of Antenna IDs in pattern
} GAPSetConnectionlessCteTransmitParametersRequest_t;

/* Enable or disable CTE Transmit */
typedef enum GAPEnableConnectionlessCteTransmitRequest_Enable_tag {
	GAPEnableConnectionlessCteTransmitRequest_Enable_gCteTransmitDisable_c = 0x00,
	GAPEnableConnectionlessCteTransmitRequest_Enable_gCteTransmitEnable_c = 0x01
} GAPEnableConnectionlessCteTransmitRequest_Enable_t;

typedef PACKED_STRUCT GAPEnableConnectionlessCteTransmitRequest_tag {
	uint8_t Handle;  // Advertising set handle
	GAPEnableConnectionlessCteTransmitRequest_Enable_t Enable;  // Enable or disable CTE Transmit
} GAPEnableConnectionlessCteTransmitRequest_t;

/* Enable or disable IQ sampling */
typedef enum GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_IqSamplingEnable_tag {
	GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_IqSamplingEnable_gIqSamplingDisable_c = 0x00,
	GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_IqSamplingEnable_gIqSamplingEnable_c = 0x01
} GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_IqSamplingEnable_t;

/* Switching and sampling slot durations */
typedef enum GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_SlotDurations_tag {
	GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_SlotDurations_gSlotDurations1us_c = 0x01,
	GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_SlotDurations_gSlotDurations2us_c = 0x02
} GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_SlotDurations_t;

typedef struct GAPEnableConnectionlessIqSamplingRequest_tag {
	uint16_t SyncHandle;  // Sync_Handle to be used to identify the advertising train
	struct {
		GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_IqSamplingEnable_t IqSamplingEnable;  // Enable or disable IQ sampling
		GAPEnableConnectionlessIqSamplingRequest_pIqSamplingParams_SlotDurations_t SlotDurations;  // Switching and sampling slot durations
		uint8_t MaxSampledCtes;  // Maximum number of CTEs to sample and report in each periodic advertising interval
		uint8_t SwitchingPatternLength;  // Number of Antenna IDs in pattern
		uint8_t *AntennaIds;  // List of Antenna IDs in pattern
	} pIqSamplingParams;  // Iq Sampling Parameters
} GAPEnableConnectionlessIqSamplingRequest_t;

/* Enable or disable IQ sampling */
typedef enum GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_IqSamplingEnable_tag {
	GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_IqSamplingEnable_gIqSamplingDisable_c = 0x00,
	GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_IqSamplingEnable_gIqSamplingEnable_c = 0x01
} GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_IqSamplingEnable_t;

/* Switching and sampling slot durations */
typedef enum GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_SlotDurations_tag {
	GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_SlotDurations_gSlotDurations1us_c = 0x01,
	GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_SlotDurations_gSlotDurations2us_c = 0x02
} GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_SlotDurations_t;

typedef struct GAPSetConnectionCteReceiveParametersRequest_tag {
	uint8_t DeviceId;  // Peer Device Id
	struct {
		GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_IqSamplingEnable_t IqSamplingEnable;  // Enable or disable IQ sampling
		GAPSetConnectionCteReceiveParametersRequest_pReceiveParams_SlotDurations_t SlotDurations;  // Switching and sampling slot durations
		uint8_t SwitchingPatternLength;  // Number of Antenna IDs in pattern
		uint8_t *AntennaIds;  // List of Antenna IDs in pattern
	} pReceiveParams;  // CTE Receive Parameters to be set
} GAPSetConnectionCteReceiveParametersRequest_t;

typedef struct GAPSetConnectionCteTransmitParametersRequest_tag {
	uint8_t DeviceId;  // Peer Device Id
	struct {
		uint8_t CteTypes;  // 0x04|0x02|0x01  // Allowed CTE Types
		uint8_t SwitchingPatternLength;  // Number of Antenna IDs in pattern
		uint8_t *AntennaIds;  // List of Antenna IDs in pattern
	} pTransmitParams;  // CTE Transmit Parameters to be set
} GAPSetConnectionCteTransmitParametersRequest_t;

/* Enable or disable CTE Request procedure */
typedef enum GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_CteReqEnable_tag {
	GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_CteReqEnable_gCteReqDisable_c = 0x00,
	GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_CteReqEnable_gCteReqEnable_c = 0x01
} GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_CteReqEnable_t;

/* Requested types of CTE */
typedef enum GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_RequestedCteType_tag {
	GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_RequestedCteType_gCteTypeAoA_c = 0x01,
	GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_RequestedCteType_gCteTypeAoD1us_c = 0x02,
	GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_RequestedCteType_gCteTypeAoD2us_c = 0x03,
	GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_RequestedCteType_gCteTypeNoCte_c = 0xFF
} GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_RequestedCteType_t;

typedef PACKED_STRUCT GAPEnableConnectionCteRequestRequest_tag {
	uint8_t DeviceId;  // Peer Device Id
	struct {
		GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_CteReqEnable_t CteReqEnable;  // Enable or disable CTE Request procedure
		uint16_t CteReqInterval;  // Requested interval for initiating CTE Request procedure
		uint8_t RequestedCteLength;  // Minimum length of CTEs requested in 8 us units
		GAPEnableConnectionCteRequestRequest_pCteReqEnableParams_RequestedCteType_t RequestedCteType;  // Requested types of CTE
	} pCteReqEnableParams;  // Connection CTE Request Enable Parameters
} GAPEnableConnectionCteRequestRequest_t;

/* Enable or disable sending CTE Responses */
typedef enum GAPEnableConnectionCteResponseRequest_Enable_tag {
	GAPEnableConnectionCteResponseRequest_Enable_gCteRspDisable_c = 0x00,
	GAPEnableConnectionCteResponseRequest_Enable_gCteRspEnable_c = 0x01
} GAPEnableConnectionCteResponseRequest_Enable_t;

typedef PACKED_STRUCT GAPEnableConnectionCteResponseRequest_tag {
	uint8_t DeviceId;  // Peer Device Id
	GAPEnableConnectionCteResponseRequest_Enable_t Enable;  // Enable or disable sending CTE Responses
} GAPEnableConnectionCteResponseRequest_t;

typedef PACKED_STRUCT WritePublicDeviceAddressRequest_tag {
	bool_t CPUReset;  // Reset CPU
	uint8_t BluetoothAddress[6];  // Public bluetooth address
} WritePublicDeviceAddressRequest_t;

typedef PACKED_STRUCT GAPPeriodicAdvReceiveEnableRequest_tag {
	uint16_t SyncHandle;  // Sync_Handle to be used to identify the periodic advertiser
} GAPPeriodicAdvReceiveEnableRequest_t;

typedef PACKED_STRUCT GAPPeriodicAdvReceiveDisableRequest_tag {
	uint16_t SyncHandle;  // Sync_Handle to be used to identify the periodic advertiser
} GAPPeriodicAdvReceiveDisableRequest_t;

typedef PACKED_STRUCT GAPPeriodicAdvSyncTransferRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t ServiceData;  // A value provided by the Host
	uint16_t SyncHandle;  // Sync Handle identifying the periodic advertising train
} GAPPeriodicAdvSyncTransferRequest_t;

typedef PACKED_STRUCT GAPPeriodicAdvSetInfoTransferRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	uint16_t ServiceData;  // A value provided by the Host
	uint16_t AdvHandle;  // Used to identify an advertising set
} GAPPeriodicAdvSetInfoTransferRequest_t;

/* Periodic Advertisement Sync Transfer mode */
typedef enum GAPSetPeriodicAdvSyncTransferParamsRequest_Mode_tag {
	GAPSetPeriodicAdvSyncTransferParamsRequest_Mode_NoSyncMode = 0x00,
	GAPSetPeriodicAdvSyncTransferParamsRequest_Mode_NoReports = 0x01,
	GAPSetPeriodicAdvSyncTransferParamsRequest_Mode_ReportsEnabled = 0x02
} GAPSetPeriodicAdvSyncTransferParamsRequest_Mode_t;

typedef PACKED_STRUCT GAPSetPeriodicAdvSyncTransferParamsRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	GAPSetPeriodicAdvSyncTransferParamsRequest_Mode_t Mode;  // Periodic Advertisement Sync Transfer mode
	uint16_t Skip;  // 
            
	uint16_t SyncTimeout;  // 
            
	uint8_t CTEType;  // 0x10|0x04|0x02|0x01  // CTE type
} GAPSetPeriodicAdvSyncTransferParamsRequest_t;

/* Periodic Advertisement Sync Transfer mode */
typedef enum GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_Mode_tag {
	GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_Mode_NoSyncMode = 0x00,
	GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_Mode_NoReports = 0x01,
	GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_Mode_ReportsEnabled = 0x02
} GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_Mode_t;

typedef PACKED_STRUCT GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
	GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_Mode_t Mode;  // Periodic Advertisement Sync Transfer mode
	uint16_t Skip;  // 
            
	uint16_t SyncTimeout;  // 
            
	uint8_t CTEType;  // 0x10|0x04|0x02|0x01  // CTE type
} GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_t;

/* Phy type */
typedef enum GAPEnhancedReadTransmitPowerLevelRequest_Phy_tag {
	GAPEnhancedReadTransmitPowerLevelRequest_Phy_gPowerControlLePhy1M_c = 0x01,
	GAPEnhancedReadTransmitPowerLevelRequest_Phy_gPowerControlLePhy2M_c = 0x02,
	GAPEnhancedReadTransmitPowerLevelRequest_Phy_gPowerControlLePhyCodedS8_c = 0x03,
	GAPEnhancedReadTransmitPowerLevelRequest_Phy_gPowerControlLePhyCodedS2_c = 0x04
} GAPEnhancedReadTransmitPowerLevelRequest_Phy_t;

typedef PACKED_STRUCT GAPEnhancedReadTransmitPowerLevelRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	GAPEnhancedReadTransmitPowerLevelRequest_Phy_t Phy;  // Phy type
} GAPEnhancedReadTransmitPowerLevelRequest_t;

/* Phy type */
typedef enum GAPReadRemoteTransmitPowerLevelRequest_Phy_tag {
	GAPReadRemoteTransmitPowerLevelRequest_Phy_gPowerControlLePhy1M_c = 0x01,
	GAPReadRemoteTransmitPowerLevelRequest_Phy_gPowerControlLePhy2M_c = 0x02,
	GAPReadRemoteTransmitPowerLevelRequest_Phy_gPowerControlLePhyCodedS8_c = 0x03,
	GAPReadRemoteTransmitPowerLevelRequest_Phy_gPowerControlLePhyCodedS2_c = 0x04
} GAPReadRemoteTransmitPowerLevelRequest_Phy_t;

typedef PACKED_STRUCT GAPReadRemoteTransmitPowerLevelRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	GAPReadRemoteTransmitPowerLevelRequest_Phy_t Phy;  // Phy type
} GAPReadRemoteTransmitPowerLevelRequest_t;

typedef PACKED_STRUCT GAPSetPathLossReportingParametersRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	struct {
		uint8_t HighThreshold;  // High threshold for the path loss
		uint8_t HighHysteresis;  // Hysteresis value for the high threshold
		uint8_t LowThreshold;  // Low threshold for the path loss
		uint8_t LowHysteresis;  // Hysteresis value for the low threshold
		uint16_t minTimeSpent;  // Minimum time in number of connection events to be observed once the crosses the threshold before an event is generated
	} pPathLossReportingParams;  // Path Loss Reporting Parameters to set
} GAPSetPathLossReportingParametersRequest_t;

/* Enable or disable path loss threshold reporting */
typedef enum GAPEnablePathLossReportingRequest_Enable_tag {
	GAPEnablePathLossReportingRequest_Enable_gPathLossReportingDisable_c = 0x00,
	GAPEnablePathLossReportingRequest_Enable_gPathLossReportingEnable_c = 0x01
} GAPEnablePathLossReportingRequest_Enable_t;

typedef PACKED_STRUCT GAPEnablePathLossReportingRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	GAPEnablePathLossReportingRequest_Enable_t Enable;  // Enable or disable path loss threshold reporting
} GAPEnablePathLossReportingRequest_t;

/* Enable or disable local tx power reports */
typedef enum GAPEnableTransmitPowerReportingRequest_LocalEnable_tag {
	GAPEnableTransmitPowerReportingRequest_LocalEnable_gTxPowerReportingDisable_c = 0x00,
	GAPEnableTransmitPowerReportingRequest_LocalEnable_gTxPowerReportingEnable_c = 0x01
} GAPEnableTransmitPowerReportingRequest_LocalEnable_t;

/* Enable or disable remote tx power reports */
typedef enum GAPEnableTransmitPowerReportingRequest_RemoteEnable_tag {
	GAPEnableTransmitPowerReportingRequest_RemoteEnable_gTxPowerReportingDisable_c = 0x00,
	GAPEnableTransmitPowerReportingRequest_RemoteEnable_gTxPowerReportingEnable_c = 0x01
} GAPEnableTransmitPowerReportingRequest_RemoteEnable_t;

typedef PACKED_STRUCT GAPEnableTransmitPowerReportingRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	GAPEnableTransmitPowerReportingRequest_LocalEnable_t LocalEnable;  // Enable or disable local tx power reports
	GAPEnableTransmitPowerReportingRequest_RemoteEnable_t RemoteEnable;  // Enable or disable remote tx power reports
} GAPEnableTransmitPowerReportingRequest_t;

typedef PACKED_STRUCT GAPEattConnectionRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	uint16_t MTU;  // MTU
	uint8_t NoOfBearers;  // Number of bearers to be established
	uint16_t InitialCredits;  // Initial credits
	bool_t AutoCreditsMgmt;  // Set to TRUE if EATT should manage the enhanced bearers credits
} GAPEattConnectionRequest_t;

typedef PACKED_STRUCT GAPEattConnectionAccept_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	bool_t Accept;  // Accept or reject a received Eatt Connection Request
	uint16_t MTU;  // MTU
	uint16_t InitialCredits;  // Initial credits
	bool_t AutoCreditsMgmt;  // Set to TRUE if EATT should manage the enhanced bearers credits
} GAPEattConnectionAccept_t;

typedef struct GAPEattReconfigureRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	uint16_t MTU;  // MTU
	uint16_t MPS;  // MPS
	uint8_t NoOfBearers;  // Number of bearers to be established
	uint8_t *BearerIds;  // Array of bearer Ids to be reconfigured
} GAPEattReconfigureRequest_t;

typedef PACKED_STRUCT GAPEattSendCreditsRequest_tag {
	uint8_t DeviceId;  // Device ID for the active peer
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	uint16_t Credits;  // Credits
} GAPEattSendCreditsRequest_t;

#endif  /* GAP_ENABLE */

#if FSCI_ENABLE
/* Signal Host When Wake Up */
typedef enum FSCIAllowDeviceToSleepRequest_SignalHostWhenWakeUp_tag {
	FSCIAllowDeviceToSleepRequest_SignalHostWhenWakeUp_FALSE = 0x00,
	FSCIAllowDeviceToSleepRequest_SignalHostWhenWakeUp_TRUE = 0x01
} FSCIAllowDeviceToSleepRequest_SignalHostWhenWakeUp_t;

typedef PACKED_STRUCT FSCIAllowDeviceToSleepRequest_tag {
	FSCIAllowDeviceToSleepRequest_SignalHostWhenWakeUp_t SignalHostWhenWakeUp;  // Signal Host When Wake Up
	uint32_t DeepSleepDuration;  // Deep sleep duration in units of 16 us. Default value is 5 s
	uint8_t DeepSleepMode;  // Deep sleep mode selection.
} FSCIAllowDeviceToSleepRequest_t;

#endif  /* FSCI_ENABLE */

#if NVM_ENABLE
#endif  /* NVM_ENABLE */

#if FSCI_ENABLE
typedef PACKED_STRUCT FSCIGetNumberOfFreeBuffersResponse_tag {
	uint16_t FreeBuffers;  // Number of free buffers
} FSCIGetNumberOfFreeBuffersResponse_t;

typedef PACKED_STRUCT FSCIAckIndication_tag {
	uint8_t Checksum;  // Checksum of the FSCI packet being acknowledged
} FSCIAckIndication_t;

/* Occurred error */
typedef enum FSCIErrorIndication_Status_tag {
	FSCIErrorIndication_Status_gFsciSuccess_c = 0x00,
	FSCIErrorIndication_Status_gFsciSAPDisabled_c = 0xF0,
	FSCIErrorIndication_Status_gFsciAppMsgTooBig_c = 0xF3,
	FSCIErrorIndication_Status_gFsciOutOfMessages_c = 0xF4,
	FSCIErrorIndication_Status_gFsciUnknownOpcodeGroup_c = 0xF7,
	FSCIErrorIndication_Status_gFsciOpcodeGroupIsDisabled_c = 0xF8,
	FSCIErrorIndication_Status_gFsciUnknownOpcode_c = 0xFD,
	FSCIErrorIndication_Status_gFsciTooBig_c = 0xFE,
	FSCIErrorIndication_Status_gFsciError_c = 0xFF
} FSCIErrorIndication_Status_t;

typedef PACKED_STRUCT FSCIErrorIndication_tag {
	FSCIErrorIndication_Status_t Status;  // Occurred error
} FSCIErrorIndication_t;

/* Status */
typedef enum FSCIAllowDeviceToSleepConfirm_Status_tag {
	FSCIAllowDeviceToSleepConfirm_Status_gSuccess = 0x00,
	FSCIAllowDeviceToSleepConfirm_Status_gError = 0xFF
} FSCIAllowDeviceToSleepConfirm_Status_t;

typedef PACKED_STRUCT FSCIAllowDeviceToSleepConfirm_tag {
	FSCIAllowDeviceToSleepConfirm_Status_t Status;  // Status
} FSCIAllowDeviceToSleepConfirm_t;

typedef PACKED_STRUCT FSCIWakeUpIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} FSCIWakeUpIndication_t;

typedef PACKED_STRUCT FSCIGetWakeupReasonResponse_tag {
	uint8_t WakeUpReason;  // 0x80|0x40|0x20|0x10|0x08|0x04|0x02|0x01  // Wake Up Reason
} FSCIGetWakeupReasonResponse_t;

typedef PACKED_STRUCT FSCIGetNbuVersionResponse_tag {
	uint8_t versionNumber[3];  // Major/Minor/Subminor
	uint8_t ShaLength;  // Length of digest
	uint8_t TagLength;  // Length of tag
	uint8_t TagString[1];  // tag
} FSCIGetNbuVersionResponse_t;

#endif  /* FSCI_ENABLE */

#if L2CAPCB_ENABLE
/* The status of the L2CAP request */
typedef enum L2CAPCBConfirm_Status_tag {
	L2CAPCBConfirm_Status_gBleSuccess_c = 0x0000,
	L2CAPCBConfirm_Status_gBleInvalidParameter_c = 0x0001,
	L2CAPCBConfirm_Status_gBleOverflow_c = 0x0002,
	L2CAPCBConfirm_Status_gBleUnavailable_c = 0x0003,
	L2CAPCBConfirm_Status_gBleFeatureNotSupported_c = 0x0004,
	L2CAPCBConfirm_Status_gBleOutOfMemory_c = 0x0005,
	L2CAPCBConfirm_Status_gBleAlreadyInitialized_c = 0x0006,
	L2CAPCBConfirm_Status_gBleOsError_c = 0x0007,
	L2CAPCBConfirm_Status_gBleUnexpectedError_c = 0x0008,
	L2CAPCBConfirm_Status_gBleInvalidState_c = 0x0009,
	L2CAPCBConfirm_Status_gBleTimerError_c = 0x000A,
	L2CAPCBConfirm_Status_gHciUnknownHciCommand_c = 0x0101,
	L2CAPCBConfirm_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	L2CAPCBConfirm_Status_gHciHardwareFailure_c = 0x0103,
	L2CAPCBConfirm_Status_gHciPageTimeout_c = 0x0104,
	L2CAPCBConfirm_Status_gHciAuthenticationFailure_c = 0x0105,
	L2CAPCBConfirm_Status_gHciPinOrKeyMissing_c = 0x0106,
	L2CAPCBConfirm_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	L2CAPCBConfirm_Status_gHciConnectionTimeout_c = 0x0108,
	L2CAPCBConfirm_Status_gHciConnectionLimitExceeded_c = 0x0109,
	L2CAPCBConfirm_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	L2CAPCBConfirm_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	L2CAPCBConfirm_Status_gHciCommandDisallowed_c = 0x010C,
	L2CAPCBConfirm_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	L2CAPCBConfirm_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	L2CAPCBConfirm_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	L2CAPCBConfirm_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	L2CAPCBConfirm_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	L2CAPCBConfirm_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	L2CAPCBConfirm_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	L2CAPCBConfirm_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	L2CAPCBConfirm_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	L2CAPCBConfirm_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	L2CAPCBConfirm_Status_gHciRepeatedAttempts_c = 0x0117,
	L2CAPCBConfirm_Status_gHciPairingNotAllowed_c = 0x0118,
	L2CAPCBConfirm_Status_gHciUnknownLpmPdu_c = 0x0119,
	L2CAPCBConfirm_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	L2CAPCBConfirm_Status_gHciScoOffsetRejected_c = 0x011B,
	L2CAPCBConfirm_Status_gHciScoIntervalRejected_c = 0x011C,
	L2CAPCBConfirm_Status_gHciScoAirModeRejected_c = 0x011D,
	L2CAPCBConfirm_Status_gHciInvalidLpmParameters_c = 0x011E,
	L2CAPCBConfirm_Status_gHciUnspecifiedError_c = 0x011F,
	L2CAPCBConfirm_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	L2CAPCBConfirm_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	L2CAPCBConfirm_Status_gHciLLResponseTimeout_c = 0x0122,
	L2CAPCBConfirm_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	L2CAPCBConfirm_Status_gHciLmpPduNotAllowed_c = 0x0124,
	L2CAPCBConfirm_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	L2CAPCBConfirm_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	L2CAPCBConfirm_Status_gHciRequestedQosNotSupported_c = 0x0127,
	L2CAPCBConfirm_Status_gHciInstantPassed_c = 0x0128,
	L2CAPCBConfirm_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	L2CAPCBConfirm_Status_gHciDifferentTransactionCollision_c = 0x012A,
	L2CAPCBConfirm_Status_gHciReserved_0x2B_c = 0x012B,
	L2CAPCBConfirm_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	L2CAPCBConfirm_Status_gHciQosRejected_c = 0x012D,
	L2CAPCBConfirm_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	L2CAPCBConfirm_Status_gHciInsufficientSecurity_c = 0x012F,
	L2CAPCBConfirm_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	L2CAPCBConfirm_Status_gHciReserved_0x31_c = 0x0131,
	L2CAPCBConfirm_Status_gHciRoleSwitchPending_c = 0x0132,
	L2CAPCBConfirm_Status_gHciReserved_0x33_c = 0x0133,
	L2CAPCBConfirm_Status_gHciReservedSlotViolation_c = 0x0134,
	L2CAPCBConfirm_Status_gHciRoleSwitchFailed_c = 0x0135,
	L2CAPCBConfirm_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	L2CAPCBConfirm_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	L2CAPCBConfirm_Status_gHciHostBusyPairing_c = 0x0138,
	L2CAPCBConfirm_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	L2CAPCBConfirm_Status_gHciControllerBusy_c = 0x013A,
	L2CAPCBConfirm_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	L2CAPCBConfirm_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	L2CAPCBConfirm_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	L2CAPCBConfirm_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	L2CAPCBConfirm_Status_gHciMacConnectionFailed_c = 0x013F,
	L2CAPCBConfirm_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	L2CAPCBConfirm_Status_gHciType0SubmapNotDefined_c = 0x0141,
	L2CAPCBConfirm_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	L2CAPCBConfirm_Status_gHciLimitReached_c = 0x0143,
	L2CAPCBConfirm_Status_gHciOperationCancelledByHost_c = 0x0144,
	L2CAPCBConfirm_Status_gHciAlreadyInit_c = 0x01A0,
	L2CAPCBConfirm_Status_gHciInvalidParameter_c = 0x01A1,
	L2CAPCBConfirm_Status_gHciCallbackNotInstalled_c = 0x01A2,
	L2CAPCBConfirm_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	L2CAPCBConfirm_Status_gHciCommandNotSupported_c = 0x01A4,
	L2CAPCBConfirm_Status_gHciEventNotSupported_c = 0x01A5,
	L2CAPCBConfirm_Status_gHciTransportError_c = 0x01A6,
	L2CAPCBConfirm_Status_gL2caAlreadyInit_c = 0x03F0,
	L2CAPCBConfirm_Status_gL2caInsufficientResources_c = 0x03F1,
	L2CAPCBConfirm_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	L2CAPCBConfirm_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	L2CAPCBConfirm_Status_gL2caLePsmInvalid_c = 0x03F4,
	L2CAPCBConfirm_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	L2CAPCBConfirm_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	L2CAPCBConfirm_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	L2CAPCBConfirm_Status_gL2caChannelInvalid_c = 0x03F8,
	L2CAPCBConfirm_Status_gL2caChannelClosed_c = 0x03F9,
	L2CAPCBConfirm_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	L2CAPCBConfirm_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	L2CAPCBConfirm_Status_gL2caChannelBusy_c = 0x03FC,
	L2CAPCBConfirm_Status_gL2caInvalidParameter_c = 0x03FE,
	L2CAPCBConfirm_Status_gL2caError_c = 0x03FF,
	L2CAPCBConfirm_Status_gSmNullCBFunction_c = 0x0401,
	L2CAPCBConfirm_Status_gSmCommandNotSupported_c = 0x0402,
	L2CAPCBConfirm_Status_gSmUnexpectedCommand_c = 0x0403,
	L2CAPCBConfirm_Status_gSmInvalidCommandCode_c = 0x0404,
	L2CAPCBConfirm_Status_gSmInvalidCommandLength_c = 0x0405,
	L2CAPCBConfirm_Status_gSmInvalidCommandParameter_c = 0x0406,
	L2CAPCBConfirm_Status_gSmInvalidDeviceId_c = 0x0407,
	L2CAPCBConfirm_Status_gSmInvalidInternalOperation_c = 0x0408,
	L2CAPCBConfirm_Status_gSmInvalidConnectionHandle_c = 0x0409,
	L2CAPCBConfirm_Status_gSmInproperKeyDistributionField_c = 0x040A,
	L2CAPCBConfirm_Status_gSmUnexpectedKeyType_c = 0x040B,
	L2CAPCBConfirm_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	L2CAPCBConfirm_Status_gSmUnexpectedKeyset_c = 0x040D,
	L2CAPCBConfirm_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	L2CAPCBConfirm_Status_gSmUnknownSmpPacketType_c = 0x040F,
	L2CAPCBConfirm_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	L2CAPCBConfirm_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	L2CAPCBConfirm_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	L2CAPCBConfirm_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	L2CAPCBConfirm_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	L2CAPCBConfirm_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	L2CAPCBConfirm_Status_gSmInvalidHciEventParameter_c = 0x0416,
	L2CAPCBConfirm_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	L2CAPCBConfirm_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	L2CAPCBConfirm_Status_gSmInsufficientResources_c = 0x0419,
	L2CAPCBConfirm_Status_gSmOobDataAddressMismatch_c = 0x041A,
	L2CAPCBConfirm_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	L2CAPCBConfirm_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	L2CAPCBConfirm_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	L2CAPCBConfirm_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	L2CAPCBConfirm_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	L2CAPCBConfirm_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	L2CAPCBConfirm_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	L2CAPCBConfirm_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	L2CAPCBConfirm_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	L2CAPCBConfirm_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	L2CAPCBConfirm_Status_gSmPairingErrorTimeout_c = 0x0436,
	L2CAPCBConfirm_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	L2CAPCBConfirm_Status_gSmPairingAlreadyStarted_c = 0x0438,
	L2CAPCBConfirm_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	L2CAPCBConfirm_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	L2CAPCBConfirm_Status_gSmTbInvalidDataSignature_c = 0x0461,
	L2CAPCBConfirm_Status_gAttInvalidHandle_c = 0x0501,
	L2CAPCBConfirm_Status_gAttReadNotPermitted_c = 0x0502,
	L2CAPCBConfirm_Status_gAttWriteNotPermitted_c = 0x0503,
	L2CAPCBConfirm_Status_gAttInvalidPdu_c = 0x0504,
	L2CAPCBConfirm_Status_gAttInsufficientAuthentication_c = 0x0505,
	L2CAPCBConfirm_Status_gAttRequestNotSupported_c = 0x0506,
	L2CAPCBConfirm_Status_gAttInvalidOffset_c = 0x0507,
	L2CAPCBConfirm_Status_gAttInsufficientAuthorization_c = 0x0508,
	L2CAPCBConfirm_Status_gAttPrepareQueueFull_c = 0x0509,
	L2CAPCBConfirm_Status_gAttAttributeNotFound_c = 0x050A,
	L2CAPCBConfirm_Status_gAttAttributeNotLong_c = 0x050B,
	L2CAPCBConfirm_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	L2CAPCBConfirm_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	L2CAPCBConfirm_Status_gAttUnlikelyor_c = 0x050E,
	L2CAPCBConfirm_Status_gAttInsufficientEncryption_c = 0x050F,
	L2CAPCBConfirm_Status_gAttUnsupportedGroupType_c = 0x0510,
	L2CAPCBConfirm_Status_gAttInsufficientResources_c = 0x0511,
	L2CAPCBConfirm_Status_gAttDatabaseOutOfSync_c = 0x0512,
	L2CAPCBConfirm_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	L2CAPCBConfirm_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	L2CAPCBConfirm_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	L2CAPCBConfirm_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	L2CAPCBConfirm_Status_gGattInvalidValueLength_c = 0x0605,
	L2CAPCBConfirm_Status_gGattServerTimeout_c = 0x0606,
	L2CAPCBConfirm_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	L2CAPCBConfirm_Status_gGattClientConfirmationTimeout_c = 0x0608,
	L2CAPCBConfirm_Status_gGattInvalidPduReceived_c = 0x0609,
	L2CAPCBConfirm_Status_gGattPeerDisconnected_c = 0x060A,
	L2CAPCBConfirm_Status_gGattMtuExchangeInProgress_c = 0x060B,
	L2CAPCBConfirm_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	L2CAPCBConfirm_Status_gGapAdvDataTooLong_c = 0x0701,
	L2CAPCBConfirm_Status_gGapScanRspDataTooLong_c = 0x0702,
	L2CAPCBConfirm_Status_gGapDeviceNotBonded_c = 0x0703,
	L2CAPCBConfirm_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	L2CAPCBConfirm_Status_gDevDbCccdLimitReached_c = 0x0801,
	L2CAPCBConfirm_Status_gDevDbCccdNotFound_c = 0x0802,
	L2CAPCBConfirm_Status_gGattDbInvalidHandle_c = 0x0901,
	L2CAPCBConfirm_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	L2CAPCBConfirm_Status_gGattDbCccdNotFound_c = 0x0903,
	L2CAPCBConfirm_Status_gGattDbServiceNotFound_c = 0x0904,
	L2CAPCBConfirm_Status_gGattDbDescriptorNotFound_c = 0x0905,
	L2CAPCBConfirm_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} L2CAPCBConfirm_Status_t;

typedef PACKED_STRUCT L2CAPCBConfirm_tag {
	L2CAPCBConfirm_Status_t Status;  // The status of the L2CAP request
} L2CAPCBConfirm_t;

typedef struct L2CAPCBLePsmConnectionRequestIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
		uint16_t PeerMtu;  // Peer MTU
		uint16_t PeerMps;  // Peer MPS
		uint16_t InitialCredits;  // Initial credits
	} LeCbConnectionRequest;  // Connection request event data
} L2CAPCBLePsmConnectionRequestIndication_t;

/* Connection result */
typedef enum L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_tag {
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gSuccessful_c = 0x0000,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gLePsmNotSupported_c = 0x0002,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gNoResourcesAvailable_c = 0x0004,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gInsufficientAuthentication_c = 0x0005,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gInsufficientAuthorization_c = 0x0006,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gInsufficientEncryptionKeySize_c = 0x0007,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gInsufficientEncryption_c = 0x0008,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gInvalidSourceCid_c = 0x0009,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gSourceCidAlreadyAllocated_c = 0x000A,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gTimerError_c = 0x000B,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gCommandRejected_c = 0x0100,
	L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_gResponseTimeout_c = 0xFFFE
} L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_t;

typedef struct L2CAPCBLePsmConnectionCompleteIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t ChannelId;  // The L2CAP Channel Id assigned on the initiator
		uint16_t PeerMtu;  // Peer's MTU
		uint16_t PeerMps;  // Peer's MPS
		uint16_t InitialCredits;  // Initial credits
		L2CAPCBLePsmConnectionCompleteIndication_LeCbConnectionComplete_Result_t Result;  // Connection result
	} LeCbConnectionComplete;  // Connection complete event data
} L2CAPCBLePsmConnectionCompleteIndication_t;

typedef struct L2CAPCBLePsmDisconnectNotificationIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t ChannelId;  // The L2CAP Channel Id assigned on the initiator
	} LeCbDisconnection;  // Disconnection event data
} L2CAPCBLePsmDisconnectNotificationIndication_t;

typedef struct L2CAPCBNoPeerCreditsIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t ChannelId;  // The L2CAP Channel Id assigned on the initiator
	} LeCbNoPeerCredits;  // No peer credits event data
} L2CAPCBNoPeerCreditsIndication_t;

typedef struct L2CAPCBLocalCreditsNotificationIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t ChannelId;  // The L2CAP Channel Id assigned on the initiator
		uint16_t LocalCredits;  // Local credits
	} LeCbLocalCreditsNotification;  // Local credits notification event data
} L2CAPCBLocalCreditsNotificationIndication_t;

typedef struct L2CAPCBLeCbDataIndication_tag {
	uint8_t DeviceId;  // The DeviceId from which data packet is received
	uint16_t SrcCid;  // Source Channel Id
	uint16_t PacketLength;  // Length of the received data packet
	uint8_t *Packet;  // The received data packet
} L2CAPCBLeCbDataIndication_t;

/* Internal error reported by the L2CAP layer */
typedef enum L2CAPCBErrorIndication_LeCbError_Error_tag {
	L2CAPCBErrorIndication_LeCbError_Error_gBleSuccess_c = 0x0000,
	L2CAPCBErrorIndication_LeCbError_Error_gBleInvalidParameter_c = 0x0001,
	L2CAPCBErrorIndication_LeCbError_Error_gBleOverflow_c = 0x0002,
	L2CAPCBErrorIndication_LeCbError_Error_gBleUnavailable_c = 0x0003,
	L2CAPCBErrorIndication_LeCbError_Error_gBleFeatureNotSupported_c = 0x0004,
	L2CAPCBErrorIndication_LeCbError_Error_gBleOutOfMemory_c = 0x0005,
	L2CAPCBErrorIndication_LeCbError_Error_gBleAlreadyInitialized_c = 0x0006,
	L2CAPCBErrorIndication_LeCbError_Error_gBleOsError_c = 0x0007,
	L2CAPCBErrorIndication_LeCbError_Error_gBleUnexpectedError_c = 0x0008,
	L2CAPCBErrorIndication_LeCbError_Error_gBleInvalidState_c = 0x0009,
	L2CAPCBErrorIndication_LeCbError_Error_gBleTimerError_c = 0x000A,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnknownHciCommand_c = 0x0101,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	L2CAPCBErrorIndication_LeCbError_Error_gHciHardwareFailure_c = 0x0103,
	L2CAPCBErrorIndication_LeCbError_Error_gHciPageTimeout_c = 0x0104,
	L2CAPCBErrorIndication_LeCbError_Error_gHciAuthenticationFailure_c = 0x0105,
	L2CAPCBErrorIndication_LeCbError_Error_gHciPinOrKeyMissing_c = 0x0106,
	L2CAPCBErrorIndication_LeCbError_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionTimeout_c = 0x0108,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionLimitExceeded_c = 0x0109,
	L2CAPCBErrorIndication_LeCbError_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	L2CAPCBErrorIndication_LeCbError_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	L2CAPCBErrorIndication_LeCbError_Error_gHciCommandDisallowed_c = 0x010C,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	L2CAPCBErrorIndication_LeCbError_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRepeatedAttempts_c = 0x0117,
	L2CAPCBErrorIndication_LeCbError_Error_gHciPairingNotAllowed_c = 0x0118,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnknownLpmPdu_c = 0x0119,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	L2CAPCBErrorIndication_LeCbError_Error_gHciScoOffsetRejected_c = 0x011B,
	L2CAPCBErrorIndication_LeCbError_Error_gHciScoIntervalRejected_c = 0x011C,
	L2CAPCBErrorIndication_LeCbError_Error_gHciScoAirModeRejected_c = 0x011D,
	L2CAPCBErrorIndication_LeCbError_Error_gHciInvalidLpmParameters_c = 0x011E,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnspecifiedError_c = 0x011F,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	L2CAPCBErrorIndication_LeCbError_Error_gHciLLResponseTimeout_c = 0x0122,
	L2CAPCBErrorIndication_LeCbError_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	L2CAPCBErrorIndication_LeCbError_Error_gHciLmpPduNotAllowed_c = 0x0124,
	L2CAPCBErrorIndication_LeCbError_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	L2CAPCBErrorIndication_LeCbError_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRequestedQosNotSupported_c = 0x0127,
	L2CAPCBErrorIndication_LeCbError_Error_gHciInstantPassed_c = 0x0128,
	L2CAPCBErrorIndication_LeCbError_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	L2CAPCBErrorIndication_LeCbError_Error_gHciDifferentTransactionCollision_c = 0x012A,
	L2CAPCBErrorIndication_LeCbError_Error_gHciReserved_0x2B_c = 0x012B,
	L2CAPCBErrorIndication_LeCbError_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	L2CAPCBErrorIndication_LeCbError_Error_gHciQosRejected_c = 0x012D,
	L2CAPCBErrorIndication_LeCbError_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	L2CAPCBErrorIndication_LeCbError_Error_gHciInsufficientSecurity_c = 0x012F,
	L2CAPCBErrorIndication_LeCbError_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	L2CAPCBErrorIndication_LeCbError_Error_gHciReserved_0x31_c = 0x0131,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRoleSwitchPending_c = 0x0132,
	L2CAPCBErrorIndication_LeCbError_Error_gHciReserved_0x33_c = 0x0133,
	L2CAPCBErrorIndication_LeCbError_Error_gHciReservedSlotViolation_c = 0x0134,
	L2CAPCBErrorIndication_LeCbError_Error_gHciRoleSwitchFailed_c = 0x0135,
	L2CAPCBErrorIndication_LeCbError_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	L2CAPCBErrorIndication_LeCbError_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	L2CAPCBErrorIndication_LeCbError_Error_gHciHostBusyPairing_c = 0x0138,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	L2CAPCBErrorIndication_LeCbError_Error_gHciControllerBusy_c = 0x013A,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	L2CAPCBErrorIndication_LeCbError_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	L2CAPCBErrorIndication_LeCbError_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	L2CAPCBErrorIndication_LeCbError_Error_gHciMacConnectionFailed_c = 0x013F,
	L2CAPCBErrorIndication_LeCbError_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	L2CAPCBErrorIndication_LeCbError_Error_gHciType0SubmapNotDefined_c = 0x0141,
	L2CAPCBErrorIndication_LeCbError_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	L2CAPCBErrorIndication_LeCbError_Error_gHciLimitReached_c = 0x0143,
	L2CAPCBErrorIndication_LeCbError_Error_gHciOperationCancelledByHost_c = 0x0144,
	L2CAPCBErrorIndication_LeCbError_Error_gHciAlreadyInit_c = 0x01A0,
	L2CAPCBErrorIndication_LeCbError_Error_gHciInvalidParameter_c = 0x01A1,
	L2CAPCBErrorIndication_LeCbError_Error_gHciCallbackNotInstalled_c = 0x01A2,
	L2CAPCBErrorIndication_LeCbError_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	L2CAPCBErrorIndication_LeCbError_Error_gHciCommandNotSupported_c = 0x01A4,
	L2CAPCBErrorIndication_LeCbError_Error_gHciEventNotSupported_c = 0x01A5,
	L2CAPCBErrorIndication_LeCbError_Error_gHciTransportError_c = 0x01A6,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caAlreadyInit_c = 0x03F0,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caInsufficientResources_c = 0x03F1,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caLePsmInvalid_c = 0x03F4,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caChannelInvalid_c = 0x03F8,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caChannelClosed_c = 0x03F9,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caChannelBusy_c = 0x03FC,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caInvalidParameter_c = 0x03FE,
	L2CAPCBErrorIndication_LeCbError_Error_gL2caError_c = 0x03FF,
	L2CAPCBErrorIndication_LeCbError_Error_gSmNullCBFunction_c = 0x0401,
	L2CAPCBErrorIndication_LeCbError_Error_gSmCommandNotSupported_c = 0x0402,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnexpectedCommand_c = 0x0403,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidCommandCode_c = 0x0404,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidCommandLength_c = 0x0405,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidCommandParameter_c = 0x0406,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidDeviceId_c = 0x0407,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidInternalOperation_c = 0x0408,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidConnectionHandle_c = 0x0409,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInproperKeyDistributionField_c = 0x040A,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnexpectedKeyType_c = 0x040B,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnexpectedKeyset_c = 0x040D,
	L2CAPCBErrorIndication_LeCbError_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnknownSmpPacketType_c = 0x040F,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	L2CAPCBErrorIndication_LeCbError_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	L2CAPCBErrorIndication_LeCbError_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	L2CAPCBErrorIndication_LeCbError_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	L2CAPCBErrorIndication_LeCbError_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInvalidHciEventParameter_c = 0x0416,
	L2CAPCBErrorIndication_LeCbError_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	L2CAPCBErrorIndication_LeCbError_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	L2CAPCBErrorIndication_LeCbError_Error_gSmInsufficientResources_c = 0x0419,
	L2CAPCBErrorIndication_LeCbError_Error_gSmOobDataAddressMismatch_c = 0x041A,
	L2CAPCBErrorIndication_LeCbError_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	L2CAPCBErrorIndication_LeCbError_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	L2CAPCBErrorIndication_LeCbError_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorTimeout_c = 0x0436,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingAlreadyStarted_c = 0x0438,
	L2CAPCBErrorIndication_LeCbError_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	L2CAPCBErrorIndication_LeCbError_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	L2CAPCBErrorIndication_LeCbError_Error_gSmTbInvalidDataSignature_c = 0x0461,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInvalidHandle_c = 0x0501,
	L2CAPCBErrorIndication_LeCbError_Error_gAttReadNotPermitted_c = 0x0502,
	L2CAPCBErrorIndication_LeCbError_Error_gAttWriteNotPermitted_c = 0x0503,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInvalidPdu_c = 0x0504,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInsufficientAuthentication_c = 0x0505,
	L2CAPCBErrorIndication_LeCbError_Error_gAttRequestNotSupported_c = 0x0506,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInvalidOffset_c = 0x0507,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInsufficientAuthorization_c = 0x0508,
	L2CAPCBErrorIndication_LeCbError_Error_gAttPrepareQueueFull_c = 0x0509,
	L2CAPCBErrorIndication_LeCbError_Error_gAttAttributeNotFound_c = 0x050A,
	L2CAPCBErrorIndication_LeCbError_Error_gAttAttributeNotLong_c = 0x050B,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	L2CAPCBErrorIndication_LeCbError_Error_gAttUnlikelyor_c = 0x050E,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInsufficientEncryption_c = 0x050F,
	L2CAPCBErrorIndication_LeCbError_Error_gAttUnsupportedGroupType_c = 0x0510,
	L2CAPCBErrorIndication_LeCbError_Error_gAttInsufficientResources_c = 0x0511,
	L2CAPCBErrorIndication_LeCbError_Error_gAttDatabaseOutOfSync_c = 0x0512,
	L2CAPCBErrorIndication_LeCbError_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	L2CAPCBErrorIndication_LeCbError_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	L2CAPCBErrorIndication_LeCbError_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	L2CAPCBErrorIndication_LeCbError_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	L2CAPCBErrorIndication_LeCbError_Error_gGattInvalidValueLength_c = 0x0605,
	L2CAPCBErrorIndication_LeCbError_Error_gGattServerTimeout_c = 0x0606,
	L2CAPCBErrorIndication_LeCbError_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	L2CAPCBErrorIndication_LeCbError_Error_gGattClientConfirmationTimeout_c = 0x0608,
	L2CAPCBErrorIndication_LeCbError_Error_gGattInvalidPduReceived_c = 0x0609,
	L2CAPCBErrorIndication_LeCbError_Error_gGattPeerDisconnected_c = 0x060A,
	L2CAPCBErrorIndication_LeCbError_Error_gGattMtuExchangeInProgress_c = 0x060B,
	L2CAPCBErrorIndication_LeCbError_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	L2CAPCBErrorIndication_LeCbError_Error_gGapAdvDataTooLong_c = 0x0701,
	L2CAPCBErrorIndication_LeCbError_Error_gGapScanRspDataTooLong_c = 0x0702,
	L2CAPCBErrorIndication_LeCbError_Error_gGapDeviceNotBonded_c = 0x0703,
	L2CAPCBErrorIndication_LeCbError_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	L2CAPCBErrorIndication_LeCbError_Error_gDevDbCccdLimitReached_c = 0x0801,
	L2CAPCBErrorIndication_LeCbError_Error_gDevDbCccdNotFound_c = 0x0802,
	L2CAPCBErrorIndication_LeCbError_Error_gGattDbInvalidHandle_c = 0x0901,
	L2CAPCBErrorIndication_LeCbError_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	L2CAPCBErrorIndication_LeCbError_Error_gGattDbCccdNotFound_c = 0x0903,
	L2CAPCBErrorIndication_LeCbError_Error_gGattDbServiceNotFound_c = 0x0904,
	L2CAPCBErrorIndication_LeCbError_Error_gGattDbDescriptorNotFound_c = 0x0905,
	L2CAPCBErrorIndication_LeCbError_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} L2CAPCBErrorIndication_LeCbError_Error_t;

/* Function which triggered the errror */
typedef enum L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_tag {
	L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_gL2ca_CancelConnection_c = 0x00,
	L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_gL2ca_SendLeCredit_c = 0x01,
	L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_gL2ca_DisconnectLePsm_c = 0x02,
	L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_gL2ca_HandleSendLeCbData_c = 0x03,
	L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_gL2ca_HandleRecvLeCbData_c = 0x04,
	L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_gL2ca_HandleLeFlowControlCredit_c = 0x05
} L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_t;

typedef struct L2CAPCBErrorIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		L2CAPCBErrorIndication_LeCbError_Error_t Error;  // Internal error reported by the L2CAP layer
		L2CAPCBErrorIndication_LeCbError_l2caErrorSource_t_t l2caErrorSource_t;  // Function which triggered the errror
	} LeCbError;  // Internal error event data
} L2CAPCBErrorIndication_t;

/* Channel status */
typedef enum L2CAPCBChannelStatusNotificationIndication_ChannelStatus_tag {
	L2CAPCBChannelStatusNotificationIndication_ChannelStatus_gL2ca_ChannelIdle_c = 0x00,
	L2CAPCBChannelStatusNotificationIndication_ChannelStatus_gL2ca_ChannelBusy_c = 0x01
} L2CAPCBChannelStatusNotificationIndication_ChannelStatus_t;

typedef PACKED_STRUCT L2CAPCBChannelStatusNotificationIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	uint8_t DeviceId;  // The DeviceId from which data packet is received
	uint16_t SrcCid;  // Source Channel Id
	L2CAPCBChannelStatusNotificationIndication_ChannelStatus_t ChannelStatus;  // Channel status
} L2CAPCBChannelStatusNotificationIndication_t;

typedef struct L2CAPCBLePsmEnhancedConnectRequestIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t LePsm;  // Bluetooth SIG or Vendor LE_PSM
		uint16_t PeerMtu;  // Peer MTU
		uint16_t PeerMps;  // Peer MPS
		uint16_t InitialCredits;  // Initial number of credits
		uint8_t NoOfChannels;  // Number of requested channels to connect - max 5
		uint8_t *aCids;  // List of CIDs
	} EnhancedConnRequest;  // Enhanced Connection Request event data
} L2CAPCBLePsmEnhancedConnectRequestIndication_t;

/* Result of the connection procedure */
typedef enum L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_tag {
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gSuccessful_c = 0x0000,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gLePsmNotSupported_c = 0x0002,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gNoResourcesAvailable_c = 0x0004,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gInsufficientAuthentication_c = 0x0005,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gInsufficientAuthorization_c = 0x0006,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gInsufficientEncryptionKeySize_c = 0x0007,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gInsufficientEncryption_c = 0x0008,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gInvalidSourceCid_c = 0x0009,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gSourceCidAlreadyAllocated_c = 0x000A,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gTimerError_c = 0x000B,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gCommandRejected_c = 0x0100,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_gResponseTimeout_c = 0xFFFE
} L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_t;

typedef struct L2CAPCBLePsmEnhancedConnectionCompleteIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t PeerMtu;  // Peer MTU
		uint16_t PeerMps;  // Peer MPS
		uint16_t InitialCredits;  // Initial number of credits
		L2CAPCBLePsmEnhancedConnectionCompleteIndication_EnhancedConnComplete_Result_t Result;  // Result of the connection procedure
		uint8_t NoOfChannels;  // Number of requested channels to connect - max 5
		uint8_t *aCids;  // List of CIDs
	} EnhancedConnComplete;  // Enhanced Connection Complete event data
} L2CAPCBLePsmEnhancedConnectionCompleteIndication_t;

/* Result of the reconfiguration */
typedef enum L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_tag {
	L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_gReconfigurationSuccessful_c = 0x0000,
	L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_gMtuReductionNotAllowed_c = 0x0001,
	L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_gMultipleChannelMpsReductionNotAllowed_c = 0x0002,
	L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_gDestinationCidInvalid_c = 0x0003,
	L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_gUnacceptableParameters_c = 0x0004,
	L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_gReconfigurationTimeout_c = 0x0005
} L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_t;

typedef struct L2CAPCBEnhancedReconfigureRequestIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		uint16_t NewMtu;  // New MTU value
		uint16_t NewMps;  // New MPS value
		L2CAPCBEnhancedReconfigureRequestIndication_EnhancedReconfigureRequest_Result_t Result;  // Result of the reconfiguration
		uint8_t NoOfChannels;  // Number of requested channels to connect - max 5
		uint8_t *aCids;  // List of CIDs
	} EnhancedReconfigureRequest;  // Enhanced Reconfigure Request event data
} L2CAPCBEnhancedReconfigureRequestIndication_t;

/* Result of the reconfiguration */
typedef enum L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_tag {
	L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_gReconfigurationSuccessful_c = 0x0000,
	L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_gMtuReductionNotAllowed_c = 0x0001,
	L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_gMultipleChannelMpsReductionNotAllowed_c = 0x0002,
	L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_gDestinationCidInvalid_c = 0x0003,
	L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_gUnacceptableParameters_c = 0x0004,
	L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_gReconfigurationTimeout_c = 0x0005
} L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_t;

typedef struct L2CAPCBEnhancedReconfigureResponseIndication_tag {
	bool_t InformationIncluded;  // Information is included or not
	struct {
		uint8_t DeviceId;  // The DeviceId for which the command is intended
		L2CAPCBEnhancedReconfigureResponseIndication_EnhancedReconfigureResponse_Result_t Result;  // Result of the reconfiguration
	} EnhancedReconfigureResponse;  // Enhanced Reconfigure Response event data
} L2CAPCBEnhancedReconfigureResponseIndication_t;

#endif  /* L2CAPCB_ENABLE */

#if GATT_ENABLE
/* The status of the GATT request */
typedef enum GATTConfirm_Status_tag {
	GATTConfirm_Status_gBleSuccess_c = 0x0000,
	GATTConfirm_Status_gBleInvalidParameter_c = 0x0001,
	GATTConfirm_Status_gBleOverflow_c = 0x0002,
	GATTConfirm_Status_gBleUnavailable_c = 0x0003,
	GATTConfirm_Status_gBleFeatureNotSupported_c = 0x0004,
	GATTConfirm_Status_gBleOutOfMemory_c = 0x0005,
	GATTConfirm_Status_gBleAlreadyInitialized_c = 0x0006,
	GATTConfirm_Status_gBleOsError_c = 0x0007,
	GATTConfirm_Status_gBleUnexpectedError_c = 0x0008,
	GATTConfirm_Status_gBleInvalidState_c = 0x0009,
	GATTConfirm_Status_gBleTimerError_c = 0x000A,
	GATTConfirm_Status_gHciUnknownHciCommand_c = 0x0101,
	GATTConfirm_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTConfirm_Status_gHciHardwareFailure_c = 0x0103,
	GATTConfirm_Status_gHciPageTimeout_c = 0x0104,
	GATTConfirm_Status_gHciAuthenticationFailure_c = 0x0105,
	GATTConfirm_Status_gHciPinOrKeyMissing_c = 0x0106,
	GATTConfirm_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTConfirm_Status_gHciConnectionTimeout_c = 0x0108,
	GATTConfirm_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GATTConfirm_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTConfirm_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTConfirm_Status_gHciCommandDisallowed_c = 0x010C,
	GATTConfirm_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTConfirm_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTConfirm_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTConfirm_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTConfirm_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTConfirm_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTConfirm_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTConfirm_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTConfirm_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTConfirm_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTConfirm_Status_gHciRepeatedAttempts_c = 0x0117,
	GATTConfirm_Status_gHciPairingNotAllowed_c = 0x0118,
	GATTConfirm_Status_gHciUnknownLpmPdu_c = 0x0119,
	GATTConfirm_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTConfirm_Status_gHciScoOffsetRejected_c = 0x011B,
	GATTConfirm_Status_gHciScoIntervalRejected_c = 0x011C,
	GATTConfirm_Status_gHciScoAirModeRejected_c = 0x011D,
	GATTConfirm_Status_gHciInvalidLpmParameters_c = 0x011E,
	GATTConfirm_Status_gHciUnspecifiedError_c = 0x011F,
	GATTConfirm_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTConfirm_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTConfirm_Status_gHciLLResponseTimeout_c = 0x0122,
	GATTConfirm_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTConfirm_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GATTConfirm_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTConfirm_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTConfirm_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GATTConfirm_Status_gHciInstantPassed_c = 0x0128,
	GATTConfirm_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTConfirm_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GATTConfirm_Status_gHciReserved_0x2B_c = 0x012B,
	GATTConfirm_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTConfirm_Status_gHciQosRejected_c = 0x012D,
	GATTConfirm_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTConfirm_Status_gHciInsufficientSecurity_c = 0x012F,
	GATTConfirm_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTConfirm_Status_gHciReserved_0x31_c = 0x0131,
	GATTConfirm_Status_gHciRoleSwitchPending_c = 0x0132,
	GATTConfirm_Status_gHciReserved_0x33_c = 0x0133,
	GATTConfirm_Status_gHciReservedSlotViolation_c = 0x0134,
	GATTConfirm_Status_gHciRoleSwitchFailed_c = 0x0135,
	GATTConfirm_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTConfirm_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTConfirm_Status_gHciHostBusyPairing_c = 0x0138,
	GATTConfirm_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTConfirm_Status_gHciControllerBusy_c = 0x013A,
	GATTConfirm_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTConfirm_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTConfirm_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTConfirm_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTConfirm_Status_gHciMacConnectionFailed_c = 0x013F,
	GATTConfirm_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTConfirm_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GATTConfirm_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTConfirm_Status_gHciLimitReached_c = 0x0143,
	GATTConfirm_Status_gHciOperationCancelledByHost_c = 0x0144,
	GATTConfirm_Status_gHciAlreadyInit_c = 0x01A0,
	GATTConfirm_Status_gHciInvalidParameter_c = 0x01A1,
	GATTConfirm_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GATTConfirm_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTConfirm_Status_gHciCommandNotSupported_c = 0x01A4,
	GATTConfirm_Status_gHciEventNotSupported_c = 0x01A5,
	GATTConfirm_Status_gHciTransportError_c = 0x01A6,
	GATTConfirm_Status_gL2caAlreadyInit_c = 0x03F0,
	GATTConfirm_Status_gL2caInsufficientResources_c = 0x03F1,
	GATTConfirm_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTConfirm_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTConfirm_Status_gL2caLePsmInvalid_c = 0x03F4,
	GATTConfirm_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTConfirm_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTConfirm_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTConfirm_Status_gL2caChannelInvalid_c = 0x03F8,
	GATTConfirm_Status_gL2caChannelClosed_c = 0x03F9,
	GATTConfirm_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTConfirm_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTConfirm_Status_gL2caChannelBusy_c = 0x03FC,
	GATTConfirm_Status_gL2caInvalidParameter_c = 0x03FE,
	GATTConfirm_Status_gL2caError_c = 0x03FF,
	GATTConfirm_Status_gSmNullCBFunction_c = 0x0401,
	GATTConfirm_Status_gSmCommandNotSupported_c = 0x0402,
	GATTConfirm_Status_gSmUnexpectedCommand_c = 0x0403,
	GATTConfirm_Status_gSmInvalidCommandCode_c = 0x0404,
	GATTConfirm_Status_gSmInvalidCommandLength_c = 0x0405,
	GATTConfirm_Status_gSmInvalidCommandParameter_c = 0x0406,
	GATTConfirm_Status_gSmInvalidDeviceId_c = 0x0407,
	GATTConfirm_Status_gSmInvalidInternalOperation_c = 0x0408,
	GATTConfirm_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GATTConfirm_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GATTConfirm_Status_gSmUnexpectedKeyType_c = 0x040B,
	GATTConfirm_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTConfirm_Status_gSmUnexpectedKeyset_c = 0x040D,
	GATTConfirm_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTConfirm_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GATTConfirm_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTConfirm_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTConfirm_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTConfirm_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTConfirm_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTConfirm_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTConfirm_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GATTConfirm_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTConfirm_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTConfirm_Status_gSmInsufficientResources_c = 0x0419,
	GATTConfirm_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GATTConfirm_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTConfirm_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTConfirm_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTConfirm_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTConfirm_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTConfirm_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTConfirm_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTConfirm_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTConfirm_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTConfirm_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTConfirm_Status_gSmPairingErrorTimeout_c = 0x0436,
	GATTConfirm_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTConfirm_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GATTConfirm_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTConfirm_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTConfirm_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GATTConfirm_Status_gAttInvalidHandle_c = 0x0501,
	GATTConfirm_Status_gAttReadNotPermitted_c = 0x0502,
	GATTConfirm_Status_gAttWriteNotPermitted_c = 0x0503,
	GATTConfirm_Status_gAttInvalidPdu_c = 0x0504,
	GATTConfirm_Status_gAttInsufficientAuthentication_c = 0x0505,
	GATTConfirm_Status_gAttRequestNotSupported_c = 0x0506,
	GATTConfirm_Status_gAttInvalidOffset_c = 0x0507,
	GATTConfirm_Status_gAttInsufficientAuthorization_c = 0x0508,
	GATTConfirm_Status_gAttPrepareQueueFull_c = 0x0509,
	GATTConfirm_Status_gAttAttributeNotFound_c = 0x050A,
	GATTConfirm_Status_gAttAttributeNotLong_c = 0x050B,
	GATTConfirm_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTConfirm_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTConfirm_Status_gAttUnlikelyor_c = 0x050E,
	GATTConfirm_Status_gAttInsufficientEncryption_c = 0x050F,
	GATTConfirm_Status_gAttUnsupportedGroupType_c = 0x0510,
	GATTConfirm_Status_gAttInsufficientResources_c = 0x0511,
	GATTConfirm_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GATTConfirm_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTConfirm_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTConfirm_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTConfirm_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTConfirm_Status_gGattInvalidValueLength_c = 0x0605,
	GATTConfirm_Status_gGattServerTimeout_c = 0x0606,
	GATTConfirm_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTConfirm_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GATTConfirm_Status_gGattInvalidPduReceived_c = 0x0609,
	GATTConfirm_Status_gGattPeerDisconnected_c = 0x060A,
	GATTConfirm_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GATTConfirm_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTConfirm_Status_gGapAdvDataTooLong_c = 0x0701,
	GATTConfirm_Status_gGapScanRspDataTooLong_c = 0x0702,
	GATTConfirm_Status_gGapDeviceNotBonded_c = 0x0703,
	GATTConfirm_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTConfirm_Status_gDevDbCccdLimitReached_c = 0x0801,
	GATTConfirm_Status_gDevDbCccdNotFound_c = 0x0802,
	GATTConfirm_Status_gGattDbInvalidHandle_c = 0x0901,
	GATTConfirm_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTConfirm_Status_gGattDbCccdNotFound_c = 0x0903,
	GATTConfirm_Status_gGattDbServiceNotFound_c = 0x0904,
	GATTConfirm_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GATTConfirm_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTConfirm_Status_t;

typedef PACKED_STRUCT GATTConfirm_tag {
	GATTConfirm_Status_t Status;  // The status of the GATT request
} GATTConfirm_t;

typedef PACKED_STRUCT GATTGetMtuIndication_tag {
	uint16_t Mtu;  // MTU value
} GATTGetMtuIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureExchangeMtuIndication_ProcedureResult_tag {
	GATTClientProcedureExchangeMtuIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureExchangeMtuIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureExchangeMtuIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureExchangeMtuIndication_Error_tag {
	GATTClientProcedureExchangeMtuIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureExchangeMtuIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureExchangeMtuIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureExchangeMtuIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureExchangeMtuIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureExchangeMtuIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureExchangeMtuIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureExchangeMtuIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureExchangeMtuIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureExchangeMtuIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureExchangeMtuIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureExchangeMtuIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureExchangeMtuIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureExchangeMtuIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureExchangeMtuIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureExchangeMtuIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureExchangeMtuIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureExchangeMtuIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureExchangeMtuIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureExchangeMtuIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureExchangeMtuIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureExchangeMtuIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureExchangeMtuIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureExchangeMtuIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureExchangeMtuIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureExchangeMtuIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureExchangeMtuIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureExchangeMtuIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureExchangeMtuIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureExchangeMtuIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureExchangeMtuIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureExchangeMtuIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureExchangeMtuIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureExchangeMtuIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureExchangeMtuIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureExchangeMtuIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureExchangeMtuIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureExchangeMtuIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureExchangeMtuIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureExchangeMtuIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureExchangeMtuIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureExchangeMtuIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureExchangeMtuIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureExchangeMtuIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureExchangeMtuIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureExchangeMtuIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureExchangeMtuIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureExchangeMtuIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureExchangeMtuIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureExchangeMtuIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureExchangeMtuIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureExchangeMtuIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureExchangeMtuIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureExchangeMtuIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureExchangeMtuIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureExchangeMtuIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureExchangeMtuIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureExchangeMtuIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureExchangeMtuIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureExchangeMtuIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureExchangeMtuIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureExchangeMtuIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureExchangeMtuIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureExchangeMtuIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureExchangeMtuIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureExchangeMtuIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureExchangeMtuIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureExchangeMtuIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureExchangeMtuIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureExchangeMtuIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureExchangeMtuIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureExchangeMtuIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureExchangeMtuIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureExchangeMtuIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureExchangeMtuIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureExchangeMtuIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureExchangeMtuIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureExchangeMtuIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureExchangeMtuIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureExchangeMtuIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureExchangeMtuIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureExchangeMtuIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureExchangeMtuIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureExchangeMtuIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureExchangeMtuIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureExchangeMtuIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureExchangeMtuIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureExchangeMtuIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureExchangeMtuIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureExchangeMtuIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureExchangeMtuIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureExchangeMtuIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureExchangeMtuIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureExchangeMtuIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureExchangeMtuIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureExchangeMtuIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureExchangeMtuIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureExchangeMtuIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureExchangeMtuIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureExchangeMtuIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureExchangeMtuIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureExchangeMtuIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureExchangeMtuIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureExchangeMtuIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureExchangeMtuIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureExchangeMtuIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureExchangeMtuIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureExchangeMtuIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureExchangeMtuIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureExchangeMtuIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureExchangeMtuIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureExchangeMtuIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureExchangeMtuIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureExchangeMtuIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureExchangeMtuIndication_Error_t;

typedef PACKED_STRUCT GATTClientProcedureExchangeMtuIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureExchangeMtuIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureExchangeMtuIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
} GATTClientProcedureExchangeMtuIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureDiscoverAllPrimaryServicesIndication_ProcedureResult_tag {
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureDiscoverAllPrimaryServicesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_tag {
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_t;

typedef struct GATTClientProcedureDiscoverAllPrimaryServicesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfDiscoveredServices;  // Number of discovered services
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} *DiscoveredServices;  // Discovered services
} GATTClientProcedureDiscoverAllPrimaryServicesIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_ProcedureResult_tag {
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_tag {
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_t;

typedef struct GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfDiscoveredServices;  // Number of discovered services
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} *DiscoveredServices;  // Discovered services
} GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureFindIncludedServicesIndication_ProcedureResult_tag {
	GATTClientProcedureFindIncludedServicesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureFindIncludedServicesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureFindIncludedServicesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureFindIncludedServicesIndication_Error_tag {
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureFindIncludedServicesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureFindIncludedServicesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureFindIncludedServicesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureFindIncludedServicesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureFindIncludedServicesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureFindIncludedServicesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureFindIncludedServicesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureFindIncludedServicesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureFindIncludedServicesIndication_Error_t;

typedef struct GATTClientProcedureFindIncludedServicesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureFindIncludedServicesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureFindIncludedServicesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which inclusions were searched
} GATTClientProcedureFindIncludedServicesIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureDiscoverAllCharacteristicsIndication_ProcedureResult_tag {
	GATTClientProcedureDiscoverAllCharacteristicsIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureDiscoverAllCharacteristicsIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_tag {
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_t;

typedef struct GATTClientProcedureDiscoverAllCharacteristicsIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureDiscoverAllCharacteristicsIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureDiscoverAllCharacteristicsIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which characteristics were searched
} GATTClientProcedureDiscoverAllCharacteristicsIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureDiscoverCharacteristicByUuidIndication_ProcedureResult_tag {
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureDiscoverCharacteristicByUuidIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_tag {
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_t;

typedef struct GATTClientProcedureDiscoverCharacteristicByUuidIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfCharacteristics;  // Number of discovered characteristics
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Characteristics that were filled
} GATTClientProcedureDiscoverCharacteristicByUuidIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_tag {
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_tag {
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_t;

typedef struct GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic within which descriptors were searched
} GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureReadCharacteristicValueIndication_ProcedureResult_tag {
	GATTClientProcedureReadCharacteristicValueIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureReadCharacteristicValueIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureReadCharacteristicValueIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureReadCharacteristicValueIndication_Error_tag {
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureReadCharacteristicValueIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureReadCharacteristicValueIndication_Error_t;

typedef struct GATTClientProcedureReadCharacteristicValueIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureReadCharacteristicValueIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureReadCharacteristicValueIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic whose value was read
} GATTClientProcedureReadCharacteristicValueIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureReadUsingCharacteristicUuidIndication_ProcedureResult_tag {
	GATTClientProcedureReadUsingCharacteristicUuidIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureReadUsingCharacteristicUuidIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_tag {
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_t;

typedef struct GATTClientProcedureReadUsingCharacteristicUuidIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureReadUsingCharacteristicUuidIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureReadUsingCharacteristicUuidIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint16_t NbOfReadBytes;  // The number of bytes that were read
	uint8_t *ReadBytes;  // The bytes that were read
} GATTClientProcedureReadUsingCharacteristicUuidIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureReadMultipleCharacteristicValuesIndication_ProcedureResult_tag {
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureReadMultipleCharacteristicValuesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_tag {
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_t;

typedef struct GATTClientProcedureReadMultipleCharacteristicValuesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfCharacteristics;  // Number of characteristics whose values were read
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Characteristics whose values were read
} GATTClientProcedureReadMultipleCharacteristicValuesIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureWriteCharacteristicValueIndication_ProcedureResult_tag {
	GATTClientProcedureWriteCharacteristicValueIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureWriteCharacteristicValueIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureWriteCharacteristicValueIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureWriteCharacteristicValueIndication_Error_tag {
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureWriteCharacteristicValueIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureWriteCharacteristicValueIndication_Error_t;

typedef PACKED_STRUCT GATTClientProcedureWriteCharacteristicValueIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureWriteCharacteristicValueIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureWriteCharacteristicValueIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
} GATTClientProcedureWriteCharacteristicValueIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureReadCharacteristicDescriptorIndication_ProcedureResult_tag {
	GATTClientProcedureReadCharacteristicDescriptorIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureReadCharacteristicDescriptorIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureReadCharacteristicDescriptorIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureReadCharacteristicDescriptorIndication_Error_tag {
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureReadCharacteristicDescriptorIndication_Error_t;

typedef struct GATTClientProcedureReadCharacteristicDescriptorIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureReadCharacteristicDescriptorIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureReadCharacteristicDescriptorIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		uint16_t Handle;  // Attribute handle
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint16_t ValueLength;  // Length of the attribute value array
		uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
		uint8_t *Value;  // Attribute value array
	} Descriptor;  // The characteristic descriptor whose value was read
} GATTClientProcedureReadCharacteristicDescriptorIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureWriteCharacteristicDescriptorIndication_ProcedureResult_tag {
	GATTClientProcedureWriteCharacteristicDescriptorIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureWriteCharacteristicDescriptorIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_tag {
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_t;

typedef PACKED_STRUCT GATTClientProcedureWriteCharacteristicDescriptorIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureWriteCharacteristicDescriptorIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureWriteCharacteristicDescriptorIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
} GATTClientProcedureWriteCharacteristicDescriptorIndication_t;

typedef struct GATTClientNotificationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint16_t CharacteristicValueHandle;  // Handle of the Characteristic Value attribute to be notified
	uint16_t ValueLength;  // Value array size
	uint8_t *Value;  // The Characteristic value array
} GATTClientNotificationIndication_t;

typedef struct GATTClientMultipleHandleValueNotificationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint16_t HandleCount;  // Number of handles to be notified
	struct {
		uint16_t Handle;  // Handle of the Value of the Characteristic to be indicated
		uint16_t ValueLength;  // Length of data to be indicated
		uint8_t *Value;  // Data to be indicated
	} *HandleLengthValueList;  // List of handle, length, value tuples
} GATTClientMultipleHandleValueNotificationIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureReadMultipleVariableLenCharValuesIndication_ProcedureResult_tag {
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureReadMultipleVariableLenCharValuesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_tag {
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_t;

typedef struct GATTClientProcedureReadMultipleVariableLenCharValuesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfCharacteristics;  // Number of characteristics whose values were read
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Characteristics whose values were read
} GATTClientProcedureReadMultipleVariableLenCharValuesIndication_t;

typedef struct GATTClientIndicationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint16_t CharacteristicValueHandle;  // Handle of the Characteristic Value attribute to be indicated
	uint16_t ValueLength;  // Value array size
	uint8_t *Value;  // The Characteristic value array
} GATTClientIndicationIndication_t;

typedef PACKED_STRUCT GATTServerMtuChangedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint16_t MtuChangedEvent_NewMtu;

} GATTServerMtuChangedIndication_t;

typedef PACKED_STRUCT GATTServerHandleValueConfirmationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
} GATTServerHandleValueConfirmationIndication_t;

typedef struct GATTServerAttributeWrittenIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	struct {
		uint16_t Handle;  // Handle of the attribute
		uint16_t ValueLength;  // Length of the attribute value array
		uint8_t *Value;  // Attribute value array attempted to be written
	} AttributeWrittenEvent;  // Attribute written event
} GATTServerAttributeWrittenIndication_t;

/* New value of the CCCD */
typedef enum GATTServerCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_tag {
	GATTServerCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_gCccdEmpty_c = 0x00,
	GATTServerCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_gCccdNotification_c = 0x01,
	GATTServerCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_gCccdIndication_c = 0x02
} GATTServerCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_t;

typedef PACKED_STRUCT GATTServerCharacteristicCccdWrittenIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	struct {
		uint16_t Handle;  // Handle of the CCCD attribute
		GATTServerCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_t NewCccd;  // New value of the CCCD
	} CharCccdWrittenEvent;  // Characteristic cccd written event
} GATTServerCharacteristicCccdWrittenIndication_t;

typedef struct GATTServerAttributeWrittenWithoutResponseIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	struct {
		uint16_t Handle;  // Handle of the attribute
		uint16_t ValueLength;  // Length of the attribute value array
		uint8_t *Value;  // Attribute value array attempted to be written
	} AttributeWrittenEvent;  // Attribute written event
} GATTServerAttributeWrittenWithoutResponseIndication_t;

/* Procedure that generated error */
typedef enum GATTServerErrorIndication_ProcedureError_ProcedureType_tag {
	GATTServerErrorIndication_ProcedureError_ProcedureType_gSendAttributeWrittenStatus_c = 0x00,
	GATTServerErrorIndication_ProcedureError_ProcedureType_gSendAttributeReadStatus_c = 0x01,
	GATTServerErrorIndication_ProcedureError_ProcedureType_gSendNotification_c = 0x02,
	GATTServerErrorIndication_ProcedureError_ProcedureType_gSendIndication_c = 0x03
} GATTServerErrorIndication_ProcedureError_ProcedureType_t;

/* Error generated */
typedef enum GATTServerErrorIndication_ProcedureError_Error_tag {
	GATTServerErrorIndication_ProcedureError_Error_gBleSuccess_c = 0x0000,
	GATTServerErrorIndication_ProcedureError_Error_gBleInvalidParameter_c = 0x0001,
	GATTServerErrorIndication_ProcedureError_Error_gBleOverflow_c = 0x0002,
	GATTServerErrorIndication_ProcedureError_Error_gBleUnavailable_c = 0x0003,
	GATTServerErrorIndication_ProcedureError_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTServerErrorIndication_ProcedureError_Error_gBleOutOfMemory_c = 0x0005,
	GATTServerErrorIndication_ProcedureError_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTServerErrorIndication_ProcedureError_Error_gBleOsError_c = 0x0007,
	GATTServerErrorIndication_ProcedureError_Error_gBleUnexpectedError_c = 0x0008,
	GATTServerErrorIndication_ProcedureError_Error_gBleInvalidState_c = 0x0009,
	GATTServerErrorIndication_ProcedureError_Error_gBleTimerError_c = 0x000A,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTServerErrorIndication_ProcedureError_Error_gHciHardwareFailure_c = 0x0103,
	GATTServerErrorIndication_ProcedureError_Error_gHciPageTimeout_c = 0x0104,
	GATTServerErrorIndication_ProcedureError_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTServerErrorIndication_ProcedureError_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTServerErrorIndication_ProcedureError_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionTimeout_c = 0x0108,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTServerErrorIndication_ProcedureError_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTServerErrorIndication_ProcedureError_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTServerErrorIndication_ProcedureError_Error_gHciCommandDisallowed_c = 0x010C,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTServerErrorIndication_ProcedureError_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTServerErrorIndication_ProcedureError_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTServerErrorIndication_ProcedureError_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTServerErrorIndication_ProcedureError_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTServerErrorIndication_ProcedureError_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTServerErrorIndication_ProcedureError_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTServerErrorIndication_ProcedureError_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTServerErrorIndication_ProcedureError_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTServerErrorIndication_ProcedureError_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTServerErrorIndication_ProcedureError_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnspecifiedError_c = 0x011F,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTServerErrorIndication_ProcedureError_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTServerErrorIndication_ProcedureError_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTServerErrorIndication_ProcedureError_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTServerErrorIndication_ProcedureError_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTServerErrorIndication_ProcedureError_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTServerErrorIndication_ProcedureError_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTServerErrorIndication_ProcedureError_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTServerErrorIndication_ProcedureError_Error_gHciInstantPassed_c = 0x0128,
	GATTServerErrorIndication_ProcedureError_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTServerErrorIndication_ProcedureError_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTServerErrorIndication_ProcedureError_Error_gHciReserved_0x2B_c = 0x012B,
	GATTServerErrorIndication_ProcedureError_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTServerErrorIndication_ProcedureError_Error_gHciQosRejected_c = 0x012D,
	GATTServerErrorIndication_ProcedureError_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTServerErrorIndication_ProcedureError_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTServerErrorIndication_ProcedureError_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTServerErrorIndication_ProcedureError_Error_gHciReserved_0x31_c = 0x0131,
	GATTServerErrorIndication_ProcedureError_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTServerErrorIndication_ProcedureError_Error_gHciReserved_0x33_c = 0x0133,
	GATTServerErrorIndication_ProcedureError_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTServerErrorIndication_ProcedureError_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTServerErrorIndication_ProcedureError_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTServerErrorIndication_ProcedureError_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTServerErrorIndication_ProcedureError_Error_gHciHostBusyPairing_c = 0x0138,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTServerErrorIndication_ProcedureError_Error_gHciControllerBusy_c = 0x013A,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTServerErrorIndication_ProcedureError_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTServerErrorIndication_ProcedureError_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTServerErrorIndication_ProcedureError_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTServerErrorIndication_ProcedureError_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTServerErrorIndication_ProcedureError_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTServerErrorIndication_ProcedureError_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTServerErrorIndication_ProcedureError_Error_gHciLimitReached_c = 0x0143,
	GATTServerErrorIndication_ProcedureError_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTServerErrorIndication_ProcedureError_Error_gHciAlreadyInit_c = 0x01A0,
	GATTServerErrorIndication_ProcedureError_Error_gHciInvalidParameter_c = 0x01A1,
	GATTServerErrorIndication_ProcedureError_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTServerErrorIndication_ProcedureError_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTServerErrorIndication_ProcedureError_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTServerErrorIndication_ProcedureError_Error_gHciEventNotSupported_c = 0x01A5,
	GATTServerErrorIndication_ProcedureError_Error_gHciTransportError_c = 0x01A6,
	GATTServerErrorIndication_ProcedureError_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTServerErrorIndication_ProcedureError_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTServerErrorIndication_ProcedureError_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTServerErrorIndication_ProcedureError_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTServerErrorIndication_ProcedureError_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTServerErrorIndication_ProcedureError_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTServerErrorIndication_ProcedureError_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTServerErrorIndication_ProcedureError_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTServerErrorIndication_ProcedureError_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTServerErrorIndication_ProcedureError_Error_gL2caChannelClosed_c = 0x03F9,
	GATTServerErrorIndication_ProcedureError_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTServerErrorIndication_ProcedureError_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTServerErrorIndication_ProcedureError_Error_gL2caChannelBusy_c = 0x03FC,
	GATTServerErrorIndication_ProcedureError_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTServerErrorIndication_ProcedureError_Error_gL2caError_c = 0x03FF,
	GATTServerErrorIndication_ProcedureError_Error_gSmNullCBFunction_c = 0x0401,
	GATTServerErrorIndication_ProcedureError_Error_gSmCommandNotSupported_c = 0x0402,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTServerErrorIndication_ProcedureError_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTServerErrorIndication_ProcedureError_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTServerErrorIndication_ProcedureError_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTServerErrorIndication_ProcedureError_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTServerErrorIndication_ProcedureError_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTServerErrorIndication_ProcedureError_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTServerErrorIndication_ProcedureError_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTServerErrorIndication_ProcedureError_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTServerErrorIndication_ProcedureError_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTServerErrorIndication_ProcedureError_Error_gSmInsufficientResources_c = 0x0419,
	GATTServerErrorIndication_ProcedureError_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTServerErrorIndication_ProcedureError_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTServerErrorIndication_ProcedureError_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTServerErrorIndication_ProcedureError_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTServerErrorIndication_ProcedureError_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTServerErrorIndication_ProcedureError_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTServerErrorIndication_ProcedureError_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTServerErrorIndication_ProcedureError_Error_gAttInvalidHandle_c = 0x0501,
	GATTServerErrorIndication_ProcedureError_Error_gAttReadNotPermitted_c = 0x0502,
	GATTServerErrorIndication_ProcedureError_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTServerErrorIndication_ProcedureError_Error_gAttInvalidPdu_c = 0x0504,
	GATTServerErrorIndication_ProcedureError_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTServerErrorIndication_ProcedureError_Error_gAttRequestNotSupported_c = 0x0506,
	GATTServerErrorIndication_ProcedureError_Error_gAttInvalidOffset_c = 0x0507,
	GATTServerErrorIndication_ProcedureError_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTServerErrorIndication_ProcedureError_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTServerErrorIndication_ProcedureError_Error_gAttAttributeNotFound_c = 0x050A,
	GATTServerErrorIndication_ProcedureError_Error_gAttAttributeNotLong_c = 0x050B,
	GATTServerErrorIndication_ProcedureError_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTServerErrorIndication_ProcedureError_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTServerErrorIndication_ProcedureError_Error_gAttUnlikelyor_c = 0x050E,
	GATTServerErrorIndication_ProcedureError_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTServerErrorIndication_ProcedureError_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTServerErrorIndication_ProcedureError_Error_gAttInsufficientResources_c = 0x0511,
	GATTServerErrorIndication_ProcedureError_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTServerErrorIndication_ProcedureError_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTServerErrorIndication_ProcedureError_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTServerErrorIndication_ProcedureError_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTServerErrorIndication_ProcedureError_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTServerErrorIndication_ProcedureError_Error_gGattInvalidValueLength_c = 0x0605,
	GATTServerErrorIndication_ProcedureError_Error_gGattServerTimeout_c = 0x0606,
	GATTServerErrorIndication_ProcedureError_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTServerErrorIndication_ProcedureError_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTServerErrorIndication_ProcedureError_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTServerErrorIndication_ProcedureError_Error_gGattPeerDisconnected_c = 0x060A,
	GATTServerErrorIndication_ProcedureError_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTServerErrorIndication_ProcedureError_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTServerErrorIndication_ProcedureError_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTServerErrorIndication_ProcedureError_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTServerErrorIndication_ProcedureError_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTServerErrorIndication_ProcedureError_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTServerErrorIndication_ProcedureError_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTServerErrorIndication_ProcedureError_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTServerErrorIndication_ProcedureError_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTServerErrorIndication_ProcedureError_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTServerErrorIndication_ProcedureError_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTServerErrorIndication_ProcedureError_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTServerErrorIndication_ProcedureError_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTServerErrorIndication_ProcedureError_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTServerErrorIndication_ProcedureError_Error_t;

typedef PACKED_STRUCT GATTServerErrorIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	struct {
		GATTServerErrorIndication_ProcedureError_ProcedureType_t ProcedureType;  // Procedure that generated error
		GATTServerErrorIndication_ProcedureError_Error_t Error;  // Error generated
	} ProcedureError;  // Procedure error
} GATTServerErrorIndication_t;

typedef struct GATTServerLongCharacteristicWrittenIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	struct {
		uint16_t Handle;  // Handle of the characteristic value
		uint16_t ValueLength;  // Length of the characteristic value
		uint8_t *Value;  // Characteristic value array
	} LongCharacteristicWrittenEvent;  // Long characteristic written event
} GATTServerLongCharacteristicWrittenIndication_t;

typedef PACKED_STRUCT GATTServerAttributeReadIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint16_t AttributeReadEvent_Handle;

} GATTServerAttributeReadIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_tag {
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_t;

typedef struct GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfDiscoveredServices;  // Number of discovered services
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} *DiscoveredServices;  // Discovered services
} GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_tag {
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_t;

typedef struct GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfDiscoveredServices;  // Number of discovered services
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} *DiscoveredServices;  // Discovered services
} GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedFindIncludedServicesIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedFindIncludedServicesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedFindIncludedServicesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_tag {
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_t;

typedef struct GATTClientProcedureEnhancedFindIncludedServicesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedFindIncludedServicesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedFindIncludedServicesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which inclusions were searched
} GATTClientProcedureEnhancedFindIncludedServicesIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_tag {
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_t;

typedef struct GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		uint16_t StartHandle;  // The handle of the Service Declaration attribute
		uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint8_t NbOfCharacteristics;  // Size of the Characteristic array
		struct {
			Properties_t Properties;  // Characteristic Properties as defined by GATT
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} Value;  // Characteristic Value attribute
			uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
			struct {
				uint16_t Handle;  // Attribute handle
				UuidType_t UuidType;  // UUID type
				union {
					uint8_t Uuid16Bits[2];  // UUID 16 bits
					uint8_t Uuid128Bits[16];  // UUID 128 bits
					uint8_t Uuid32Bits[4];  // UUID 32 bits
				} Uuid;  // UUID value
				uint16_t ValueLength;  // Length of the attribute value array
				uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
				uint8_t *Value;  // Attribute value array
			} *Descriptors;  // Characteristic Descriptors array
		} *Characteristics;  // Characteristic array
		uint8_t NbOfIncludedServices;  // Size of the Included Services array
		struct {
			uint16_t StartHandle;  // The handle of the Service Declaration attribute
			uint16_t EndHandle;  // The last handle belonging to this Service (followed by another Service declaration of the end of the database)
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint8_t NbOfCharacteristics;  // Size of the Characteristic array
			uint8_t NbOfIncludedServices;  // Size of the Included Services array
		} *IncludedServices;  // Included Services array
	} Service;  // The service within which characteristics were searched
} GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_tag {
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_t;

typedef struct GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfCharacteristics;  // Number of discovered characteristics
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Characteristics that were filled
} GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_tag {
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_t;

typedef struct GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic within which descriptors were searched
} GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedReadCharacteristicValueIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedReadCharacteristicValueIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_tag {
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_t;

typedef struct GATTClientProcedureEnhancedReadCharacteristicValueIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} Characteristic;  // The characteristic whose value was read
} GATTClientProcedureEnhancedReadCharacteristicValueIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_tag {
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_t;

typedef struct GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint16_t NbOfReadBytes;  // The number of bytes that were read
	uint8_t *ReadBytes;  // The bytes that were read
} GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_tag {
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_t;

typedef struct GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfCharacteristics;  // Number of characteristics whose values were read
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Characteristics whose values were read
} GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedWriteCharacteristicValueIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedWriteCharacteristicValueIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_tag {
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_t;

typedef PACKED_STRUCT GATTClientProcedureEnhancedWriteCharacteristicValueIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
} GATTClientProcedureEnhancedWriteCharacteristicValueIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_tag {
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_t;

typedef struct GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	struct {
		uint16_t Handle;  // Attribute handle
		UuidType_t UuidType;  // UUID type
		union {
			uint8_t Uuid16Bits[2];  // UUID 16 bits
			uint8_t Uuid128Bits[16];  // UUID 128 bits
			uint8_t Uuid32Bits[4];  // UUID 32 bits
		} Uuid;  // UUID value
		uint16_t ValueLength;  // Length of the attribute value array
		uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
		uint8_t *Value;  // Attribute value array
	} Descriptor;  // The characteristic descriptor whose value was read
} GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_tag {
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_t;

typedef PACKED_STRUCT GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
} GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_t;

/* The result of the completed procedure */
typedef enum GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_ProcedureResult_tag {
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_ProcedureResult_gGattProcSuccess_c = 0x00,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_ProcedureResult_gProcedureError_c = 0x01
} GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_ProcedureResult_t;

/* If procedureResult is not gSuccess_c, this contains the error that terminated the procedure */
typedef enum GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_tag {
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleSuccess_c = 0x0000,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleInvalidParameter_c = 0x0001,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleOverflow_c = 0x0002,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleUnavailable_c = 0x0003,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleOutOfMemory_c = 0x0005,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleOsError_c = 0x0007,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleUnexpectedError_c = 0x0008,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleInvalidState_c = 0x0009,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gBleTimerError_c = 0x000A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciHardwareFailure_c = 0x0103,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciPageTimeout_c = 0x0104,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionTimeout_c = 0x0108,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciCommandDisallowed_c = 0x010C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnspecifiedError_c = 0x011F,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciInstantPassed_c = 0x0128,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciReserved_0x2B_c = 0x012B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciQosRejected_c = 0x012D,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciReserved_0x31_c = 0x0131,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciReserved_0x33_c = 0x0133,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciHostBusyPairing_c = 0x0138,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciControllerBusy_c = 0x013A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciLimitReached_c = 0x0143,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciAlreadyInit_c = 0x01A0,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciInvalidParameter_c = 0x01A1,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciEventNotSupported_c = 0x01A5,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gHciTransportError_c = 0x01A6,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelClosed_c = 0x03F9,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caChannelBusy_c = 0x03FC,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gL2caError_c = 0x03FF,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmNullCBFunction_c = 0x0401,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmCommandNotSupported_c = 0x0402,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmInsufficientResources_c = 0x0419,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidHandle_c = 0x0501,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttReadNotPermitted_c = 0x0502,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidPdu_c = 0x0504,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttRequestNotSupported_c = 0x0506,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidOffset_c = 0x0507,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttAttributeNotFound_c = 0x050A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttAttributeNotLong_c = 0x050B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttUnlikelyor_c = 0x050E,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttInsufficientResources_c = 0x0511,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattInvalidValueLength_c = 0x0605,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattServerTimeout_c = 0x0606,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattPeerDisconnected_c = 0x060A,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_t;

typedef struct GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_ProcedureResult_t ProcedureResult;  // The result of the completed procedure
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_Error_t Error;  // If procedureResult is not gSuccess_c, this contains the error that terminated the procedure
	uint8_t NbOfCharacteristics;  // Number of characteristics whose values were read
	struct {
		Properties_t Properties;  // Characteristic Properties as defined by GATT
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} Value;  // Characteristic Value attribute
		uint8_t NbOfDescriptors;  // Size of the Characteristic Descriptors array
		struct {
			uint16_t Handle;  // Attribute handle
			UuidType_t UuidType;  // UUID type
			union {
				uint8_t Uuid16Bits[2];  // UUID 16 bits
				uint8_t Uuid128Bits[16];  // UUID 128 bits
				uint8_t Uuid32Bits[4];  // UUID 32 bits
			} Uuid;  // UUID value
			uint16_t ValueLength;  // Length of the attribute value array
			uint16_t MaxValueLength;  // Maximum length of the attribute value array; if this is set to 0, then the attribute's length is fixed and cannot be changed
			uint8_t *Value;  // Attribute value array
		} *Descriptors;  // Characteristic Descriptors array
	} *Characteristics;  // Characteristics whose values were read
} GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_t;

typedef struct GATTClientEnhancedMultipleHandleValueNotificationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	uint16_t HandleCount;  // Number of handles to be notified
	struct {
		uint16_t Handle;  // Handle of the Value of the Characteristic to be indicated
		uint16_t ValueLength;  // Length of data to be indicated
		uint8_t *Value;  // Data to be indicated
	} *HandleLengthValueList;  // List of handle, length, value tuples
} GATTClientEnhancedMultipleHandleValueNotificationIndication_t;

typedef struct GATTClientEnhancedNotificationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	uint16_t CharacteristicValueHandle;  // Handle of the Characteristic Value attribute to be notified
	uint16_t ValueLength;  // Value array size
	uint8_t *Value;  // The Characteristic value array
} GATTClientEnhancedNotificationIndication_t;

typedef struct GATTClientEnhancedIndicationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	uint16_t CharacteristicValueHandle;  // Handle of the Characteristic Value attribute to be indicated
	uint16_t ValueLength;  // Value array size
	uint8_t *Value;  // The Characteristic value array
} GATTClientEnhancedIndicationIndication_t;

typedef PACKED_STRUCT GATTServerEnhancedHandleValueConfirmationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
} GATTServerEnhancedHandleValueConfirmationIndication_t;

typedef struct GATTServerEnhancedAttributeWrittenIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	struct {
		uint16_t Handle;  // Handle of the attribute
		uint16_t ValueLength;  // Length of the attribute value array
		uint8_t *Value;  // Attribute value array attempted to be written
	} AttributeWrittenEvent;  // Attribute written event
} GATTServerEnhancedAttributeWrittenIndication_t;

/* New value of the CCCD */
typedef enum GATTServerEnhancedCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_tag {
	GATTServerEnhancedCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_gCccdEmpty_c = 0x00,
	GATTServerEnhancedCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_gCccdNotification_c = 0x01,
	GATTServerEnhancedCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_gCccdIndication_c = 0x02
} GATTServerEnhancedCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_t;

typedef PACKED_STRUCT GATTServerEnhancedCharacteristicCccdWrittenIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	struct {
		uint16_t Handle;  // Handle of the CCCD attribute
		GATTServerEnhancedCharacteristicCccdWrittenIndication_CharCccdWrittenEvent_NewCccd_t NewCccd;  // New value of the CCCD
	} CharCccdWrittenEvent;  // Characteristic cccd written event
} GATTServerEnhancedCharacteristicCccdWrittenIndication_t;

typedef struct GATTServerEnhancedAttributeWrittenWithoutResponseIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	struct {
		uint16_t Handle;  // Handle of the attribute
		uint16_t ValueLength;  // Length of the attribute value array
		uint8_t *Value;  // Attribute value array attempted to be written
	} AttributeWrittenEvent;  // Attribute written event
} GATTServerEnhancedAttributeWrittenWithoutResponseIndication_t;

/* Procedure that generated error */
typedef enum GATTServerEnhancedErrorIndication_ProcedureError_ProcedureType_tag {
	GATTServerEnhancedErrorIndication_ProcedureError_ProcedureType_gSendAttributeWrittenStatus_c = 0x00,
	GATTServerEnhancedErrorIndication_ProcedureError_ProcedureType_gSendAttributeReadStatus_c = 0x01,
	GATTServerEnhancedErrorIndication_ProcedureError_ProcedureType_gSendNotification_c = 0x02,
	GATTServerEnhancedErrorIndication_ProcedureError_ProcedureType_gSendIndication_c = 0x03
} GATTServerEnhancedErrorIndication_ProcedureError_ProcedureType_t;

/* Error generated */
typedef enum GATTServerEnhancedErrorIndication_ProcedureError_Error_tag {
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleSuccess_c = 0x0000,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleInvalidParameter_c = 0x0001,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleOverflow_c = 0x0002,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleUnavailable_c = 0x0003,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleFeatureNotSupported_c = 0x0004,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleOutOfMemory_c = 0x0005,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleAlreadyInitialized_c = 0x0006,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleOsError_c = 0x0007,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleUnexpectedError_c = 0x0008,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleInvalidState_c = 0x0009,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gBleTimerError_c = 0x000A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnknownHciCommand_c = 0x0101,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciHardwareFailure_c = 0x0103,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciPageTimeout_c = 0x0104,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciAuthenticationFailure_c = 0x0105,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciPinOrKeyMissing_c = 0x0106,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionTimeout_c = 0x0108,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionLimitExceeded_c = 0x0109,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciCommandDisallowed_c = 0x010C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRepeatedAttempts_c = 0x0117,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciPairingNotAllowed_c = 0x0118,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnknownLpmPdu_c = 0x0119,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciScoOffsetRejected_c = 0x011B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciScoIntervalRejected_c = 0x011C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciScoAirModeRejected_c = 0x011D,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciInvalidLpmParameters_c = 0x011E,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnspecifiedError_c = 0x011F,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciLLResponseTimeout_c = 0x0122,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciLmpPduNotAllowed_c = 0x0124,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRequestedQosNotSupported_c = 0x0127,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciInstantPassed_c = 0x0128,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciDifferentTransactionCollision_c = 0x012A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciReserved_0x2B_c = 0x012B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciQosRejected_c = 0x012D,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciInsufficientSecurity_c = 0x012F,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciReserved_0x31_c = 0x0131,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRoleSwitchPending_c = 0x0132,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciReserved_0x33_c = 0x0133,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciReservedSlotViolation_c = 0x0134,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciRoleSwitchFailed_c = 0x0135,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciHostBusyPairing_c = 0x0138,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciControllerBusy_c = 0x013A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciMacConnectionFailed_c = 0x013F,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciType0SubmapNotDefined_c = 0x0141,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciLimitReached_c = 0x0143,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciOperationCancelledByHost_c = 0x0144,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciAlreadyInit_c = 0x01A0,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciInvalidParameter_c = 0x01A1,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciCallbackNotInstalled_c = 0x01A2,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciCommandNotSupported_c = 0x01A4,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciEventNotSupported_c = 0x01A5,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gHciTransportError_c = 0x01A6,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caAlreadyInit_c = 0x03F0,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caInsufficientResources_c = 0x03F1,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caLePsmInvalid_c = 0x03F4,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caChannelInvalid_c = 0x03F8,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caChannelClosed_c = 0x03F9,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caChannelBusy_c = 0x03FC,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caInvalidParameter_c = 0x03FE,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gL2caError_c = 0x03FF,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmNullCBFunction_c = 0x0401,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmCommandNotSupported_c = 0x0402,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnexpectedCommand_c = 0x0403,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidCommandCode_c = 0x0404,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidCommandLength_c = 0x0405,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidCommandParameter_c = 0x0406,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidDeviceId_c = 0x0407,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidInternalOperation_c = 0x0408,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidConnectionHandle_c = 0x0409,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInproperKeyDistributionField_c = 0x040A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnexpectedKeyType_c = 0x040B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnexpectedKeyset_c = 0x040D,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnknownSmpPacketType_c = 0x040F,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInvalidHciEventParameter_c = 0x0416,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmInsufficientResources_c = 0x0419,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmOobDataAddressMismatch_c = 0x041A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorTimeout_c = 0x0436,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingAlreadyStarted_c = 0x0438,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gSmTbInvalidDataSignature_c = 0x0461,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInvalidHandle_c = 0x0501,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttReadNotPermitted_c = 0x0502,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttWriteNotPermitted_c = 0x0503,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInvalidPdu_c = 0x0504,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInsufficientAuthentication_c = 0x0505,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttRequestNotSupported_c = 0x0506,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInvalidOffset_c = 0x0507,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInsufficientAuthorization_c = 0x0508,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttPrepareQueueFull_c = 0x0509,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttAttributeNotFound_c = 0x050A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttAttributeNotLong_c = 0x050B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttUnlikelyor_c = 0x050E,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInsufficientEncryption_c = 0x050F,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttUnsupportedGroupType_c = 0x0510,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttInsufficientResources_c = 0x0511,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gAttDatabaseOutOfSync_c = 0x0512,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattInvalidValueLength_c = 0x0605,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattServerTimeout_c = 0x0606,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattClientConfirmationTimeout_c = 0x0608,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattInvalidPduReceived_c = 0x0609,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattPeerDisconnected_c = 0x060A,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattMtuExchangeInProgress_c = 0x060B,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGapAdvDataTooLong_c = 0x0701,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGapScanRspDataTooLong_c = 0x0702,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGapDeviceNotBonded_c = 0x0703,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gDevDbCccdLimitReached_c = 0x0801,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gDevDbCccdNotFound_c = 0x0802,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattDbInvalidHandle_c = 0x0901,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattDbCccdNotFound_c = 0x0903,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattDbServiceNotFound_c = 0x0904,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattDbDescriptorNotFound_c = 0x0905,
	GATTServerEnhancedErrorIndication_ProcedureError_Error_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTServerEnhancedErrorIndication_ProcedureError_Error_t;

typedef PACKED_STRUCT GATTServerEnhancedErrorIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	struct {
		GATTServerEnhancedErrorIndication_ProcedureError_ProcedureType_t ProcedureType;  // Procedure that generated error
		GATTServerEnhancedErrorIndication_ProcedureError_Error_t Error;  // Error generated
	} ProcedureError;  // Procedure error
} GATTServerEnhancedErrorIndication_t;

typedef struct GATTServerEnhancedLongCharacteristicWrittenIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	struct {
		uint16_t Handle;  // Handle of the characteristic value
		uint16_t ValueLength;  // Length of the characteristic value
		uint8_t *Value;  // Characteristic value array
	} LongCharacteristicWrittenEvent;  // Long characteristic written event
} GATTServerEnhancedLongCharacteristicWrittenIndication_t;

typedef PACKED_STRUCT GATTServerEnhancedAttributeReadIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the active connection
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	uint16_t AttributeReadEvent_Handle;

} GATTServerEnhancedAttributeReadIndication_t;

#endif  /* GATT_ENABLE */

#if GATTDB_APP_ENABLE
/* The status of the GATT Database request */
typedef enum GATTDBConfirm_Status_tag {
	GATTDBConfirm_Status_gBleSuccess_c = 0x0000,
	GATTDBConfirm_Status_gBleInvalidParameter_c = 0x0001,
	GATTDBConfirm_Status_gBleOverflow_c = 0x0002,
	GATTDBConfirm_Status_gBleUnavailable_c = 0x0003,
	GATTDBConfirm_Status_gBleFeatureNotSupported_c = 0x0004,
	GATTDBConfirm_Status_gBleOutOfMemory_c = 0x0005,
	GATTDBConfirm_Status_gBleAlreadyInitialized_c = 0x0006,
	GATTDBConfirm_Status_gBleOsError_c = 0x0007,
	GATTDBConfirm_Status_gBleUnexpectedError_c = 0x0008,
	GATTDBConfirm_Status_gBleInvalidState_c = 0x0009,
	GATTDBConfirm_Status_gBleTimerError_c = 0x000A,
	GATTDBConfirm_Status_gHciUnknownHciCommand_c = 0x0101,
	GATTDBConfirm_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTDBConfirm_Status_gHciHardwareFailure_c = 0x0103,
	GATTDBConfirm_Status_gHciPageTimeout_c = 0x0104,
	GATTDBConfirm_Status_gHciAuthenticationFailure_c = 0x0105,
	GATTDBConfirm_Status_gHciPinOrKeyMissing_c = 0x0106,
	GATTDBConfirm_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTDBConfirm_Status_gHciConnectionTimeout_c = 0x0108,
	GATTDBConfirm_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GATTDBConfirm_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTDBConfirm_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTDBConfirm_Status_gHciCommandDisallowed_c = 0x010C,
	GATTDBConfirm_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTDBConfirm_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTDBConfirm_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTDBConfirm_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTDBConfirm_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTDBConfirm_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTDBConfirm_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTDBConfirm_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTDBConfirm_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTDBConfirm_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTDBConfirm_Status_gHciRepeatedAttempts_c = 0x0117,
	GATTDBConfirm_Status_gHciPairingNotAllowed_c = 0x0118,
	GATTDBConfirm_Status_gHciUnknownLpmPdu_c = 0x0119,
	GATTDBConfirm_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTDBConfirm_Status_gHciScoOffsetRejected_c = 0x011B,
	GATTDBConfirm_Status_gHciScoIntervalRejected_c = 0x011C,
	GATTDBConfirm_Status_gHciScoAirModeRejected_c = 0x011D,
	GATTDBConfirm_Status_gHciInvalidLpmParameters_c = 0x011E,
	GATTDBConfirm_Status_gHciUnspecifiedError_c = 0x011F,
	GATTDBConfirm_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTDBConfirm_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTDBConfirm_Status_gHciLLResponseTimeout_c = 0x0122,
	GATTDBConfirm_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTDBConfirm_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GATTDBConfirm_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTDBConfirm_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTDBConfirm_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GATTDBConfirm_Status_gHciInstantPassed_c = 0x0128,
	GATTDBConfirm_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTDBConfirm_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GATTDBConfirm_Status_gHciReserved_0x2B_c = 0x012B,
	GATTDBConfirm_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTDBConfirm_Status_gHciQosRejected_c = 0x012D,
	GATTDBConfirm_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTDBConfirm_Status_gHciInsufficientSecurity_c = 0x012F,
	GATTDBConfirm_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTDBConfirm_Status_gHciReserved_0x31_c = 0x0131,
	GATTDBConfirm_Status_gHciRoleSwitchPending_c = 0x0132,
	GATTDBConfirm_Status_gHciReserved_0x33_c = 0x0133,
	GATTDBConfirm_Status_gHciReservedSlotViolation_c = 0x0134,
	GATTDBConfirm_Status_gHciRoleSwitchFailed_c = 0x0135,
	GATTDBConfirm_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTDBConfirm_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTDBConfirm_Status_gHciHostBusyPairing_c = 0x0138,
	GATTDBConfirm_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTDBConfirm_Status_gHciControllerBusy_c = 0x013A,
	GATTDBConfirm_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTDBConfirm_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTDBConfirm_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTDBConfirm_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTDBConfirm_Status_gHciMacConnectionFailed_c = 0x013F,
	GATTDBConfirm_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTDBConfirm_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GATTDBConfirm_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTDBConfirm_Status_gHciLimitReached_c = 0x0143,
	GATTDBConfirm_Status_gHciOperationCancelledByHost_c = 0x0144,
	GATTDBConfirm_Status_gHciAlreadyInit_c = 0x01A0,
	GATTDBConfirm_Status_gHciInvalidParameter_c = 0x01A1,
	GATTDBConfirm_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GATTDBConfirm_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTDBConfirm_Status_gHciCommandNotSupported_c = 0x01A4,
	GATTDBConfirm_Status_gHciEventNotSupported_c = 0x01A5,
	GATTDBConfirm_Status_gHciTransportError_c = 0x01A6,
	GATTDBConfirm_Status_gL2caAlreadyInit_c = 0x03F0,
	GATTDBConfirm_Status_gL2caInsufficientResources_c = 0x03F1,
	GATTDBConfirm_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTDBConfirm_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTDBConfirm_Status_gL2caLePsmInvalid_c = 0x03F4,
	GATTDBConfirm_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTDBConfirm_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTDBConfirm_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTDBConfirm_Status_gL2caChannelInvalid_c = 0x03F8,
	GATTDBConfirm_Status_gL2caChannelClosed_c = 0x03F9,
	GATTDBConfirm_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTDBConfirm_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTDBConfirm_Status_gL2caChannelBusy_c = 0x03FC,
	GATTDBConfirm_Status_gL2caInvalidParameter_c = 0x03FE,
	GATTDBConfirm_Status_gL2caError_c = 0x03FF,
	GATTDBConfirm_Status_gSmNullCBFunction_c = 0x0401,
	GATTDBConfirm_Status_gSmCommandNotSupported_c = 0x0402,
	GATTDBConfirm_Status_gSmUnexpectedCommand_c = 0x0403,
	GATTDBConfirm_Status_gSmInvalidCommandCode_c = 0x0404,
	GATTDBConfirm_Status_gSmInvalidCommandLength_c = 0x0405,
	GATTDBConfirm_Status_gSmInvalidCommandParameter_c = 0x0406,
	GATTDBConfirm_Status_gSmInvalidDeviceId_c = 0x0407,
	GATTDBConfirm_Status_gSmInvalidInternalOperation_c = 0x0408,
	GATTDBConfirm_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GATTDBConfirm_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GATTDBConfirm_Status_gSmUnexpectedKeyType_c = 0x040B,
	GATTDBConfirm_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTDBConfirm_Status_gSmUnexpectedKeyset_c = 0x040D,
	GATTDBConfirm_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTDBConfirm_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GATTDBConfirm_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTDBConfirm_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTDBConfirm_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTDBConfirm_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTDBConfirm_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTDBConfirm_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTDBConfirm_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GATTDBConfirm_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTDBConfirm_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTDBConfirm_Status_gSmInsufficientResources_c = 0x0419,
	GATTDBConfirm_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GATTDBConfirm_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTDBConfirm_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTDBConfirm_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTDBConfirm_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTDBConfirm_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTDBConfirm_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTDBConfirm_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTDBConfirm_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTDBConfirm_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTDBConfirm_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTDBConfirm_Status_gSmPairingErrorTimeout_c = 0x0436,
	GATTDBConfirm_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTDBConfirm_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GATTDBConfirm_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTDBConfirm_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTDBConfirm_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GATTDBConfirm_Status_gAttInvalidHandle_c = 0x0501,
	GATTDBConfirm_Status_gAttReadNotPermitted_c = 0x0502,
	GATTDBConfirm_Status_gAttWriteNotPermitted_c = 0x0503,
	GATTDBConfirm_Status_gAttInvalidPdu_c = 0x0504,
	GATTDBConfirm_Status_gAttInsufficientAuthentication_c = 0x0505,
	GATTDBConfirm_Status_gAttRequestNotSupported_c = 0x0506,
	GATTDBConfirm_Status_gAttInvalidOffset_c = 0x0507,
	GATTDBConfirm_Status_gAttInsufficientAuthorization_c = 0x0508,
	GATTDBConfirm_Status_gAttPrepareQueueFull_c = 0x0509,
	GATTDBConfirm_Status_gAttAttributeNotFound_c = 0x050A,
	GATTDBConfirm_Status_gAttAttributeNotLong_c = 0x050B,
	GATTDBConfirm_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTDBConfirm_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTDBConfirm_Status_gAttUnlikelyor_c = 0x050E,
	GATTDBConfirm_Status_gAttInsufficientEncryption_c = 0x050F,
	GATTDBConfirm_Status_gAttUnsupportedGroupType_c = 0x0510,
	GATTDBConfirm_Status_gAttInsufficientResources_c = 0x0511,
	GATTDBConfirm_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GATTDBConfirm_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTDBConfirm_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTDBConfirm_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTDBConfirm_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTDBConfirm_Status_gGattInvalidValueLength_c = 0x0605,
	GATTDBConfirm_Status_gGattServerTimeout_c = 0x0606,
	GATTDBConfirm_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTDBConfirm_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GATTDBConfirm_Status_gGattInvalidPduReceived_c = 0x0609,
	GATTDBConfirm_Status_gGattPeerDisconnected_c = 0x060A,
	GATTDBConfirm_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GATTDBConfirm_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTDBConfirm_Status_gGapAdvDataTooLong_c = 0x0701,
	GATTDBConfirm_Status_gGapScanRspDataTooLong_c = 0x0702,
	GATTDBConfirm_Status_gGapDeviceNotBonded_c = 0x0703,
	GATTDBConfirm_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTDBConfirm_Status_gDevDbCccdLimitReached_c = 0x0801,
	GATTDBConfirm_Status_gDevDbCccdNotFound_c = 0x0802,
	GATTDBConfirm_Status_gGattDbInvalidHandle_c = 0x0901,
	GATTDBConfirm_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTDBConfirm_Status_gGattDbCccdNotFound_c = 0x0903,
	GATTDBConfirm_Status_gGattDbServiceNotFound_c = 0x0904,
	GATTDBConfirm_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GATTDBConfirm_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTDBConfirm_Status_t;

typedef PACKED_STRUCT GATTDBConfirm_tag {
	GATTDBConfirm_Status_t Status;  // The status of the GATT Database request
} GATTDBConfirm_t;

typedef struct GATTDBReadAttributeIndication_tag {
	uint16_t ValueLength;  // Attribute value length
	uint8_t *Value;  // Attribute value
} GATTDBReadAttributeIndication_t;

typedef PACKED_STRUCT GATTDBFindServiceHandleIndication_tag {
	uint16_t ServiceHandle;  // Service handle
} GATTDBFindServiceHandleIndication_t;

typedef PACKED_STRUCT GATTDBFindCharValueHandleInServiceIndication_tag {
	uint16_t CharValueHandle;  // Characteristic value handle
} GATTDBFindCharValueHandleInServiceIndication_t;

typedef PACKED_STRUCT GATTDBFindCccdHandleForCharValueHandleIndication_tag {
	uint16_t CccdHandle;  // CCCD handle
} GATTDBFindCccdHandleForCharValueHandleIndication_t;

typedef PACKED_STRUCT GATTDBFindDescriptorHandleForCharValueHandleIndication_tag {
	uint16_t DescriptorHandle;  // Descriptor handle
} GATTDBFindDescriptorHandleForCharValueHandleIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddPrimaryServiceDeclarationIndication_tag {
	uint16_t ServiceHandle;  // Handle of the Service declaration
} GATTDBDynamicAddPrimaryServiceDeclarationIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddSecondaryServiceDeclarationIndication_tag {
	uint16_t ServiceHandle;  // Handle of the Service declaration
} GATTDBDynamicAddSecondaryServiceDeclarationIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddIncludeDeclarationIndication_tag {
	uint16_t IncludeHandle;  // Handle of the Include declaration
} GATTDBDynamicAddIncludeDeclarationIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddCharacteristicDeclarationAndValueIndication_tag {
	uint16_t CharacteristicHandle;  // Handle of the Characteristic declaration
} GATTDBDynamicAddCharacteristicDeclarationAndValueIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddCharacteristicDescriptorIndication_tag {
	uint16_t DescriptorHandle;  // Handle of the Descriptor declaration
} GATTDBDynamicAddCharacteristicDescriptorIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddCccdIndication_tag {
	uint16_t CCCDHandle;  // Handle of the CCCD declaration
} GATTDBDynamicAddCccdIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueIndication_tag {
	uint16_t CharacteristicHandle;  // Handle of the Characteristic declaration
} GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueIndication_t;

typedef PACKED_STRUCT GATTDBDynamicAddCharDescriptorWithUniqueValueIndication_tag {
	uint16_t DescriptorHandle;  // Handle of the Descriptor declaration
} GATTDBDynamicAddCharDescriptorWithUniqueValueIndication_t;

#endif  /* GATTDB_APP_ENABLE */

#if GATTDB_ATT_ENABLE
/* The status of the GATT Database request */
typedef enum GATTDBAttConfirm_Status_tag {
	GATTDBAttConfirm_Status_gBleSuccess_c = 0x0000,
	GATTDBAttConfirm_Status_gBleInvalidParameter_c = 0x0001,
	GATTDBAttConfirm_Status_gBleOverflow_c = 0x0002,
	GATTDBAttConfirm_Status_gBleUnavailable_c = 0x0003,
	GATTDBAttConfirm_Status_gBleFeatureNotSupported_c = 0x0004,
	GATTDBAttConfirm_Status_gBleOutOfMemory_c = 0x0005,
	GATTDBAttConfirm_Status_gBleAlreadyInitialized_c = 0x0006,
	GATTDBAttConfirm_Status_gBleOsError_c = 0x0007,
	GATTDBAttConfirm_Status_gBleUnexpectedError_c = 0x0008,
	GATTDBAttConfirm_Status_gBleInvalidState_c = 0x0009,
	GATTDBAttConfirm_Status_gBleTimerError_c = 0x000A,
	GATTDBAttConfirm_Status_gHciUnknownHciCommand_c = 0x0101,
	GATTDBAttConfirm_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GATTDBAttConfirm_Status_gHciHardwareFailure_c = 0x0103,
	GATTDBAttConfirm_Status_gHciPageTimeout_c = 0x0104,
	GATTDBAttConfirm_Status_gHciAuthenticationFailure_c = 0x0105,
	GATTDBAttConfirm_Status_gHciPinOrKeyMissing_c = 0x0106,
	GATTDBAttConfirm_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GATTDBAttConfirm_Status_gHciConnectionTimeout_c = 0x0108,
	GATTDBAttConfirm_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GATTDBAttConfirm_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GATTDBAttConfirm_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GATTDBAttConfirm_Status_gHciCommandDisallowed_c = 0x010C,
	GATTDBAttConfirm_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GATTDBAttConfirm_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GATTDBAttConfirm_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GATTDBAttConfirm_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GATTDBAttConfirm_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GATTDBAttConfirm_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GATTDBAttConfirm_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GATTDBAttConfirm_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GATTDBAttConfirm_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GATTDBAttConfirm_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GATTDBAttConfirm_Status_gHciRepeatedAttempts_c = 0x0117,
	GATTDBAttConfirm_Status_gHciPairingNotAllowed_c = 0x0118,
	GATTDBAttConfirm_Status_gHciUnknownLpmPdu_c = 0x0119,
	GATTDBAttConfirm_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GATTDBAttConfirm_Status_gHciScoOffsetRejected_c = 0x011B,
	GATTDBAttConfirm_Status_gHciScoIntervalRejected_c = 0x011C,
	GATTDBAttConfirm_Status_gHciScoAirModeRejected_c = 0x011D,
	GATTDBAttConfirm_Status_gHciInvalidLpmParameters_c = 0x011E,
	GATTDBAttConfirm_Status_gHciUnspecifiedError_c = 0x011F,
	GATTDBAttConfirm_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GATTDBAttConfirm_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GATTDBAttConfirm_Status_gHciLLResponseTimeout_c = 0x0122,
	GATTDBAttConfirm_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GATTDBAttConfirm_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GATTDBAttConfirm_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GATTDBAttConfirm_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GATTDBAttConfirm_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GATTDBAttConfirm_Status_gHciInstantPassed_c = 0x0128,
	GATTDBAttConfirm_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GATTDBAttConfirm_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GATTDBAttConfirm_Status_gHciReserved_0x2B_c = 0x012B,
	GATTDBAttConfirm_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GATTDBAttConfirm_Status_gHciQosRejected_c = 0x012D,
	GATTDBAttConfirm_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GATTDBAttConfirm_Status_gHciInsufficientSecurity_c = 0x012F,
	GATTDBAttConfirm_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GATTDBAttConfirm_Status_gHciReserved_0x31_c = 0x0131,
	GATTDBAttConfirm_Status_gHciRoleSwitchPending_c = 0x0132,
	GATTDBAttConfirm_Status_gHciReserved_0x33_c = 0x0133,
	GATTDBAttConfirm_Status_gHciReservedSlotViolation_c = 0x0134,
	GATTDBAttConfirm_Status_gHciRoleSwitchFailed_c = 0x0135,
	GATTDBAttConfirm_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GATTDBAttConfirm_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GATTDBAttConfirm_Status_gHciHostBusyPairing_c = 0x0138,
	GATTDBAttConfirm_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GATTDBAttConfirm_Status_gHciControllerBusy_c = 0x013A,
	GATTDBAttConfirm_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GATTDBAttConfirm_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GATTDBAttConfirm_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GATTDBAttConfirm_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GATTDBAttConfirm_Status_gHciMacConnectionFailed_c = 0x013F,
	GATTDBAttConfirm_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GATTDBAttConfirm_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GATTDBAttConfirm_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GATTDBAttConfirm_Status_gHciLimitReached_c = 0x0143,
	GATTDBAttConfirm_Status_gHciOperationCancelledByHost_c = 0x0144,
	GATTDBAttConfirm_Status_gHciAlreadyInit_c = 0x01A0,
	GATTDBAttConfirm_Status_gHciInvalidParameter_c = 0x01A1,
	GATTDBAttConfirm_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GATTDBAttConfirm_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GATTDBAttConfirm_Status_gHciCommandNotSupported_c = 0x01A4,
	GATTDBAttConfirm_Status_gHciEventNotSupported_c = 0x01A5,
	GATTDBAttConfirm_Status_gHciTransportError_c = 0x01A6,
	GATTDBAttConfirm_Status_gL2caAlreadyInit_c = 0x03F0,
	GATTDBAttConfirm_Status_gL2caInsufficientResources_c = 0x03F1,
	GATTDBAttConfirm_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GATTDBAttConfirm_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GATTDBAttConfirm_Status_gL2caLePsmInvalid_c = 0x03F4,
	GATTDBAttConfirm_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GATTDBAttConfirm_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GATTDBAttConfirm_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GATTDBAttConfirm_Status_gL2caChannelInvalid_c = 0x03F8,
	GATTDBAttConfirm_Status_gL2caChannelClosed_c = 0x03F9,
	GATTDBAttConfirm_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GATTDBAttConfirm_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GATTDBAttConfirm_Status_gL2caChannelBusy_c = 0x03FC,
	GATTDBAttConfirm_Status_gL2caInvalidParameter_c = 0x03FE,
	GATTDBAttConfirm_Status_gL2caError_c = 0x03FF,
	GATTDBAttConfirm_Status_gSmNullCBFunction_c = 0x0401,
	GATTDBAttConfirm_Status_gSmCommandNotSupported_c = 0x0402,
	GATTDBAttConfirm_Status_gSmUnexpectedCommand_c = 0x0403,
	GATTDBAttConfirm_Status_gSmInvalidCommandCode_c = 0x0404,
	GATTDBAttConfirm_Status_gSmInvalidCommandLength_c = 0x0405,
	GATTDBAttConfirm_Status_gSmInvalidCommandParameter_c = 0x0406,
	GATTDBAttConfirm_Status_gSmInvalidDeviceId_c = 0x0407,
	GATTDBAttConfirm_Status_gSmInvalidInternalOperation_c = 0x0408,
	GATTDBAttConfirm_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GATTDBAttConfirm_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GATTDBAttConfirm_Status_gSmUnexpectedKeyType_c = 0x040B,
	GATTDBAttConfirm_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GATTDBAttConfirm_Status_gSmUnexpectedKeyset_c = 0x040D,
	GATTDBAttConfirm_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GATTDBAttConfirm_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GATTDBAttConfirm_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GATTDBAttConfirm_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GATTDBAttConfirm_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GATTDBAttConfirm_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GATTDBAttConfirm_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GATTDBAttConfirm_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GATTDBAttConfirm_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GATTDBAttConfirm_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GATTDBAttConfirm_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GATTDBAttConfirm_Status_gSmInsufficientResources_c = 0x0419,
	GATTDBAttConfirm_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GATTDBAttConfirm_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GATTDBAttConfirm_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GATTDBAttConfirm_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GATTDBAttConfirm_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GATTDBAttConfirm_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GATTDBAttConfirm_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GATTDBAttConfirm_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GATTDBAttConfirm_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GATTDBAttConfirm_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GATTDBAttConfirm_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GATTDBAttConfirm_Status_gSmPairingErrorTimeout_c = 0x0436,
	GATTDBAttConfirm_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GATTDBAttConfirm_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GATTDBAttConfirm_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GATTDBAttConfirm_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GATTDBAttConfirm_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GATTDBAttConfirm_Status_gAttInvalidHandle_c = 0x0501,
	GATTDBAttConfirm_Status_gAttReadNotPermitted_c = 0x0502,
	GATTDBAttConfirm_Status_gAttWriteNotPermitted_c = 0x0503,
	GATTDBAttConfirm_Status_gAttInvalidPdu_c = 0x0504,
	GATTDBAttConfirm_Status_gAttInsufficientAuthentication_c = 0x0505,
	GATTDBAttConfirm_Status_gAttRequestNotSupported_c = 0x0506,
	GATTDBAttConfirm_Status_gAttInvalidOffset_c = 0x0507,
	GATTDBAttConfirm_Status_gAttInsufficientAuthorization_c = 0x0508,
	GATTDBAttConfirm_Status_gAttPrepareQueueFull_c = 0x0509,
	GATTDBAttConfirm_Status_gAttAttributeNotFound_c = 0x050A,
	GATTDBAttConfirm_Status_gAttAttributeNotLong_c = 0x050B,
	GATTDBAttConfirm_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GATTDBAttConfirm_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GATTDBAttConfirm_Status_gAttUnlikelyor_c = 0x050E,
	GATTDBAttConfirm_Status_gAttInsufficientEncryption_c = 0x050F,
	GATTDBAttConfirm_Status_gAttUnsupportedGroupType_c = 0x0510,
	GATTDBAttConfirm_Status_gAttInsufficientResources_c = 0x0511,
	GATTDBAttConfirm_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GATTDBAttConfirm_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GATTDBAttConfirm_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GATTDBAttConfirm_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GATTDBAttConfirm_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GATTDBAttConfirm_Status_gGattInvalidValueLength_c = 0x0605,
	GATTDBAttConfirm_Status_gGattServerTimeout_c = 0x0606,
	GATTDBAttConfirm_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GATTDBAttConfirm_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GATTDBAttConfirm_Status_gGattInvalidPduReceived_c = 0x0609,
	GATTDBAttConfirm_Status_gGattPeerDisconnected_c = 0x060A,
	GATTDBAttConfirm_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GATTDBAttConfirm_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GATTDBAttConfirm_Status_gGapAdvDataTooLong_c = 0x0701,
	GATTDBAttConfirm_Status_gGapScanRspDataTooLong_c = 0x0702,
	GATTDBAttConfirm_Status_gGapDeviceNotBonded_c = 0x0703,
	GATTDBAttConfirm_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GATTDBAttConfirm_Status_gDevDbCccdLimitReached_c = 0x0801,
	GATTDBAttConfirm_Status_gDevDbCccdNotFound_c = 0x0802,
	GATTDBAttConfirm_Status_gGattDbInvalidHandle_c = 0x0901,
	GATTDBAttConfirm_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GATTDBAttConfirm_Status_gGattDbCccdNotFound_c = 0x0903,
	GATTDBAttConfirm_Status_gGattDbServiceNotFound_c = 0x0904,
	GATTDBAttConfirm_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GATTDBAttConfirm_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GATTDBAttConfirm_Status_t;

typedef PACKED_STRUCT GATTDBAttConfirm_tag {
	GATTDBAttConfirm_Status_t Status;  // The status of the GATT Database request
} GATTDBAttConfirm_t;

/* ATT Uuid format */
typedef enum GATTDBAttFindInformationIndication_Params_Format_tag {
	GATTDBAttFindInformationIndication_Params_Format_Uuid16BitFormat = 0x01,
	GATTDBAttFindInformationIndication_Params_Format_Uuid128BitFormat = 0x02
} GATTDBAttFindInformationIndication_Params_Format_t;

typedef struct GATTDBAttFindInformationIndication_tag {
	struct {
		GATTDBAttFindInformationIndication_Params_Format_t Format;  // ATT Uuid format
		union {
			struct {
				uint8_t HandleUuid16PairCount;  // Number of UUID 16 bits format handles
				struct {
					uint16_t Handle;  // Handle
					uint16_t Uuid16;  // UUID 16 bits
				} *HandleUuid16;  // Handle UUID 16 bits format
			} Uuid16BitFormat;  // UUID 16 bits format
			struct {
				uint8_t HandleUuid128PairCount;  // Number of UUID 128 bits format handles
				struct {
					uint16_t Handle;  // Handle
					uint8_t Uuid128[16];  // Uuid 128 bits
				} *HandleUuid128;  // Handle UUID 128 bits format
			} Uuid128BitFormat;  // UUID 128 bits format data
		} InformationData;  // Information data
	} Params;  // ATT find information response parameters
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttFindInformationIndication_t;

typedef struct GATTDBAttFindByTypeValueIndication_tag {
	struct {
		uint16_t GroupCount;  // Number of groups
		struct {
			uint16_t StartingHandle;  // Starting handle
			uint16_t EndingHandle;  // Ending handle
		} *HandleGroup;  // ATT find information request parameters
	} Params;  // ATT find by type value response parameters
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttFindByTypeValueIndication_t;

typedef struct GATTDBAttReadByTypeIndication_tag {
	struct {
		uint8_t Length;  // Fixed length of each handle+value pair in the AttributeDataList
		uint16_t AttributeDataListLength;  // Attribute data list length
		uint8_t *AttributeDataList;  // Attribute data list
	} Params;  // ATT read by type response parameters
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttReadByTypeIndication_t;

typedef struct GATTDBAttReadIndication_tag {
	struct {
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT read response parameters
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttReadIndication_t;

typedef PACKED_STRUCT GATTDBAttReadBlobIndication_tag {
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttReadBlobIndication_t;

typedef struct GATTDBAttReadMultipleIndication_tag {
	struct {
		uint16_t ListLength;  // List length
		uint8_t *ListOfValues;  // List of values
	} Params;  // ATT read multiple response parameters
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttReadMultipleIndication_t;

typedef struct GATTDBAttReadByGroupTypeIndication_tag {
	struct {
		uint8_t Length;  // Fixed length of each handle+value pair in the AttributeDataList
		uint16_t AttributeDataListLength;  // Attribute data list length
		uint8_t *AttributeDataList;  // Attribute data list
	} Params;  // ATT read by group type response parameters
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttReadByGroupTypeIndication_t;

typedef PACKED_STRUCT GATTDBAttWriteIndication_tag {
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttWriteIndication_t;

typedef struct GATTDBAttPrepareWriteIndication_tag {
	struct {
		uint16_t AttributeHandle;  // Attribute handle
		uint16_t ValueOffset;  // Value offset
		uint16_t AttributeLength;  // Attribute length
		uint8_t *AttributeValue;  // Attribute value
	} Params;  // ATT prepare write response parameters
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttPrepareWriteIndication_t;

typedef PACKED_STRUCT GATTDBAttExecuteWriteIndication_tag {
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttExecuteWriteIndication_t;

typedef PACKED_STRUCT GATTDBAttExecuteWriteFromQueueIndication_tag {
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttExecuteWriteFromQueueIndication_t;

typedef PACKED_STRUCT GATTDBAttPrepareNotificationIndicationIndication_tag {
	uint16_t ErrorAttributeHandle;  // The attribute handle where an error occurred
} GATTDBAttPrepareNotificationIndicationIndication_t;

#endif  /* GATTDB_ATT_ENABLE */

#if GAP_ENABLE
/* The status of the GAP request */
typedef enum GAPConfirm_Status_tag {
	GAPConfirm_Status_gBleSuccess_c = 0x0000,
	GAPConfirm_Status_gBleInvalidParameter_c = 0x0001,
	GAPConfirm_Status_gBleOverflow_c = 0x0002,
	GAPConfirm_Status_gBleUnavailable_c = 0x0003,
	GAPConfirm_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPConfirm_Status_gBleOutOfMemory_c = 0x0005,
	GAPConfirm_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPConfirm_Status_gBleOsError_c = 0x0007,
	GAPConfirm_Status_gBleUnexpectedError_c = 0x0008,
	GAPConfirm_Status_gBleInvalidState_c = 0x0009,
	GAPConfirm_Status_gBleTimerError_c = 0x000A,
	GAPConfirm_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPConfirm_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPConfirm_Status_gHciHardwareFailure_c = 0x0103,
	GAPConfirm_Status_gHciPageTimeout_c = 0x0104,
	GAPConfirm_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPConfirm_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPConfirm_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPConfirm_Status_gHciConnectionTimeout_c = 0x0108,
	GAPConfirm_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPConfirm_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPConfirm_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPConfirm_Status_gHciCommandDisallowed_c = 0x010C,
	GAPConfirm_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPConfirm_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPConfirm_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPConfirm_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPConfirm_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPConfirm_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPConfirm_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPConfirm_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPConfirm_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPConfirm_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPConfirm_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPConfirm_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPConfirm_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPConfirm_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPConfirm_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPConfirm_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPConfirm_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPConfirm_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPConfirm_Status_gHciUnspecifiedError_c = 0x011F,
	GAPConfirm_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPConfirm_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPConfirm_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPConfirm_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPConfirm_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPConfirm_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPConfirm_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPConfirm_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPConfirm_Status_gHciInstantPassed_c = 0x0128,
	GAPConfirm_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPConfirm_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPConfirm_Status_gHciReserved_0x2B_c = 0x012B,
	GAPConfirm_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPConfirm_Status_gHciQosRejected_c = 0x012D,
	GAPConfirm_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPConfirm_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPConfirm_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPConfirm_Status_gHciReserved_0x31_c = 0x0131,
	GAPConfirm_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPConfirm_Status_gHciReserved_0x33_c = 0x0133,
	GAPConfirm_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPConfirm_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPConfirm_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPConfirm_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPConfirm_Status_gHciHostBusyPairing_c = 0x0138,
	GAPConfirm_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPConfirm_Status_gHciControllerBusy_c = 0x013A,
	GAPConfirm_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPConfirm_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPConfirm_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPConfirm_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPConfirm_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPConfirm_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPConfirm_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPConfirm_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPConfirm_Status_gHciLimitReached_c = 0x0143,
	GAPConfirm_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPConfirm_Status_gHciAlreadyInit_c = 0x01A0,
	GAPConfirm_Status_gHciInvalidParameter_c = 0x01A1,
	GAPConfirm_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPConfirm_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPConfirm_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPConfirm_Status_gHciEventNotSupported_c = 0x01A5,
	GAPConfirm_Status_gHciTransportError_c = 0x01A6,
	GAPConfirm_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPConfirm_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPConfirm_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPConfirm_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPConfirm_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPConfirm_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPConfirm_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPConfirm_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPConfirm_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPConfirm_Status_gL2caChannelClosed_c = 0x03F9,
	GAPConfirm_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPConfirm_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPConfirm_Status_gL2caChannelBusy_c = 0x03FC,
	GAPConfirm_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPConfirm_Status_gL2caError_c = 0x03FF,
	GAPConfirm_Status_gSmNullCBFunction_c = 0x0401,
	GAPConfirm_Status_gSmCommandNotSupported_c = 0x0402,
	GAPConfirm_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPConfirm_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPConfirm_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPConfirm_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPConfirm_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPConfirm_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPConfirm_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPConfirm_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPConfirm_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPConfirm_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPConfirm_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPConfirm_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPConfirm_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPConfirm_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPConfirm_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPConfirm_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPConfirm_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPConfirm_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPConfirm_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPConfirm_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPConfirm_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPConfirm_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPConfirm_Status_gSmInsufficientResources_c = 0x0419,
	GAPConfirm_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPConfirm_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPConfirm_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPConfirm_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPConfirm_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPConfirm_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPConfirm_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPConfirm_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPConfirm_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPConfirm_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPConfirm_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPConfirm_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPConfirm_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPConfirm_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPConfirm_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPConfirm_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPConfirm_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPConfirm_Status_gAttInvalidHandle_c = 0x0501,
	GAPConfirm_Status_gAttReadNotPermitted_c = 0x0502,
	GAPConfirm_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPConfirm_Status_gAttInvalidPdu_c = 0x0504,
	GAPConfirm_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPConfirm_Status_gAttRequestNotSupported_c = 0x0506,
	GAPConfirm_Status_gAttInvalidOffset_c = 0x0507,
	GAPConfirm_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPConfirm_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPConfirm_Status_gAttAttributeNotFound_c = 0x050A,
	GAPConfirm_Status_gAttAttributeNotLong_c = 0x050B,
	GAPConfirm_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPConfirm_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPConfirm_Status_gAttUnlikelyor_c = 0x050E,
	GAPConfirm_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPConfirm_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPConfirm_Status_gAttInsufficientResources_c = 0x0511,
	GAPConfirm_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPConfirm_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPConfirm_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPConfirm_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPConfirm_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPConfirm_Status_gGattInvalidValueLength_c = 0x0605,
	GAPConfirm_Status_gGattServerTimeout_c = 0x0606,
	GAPConfirm_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPConfirm_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPConfirm_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPConfirm_Status_gGattPeerDisconnected_c = 0x060A,
	GAPConfirm_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPConfirm_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPConfirm_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPConfirm_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPConfirm_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPConfirm_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPConfirm_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPConfirm_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPConfirm_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPConfirm_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPConfirm_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPConfirm_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPConfirm_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPConfirm_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPConfirm_Status_t;

typedef PACKED_STRUCT GAPConfirm_tag {
	GAPConfirm_Status_t Status;  // The status of the GAP request
} GAPConfirm_t;

typedef PACKED_STRUCT GAPCheckNotificationStatusIndication_tag {
	bool_t IsActive;  // Notification status
} GAPCheckNotificationStatusIndication_t;

typedef PACKED_STRUCT GAPCheckIndicationStatusIndication_tag {
	bool_t IsActive;  // Indication status
} GAPCheckIndicationStatusIndication_t;

/* Public or Random address */
typedef enum GAPLoadKeysIndication_Keys_AddressInfo_DeviceAddressType_tag {
	GAPLoadKeysIndication_Keys_AddressInfo_DeviceAddressType_gPublic_c = 0x00,
	GAPLoadKeysIndication_Keys_AddressInfo_DeviceAddressType_gRandom_c = 0x01
} GAPLoadKeysIndication_Keys_AddressInfo_DeviceAddressType_t;

typedef struct GAPLoadKeysIndication_tag {
	struct {
		bool_t LtkIncluded;  // Boolean value which indicates if LTK size and LTK are included or not
		struct {
			uint8_t LtkSize;  // Encryption Key Size
			uint8_t *Ltk;  // Long Term (Encryption) Key
		} LtkInfo;  // LTK information (size and value)
		bool_t IrkIncluded;  // Boolean value which indicates if IRK is included or not
		uint8_t Irk[16];  // Identity Resolving Key
		bool_t CsrkIncluded;  // Boolean value which indicates if CSRK is included or not
		uint8_t Csrk[16];  // Connection Signature Resolving Key
		struct {
			uint8_t RandSize;  // Random value size
			uint8_t *Rand;  // Random value used to identify the LTK
			uint16_t Ediv;  // EDIV value used to identify the LTK
		} RandEdivInfo;  // Rand information (size and value) and Ediv
		bool_t AddressIncluded;  // Boolean value which indicates if Address is included or not
		struct {
			GAPLoadKeysIndication_Keys_AddressInfo_DeviceAddressType_t DeviceAddressType;  // Public or Random address
			uint8_t DeviceAddress[6];  // Device Address
		} AddressInfo;  // Address information (type and value)
	} Keys;  // The SMP keys distributed by the peer
	uint8_t KeyFlags;  // 0x04|0x02|0x01  // Mask identifying the keys stored
	bool_t LeSc;  // LE SC pairing
	bool_t Authenticated;  // Peer is authenticated during pairing
} GAPLoadKeysIndication_t;

typedef struct GAPLoadEncryptionInformationIndication_tag {
	uint8_t LtkSize;  // LTK size
	uint8_t *Ltk;  // LTK value
} GAPLoadEncryptionInformationIndication_t;

typedef struct GAPLoadCustomPeerInformationIndication_tag {
	uint16_t InfoSize;  // Data size
	uint8_t *Info;  // Data value
} GAPLoadCustomPeerInformationIndication_t;

typedef PACKED_STRUCT GAPCheckIfBondedIndication_tag {
	bool_t IsBonded;  // Bonded flag
	uint8_t NvmIndex;  // NVM Index
} GAPCheckIfBondedIndication_t;

typedef PACKED_STRUCT GAPGetBondedDevicesCountIndication_tag {
	uint8_t NbOfBondedDevices;  // Number of bonded devices
} GAPGetBondedDevicesCountIndication_t;

typedef struct GAPGetBondedDeviceNameIndication_tag {
	uint8_t NameSize;  // The name size
	uint8_t *Name;  // The name value
} GAPGetBondedDeviceNameIndication_t;

typedef PACKED_STRUCT GAPGenericEventInitializationCompleteIndication_tag {
	uint32_t SupportedFeatures;  // local supported features
	uint16_t MaxAdvDataSize;  // The maximum length of data supported by the Controller for use as advertisement data or scan response data in an advertising event or as periodic advertisement data
	uint8_t NumOfSupportedAdvSets;  // number of supported advertising sets
	uint8_t PeriodicAdvListSize;  // periodic advertiser list size
} GAPGenericEventInitializationCompleteIndication_t;

/* Error code */
typedef enum GAPGenericEventInternalErrorIndication_ErrorCode_tag {
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleSuccess_c = 0x0000,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleInvalidParameter_c = 0x0001,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleOverflow_c = 0x0002,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleUnavailable_c = 0x0003,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleFeatureNotSupported_c = 0x0004,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleOutOfMemory_c = 0x0005,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleAlreadyInitialized_c = 0x0006,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleOsError_c = 0x0007,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleUnexpectedError_c = 0x0008,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleInvalidState_c = 0x0009,
	GAPGenericEventInternalErrorIndication_ErrorCode_gBleTimerError_c = 0x000A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnknownHciCommand_c = 0x0101,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciHardwareFailure_c = 0x0103,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciPageTimeout_c = 0x0104,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciAuthenticationFailure_c = 0x0105,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciPinOrKeyMissing_c = 0x0106,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionTimeout_c = 0x0108,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionLimitExceeded_c = 0x0109,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciCommandDisallowed_c = 0x010C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRepeatedAttempts_c = 0x0117,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciPairingNotAllowed_c = 0x0118,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnknownLpmPdu_c = 0x0119,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciScoOffsetRejected_c = 0x011B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciScoIntervalRejected_c = 0x011C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciScoAirModeRejected_c = 0x011D,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciInvalidLpmParameters_c = 0x011E,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnspecifiedError_c = 0x011F,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciLLResponseTimeout_c = 0x0122,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciLmpPduNotAllowed_c = 0x0124,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRequestedQosNotSupported_c = 0x0127,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciInstantPassed_c = 0x0128,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciDifferentTransactionCollision_c = 0x012A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciReserved_0x2B_c = 0x012B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciQosRejected_c = 0x012D,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciInsufficientSecurity_c = 0x012F,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciReserved_0x31_c = 0x0131,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRoleSwitchPending_c = 0x0132,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciReserved_0x33_c = 0x0133,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciReservedSlotViolation_c = 0x0134,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciRoleSwitchFailed_c = 0x0135,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciHostBusyPairing_c = 0x0138,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciControllerBusy_c = 0x013A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciMacConnectionFailed_c = 0x013F,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciType0SubmapNotDefined_c = 0x0141,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciLimitReached_c = 0x0143,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciOperationCancelledByHost_c = 0x0144,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciAlreadyInit_c = 0x01A0,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciInvalidParameter_c = 0x01A1,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciCallbackNotInstalled_c = 0x01A2,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciCommandNotSupported_c = 0x01A4,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciEventNotSupported_c = 0x01A5,
	GAPGenericEventInternalErrorIndication_ErrorCode_gHciTransportError_c = 0x01A6,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caAlreadyInit_c = 0x03F0,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caInsufficientResources_c = 0x03F1,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caLePsmInvalid_c = 0x03F4,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caChannelInvalid_c = 0x03F8,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caChannelClosed_c = 0x03F9,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caChannelBusy_c = 0x03FC,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caInvalidParameter_c = 0x03FE,
	GAPGenericEventInternalErrorIndication_ErrorCode_gL2caError_c = 0x03FF,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmNullCBFunction_c = 0x0401,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmCommandNotSupported_c = 0x0402,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnexpectedCommand_c = 0x0403,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidCommandCode_c = 0x0404,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidCommandLength_c = 0x0405,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidCommandParameter_c = 0x0406,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidDeviceId_c = 0x0407,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidInternalOperation_c = 0x0408,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidConnectionHandle_c = 0x0409,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInproperKeyDistributionField_c = 0x040A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnexpectedKeyType_c = 0x040B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnexpectedKeyset_c = 0x040D,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnknownSmpPacketType_c = 0x040F,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInvalidHciEventParameter_c = 0x0416,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmInsufficientResources_c = 0x0419,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmOobDataAddressMismatch_c = 0x041A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorTimeout_c = 0x0436,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingAlreadyStarted_c = 0x0438,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPGenericEventInternalErrorIndication_ErrorCode_gSmTbInvalidDataSignature_c = 0x0461,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInvalidHandle_c = 0x0501,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttReadNotPermitted_c = 0x0502,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttWriteNotPermitted_c = 0x0503,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInvalidPdu_c = 0x0504,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInsufficientAuthentication_c = 0x0505,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttRequestNotSupported_c = 0x0506,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInvalidOffset_c = 0x0507,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInsufficientAuthorization_c = 0x0508,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttPrepareQueueFull_c = 0x0509,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttAttributeNotFound_c = 0x050A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttAttributeNotLong_c = 0x050B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttUnlikelyor_c = 0x050E,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInsufficientEncryption_c = 0x050F,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttUnsupportedGroupType_c = 0x0510,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttInsufficientResources_c = 0x0511,
	GAPGenericEventInternalErrorIndication_ErrorCode_gAttDatabaseOutOfSync_c = 0x0512,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattInvalidValueLength_c = 0x0605,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattServerTimeout_c = 0x0606,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattClientConfirmationTimeout_c = 0x0608,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattInvalidPduReceived_c = 0x0609,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattPeerDisconnected_c = 0x060A,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattMtuExchangeInProgress_c = 0x060B,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGapAdvDataTooLong_c = 0x0701,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGapScanRspDataTooLong_c = 0x0702,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGapDeviceNotBonded_c = 0x0703,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPGenericEventInternalErrorIndication_ErrorCode_gDevDbCccdLimitReached_c = 0x0801,
	GAPGenericEventInternalErrorIndication_ErrorCode_gDevDbCccdNotFound_c = 0x0802,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattDbInvalidHandle_c = 0x0901,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattDbCccdNotFound_c = 0x0903,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattDbServiceNotFound_c = 0x0904,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattDbDescriptorNotFound_c = 0x0905,
	GAPGenericEventInternalErrorIndication_ErrorCode_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPGenericEventInternalErrorIndication_ErrorCode_t;

/* Error source */
typedef enum GAPGenericEventInternalErrorIndication_ErrorSource_tag {
	GAPGenericEventInternalErrorIndication_ErrorSource_gHciCommandStatus_c = 0x00,
	GAPGenericEventInternalErrorIndication_ErrorSource_gCheckPrivateResolvableAddress_c = 0x01,
	GAPGenericEventInternalErrorIndication_ErrorSource_gVerifySignature_c = 0x02,
	GAPGenericEventInternalErrorIndication_ErrorSource_gAddNewConnection_c = 0x03,
	GAPGenericEventInternalErrorIndication_ErrorSource_gResetController_c = 0x04,
	GAPGenericEventInternalErrorIndication_ErrorSource_gSetEventMask_c = 0x05,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadLeBufferSize_c = 0x06,
	GAPGenericEventInternalErrorIndication_ErrorSource_gSetLeEventMask_c = 0x07,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadDeviceAddress_c = 0x08,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadLocalSupportedFeatures_c = 0x09,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadFilterAcceptListSize_c = 0x0A,
	GAPGenericEventInternalErrorIndication_ErrorSource_gClearFilterAcceptList_c = 0x0B,
	GAPGenericEventInternalErrorIndication_ErrorSource_gAddDeviceToFilterAcceptList_c = 0x0C,
	GAPGenericEventInternalErrorIndication_ErrorSource_gRemoveDeviceFromFilterAcceptList_c = 0x0D,
	GAPGenericEventInternalErrorIndication_ErrorSource_gCancelCreateConnection_c = 0x0E,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadRadioPower_c = 0x0F,
	GAPGenericEventInternalErrorIndication_ErrorSource_gSetRandomAddress_c = 0x10,
	GAPGenericEventInternalErrorIndication_ErrorSource_gCreateRandomAddress_c = 0x11,
	GAPGenericEventInternalErrorIndication_ErrorSource_gEncryptLink_c = 0x12,
	GAPGenericEventInternalErrorIndication_ErrorSource_gProvideLongTermKey_c = 0x13,
	GAPGenericEventInternalErrorIndication_ErrorSource_gDenyLongTermKey_c = 0x14,
	GAPGenericEventInternalErrorIndication_ErrorSource_gConnect_c = 0x15,
	GAPGenericEventInternalErrorIndication_ErrorSource_gDisconnect_c = 0x16,
	GAPGenericEventInternalErrorIndication_ErrorSource_gTerminatePairing_c = 0x17,
	GAPGenericEventInternalErrorIndication_ErrorSource_gSendPeripheralSecurityRequest_c = 0x18,
	GAPGenericEventInternalErrorIndication_ErrorSource_gEnterPasskey_c = 0x19,
	GAPGenericEventInternalErrorIndication_ErrorSource_gProvideOob_c = 0x1A,
	GAPGenericEventInternalErrorIndication_ErrorSource_gSendSmpKeys_c = 0x1B,
	GAPGenericEventInternalErrorIndication_ErrorSource_gWriteSuggestedDefaultDataLength_c = 0x1C,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadSuggestedDefaultDataLength_c = 0x1D,
	GAPGenericEventInternalErrorIndication_ErrorSource_gUpdateLeDataLength_c = 0x1E,
	GAPGenericEventInternalErrorIndication_ErrorSource_gEnableHostPrivacy_c = 0x1F,
	GAPGenericEventInternalErrorIndication_ErrorSource_gEnableControllerPrivacy_c = 0x20,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeScSendKeypressNotification_c = 0x21,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeScSetPeerOobData_c = 0x22,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeScGetLocalOobData_c = 0x23,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeScValidateNumericValue_c = 0x24,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeScRegeneratePublicKey_c = 0x25,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeSetResolvablePrivateAddressTimeout_c = 0x26,
	GAPGenericEventInternalErrorIndication_ErrorSource_gDefaultPairingProcedure_c = 0x27,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeControllerTest_c = 0x28,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeReadPhy_c = 0x29,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeSetPhy_c = 0x2A,
	GAPGenericEventInternalErrorIndication_ErrorSource_gSaveKeys_c = 0x2B,
	GAPGenericEventInternalErrorIndication_ErrorSource_gSetChannelMap_c = 0x2C,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadLocalSupportedCommands_c = 0x2D,
	GAPGenericEventInternalErrorIndication_ErrorSource_gEnableLdmTimer_c = 0x2E,
	GAPGenericEventInternalErrorIndication_ErrorSource_gRemoveAdvertisingSet_c = 0x2F,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLePeriodicAdvSyncEstb_c = 0x30,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLePeriodicAdvSyncLost_c = 0x31,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeRemoveDeviceFromPeriodicAdvList_c = 0x32,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeClearPeriodicAdvList_c = 0x33,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeAddDeviceToPeriodicAdvList_c = 0x34,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeReadNumOfSupportedAdvSets_c = 0x35,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeReadPeriodicAdvListSize_c = 0x36,
	GAPGenericEventInternalErrorIndication_ErrorSource_gLeReadMaxAdvDataLen_c = 0x37,
	GAPGenericEventInternalErrorIndication_ErrorSource_gPeriodicAdvCreateSync = 0x38,
	GAPGenericEventInternalErrorIndication_ErrorSource_gPeriodicAdvCancelSync = 0x39,
	GAPGenericEventInternalErrorIndication_ErrorSource_gPeriodicAdvTerminateSync = 0x3A,
	GAPGenericEventInternalErrorIndication_ErrorSource_gL2capRxPacket_c = 0x3B,
	GAPGenericEventInternalErrorIndication_ErrorSource_gExtAdvReportProcess_c = 0x3C,
	GAPGenericEventInternalErrorIndication_ErrorSource_gReadControllerLocalRPA_c = 0x3D,
	GAPGenericEventInternalErrorIndication_ErrorSource_gHciEventReceiveHandler_c = 0x3E
} GAPGenericEventInternalErrorIndication_ErrorSource_t;

typedef PACKED_STRUCT GAPGenericEventInternalErrorIndication_tag {
	GAPGenericEventInternalErrorIndication_ErrorCode_t ErrorCode;  // Error code
	GAPGenericEventInternalErrorIndication_ErrorSource_t ErrorSource;  // Error source
	uint16_t HciCommandOpcode;  // HCI command operation code
} GAPGenericEventInternalErrorIndication_t;

/* Setup fail reason */
typedef enum GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_tag {
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleSuccess_c = 0x0000,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleInvalidParameter_c = 0x0001,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleOverflow_c = 0x0002,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleUnavailable_c = 0x0003,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleFeatureNotSupported_c = 0x0004,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleOutOfMemory_c = 0x0005,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleAlreadyInitialized_c = 0x0006,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleOsError_c = 0x0007,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleUnexpectedError_c = 0x0008,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleInvalidState_c = 0x0009,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gBleTimerError_c = 0x000A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnknownHciCommand_c = 0x0101,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciHardwareFailure_c = 0x0103,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciPageTimeout_c = 0x0104,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciAuthenticationFailure_c = 0x0105,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciPinOrKeyMissing_c = 0x0106,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionTimeout_c = 0x0108,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionLimitExceeded_c = 0x0109,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciCommandDisallowed_c = 0x010C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRepeatedAttempts_c = 0x0117,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciPairingNotAllowed_c = 0x0118,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnknownLpmPdu_c = 0x0119,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciScoOffsetRejected_c = 0x011B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciScoIntervalRejected_c = 0x011C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciScoAirModeRejected_c = 0x011D,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciInvalidLpmParameters_c = 0x011E,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnspecifiedError_c = 0x011F,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciLLResponseTimeout_c = 0x0122,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciLmpPduNotAllowed_c = 0x0124,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRequestedQosNotSupported_c = 0x0127,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciInstantPassed_c = 0x0128,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciDifferentTransactionCollision_c = 0x012A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciReserved_0x2B_c = 0x012B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciQosRejected_c = 0x012D,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciInsufficientSecurity_c = 0x012F,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciReserved_0x31_c = 0x0131,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRoleSwitchPending_c = 0x0132,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciReserved_0x33_c = 0x0133,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciReservedSlotViolation_c = 0x0134,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciRoleSwitchFailed_c = 0x0135,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciHostBusyPairing_c = 0x0138,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciControllerBusy_c = 0x013A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciMacConnectionFailed_c = 0x013F,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciType0SubmapNotDefined_c = 0x0141,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciLimitReached_c = 0x0143,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciOperationCancelledByHost_c = 0x0144,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciAlreadyInit_c = 0x01A0,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciInvalidParameter_c = 0x01A1,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciCallbackNotInstalled_c = 0x01A2,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciCommandNotSupported_c = 0x01A4,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciEventNotSupported_c = 0x01A5,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gHciTransportError_c = 0x01A6,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caAlreadyInit_c = 0x03F0,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caInsufficientResources_c = 0x03F1,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caLePsmInvalid_c = 0x03F4,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caChannelInvalid_c = 0x03F8,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caChannelClosed_c = 0x03F9,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caChannelBusy_c = 0x03FC,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caInvalidParameter_c = 0x03FE,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gL2caError_c = 0x03FF,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmNullCBFunction_c = 0x0401,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmCommandNotSupported_c = 0x0402,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnexpectedCommand_c = 0x0403,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidCommandCode_c = 0x0404,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidCommandLength_c = 0x0405,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidCommandParameter_c = 0x0406,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidDeviceId_c = 0x0407,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidInternalOperation_c = 0x0408,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidConnectionHandle_c = 0x0409,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInproperKeyDistributionField_c = 0x040A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnexpectedKeyType_c = 0x040B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnexpectedKeyset_c = 0x040D,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnknownSmpPacketType_c = 0x040F,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInvalidHciEventParameter_c = 0x0416,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmInsufficientResources_c = 0x0419,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmOobDataAddressMismatch_c = 0x041A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorTimeout_c = 0x0436,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingAlreadyStarted_c = 0x0438,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gSmTbInvalidDataSignature_c = 0x0461,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInvalidHandle_c = 0x0501,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttReadNotPermitted_c = 0x0502,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttWriteNotPermitted_c = 0x0503,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInvalidPdu_c = 0x0504,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInsufficientAuthentication_c = 0x0505,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttRequestNotSupported_c = 0x0506,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInvalidOffset_c = 0x0507,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInsufficientAuthorization_c = 0x0508,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttPrepareQueueFull_c = 0x0509,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttAttributeNotFound_c = 0x050A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttAttributeNotLong_c = 0x050B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttUnlikelyor_c = 0x050E,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInsufficientEncryption_c = 0x050F,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttUnsupportedGroupType_c = 0x0510,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttInsufficientResources_c = 0x0511,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gAttDatabaseOutOfSync_c = 0x0512,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattInvalidValueLength_c = 0x0605,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattServerTimeout_c = 0x0606,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattClientConfirmationTimeout_c = 0x0608,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattInvalidPduReceived_c = 0x0609,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattPeerDisconnected_c = 0x060A,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattMtuExchangeInProgress_c = 0x060B,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGapAdvDataTooLong_c = 0x0701,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGapScanRspDataTooLong_c = 0x0702,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGapDeviceNotBonded_c = 0x0703,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gDevDbCccdLimitReached_c = 0x0801,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gDevDbCccdNotFound_c = 0x0802,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattDbInvalidHandle_c = 0x0901,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattDbCccdNotFound_c = 0x0903,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattDbServiceNotFound_c = 0x0904,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattDbDescriptorNotFound_c = 0x0905,
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_t;

typedef PACKED_STRUCT GAPGenericEventAdvertisingSetupFailedIndication_tag {
	GAPGenericEventAdvertisingSetupFailedIndication_SetupFailReason_t SetupFailReason;  // Setup fail reason
} GAPGenericEventAdvertisingSetupFailedIndication_t;

typedef PACKED_STRUCT GAPGenericEventAdvertisingParametersSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventAdvertisingParametersSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPGenericEventAdvertisingDataSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventAdvertisingDataSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPGenericEventFilterAcceptListSizeReadIndication_tag {
	uint8_t FilterAcceptListSize;  // Filter accept list size
} GAPGenericEventFilterAcceptListSizeReadIndication_t;

typedef PACKED_STRUCT GAPGenericEventDeviceAddedToFilterAcceptListIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventDeviceAddedToFilterAcceptListIndication_t;

typedef PACKED_STRUCT GAPGenericEventDeviceRemovedFromFilterAcceptListIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventDeviceRemovedFromFilterAcceptListIndication_t;

typedef PACKED_STRUCT GAPGenericEventFilterAcceptListClearedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventFilterAcceptListClearedIndication_t;

typedef PACKED_STRUCT GAPGenericEventRandomAddressReadyIndication_tag {
	uint8_t Address[6];  // Random address
	uint8_t Handle;  // Advertising set handle if the generated device address will be used on an extended set, 0xFF otherwise
} GAPGenericEventRandomAddressReadyIndication_t;

typedef PACKED_STRUCT GAPGenericEventCreateConnectionCanceledIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventCreateConnectionCanceledIndication_t;

typedef PACKED_STRUCT GAPGenericEventPublicAddressReadIndication_tag {
	uint8_t Address[6];  // Public address
} GAPGenericEventPublicAddressReadIndication_t;

typedef PACKED_STRUCT GAPGenericEventAdvTxPowerLevelReadIndication_tag {
	uint8_t AdvTxPowerLevel;  // Advertising transmission power level (in dBm)
} GAPGenericEventAdvTxPowerLevelReadIndication_t;

typedef PACKED_STRUCT GAPGenericEventPrivateResolvableAddressVerifiedIndication_tag {
	bool_t Verified;  // Private Resolvable Address verified
} GAPGenericEventPrivateResolvableAddressVerifiedIndication_t;

typedef PACKED_STRUCT GAPGenericEventRandomAddressSetIndication_tag {
	uint8_t Handle;  // Advertising handle of the configured set or 0xFF in case of legacy advertising or scanning and initiating address
} GAPGenericEventRandomAddressSetIndication_t;

typedef PACKED_STRUCT GAPAdvertisingEventStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPAdvertisingEventStateChangedIndication_t;

/* The reason of failure */
typedef enum GAPAdvertisingEventCommandFailedIndication_FailReason_tag {
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleSuccess_c = 0x0000,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleInvalidParameter_c = 0x0001,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleOverflow_c = 0x0002,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleUnavailable_c = 0x0003,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleFeatureNotSupported_c = 0x0004,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleOutOfMemory_c = 0x0005,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleAlreadyInitialized_c = 0x0006,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleOsError_c = 0x0007,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleUnexpectedError_c = 0x0008,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleInvalidState_c = 0x0009,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gBleTimerError_c = 0x000A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnknownHciCommand_c = 0x0101,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciHardwareFailure_c = 0x0103,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciPageTimeout_c = 0x0104,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciAuthenticationFailure_c = 0x0105,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciPinOrKeyMissing_c = 0x0106,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionTimeout_c = 0x0108,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionLimitExceeded_c = 0x0109,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciCommandDisallowed_c = 0x010C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRepeatedAttempts_c = 0x0117,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciPairingNotAllowed_c = 0x0118,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnknownLpmPdu_c = 0x0119,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciScoOffsetRejected_c = 0x011B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciScoIntervalRejected_c = 0x011C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciScoAirModeRejected_c = 0x011D,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciInvalidLpmParameters_c = 0x011E,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnspecifiedError_c = 0x011F,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciLLResponseTimeout_c = 0x0122,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciLmpPduNotAllowed_c = 0x0124,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRequestedQosNotSupported_c = 0x0127,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciInstantPassed_c = 0x0128,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciDifferentTransactionCollision_c = 0x012A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciReserved_0x2B_c = 0x012B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciQosRejected_c = 0x012D,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciInsufficientSecurity_c = 0x012F,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciReserved_0x31_c = 0x0131,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRoleSwitchPending_c = 0x0132,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciReserved_0x33_c = 0x0133,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciReservedSlotViolation_c = 0x0134,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciRoleSwitchFailed_c = 0x0135,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciHostBusyPairing_c = 0x0138,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciControllerBusy_c = 0x013A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciMacConnectionFailed_c = 0x013F,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciType0SubmapNotDefined_c = 0x0141,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciLimitReached_c = 0x0143,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciOperationCancelledByHost_c = 0x0144,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciAlreadyInit_c = 0x01A0,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciInvalidParameter_c = 0x01A1,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciCallbackNotInstalled_c = 0x01A2,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciCommandNotSupported_c = 0x01A4,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciEventNotSupported_c = 0x01A5,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gHciTransportError_c = 0x01A6,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caAlreadyInit_c = 0x03F0,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caInsufficientResources_c = 0x03F1,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caLePsmInvalid_c = 0x03F4,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caChannelInvalid_c = 0x03F8,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caChannelClosed_c = 0x03F9,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caChannelBusy_c = 0x03FC,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caInvalidParameter_c = 0x03FE,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gL2caError_c = 0x03FF,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmNullCBFunction_c = 0x0401,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmCommandNotSupported_c = 0x0402,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnexpectedCommand_c = 0x0403,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidCommandCode_c = 0x0404,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidCommandLength_c = 0x0405,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidCommandParameter_c = 0x0406,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidDeviceId_c = 0x0407,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidInternalOperation_c = 0x0408,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidConnectionHandle_c = 0x0409,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInproperKeyDistributionField_c = 0x040A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnexpectedKeyType_c = 0x040B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnexpectedKeyset_c = 0x040D,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnknownSmpPacketType_c = 0x040F,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInvalidHciEventParameter_c = 0x0416,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmInsufficientResources_c = 0x0419,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmOobDataAddressMismatch_c = 0x041A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorTimeout_c = 0x0436,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingAlreadyStarted_c = 0x0438,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gSmTbInvalidDataSignature_c = 0x0461,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInvalidHandle_c = 0x0501,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttReadNotPermitted_c = 0x0502,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttWriteNotPermitted_c = 0x0503,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInvalidPdu_c = 0x0504,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInsufficientAuthentication_c = 0x0505,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttRequestNotSupported_c = 0x0506,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInvalidOffset_c = 0x0507,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInsufficientAuthorization_c = 0x0508,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttPrepareQueueFull_c = 0x0509,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttAttributeNotFound_c = 0x050A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttAttributeNotLong_c = 0x050B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttUnlikelyor_c = 0x050E,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInsufficientEncryption_c = 0x050F,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttUnsupportedGroupType_c = 0x0510,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttInsufficientResources_c = 0x0511,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gAttDatabaseOutOfSync_c = 0x0512,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattInvalidValueLength_c = 0x0605,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattServerTimeout_c = 0x0606,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattClientConfirmationTimeout_c = 0x0608,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattInvalidPduReceived_c = 0x0609,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattPeerDisconnected_c = 0x060A,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattMtuExchangeInProgress_c = 0x060B,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGapAdvDataTooLong_c = 0x0701,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGapScanRspDataTooLong_c = 0x0702,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGapDeviceNotBonded_c = 0x0703,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gDevDbCccdLimitReached_c = 0x0801,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gDevDbCccdNotFound_c = 0x0802,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattDbInvalidHandle_c = 0x0901,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattDbCccdNotFound_c = 0x0903,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattDbServiceNotFound_c = 0x0904,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattDbDescriptorNotFound_c = 0x0905,
	GAPAdvertisingEventCommandFailedIndication_FailReason_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPAdvertisingEventCommandFailedIndication_FailReason_t;

typedef PACKED_STRUCT GAPAdvertisingEventCommandFailedIndication_tag {
	GAPAdvertisingEventCommandFailedIndication_FailReason_t FailReason;  // The reason of failure
} GAPAdvertisingEventCommandFailedIndication_t;

typedef PACKED_STRUCT GAPScanningEventStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPScanningEventStateChangedIndication_t;

/* The reason of failure */
typedef enum GAPScanningEventCommandFailedIndication_FailReason_tag {
	GAPScanningEventCommandFailedIndication_FailReason_gBleSuccess_c = 0x0000,
	GAPScanningEventCommandFailedIndication_FailReason_gBleInvalidParameter_c = 0x0001,
	GAPScanningEventCommandFailedIndication_FailReason_gBleOverflow_c = 0x0002,
	GAPScanningEventCommandFailedIndication_FailReason_gBleUnavailable_c = 0x0003,
	GAPScanningEventCommandFailedIndication_FailReason_gBleFeatureNotSupported_c = 0x0004,
	GAPScanningEventCommandFailedIndication_FailReason_gBleOutOfMemory_c = 0x0005,
	GAPScanningEventCommandFailedIndication_FailReason_gBleAlreadyInitialized_c = 0x0006,
	GAPScanningEventCommandFailedIndication_FailReason_gBleOsError_c = 0x0007,
	GAPScanningEventCommandFailedIndication_FailReason_gBleUnexpectedError_c = 0x0008,
	GAPScanningEventCommandFailedIndication_FailReason_gBleInvalidState_c = 0x0009,
	GAPScanningEventCommandFailedIndication_FailReason_gBleTimerError_c = 0x000A,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnknownHciCommand_c = 0x0101,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPScanningEventCommandFailedIndication_FailReason_gHciHardwareFailure_c = 0x0103,
	GAPScanningEventCommandFailedIndication_FailReason_gHciPageTimeout_c = 0x0104,
	GAPScanningEventCommandFailedIndication_FailReason_gHciAuthenticationFailure_c = 0x0105,
	GAPScanningEventCommandFailedIndication_FailReason_gHciPinOrKeyMissing_c = 0x0106,
	GAPScanningEventCommandFailedIndication_FailReason_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionTimeout_c = 0x0108,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionLimitExceeded_c = 0x0109,
	GAPScanningEventCommandFailedIndication_FailReason_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPScanningEventCommandFailedIndication_FailReason_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPScanningEventCommandFailedIndication_FailReason_gHciCommandDisallowed_c = 0x010C,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPScanningEventCommandFailedIndication_FailReason_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRepeatedAttempts_c = 0x0117,
	GAPScanningEventCommandFailedIndication_FailReason_gHciPairingNotAllowed_c = 0x0118,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnknownLpmPdu_c = 0x0119,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPScanningEventCommandFailedIndication_FailReason_gHciScoOffsetRejected_c = 0x011B,
	GAPScanningEventCommandFailedIndication_FailReason_gHciScoIntervalRejected_c = 0x011C,
	GAPScanningEventCommandFailedIndication_FailReason_gHciScoAirModeRejected_c = 0x011D,
	GAPScanningEventCommandFailedIndication_FailReason_gHciInvalidLpmParameters_c = 0x011E,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnspecifiedError_c = 0x011F,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPScanningEventCommandFailedIndication_FailReason_gHciLLResponseTimeout_c = 0x0122,
	GAPScanningEventCommandFailedIndication_FailReason_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPScanningEventCommandFailedIndication_FailReason_gHciLmpPduNotAllowed_c = 0x0124,
	GAPScanningEventCommandFailedIndication_FailReason_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPScanningEventCommandFailedIndication_FailReason_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRequestedQosNotSupported_c = 0x0127,
	GAPScanningEventCommandFailedIndication_FailReason_gHciInstantPassed_c = 0x0128,
	GAPScanningEventCommandFailedIndication_FailReason_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPScanningEventCommandFailedIndication_FailReason_gHciDifferentTransactionCollision_c = 0x012A,
	GAPScanningEventCommandFailedIndication_FailReason_gHciReserved_0x2B_c = 0x012B,
	GAPScanningEventCommandFailedIndication_FailReason_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPScanningEventCommandFailedIndication_FailReason_gHciQosRejected_c = 0x012D,
	GAPScanningEventCommandFailedIndication_FailReason_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPScanningEventCommandFailedIndication_FailReason_gHciInsufficientSecurity_c = 0x012F,
	GAPScanningEventCommandFailedIndication_FailReason_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPScanningEventCommandFailedIndication_FailReason_gHciReserved_0x31_c = 0x0131,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRoleSwitchPending_c = 0x0132,
	GAPScanningEventCommandFailedIndication_FailReason_gHciReserved_0x33_c = 0x0133,
	GAPScanningEventCommandFailedIndication_FailReason_gHciReservedSlotViolation_c = 0x0134,
	GAPScanningEventCommandFailedIndication_FailReason_gHciRoleSwitchFailed_c = 0x0135,
	GAPScanningEventCommandFailedIndication_FailReason_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPScanningEventCommandFailedIndication_FailReason_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPScanningEventCommandFailedIndication_FailReason_gHciHostBusyPairing_c = 0x0138,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPScanningEventCommandFailedIndication_FailReason_gHciControllerBusy_c = 0x013A,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPScanningEventCommandFailedIndication_FailReason_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPScanningEventCommandFailedIndication_FailReason_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPScanningEventCommandFailedIndication_FailReason_gHciMacConnectionFailed_c = 0x013F,
	GAPScanningEventCommandFailedIndication_FailReason_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPScanningEventCommandFailedIndication_FailReason_gHciType0SubmapNotDefined_c = 0x0141,
	GAPScanningEventCommandFailedIndication_FailReason_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPScanningEventCommandFailedIndication_FailReason_gHciLimitReached_c = 0x0143,
	GAPScanningEventCommandFailedIndication_FailReason_gHciOperationCancelledByHost_c = 0x0144,
	GAPScanningEventCommandFailedIndication_FailReason_gHciAlreadyInit_c = 0x01A0,
	GAPScanningEventCommandFailedIndication_FailReason_gHciInvalidParameter_c = 0x01A1,
	GAPScanningEventCommandFailedIndication_FailReason_gHciCallbackNotInstalled_c = 0x01A2,
	GAPScanningEventCommandFailedIndication_FailReason_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPScanningEventCommandFailedIndication_FailReason_gHciCommandNotSupported_c = 0x01A4,
	GAPScanningEventCommandFailedIndication_FailReason_gHciEventNotSupported_c = 0x01A5,
	GAPScanningEventCommandFailedIndication_FailReason_gHciTransportError_c = 0x01A6,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caAlreadyInit_c = 0x03F0,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caInsufficientResources_c = 0x03F1,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caLePsmInvalid_c = 0x03F4,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caChannelInvalid_c = 0x03F8,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caChannelClosed_c = 0x03F9,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caChannelBusy_c = 0x03FC,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caInvalidParameter_c = 0x03FE,
	GAPScanningEventCommandFailedIndication_FailReason_gL2caError_c = 0x03FF,
	GAPScanningEventCommandFailedIndication_FailReason_gSmNullCBFunction_c = 0x0401,
	GAPScanningEventCommandFailedIndication_FailReason_gSmCommandNotSupported_c = 0x0402,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnexpectedCommand_c = 0x0403,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidCommandCode_c = 0x0404,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidCommandLength_c = 0x0405,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidCommandParameter_c = 0x0406,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidDeviceId_c = 0x0407,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidInternalOperation_c = 0x0408,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidConnectionHandle_c = 0x0409,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInproperKeyDistributionField_c = 0x040A,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnexpectedKeyType_c = 0x040B,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnexpectedKeyset_c = 0x040D,
	GAPScanningEventCommandFailedIndication_FailReason_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnknownSmpPacketType_c = 0x040F,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPScanningEventCommandFailedIndication_FailReason_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPScanningEventCommandFailedIndication_FailReason_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPScanningEventCommandFailedIndication_FailReason_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPScanningEventCommandFailedIndication_FailReason_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInvalidHciEventParameter_c = 0x0416,
	GAPScanningEventCommandFailedIndication_FailReason_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPScanningEventCommandFailedIndication_FailReason_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPScanningEventCommandFailedIndication_FailReason_gSmInsufficientResources_c = 0x0419,
	GAPScanningEventCommandFailedIndication_FailReason_gSmOobDataAddressMismatch_c = 0x041A,
	GAPScanningEventCommandFailedIndication_FailReason_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPScanningEventCommandFailedIndication_FailReason_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPScanningEventCommandFailedIndication_FailReason_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorTimeout_c = 0x0436,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingAlreadyStarted_c = 0x0438,
	GAPScanningEventCommandFailedIndication_FailReason_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPScanningEventCommandFailedIndication_FailReason_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPScanningEventCommandFailedIndication_FailReason_gSmTbInvalidDataSignature_c = 0x0461,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInvalidHandle_c = 0x0501,
	GAPScanningEventCommandFailedIndication_FailReason_gAttReadNotPermitted_c = 0x0502,
	GAPScanningEventCommandFailedIndication_FailReason_gAttWriteNotPermitted_c = 0x0503,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInvalidPdu_c = 0x0504,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInsufficientAuthentication_c = 0x0505,
	GAPScanningEventCommandFailedIndication_FailReason_gAttRequestNotSupported_c = 0x0506,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInvalidOffset_c = 0x0507,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInsufficientAuthorization_c = 0x0508,
	GAPScanningEventCommandFailedIndication_FailReason_gAttPrepareQueueFull_c = 0x0509,
	GAPScanningEventCommandFailedIndication_FailReason_gAttAttributeNotFound_c = 0x050A,
	GAPScanningEventCommandFailedIndication_FailReason_gAttAttributeNotLong_c = 0x050B,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPScanningEventCommandFailedIndication_FailReason_gAttUnlikelyor_c = 0x050E,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInsufficientEncryption_c = 0x050F,
	GAPScanningEventCommandFailedIndication_FailReason_gAttUnsupportedGroupType_c = 0x0510,
	GAPScanningEventCommandFailedIndication_FailReason_gAttInsufficientResources_c = 0x0511,
	GAPScanningEventCommandFailedIndication_FailReason_gAttDatabaseOutOfSync_c = 0x0512,
	GAPScanningEventCommandFailedIndication_FailReason_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPScanningEventCommandFailedIndication_FailReason_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPScanningEventCommandFailedIndication_FailReason_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPScanningEventCommandFailedIndication_FailReason_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPScanningEventCommandFailedIndication_FailReason_gGattInvalidValueLength_c = 0x0605,
	GAPScanningEventCommandFailedIndication_FailReason_gGattServerTimeout_c = 0x0606,
	GAPScanningEventCommandFailedIndication_FailReason_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPScanningEventCommandFailedIndication_FailReason_gGattClientConfirmationTimeout_c = 0x0608,
	GAPScanningEventCommandFailedIndication_FailReason_gGattInvalidPduReceived_c = 0x0609,
	GAPScanningEventCommandFailedIndication_FailReason_gGattPeerDisconnected_c = 0x060A,
	GAPScanningEventCommandFailedIndication_FailReason_gGattMtuExchangeInProgress_c = 0x060B,
	GAPScanningEventCommandFailedIndication_FailReason_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPScanningEventCommandFailedIndication_FailReason_gGapAdvDataTooLong_c = 0x0701,
	GAPScanningEventCommandFailedIndication_FailReason_gGapScanRspDataTooLong_c = 0x0702,
	GAPScanningEventCommandFailedIndication_FailReason_gGapDeviceNotBonded_c = 0x0703,
	GAPScanningEventCommandFailedIndication_FailReason_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPScanningEventCommandFailedIndication_FailReason_gDevDbCccdLimitReached_c = 0x0801,
	GAPScanningEventCommandFailedIndication_FailReason_gDevDbCccdNotFound_c = 0x0802,
	GAPScanningEventCommandFailedIndication_FailReason_gGattDbInvalidHandle_c = 0x0901,
	GAPScanningEventCommandFailedIndication_FailReason_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPScanningEventCommandFailedIndication_FailReason_gGattDbCccdNotFound_c = 0x0903,
	GAPScanningEventCommandFailedIndication_FailReason_gGattDbServiceNotFound_c = 0x0904,
	GAPScanningEventCommandFailedIndication_FailReason_gGattDbDescriptorNotFound_c = 0x0905,
	GAPScanningEventCommandFailedIndication_FailReason_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPScanningEventCommandFailedIndication_FailReason_t;

typedef PACKED_STRUCT GAPScanningEventCommandFailedIndication_tag {
	GAPScanningEventCommandFailedIndication_FailReason_t FailReason;  // The reason of failure
} GAPScanningEventCommandFailedIndication_t;

/* Device's advertising address type */
typedef enum GAPScanningEventDeviceScannedIndication_AddressType_tag {
	GAPScanningEventDeviceScannedIndication_AddressType_gPublic_c = 0x00,
	GAPScanningEventDeviceScannedIndication_AddressType_gRandom_c = 0x01
} GAPScanningEventDeviceScannedIndication_AddressType_t;

/* Advertising report type, indicating what type of event generated this data (advertising, scan response) */
typedef enum GAPScanningEventDeviceScannedIndication_AdvEventType_tag {
	GAPScanningEventDeviceScannedIndication_AdvEventType_gBleAdvRepAdvInd_c = 0x00,
	GAPScanningEventDeviceScannedIndication_AdvEventType_gBleAdvRepAdvDirectInd_c = 0x01,
	GAPScanningEventDeviceScannedIndication_AdvEventType_gBleAdvRepAdvScanInd_c = 0x02,
	GAPScanningEventDeviceScannedIndication_AdvEventType_gBleAdvRepAdvNonconnInd_c = 0x03,
	GAPScanningEventDeviceScannedIndication_AdvEventType_gBleAdvRepScanRsp_c = 0x04,
	GAPScanningEventDeviceScannedIndication_AdvEventType_gExtAdvReportProcess_c = 0x3C,
	GAPScanningEventDeviceScannedIndication_AdvEventType_gReadControllerLocalRPA_c = 0x3D,
	GAPScanningEventDeviceScannedIndication_AdvEventType_gHciEventReceiveHandler_c = 0x3E
} GAPScanningEventDeviceScannedIndication_AdvEventType_t;

typedef struct GAPScanningEventDeviceScannedIndication_tag {
	GAPScanningEventDeviceScannedIndication_AddressType_t AddressType;  // Device's advertising address type
	uint8_t Address[6];  // Device's advertising address
	uint8_t Rssi;  // RSSI on the advertising channel; may be compared to the TX power contained in the AD Structure of type gAdTxPowerLevel_c to estimate distance from the advertiser
	uint8_t DataLength;  // Length of the advertising or scan response data
	uint8_t *Data;  // Advertising or scan response data
	GAPScanningEventDeviceScannedIndication_AdvEventType_t AdvEventType;  // Advertising report type, indicating what type of event generated this data (advertising, scan response)
	bool_t DirectRpaUsed;  // TRUE if directed advertising with Resolvable Private Address as Direct Address was detected while Enhanced Privacy is enabled
	uint8_t DirectRpa[6];  // Resolvable Private Address set as Direct Address for directed advertising
	bool_t advertisingAddressResolved;  // TRUE if the address contained in the addressType and aAddress fields is the identity address of a resolved RPA from the Advertising Address field
} GAPScanningEventDeviceScannedIndication_t;

/* Accuracy of central's clock, allowing for frame detection optimizations */
typedef enum GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_tag {
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc500ppm_c = 0x00,
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc250ppm_c = 0x01,
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc150ppm_c = 0x02,
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc100ppm_c = 0x03,
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc75ppm_c = 0x04,
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc50ppm_c = 0x05,
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc30ppm_c = 0x06,
	GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_gCentralClkAcc20ppm_c = 0x07
} GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_t;

/* Connected device's address type */
typedef enum GAPConnectionEventConnectedIndication_PeerAddressType_tag {
	GAPConnectionEventConnectedIndication_PeerAddressType_gPublic_c = 0x00,
	GAPConnectionEventConnectedIndication_PeerAddressType_gRandom_c = 0x01
} GAPConnectionEventConnectedIndication_PeerAddressType_t;

/* The role of the device in the connection - central or peripheral  */
typedef enum GAPConnectionEventConnectedIndication_connectionRole_tag {
	GAPConnectionEventConnectedIndication_connectionRole_gBleLlConnectionCentral_c = 0x00,
	GAPConnectionEventConnectedIndication_connectionRole_gBleLlConnectionPeripheral_c = 0x01
} GAPConnectionEventConnectedIndication_connectionRole_t;

typedef struct GAPConnectionEventConnectedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		uint16_t ConnInterval;  // Interval between connection events
		uint16_t ConnLatency;  // Number of consecutive connection events the Peripheral may ignore
		uint16_t SupervisionTimeout;  // The maximum time interval between consecutive over-the-air packets; if this timer expires, the connection is dropped
		GAPConnectionEventConnectedIndication_ConnectionParameters_CentralClockAccuracy_t CentralClockAccuracy;  // Accuracy of central's clock, allowing for frame detection optimizations
	} ConnectionParameters;  // Connection parameters established by the Controller
	GAPConnectionEventConnectedIndication_PeerAddressType_t PeerAddressType;  // Connected device's address type
	uint8_t PeerAddress[6];  // Connected device's address
	bool_t peerRpaResolved;  // TRUE if the address defined by peerAddressType and peerAddress is the identity address of the peer, and the peer used an RPA that was resolved by the Controller and is contained in the peerRpa field
	uint8_t peerRpa[6];  // Peer Resolvable Private Address if Controller Privacy is active and peerRpaResolved is TRUE
	bool_t localRpaUsed;  // TRUE if the Controller has used an RPA contained in the localRpa field
	uint8_t localRpa[6];  // Local Resolvable Private Address if Controller Privacy is active and localRpaUsed is TRUE
	GAPConnectionEventConnectedIndication_connectionRole_t connectionRole;  // The role of the device in the connection - central or peripheral 
} GAPConnectionEventConnectedIndication_t;

/* The desired security mode-level */
typedef enum GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_tag {
	GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_gMode1Level1_c = 0x10,
	GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_gMode1Level2_c = 0x11,
	GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_gMode1Level3_c = 0x12,
	GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_gMode1Level4_c = 0x13,
	GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_gMode2Level1_c = 0x21,
	GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_gMode2Level2_c = 0x22
} GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_t;

/* I/O capabilities used to determine the pairing method */
typedef enum GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_tag {
	GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_gIoDisplayOnly_c = 0x00,
	GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_gIoDisplayYesNo_c = 0x01,
	GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_gIoKeyboardOnly_c = 0x02,
	GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_gIoNone_c = 0x03,
	GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_gIoKeyboardDisplay_c = 0x04
} GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_t;

typedef PACKED_STRUCT GAPConnectionEventPairingRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		bool_t WithBonding;  // TRUE if this device is able to and wants to bond after pairing, FALSE otherwise
		GAPConnectionEventPairingRequestIndication_PairingParameters_SecurityModeAndLevel_t SecurityModeAndLevel;  // The desired security mode-level
		uint8_t MaxEncryptionKeySize;  // Maximum LTK size supported by the device
		GAPConnectionEventPairingRequestIndication_PairingParameters_LocalIoCapabilities_t LocalIoCapabilities;  // I/O capabilities used to determine the pairing method
		bool_t OobAvailable;  // TRUE if this device has Out-of-Band data that can be used for authenticated pairing, FALSE otherwise
		uint8_t CentralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Central
		uint8_t PeripheralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Peripheral
		bool_t LeSecureConnectionSupported;  // indicates if device supports LE Secure Connections pairing
		bool_t UseKeypressNotifications;  // Indicates if device supports Keypress Notification PDUs during Passkey Entry pairing
	} PairingParameters;  // Pairing parameters
} GAPConnectionEventPairingRequestIndication_t;

typedef PACKED_STRUCT GAPConnectionEventPeripheralSecurityRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		bool_t BondAfterPairing;  // TRUE if the Peripheral supports bonding, FALSE otherwise
		bool_t bAuthenticationRequired;  // TRUE if the peripheral requires authentication for MITM protection, FALSE otherwise
	} PeripheralSecurityRequestParameters;  // Peripheral security request parameters
} GAPConnectionEventPeripheralSecurityRequestIndication_t;

/* The desired security mode-level */
typedef enum GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_tag {
	GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_gMode1Level1_c = 0x10,
	GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_gMode1Level2_c = 0x11,
	GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_gMode1Level3_c = 0x12,
	GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_gMode1Level4_c = 0x13,
	GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_gMode2Level1_c = 0x21,
	GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_gMode2Level2_c = 0x22
} GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_t;

/* I/O capabilities used to determine the pairing method */
typedef enum GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_tag {
	GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_gIoDisplayOnly_c = 0x00,
	GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_gIoDisplayYesNo_c = 0x01,
	GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_gIoKeyboardOnly_c = 0x02,
	GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_gIoNone_c = 0x03,
	GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_gIoKeyboardDisplay_c = 0x04
} GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_t;

typedef PACKED_STRUCT GAPConnectionEventPairingResponseIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		bool_t WithBonding;  // TRUE if this device is able to and wants to bond after pairing, FALSE otherwise
		GAPConnectionEventPairingResponseIndication_PairingParameters_SecurityModeAndLevel_t SecurityModeAndLevel;  // The desired security mode-level
		uint8_t MaxEncryptionKeySize;  // Maximum LTK size supported by the device
		GAPConnectionEventPairingResponseIndication_PairingParameters_LocalIoCapabilities_t LocalIoCapabilities;  // I/O capabilities used to determine the pairing method
		bool_t OobAvailable;  // TRUE if this device has Out-of-Band data that can be used for authenticated pairing, FALSE otherwise
		uint8_t CentralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Central
		uint8_t PeripheralKeys;  // 0x04|0x02|0x01  // SMP keys to be distributed by the Peripheral
		bool_t LeSecureConnectionSupported;  // indicates if device supports LE Secure Connections pairing
		bool_t UseKeypressNotifications;  // Indicates if device supports Keypress Notification PDUs during Passkey Entry pairing
	} PairingParameters;  // Pairing parameters
} GAPConnectionEventPairingResponseIndication_t;

/* The reason of the rejection */
typedef enum GAPConnectionEventAuthenticationRejectedIndication_RejectReason_tag {
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_gOobNotAvailable_c = 0x02,
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_gIncompatibleIoCapabilities_c = 0x03,
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_gPairingNotSupported_c = 0x05,
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_gLowEncryptionKeySize_c = 0x06,
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_gUnspecifiedReason_c = 0x08,
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_gRepeatedAttempts_c = 0x09,
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_gLinkEncryptionFailed_c = 0xF0
} GAPConnectionEventAuthenticationRejectedIndication_RejectReason_t;

typedef PACKED_STRUCT GAPConnectionEventAuthenticationRejectedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	GAPConnectionEventAuthenticationRejectedIndication_RejectReason_t RejectReason;  // The reason of the rejection
} GAPConnectionEventAuthenticationRejectedIndication_t;

typedef PACKED_STRUCT GAPConnectionEventPasskeyRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
} GAPConnectionEventPasskeyRequestIndication_t;

typedef PACKED_STRUCT GAPConnectionEventOobRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
} GAPConnectionEventOobRequestIndication_t;

typedef PACKED_STRUCT GAPConnectionEventPasskeyDisplayIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	uint32_t PasskeyForDisplay;  // Passkey to be displayed by this Peripheral
} GAPConnectionEventPasskeyDisplayIndication_t;

typedef PACKED_STRUCT GAPConnectionEventKeyExchangeRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	uint8_t RequestedKeys;  // 0x04|0x02|0x01  // Mask identifying the keys being requested
	uint8_t RequestedLtkSize;  // Requested size of the encryption key
} GAPConnectionEventKeyExchangeRequestIndication_t;

/* Public or Random address */
typedef enum GAPConnectionEventKeysReceivedIndication_Keys_AddressInfo_DeviceAddressType_tag {
	GAPConnectionEventKeysReceivedIndication_Keys_AddressInfo_DeviceAddressType_gPublic_c = 0x00,
	GAPConnectionEventKeysReceivedIndication_Keys_AddressInfo_DeviceAddressType_gRandom_c = 0x01
} GAPConnectionEventKeysReceivedIndication_Keys_AddressInfo_DeviceAddressType_t;

typedef struct GAPConnectionEventKeysReceivedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		bool_t LtkIncluded;  // Boolean value which indicates if LTK size and LTK are included or not
		struct {
			uint8_t LtkSize;  // Encryption Key Size
			uint8_t *Ltk;  // Long Term (Encryption) Key
		} LtkInfo;  // LTK information (size and value)
		bool_t IrkIncluded;  // Boolean value which indicates if IRK is included or not
		uint8_t Irk[16];  // Identity Resolving Key
		bool_t CsrkIncluded;  // Boolean value which indicates if CSRK is included or not
		uint8_t Csrk[16];  // Connection Signature Resolving Key
		struct {
			uint8_t RandSize;  // Random value size
			uint8_t *Rand;  // Random value used to identify the LTK
			uint16_t Ediv;  // EDIV value used to identify the LTK
		} RandEdivInfo;  // Rand information (size and value) and Ediv
		bool_t AddressIncluded;  // Boolean value which indicates if Address is included or not
		struct {
			GAPConnectionEventKeysReceivedIndication_Keys_AddressInfo_DeviceAddressType_t DeviceAddressType;  // Public or Random address
			uint8_t DeviceAddress[6];  // Device Address
		} AddressInfo;  // Address information (type and value)
	} Keys;  // The SMP keys distributed by the peer
} GAPConnectionEventKeysReceivedIndication_t;

typedef struct GAPConnectionEventLongTermKeyRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	uint16_t Ediv;  // The Encryption Diversifier, as defined by the SMP
	uint8_t RandSize;  // Random number size
	uint8_t *Rand;  // The Random number, as defined by the SMP
} GAPConnectionEventLongTermKeyRequestIndication_t;

typedef PACKED_STRUCT GAPConnectionEventEncryptionChangedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	bool_t NewEncryptionState;  // TRUE if link has been encrypted, FALSE if encryption was paused or removed
} GAPConnectionEventEncryptionChangedIndication_t;

/* Pairing status */
typedef enum GAPConnectionEventPairingCompleteIndication_PairingStatus_tag {
	GAPConnectionEventPairingCompleteIndication_PairingStatus_PairingSuccessful = 0x01,
	GAPConnectionEventPairingCompleteIndication_PairingStatus_PairingFailed = 0x00
} GAPConnectionEventPairingCompleteIndication_PairingStatus_t;

/* The reason of failure */
typedef enum GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_tag {
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleSuccess_c = 0x0000,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleInvalidParameter_c = 0x0001,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleOverflow_c = 0x0002,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleUnavailable_c = 0x0003,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleFeatureNotSupported_c = 0x0004,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleOutOfMemory_c = 0x0005,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleAlreadyInitialized_c = 0x0006,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleOsError_c = 0x0007,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleUnexpectedError_c = 0x0008,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleInvalidState_c = 0x0009,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gBleTimerError_c = 0x000A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnknownHciCommand_c = 0x0101,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciHardwareFailure_c = 0x0103,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciPageTimeout_c = 0x0104,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciAuthenticationFailure_c = 0x0105,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciPinOrKeyMissing_c = 0x0106,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionTimeout_c = 0x0108,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionLimitExceeded_c = 0x0109,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciCommandDisallowed_c = 0x010C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRepeatedAttempts_c = 0x0117,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciPairingNotAllowed_c = 0x0118,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnknownLpmPdu_c = 0x0119,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciScoOffsetRejected_c = 0x011B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciScoIntervalRejected_c = 0x011C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciScoAirModeRejected_c = 0x011D,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciInvalidLpmParameters_c = 0x011E,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnspecifiedError_c = 0x011F,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciLLResponseTimeout_c = 0x0122,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciLmpPduNotAllowed_c = 0x0124,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRequestedQosNotSupported_c = 0x0127,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciInstantPassed_c = 0x0128,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciDifferentTransactionCollision_c = 0x012A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciReserved_0x2B_c = 0x012B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciQosRejected_c = 0x012D,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciInsufficientSecurity_c = 0x012F,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciReserved_0x31_c = 0x0131,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRoleSwitchPending_c = 0x0132,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciReserved_0x33_c = 0x0133,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciReservedSlotViolation_c = 0x0134,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciRoleSwitchFailed_c = 0x0135,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciHostBusyPairing_c = 0x0138,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciControllerBusy_c = 0x013A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciMacConnectionFailed_c = 0x013F,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciType0SubmapNotDefined_c = 0x0141,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciLimitReached_c = 0x0143,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciOperationCancelledByHost_c = 0x0144,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciAlreadyInit_c = 0x01A0,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciInvalidParameter_c = 0x01A1,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciCallbackNotInstalled_c = 0x01A2,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciCommandNotSupported_c = 0x01A4,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciEventNotSupported_c = 0x01A5,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gHciTransportError_c = 0x01A6,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caAlreadyInit_c = 0x03F0,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caInsufficientResources_c = 0x03F1,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caLePsmInvalid_c = 0x03F4,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caChannelInvalid_c = 0x03F8,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caChannelClosed_c = 0x03F9,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caChannelBusy_c = 0x03FC,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caInvalidParameter_c = 0x03FE,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gL2caError_c = 0x03FF,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmNullCBFunction_c = 0x0401,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmCommandNotSupported_c = 0x0402,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnexpectedCommand_c = 0x0403,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidCommandCode_c = 0x0404,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidCommandLength_c = 0x0405,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidCommandParameter_c = 0x0406,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidDeviceId_c = 0x0407,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidInternalOperation_c = 0x0408,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidConnectionHandle_c = 0x0409,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInproperKeyDistributionField_c = 0x040A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnexpectedKeyType_c = 0x040B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnexpectedKeyset_c = 0x040D,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnknownSmpPacketType_c = 0x040F,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInvalidHciEventParameter_c = 0x0416,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmInsufficientResources_c = 0x0419,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmOobDataAddressMismatch_c = 0x041A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorTimeout_c = 0x0436,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingAlreadyStarted_c = 0x0438,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gSmTbInvalidDataSignature_c = 0x0461,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInvalidHandle_c = 0x0501,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttReadNotPermitted_c = 0x0502,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttWriteNotPermitted_c = 0x0503,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInvalidPdu_c = 0x0504,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInsufficientAuthentication_c = 0x0505,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttRequestNotSupported_c = 0x0506,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInvalidOffset_c = 0x0507,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInsufficientAuthorization_c = 0x0508,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttPrepareQueueFull_c = 0x0509,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttAttributeNotFound_c = 0x050A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttAttributeNotLong_c = 0x050B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttUnlikelyor_c = 0x050E,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInsufficientEncryption_c = 0x050F,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttUnsupportedGroupType_c = 0x0510,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttInsufficientResources_c = 0x0511,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gAttDatabaseOutOfSync_c = 0x0512,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattInvalidValueLength_c = 0x0605,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattServerTimeout_c = 0x0606,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattClientConfirmationTimeout_c = 0x0608,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattInvalidPduReceived_c = 0x0609,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattPeerDisconnected_c = 0x060A,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattMtuExchangeInProgress_c = 0x060B,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGapAdvDataTooLong_c = 0x0701,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGapScanRspDataTooLong_c = 0x0702,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGapDeviceNotBonded_c = 0x0703,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gDevDbCccdLimitReached_c = 0x0801,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gDevDbCccdNotFound_c = 0x0802,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattDbInvalidHandle_c = 0x0901,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattDbCccdNotFound_c = 0x0903,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattDbServiceNotFound_c = 0x0904,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattDbDescriptorNotFound_c = 0x0905,
	GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_t;

typedef struct GAPConnectionEventPairingCompleteIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	GAPConnectionEventPairingCompleteIndication_PairingStatus_t PairingStatus;  // Pairing status
	union {
		bool_t PairingSuccessful_WithBonding;

		GAPConnectionEventPairingCompleteIndication_PairingData_PairingFailed_FailReason_t PairingFailed_FailReason;

	} PairingData;  // Information of pairing completion
} GAPConnectionEventPairingCompleteIndication_t;

/* Reason for disconnection */
typedef enum GAPConnectionEventDisconnectedIndication_Reason_tag {
	GAPConnectionEventDisconnectedIndication_Reason_gBleSuccess_c = 0x0000,
	GAPConnectionEventDisconnectedIndication_Reason_gBleInvalidParameter_c = 0x0001,
	GAPConnectionEventDisconnectedIndication_Reason_gBleOverflow_c = 0x0002,
	GAPConnectionEventDisconnectedIndication_Reason_gBleUnavailable_c = 0x0003,
	GAPConnectionEventDisconnectedIndication_Reason_gBleFeatureNotSupported_c = 0x0004,
	GAPConnectionEventDisconnectedIndication_Reason_gBleOutOfMemory_c = 0x0005,
	GAPConnectionEventDisconnectedIndication_Reason_gBleAlreadyInitialized_c = 0x0006,
	GAPConnectionEventDisconnectedIndication_Reason_gBleOsError_c = 0x0007,
	GAPConnectionEventDisconnectedIndication_Reason_gBleUnexpectedError_c = 0x0008,
	GAPConnectionEventDisconnectedIndication_Reason_gBleInvalidState_c = 0x0009,
	GAPConnectionEventDisconnectedIndication_Reason_gBleTimerError_c = 0x000A,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnknownHciCommand_c = 0x0101,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPConnectionEventDisconnectedIndication_Reason_gHciHardwareFailure_c = 0x0103,
	GAPConnectionEventDisconnectedIndication_Reason_gHciPageTimeout_c = 0x0104,
	GAPConnectionEventDisconnectedIndication_Reason_gHciAuthenticationFailure_c = 0x0105,
	GAPConnectionEventDisconnectedIndication_Reason_gHciPinOrKeyMissing_c = 0x0106,
	GAPConnectionEventDisconnectedIndication_Reason_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionTimeout_c = 0x0108,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionLimitExceeded_c = 0x0109,
	GAPConnectionEventDisconnectedIndication_Reason_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPConnectionEventDisconnectedIndication_Reason_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPConnectionEventDisconnectedIndication_Reason_gHciCommandDisallowed_c = 0x010C,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPConnectionEventDisconnectedIndication_Reason_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRepeatedAttempts_c = 0x0117,
	GAPConnectionEventDisconnectedIndication_Reason_gHciPairingNotAllowed_c = 0x0118,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnknownLpmPdu_c = 0x0119,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPConnectionEventDisconnectedIndication_Reason_gHciScoOffsetRejected_c = 0x011B,
	GAPConnectionEventDisconnectedIndication_Reason_gHciScoIntervalRejected_c = 0x011C,
	GAPConnectionEventDisconnectedIndication_Reason_gHciScoAirModeRejected_c = 0x011D,
	GAPConnectionEventDisconnectedIndication_Reason_gHciInvalidLpmParameters_c = 0x011E,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnspecifiedError_c = 0x011F,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPConnectionEventDisconnectedIndication_Reason_gHciLLResponseTimeout_c = 0x0122,
	GAPConnectionEventDisconnectedIndication_Reason_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPConnectionEventDisconnectedIndication_Reason_gHciLmpPduNotAllowed_c = 0x0124,
	GAPConnectionEventDisconnectedIndication_Reason_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPConnectionEventDisconnectedIndication_Reason_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRequestedQosNotSupported_c = 0x0127,
	GAPConnectionEventDisconnectedIndication_Reason_gHciInstantPassed_c = 0x0128,
	GAPConnectionEventDisconnectedIndication_Reason_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPConnectionEventDisconnectedIndication_Reason_gHciDifferentTransactionCollision_c = 0x012A,
	GAPConnectionEventDisconnectedIndication_Reason_gHciReserved_0x2B_c = 0x012B,
	GAPConnectionEventDisconnectedIndication_Reason_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPConnectionEventDisconnectedIndication_Reason_gHciQosRejected_c = 0x012D,
	GAPConnectionEventDisconnectedIndication_Reason_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPConnectionEventDisconnectedIndication_Reason_gHciInsufficientSecurity_c = 0x012F,
	GAPConnectionEventDisconnectedIndication_Reason_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPConnectionEventDisconnectedIndication_Reason_gHciReserved_0x31_c = 0x0131,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRoleSwitchPending_c = 0x0132,
	GAPConnectionEventDisconnectedIndication_Reason_gHciReserved_0x33_c = 0x0133,
	GAPConnectionEventDisconnectedIndication_Reason_gHciReservedSlotViolation_c = 0x0134,
	GAPConnectionEventDisconnectedIndication_Reason_gHciRoleSwitchFailed_c = 0x0135,
	GAPConnectionEventDisconnectedIndication_Reason_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPConnectionEventDisconnectedIndication_Reason_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPConnectionEventDisconnectedIndication_Reason_gHciHostBusyPairing_c = 0x0138,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPConnectionEventDisconnectedIndication_Reason_gHciControllerBusy_c = 0x013A,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPConnectionEventDisconnectedIndication_Reason_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPConnectionEventDisconnectedIndication_Reason_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPConnectionEventDisconnectedIndication_Reason_gHciMacConnectionFailed_c = 0x013F,
	GAPConnectionEventDisconnectedIndication_Reason_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPConnectionEventDisconnectedIndication_Reason_gHciType0SubmapNotDefined_c = 0x0141,
	GAPConnectionEventDisconnectedIndication_Reason_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPConnectionEventDisconnectedIndication_Reason_gHciLimitReached_c = 0x0143,
	GAPConnectionEventDisconnectedIndication_Reason_gHciOperationCancelledByHost_c = 0x0144,
	GAPConnectionEventDisconnectedIndication_Reason_gHciAlreadyInit_c = 0x01A0,
	GAPConnectionEventDisconnectedIndication_Reason_gHciInvalidParameter_c = 0x01A1,
	GAPConnectionEventDisconnectedIndication_Reason_gHciCallbackNotInstalled_c = 0x01A2,
	GAPConnectionEventDisconnectedIndication_Reason_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPConnectionEventDisconnectedIndication_Reason_gHciCommandNotSupported_c = 0x01A4,
	GAPConnectionEventDisconnectedIndication_Reason_gHciEventNotSupported_c = 0x01A5,
	GAPConnectionEventDisconnectedIndication_Reason_gHciTransportError_c = 0x01A6,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caAlreadyInit_c = 0x03F0,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caInsufficientResources_c = 0x03F1,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caLePsmInvalid_c = 0x03F4,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caChannelInvalid_c = 0x03F8,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caChannelClosed_c = 0x03F9,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caChannelBusy_c = 0x03FC,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caInvalidParameter_c = 0x03FE,
	GAPConnectionEventDisconnectedIndication_Reason_gL2caError_c = 0x03FF,
	GAPConnectionEventDisconnectedIndication_Reason_gSmNullCBFunction_c = 0x0401,
	GAPConnectionEventDisconnectedIndication_Reason_gSmCommandNotSupported_c = 0x0402,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnexpectedCommand_c = 0x0403,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidCommandCode_c = 0x0404,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidCommandLength_c = 0x0405,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidCommandParameter_c = 0x0406,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidDeviceId_c = 0x0407,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidInternalOperation_c = 0x0408,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidConnectionHandle_c = 0x0409,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInproperKeyDistributionField_c = 0x040A,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnexpectedKeyType_c = 0x040B,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnexpectedKeyset_c = 0x040D,
	GAPConnectionEventDisconnectedIndication_Reason_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnknownSmpPacketType_c = 0x040F,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPConnectionEventDisconnectedIndication_Reason_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPConnectionEventDisconnectedIndication_Reason_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPConnectionEventDisconnectedIndication_Reason_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPConnectionEventDisconnectedIndication_Reason_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInvalidHciEventParameter_c = 0x0416,
	GAPConnectionEventDisconnectedIndication_Reason_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPConnectionEventDisconnectedIndication_Reason_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPConnectionEventDisconnectedIndication_Reason_gSmInsufficientResources_c = 0x0419,
	GAPConnectionEventDisconnectedIndication_Reason_gSmOobDataAddressMismatch_c = 0x041A,
	GAPConnectionEventDisconnectedIndication_Reason_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPConnectionEventDisconnectedIndication_Reason_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPConnectionEventDisconnectedIndication_Reason_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorTimeout_c = 0x0436,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingAlreadyStarted_c = 0x0438,
	GAPConnectionEventDisconnectedIndication_Reason_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPConnectionEventDisconnectedIndication_Reason_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPConnectionEventDisconnectedIndication_Reason_gSmTbInvalidDataSignature_c = 0x0461,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInvalidHandle_c = 0x0501,
	GAPConnectionEventDisconnectedIndication_Reason_gAttReadNotPermitted_c = 0x0502,
	GAPConnectionEventDisconnectedIndication_Reason_gAttWriteNotPermitted_c = 0x0503,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInvalidPdu_c = 0x0504,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInsufficientAuthentication_c = 0x0505,
	GAPConnectionEventDisconnectedIndication_Reason_gAttRequestNotSupported_c = 0x0506,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInvalidOffset_c = 0x0507,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInsufficientAuthorization_c = 0x0508,
	GAPConnectionEventDisconnectedIndication_Reason_gAttPrepareQueueFull_c = 0x0509,
	GAPConnectionEventDisconnectedIndication_Reason_gAttAttributeNotFound_c = 0x050A,
	GAPConnectionEventDisconnectedIndication_Reason_gAttAttributeNotLong_c = 0x050B,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPConnectionEventDisconnectedIndication_Reason_gAttUnlikelyor_c = 0x050E,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInsufficientEncryption_c = 0x050F,
	GAPConnectionEventDisconnectedIndication_Reason_gAttUnsupportedGroupType_c = 0x0510,
	GAPConnectionEventDisconnectedIndication_Reason_gAttInsufficientResources_c = 0x0511,
	GAPConnectionEventDisconnectedIndication_Reason_gAttDatabaseOutOfSync_c = 0x0512,
	GAPConnectionEventDisconnectedIndication_Reason_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPConnectionEventDisconnectedIndication_Reason_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPConnectionEventDisconnectedIndication_Reason_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPConnectionEventDisconnectedIndication_Reason_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPConnectionEventDisconnectedIndication_Reason_gGattInvalidValueLength_c = 0x0605,
	GAPConnectionEventDisconnectedIndication_Reason_gGattServerTimeout_c = 0x0606,
	GAPConnectionEventDisconnectedIndication_Reason_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPConnectionEventDisconnectedIndication_Reason_gGattClientConfirmationTimeout_c = 0x0608,
	GAPConnectionEventDisconnectedIndication_Reason_gGattInvalidPduReceived_c = 0x0609,
	GAPConnectionEventDisconnectedIndication_Reason_gGattPeerDisconnected_c = 0x060A,
	GAPConnectionEventDisconnectedIndication_Reason_gGattMtuExchangeInProgress_c = 0x060B,
	GAPConnectionEventDisconnectedIndication_Reason_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPConnectionEventDisconnectedIndication_Reason_gGapAdvDataTooLong_c = 0x0701,
	GAPConnectionEventDisconnectedIndication_Reason_gGapScanRspDataTooLong_c = 0x0702,
	GAPConnectionEventDisconnectedIndication_Reason_gGapDeviceNotBonded_c = 0x0703,
	GAPConnectionEventDisconnectedIndication_Reason_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPConnectionEventDisconnectedIndication_Reason_gDevDbCccdLimitReached_c = 0x0801,
	GAPConnectionEventDisconnectedIndication_Reason_gDevDbCccdNotFound_c = 0x0802,
	GAPConnectionEventDisconnectedIndication_Reason_gGattDbInvalidHandle_c = 0x0901,
	GAPConnectionEventDisconnectedIndication_Reason_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPConnectionEventDisconnectedIndication_Reason_gGattDbCccdNotFound_c = 0x0903,
	GAPConnectionEventDisconnectedIndication_Reason_gGattDbServiceNotFound_c = 0x0904,
	GAPConnectionEventDisconnectedIndication_Reason_gGattDbDescriptorNotFound_c = 0x0905,
	GAPConnectionEventDisconnectedIndication_Reason_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPConnectionEventDisconnectedIndication_Reason_t;

typedef PACKED_STRUCT GAPConnectionEventDisconnectedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	GAPConnectionEventDisconnectedIndication_Reason_t Reason;  // Reason for disconnection
} GAPConnectionEventDisconnectedIndication_t;

typedef PACKED_STRUCT GAPConnectionEventRssiReadIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	uint8_t Rssi;  // Value of the RSSI
} GAPConnectionEventRssiReadIndication_t;

typedef PACKED_STRUCT GAPConnectionEventTxPowerLevelReadIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	uint8_t TxPowerLevel;  // Value of the TX power
} GAPConnectionEventTxPowerLevelReadIndication_t;

/* Reason for power reading failure */
typedef enum GAPConnectionEventPowerReadFailureIndication_FailReason_tag {
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleSuccess_c = 0x0000,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleInvalidParameter_c = 0x0001,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleOverflow_c = 0x0002,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleUnavailable_c = 0x0003,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleFeatureNotSupported_c = 0x0004,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleOutOfMemory_c = 0x0005,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleAlreadyInitialized_c = 0x0006,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleOsError_c = 0x0007,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleUnexpectedError_c = 0x0008,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleInvalidState_c = 0x0009,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gBleTimerError_c = 0x000A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnknownHciCommand_c = 0x0101,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciHardwareFailure_c = 0x0103,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciPageTimeout_c = 0x0104,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciAuthenticationFailure_c = 0x0105,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciPinOrKeyMissing_c = 0x0106,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionTimeout_c = 0x0108,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionLimitExceeded_c = 0x0109,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciCommandDisallowed_c = 0x010C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRepeatedAttempts_c = 0x0117,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciPairingNotAllowed_c = 0x0118,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnknownLpmPdu_c = 0x0119,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciScoOffsetRejected_c = 0x011B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciScoIntervalRejected_c = 0x011C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciScoAirModeRejected_c = 0x011D,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciInvalidLpmParameters_c = 0x011E,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnspecifiedError_c = 0x011F,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciLLResponseTimeout_c = 0x0122,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciLmpPduNotAllowed_c = 0x0124,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRequestedQosNotSupported_c = 0x0127,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciInstantPassed_c = 0x0128,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciDifferentTransactionCollision_c = 0x012A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciReserved_0x2B_c = 0x012B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciQosRejected_c = 0x012D,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciInsufficientSecurity_c = 0x012F,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciReserved_0x31_c = 0x0131,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRoleSwitchPending_c = 0x0132,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciReserved_0x33_c = 0x0133,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciReservedSlotViolation_c = 0x0134,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciRoleSwitchFailed_c = 0x0135,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciHostBusyPairing_c = 0x0138,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciControllerBusy_c = 0x013A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciMacConnectionFailed_c = 0x013F,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciType0SubmapNotDefined_c = 0x0141,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciLimitReached_c = 0x0143,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciOperationCancelledByHost_c = 0x0144,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciAlreadyInit_c = 0x01A0,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciInvalidParameter_c = 0x01A1,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciCallbackNotInstalled_c = 0x01A2,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciCommandNotSupported_c = 0x01A4,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciEventNotSupported_c = 0x01A5,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gHciTransportError_c = 0x01A6,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caAlreadyInit_c = 0x03F0,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caInsufficientResources_c = 0x03F1,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caLePsmInvalid_c = 0x03F4,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caChannelInvalid_c = 0x03F8,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caChannelClosed_c = 0x03F9,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caChannelBusy_c = 0x03FC,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caInvalidParameter_c = 0x03FE,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gL2caError_c = 0x03FF,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmNullCBFunction_c = 0x0401,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmCommandNotSupported_c = 0x0402,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnexpectedCommand_c = 0x0403,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidCommandCode_c = 0x0404,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidCommandLength_c = 0x0405,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidCommandParameter_c = 0x0406,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidDeviceId_c = 0x0407,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidInternalOperation_c = 0x0408,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidConnectionHandle_c = 0x0409,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInproperKeyDistributionField_c = 0x040A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnexpectedKeyType_c = 0x040B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnexpectedKeyset_c = 0x040D,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnknownSmpPacketType_c = 0x040F,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInvalidHciEventParameter_c = 0x0416,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmInsufficientResources_c = 0x0419,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmOobDataAddressMismatch_c = 0x041A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorTimeout_c = 0x0436,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingAlreadyStarted_c = 0x0438,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gSmTbInvalidDataSignature_c = 0x0461,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInvalidHandle_c = 0x0501,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttReadNotPermitted_c = 0x0502,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttWriteNotPermitted_c = 0x0503,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInvalidPdu_c = 0x0504,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInsufficientAuthentication_c = 0x0505,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttRequestNotSupported_c = 0x0506,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInvalidOffset_c = 0x0507,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInsufficientAuthorization_c = 0x0508,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttPrepareQueueFull_c = 0x0509,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttAttributeNotFound_c = 0x050A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttAttributeNotLong_c = 0x050B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttUnlikelyor_c = 0x050E,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInsufficientEncryption_c = 0x050F,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttUnsupportedGroupType_c = 0x0510,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttInsufficientResources_c = 0x0511,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gAttDatabaseOutOfSync_c = 0x0512,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattInvalidValueLength_c = 0x0605,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattServerTimeout_c = 0x0606,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattClientConfirmationTimeout_c = 0x0608,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattInvalidPduReceived_c = 0x0609,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattPeerDisconnected_c = 0x060A,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattMtuExchangeInProgress_c = 0x060B,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGapAdvDataTooLong_c = 0x0701,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGapScanRspDataTooLong_c = 0x0702,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGapDeviceNotBonded_c = 0x0703,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gDevDbCccdLimitReached_c = 0x0801,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gDevDbCccdNotFound_c = 0x0802,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattDbInvalidHandle_c = 0x0901,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattDbCccdNotFound_c = 0x0903,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattDbServiceNotFound_c = 0x0904,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattDbDescriptorNotFound_c = 0x0905,
	GAPConnectionEventPowerReadFailureIndication_FailReason_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPConnectionEventPowerReadFailureIndication_FailReason_t;

typedef PACKED_STRUCT GAPConnectionEventPowerReadFailureIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	GAPConnectionEventPowerReadFailureIndication_FailReason_t FailReason;  // Reason for power reading failure
} GAPConnectionEventPowerReadFailureIndication_t;

typedef PACKED_STRUCT GAPConnectionEventParameterUpdateRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		uint16_t IntervalMin;  // Minimum interval
		uint16_t IntervalMax;  // Maximum interval
		uint16_t PeripheralLatency;  // Peripheral latency
		uint16_t TimeoutMultiplier;  // Timeout multiplier
	} ConnectionParameterUpdateRequest;  // Connection parameter update request event data
} GAPConnectionEventParameterUpdateRequestIndication_t;

/* The status of the request */
typedef enum GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_tag {
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleSuccess_c = 0x0000,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleInvalidParameter_c = 0x0001,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleOverflow_c = 0x0002,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleUnavailable_c = 0x0003,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleOutOfMemory_c = 0x0005,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleOsError_c = 0x0007,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleUnexpectedError_c = 0x0008,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleInvalidState_c = 0x0009,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gBleTimerError_c = 0x000A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciHardwareFailure_c = 0x0103,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciPageTimeout_c = 0x0104,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionTimeout_c = 0x0108,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciCommandDisallowed_c = 0x010C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnspecifiedError_c = 0x011F,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciInstantPassed_c = 0x0128,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciReserved_0x2B_c = 0x012B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciQosRejected_c = 0x012D,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciReserved_0x31_c = 0x0131,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciReserved_0x33_c = 0x0133,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciHostBusyPairing_c = 0x0138,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciControllerBusy_c = 0x013A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciLimitReached_c = 0x0143,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciAlreadyInit_c = 0x01A0,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciInvalidParameter_c = 0x01A1,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciEventNotSupported_c = 0x01A5,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gHciTransportError_c = 0x01A6,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caChannelClosed_c = 0x03F9,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caChannelBusy_c = 0x03FC,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gL2caError_c = 0x03FF,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmNullCBFunction_c = 0x0401,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmCommandNotSupported_c = 0x0402,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmInsufficientResources_c = 0x0419,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInvalidHandle_c = 0x0501,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttReadNotPermitted_c = 0x0502,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInvalidPdu_c = 0x0504,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttRequestNotSupported_c = 0x0506,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInvalidOffset_c = 0x0507,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttAttributeNotFound_c = 0x050A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttAttributeNotLong_c = 0x050B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttUnlikelyor_c = 0x050E,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttInsufficientResources_c = 0x0511,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattInvalidValueLength_c = 0x0605,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattServerTimeout_c = 0x0606,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattPeerDisconnected_c = 0x060A,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_t;

typedef PACKED_STRUCT GAPConnectionEventParameterUpdateCompleteIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		GAPConnectionEventParameterUpdateCompleteIndication_ConnectionParameterUpdateComplete_Status_t Status;  // The status of the request
		uint16_t ConnInterval;  // Connection interval
		uint16_t ConnLatency;  // Connection latency
		uint16_t SupervisionTimeout;  // Supervision timeout
	} ConnectionParameterUpdateComplete;  // Connection parameter update complete event data
} GAPConnectionEventParameterUpdateCompleteIndication_t;

typedef PACKED_STRUCT GAPConnectionEventLeDataLengthChangedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	struct {
		uint16_t MaxTxOctets;  // The maximum number of payload octets in a Link Layer Data Channel PDU to transmit on this connection
		uint16_t MaxTxTime;  // The maximum time that the local Controller will take to send a Link Layer Data Channel PDU on this connection
		uint16_t MaxRxOctets;  // The maximum number of payload octets in a Link Layer Data Channel PDU to receive on this connection
		uint16_t MaxRxTime;  // The maximum time that the local Controller will take to receive a Link Layer Data Channel PDU on this connection
	} LeDataLengthChanged;  // Connection Le Data Length Changed Event data
} GAPConnectionEventLeDataLengthChangedIndication_t;

typedef PACKED_STRUCT GAPConnectionEventLeScOobDataRequestIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
} GAPConnectionEventLeScOobDataRequestIndication_t;

typedef PACKED_STRUCT GAPConnectionEventLeScDisplayNumericValueIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	uint32_t ncValue;  // A number between 0 and 999999 decimal which must be confirmed by the user on both pairing devices
} GAPConnectionEventLeScDisplayNumericValueIndication_t;

/* keypressNotifType */
typedef enum GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_tag {
	GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyEntryStarted_c = 0x00,
	GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyDigitStarted_c = 0x01,
	GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyDigitErased_c = 0x02,
	GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyCleared_c = 0x03,
	GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_gKnPasskeyEntryCompleted_c = 0x04
} GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_t;

typedef PACKED_STRUCT GAPConnectionEventLeScKeypressNotificationIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	GAPConnectionEventLeScKeypressNotificationIndication_GapLeScKeypressNotificationParams_keypressNotifType_t GapLeScKeypressNotificationParams_keypressNotifType;

} GAPConnectionEventLeScKeypressNotificationIndication_t;

typedef PACKED_STRUCT GAPLeScPublicKeyRegeneratedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPLeScPublicKeyRegeneratedIndication_t;

typedef PACKED_STRUCT GAPGenericEventLeScLocalOobDataIndication_tag {
	uint8_t RandomValue[16];  // LE SC OOB r (Random value)
	uint8_t ConfirmValue[16];  // LE SC OOB Cr (Random Confirm value)
} GAPGenericEventLeScLocalOobDataIndication_t;

typedef PACKED_STRUCT GAPGenericEventHostPrivacyStateChangedIndication_tag {
	bool_t NewHostPrivacyState;  // Indicates the new state of the host privacy: TRUE if enabled, FALSE if disabled
} GAPGenericEventHostPrivacyStateChangedIndication_t;

typedef PACKED_STRUCT GAPGenericEventControllerPrivacyStateChangedIndication_tag {
	bool_t NewControllerPrivacyState;  // Indicates the new state of the controller privacy: TRUE if enabled, FALSE if disabled
} GAPGenericEventControllerPrivacyStateChangedIndication_t;

typedef PACKED_STRUCT GAPGenericEventTxPowerLevelSetCompleteIndication_tag {
	uint8_t status;  // Indicates the status of the Set Tx Power Level request
} GAPGenericEventTxPowerLevelSetCompleteIndication_t;

/* Specifies the Phy event type */
typedef enum GAPGenericEventLePhyEventIndication_eventType_tag {
	GAPGenericEventLePhyEventIndication_eventType_gPhySetDefaultComplete_c = 0x00,
	GAPGenericEventLePhyEventIndication_eventType_gPhyRead_c = 0x01,
	GAPGenericEventLePhyEventIndication_eventType_gPhyUpdateComplete_c = 0x02
} GAPGenericEventLePhyEventIndication_eventType_t;

/* Specifies the Tx Phy, valid only for gPhyRead_c or gPhyUpdateComplete_c event type. */
typedef enum GAPGenericEventLePhyEventIndication_txPhy_tag {
	GAPGenericEventLePhyEventIndication_txPhy_gLeTxPhy1M_c = 0x01,
	GAPGenericEventLePhyEventIndication_txPhy_gLeTxPhy2M_c = 0x02,
	GAPGenericEventLePhyEventIndication_txPhy_gLeTxPhyCoded_c = 0x03
} GAPGenericEventLePhyEventIndication_txPhy_t;

/* Specifies the Rx Phy, valid only for gPhyRead_c or gPhyUpdateComplete_c event type. */
typedef enum GAPGenericEventLePhyEventIndication_rxPhy_tag {
	GAPGenericEventLePhyEventIndication_rxPhy_gLeRxPhy1M_c = 0x01,
	GAPGenericEventLePhyEventIndication_rxPhy_gLeRxPhy2M_c = 0x02,
	GAPGenericEventLePhyEventIndication_rxPhy_gLeRxPhyCoded_c = 0x03
} GAPGenericEventLePhyEventIndication_rxPhy_t;

typedef PACKED_STRUCT GAPGenericEventLePhyEventIndication_tag {
	GAPGenericEventLePhyEventIndication_eventType_t eventType;  // Specifies the Phy event type
	uint8_t DeviceId;  // Device ID identifying the connection
	GAPGenericEventLePhyEventIndication_txPhy_t txPhy;  // Specifies the Tx Phy, valid only for gPhyRead_c or gPhyUpdateComplete_c event type.
	GAPGenericEventLePhyEventIndication_rxPhy_t rxPhy;  // Specifies the Rx Phy, valid only for gPhyRead_c or gPhyUpdateComplete_c event type.
} GAPGenericEventLePhyEventIndication_t;

/* Identity Address Type */
typedef enum GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_IdentityAddressType_tag {
	GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_IdentityAddressType_gPublic_c = 0x00,
	GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_IdentityAddressType_gRandom_c = 0x01
} GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_IdentityAddressType_t;

/* Privacy Mode */
typedef enum GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_PrivacyMode_tag {
	GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_PrivacyMode_gNetworkPrivacy_c = 0x00,
	GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_PrivacyMode_gDevicePrivacy_c = 0x01
} GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_PrivacyMode_t;

typedef struct GAPGetBondedDevicesIdentityInformationIndication_tag {
	uint8_t NbOfDeviceIdentityAddresses;  // The number of device identity addresses
	struct {
		GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_IdentityAddressType_t IdentityAddressType;  // Identity Address Type
		uint8_t IdentityAddress[6];  // Identity Address
		uint8_t Irk[16];  // Identity Resolving Key
		GAPGetBondedDevicesIdentityInformationIndication_IdentityAddresses_PrivacyMode_t PrivacyMode;  // Privacy Mode
	} *IdentityAddresses;  // Identity addresses list
} GAPGetBondedDevicesIdentityInformationIndication_t;

/* Event type */
typedef enum GAPControllerNotificationIndication_EventType_tag {
	GAPControllerNotificationIndication_EventType_gNotifConnEventOver_c = 0x01,
	GAPControllerNotificationIndication_EventType_gNotifConnRxPdu_c = 0x02,
	GAPControllerNotificationIndication_EventType_gNotifAdvEventOver_c = 0x04,
	GAPControllerNotificationIndication_EventType_gNotifAdvTx_c = 0x08,
	GAPControllerNotificationIndication_EventType_gNotifAdvScanReqRx_c = 0x10,
	GAPControllerNotificationIndication_EventType_gNotifAdvConnReqRx_c = 0x20,
	GAPControllerNotificationIndication_EventType_gNotifScanEventOver_c = 0x40,
	GAPControllerNotificationIndication_EventType_gNotifScanAdvPktRx_c = 0x80,
	GAPControllerNotificationIndication_EventType_gNotifScanRspRx_c = 0x0100,
	GAPControllerNotificationIndication_EventType_gNotifScanReqTx_c = 0x0200,
	GAPControllerNotificationIndication_EventType_gNotifConnCreated_c = 0x0400,
	GAPControllerNotificationIndication_EventType_gNotifChannelMatrix_c = 0x800,
	GAPControllerNotificationIndication_EventType_gNotifPhyReq_c = 0x1000,
	GAPControllerNotificationIndication_EventType_gNotifConnChannelMapUpdate_c = 0x2000,
	GAPControllerNotificationIndication_EventType_gNotifConnInd_c = 0x4000,
	GAPControllerNotificationIndication_EventType_gNotifPhyUpdateInd_c = 0x8000
} GAPControllerNotificationIndication_EventType_t;

/* Status of the request to select which events to be enabled/disabled */
typedef enum GAPControllerNotificationIndication_Status_tag {
	GAPControllerNotificationIndication_Status_gBleSuccess_c = 0x0000,
	GAPControllerNotificationIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPControllerNotificationIndication_Status_gBleOverflow_c = 0x0002,
	GAPControllerNotificationIndication_Status_gBleUnavailable_c = 0x0003,
	GAPControllerNotificationIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPControllerNotificationIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPControllerNotificationIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPControllerNotificationIndication_Status_gBleOsError_c = 0x0007,
	GAPControllerNotificationIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPControllerNotificationIndication_Status_gBleInvalidState_c = 0x0009,
	GAPControllerNotificationIndication_Status_gBleTimerError_c = 0x000A,
	GAPControllerNotificationIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPControllerNotificationIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPControllerNotificationIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPControllerNotificationIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPControllerNotificationIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPControllerNotificationIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPControllerNotificationIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPControllerNotificationIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPControllerNotificationIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPControllerNotificationIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPControllerNotificationIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPControllerNotificationIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPControllerNotificationIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPControllerNotificationIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPControllerNotificationIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPControllerNotificationIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPControllerNotificationIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPControllerNotificationIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPControllerNotificationIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPControllerNotificationIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPControllerNotificationIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPControllerNotificationIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPControllerNotificationIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPControllerNotificationIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPControllerNotificationIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPControllerNotificationIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPControllerNotificationIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPControllerNotificationIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPControllerNotificationIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPControllerNotificationIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPControllerNotificationIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPControllerNotificationIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPControllerNotificationIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPControllerNotificationIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPControllerNotificationIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPControllerNotificationIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPControllerNotificationIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPControllerNotificationIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPControllerNotificationIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPControllerNotificationIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPControllerNotificationIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPControllerNotificationIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPControllerNotificationIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPControllerNotificationIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPControllerNotificationIndication_Status_gHciQosRejected_c = 0x012D,
	GAPControllerNotificationIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPControllerNotificationIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPControllerNotificationIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPControllerNotificationIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPControllerNotificationIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPControllerNotificationIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPControllerNotificationIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPControllerNotificationIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPControllerNotificationIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPControllerNotificationIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPControllerNotificationIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPControllerNotificationIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPControllerNotificationIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPControllerNotificationIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPControllerNotificationIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPControllerNotificationIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPControllerNotificationIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPControllerNotificationIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPControllerNotificationIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPControllerNotificationIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPControllerNotificationIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPControllerNotificationIndication_Status_gHciLimitReached_c = 0x0143,
	GAPControllerNotificationIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPControllerNotificationIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPControllerNotificationIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPControllerNotificationIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPControllerNotificationIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPControllerNotificationIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPControllerNotificationIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPControllerNotificationIndication_Status_gHciTransportError_c = 0x01A6,
	GAPControllerNotificationIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPControllerNotificationIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPControllerNotificationIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPControllerNotificationIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPControllerNotificationIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPControllerNotificationIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPControllerNotificationIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPControllerNotificationIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPControllerNotificationIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPControllerNotificationIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPControllerNotificationIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPControllerNotificationIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPControllerNotificationIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPControllerNotificationIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPControllerNotificationIndication_Status_gL2caError_c = 0x03FF,
	GAPControllerNotificationIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPControllerNotificationIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPControllerNotificationIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPControllerNotificationIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPControllerNotificationIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPControllerNotificationIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPControllerNotificationIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPControllerNotificationIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPControllerNotificationIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPControllerNotificationIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPControllerNotificationIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPControllerNotificationIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPControllerNotificationIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPControllerNotificationIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPControllerNotificationIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPControllerNotificationIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPControllerNotificationIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPControllerNotificationIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPControllerNotificationIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPControllerNotificationIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPControllerNotificationIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPControllerNotificationIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPControllerNotificationIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPControllerNotificationIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPControllerNotificationIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPControllerNotificationIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPControllerNotificationIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPControllerNotificationIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPControllerNotificationIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPControllerNotificationIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPControllerNotificationIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPControllerNotificationIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPControllerNotificationIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPControllerNotificationIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPControllerNotificationIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPControllerNotificationIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPControllerNotificationIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPControllerNotificationIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPControllerNotificationIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPControllerNotificationIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPControllerNotificationIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPControllerNotificationIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPControllerNotificationIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPControllerNotificationIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPControllerNotificationIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPControllerNotificationIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPControllerNotificationIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPControllerNotificationIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPControllerNotificationIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPControllerNotificationIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPControllerNotificationIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPControllerNotificationIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPControllerNotificationIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPControllerNotificationIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPControllerNotificationIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPControllerNotificationIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPControllerNotificationIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPControllerNotificationIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPControllerNotificationIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPControllerNotificationIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPControllerNotificationIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPControllerNotificationIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPControllerNotificationIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPControllerNotificationIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPControllerNotificationIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPControllerNotificationIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPControllerNotificationIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPControllerNotificationIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPControllerNotificationIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPControllerNotificationIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPControllerNotificationIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPControllerNotificationIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPControllerNotificationIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPControllerNotificationIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPControllerNotificationIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPControllerNotificationIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPControllerNotificationIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPControllerNotificationIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPControllerNotificationIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPControllerNotificationIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPControllerNotificationIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPControllerNotificationIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPControllerNotificationIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPControllerNotificationIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPControllerNotificationIndication_Status_t;

typedef PACKED_STRUCT GAPControllerNotificationIndication_tag {
	GAPControllerNotificationIndication_EventType_t EventType;  // Event type
	uint8_t DeviceId;  // Device ID identifying the connection
	uint8_t RSSI;  // RSSI, valid for Rx event types
	uint8_t Channel;  // Channel, valid for conn event over or Rx/Tx events
	uint16_t ConnEvCounter;  // Connection event counter, valid for conn event over or Conn Rx event
	GAPControllerNotificationIndication_Status_t Status;  // Status of the request to select which events to be enabled/disabled
	uint16_t Timestamp;  // Timestamp of the event, valid for conn created or Conn Rx event
	uint8_t AdvHandle;  // Advertising Handle, valid for advertising events from multiple sets
} GAPControllerNotificationIndication_t;

/* BLE Address Type */
typedef enum GAPBondCreatedIndication_AddrType_tag {
	GAPBondCreatedIndication_AddrType_gPublic_c = 0x00,
	GAPBondCreatedIndication_AddrType_gRandom_c = 0x01
} GAPBondCreatedIndication_AddrType_t;

typedef PACKED_STRUCT GAPBondCreatedIndication_tag {
	uint8_t NvmIndex;  // Index of the device in NVM bonding area
	GAPBondCreatedIndication_AddrType_t AddrType;  // BLE Address Type
	uint8_t Addr[6];  // BLE Address
} GAPBondCreatedIndication_t;

typedef PACKED_STRUCT GAPGenericEventChannelMapSetIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventChannelMapSetIndication_t;

typedef PACKED_STRUCT GAPConnectionEventChannelMapReadIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
	uint8_t ChannelMap[5];  // Channel Map
} GAPConnectionEventChannelMapReadIndication_t;

/* Reason for channel read failure */
typedef enum GAPConnectionEventChannelMapReadFailureIndication_FailReason_tag {
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleSuccess_c = 0x0000,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleInvalidParameter_c = 0x0001,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleOverflow_c = 0x0002,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleUnavailable_c = 0x0003,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleFeatureNotSupported_c = 0x0004,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleOutOfMemory_c = 0x0005,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleAlreadyInitialized_c = 0x0006,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleOsError_c = 0x0007,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleUnexpectedError_c = 0x0008,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleInvalidState_c = 0x0009,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gBleTimerError_c = 0x000A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnknownHciCommand_c = 0x0101,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciHardwareFailure_c = 0x0103,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciPageTimeout_c = 0x0104,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciAuthenticationFailure_c = 0x0105,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciPinOrKeyMissing_c = 0x0106,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionTimeout_c = 0x0108,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionLimitExceeded_c = 0x0109,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciCommandDisallowed_c = 0x010C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRepeatedAttempts_c = 0x0117,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciPairingNotAllowed_c = 0x0118,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnknownLpmPdu_c = 0x0119,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciScoOffsetRejected_c = 0x011B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciScoIntervalRejected_c = 0x011C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciScoAirModeRejected_c = 0x011D,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciInvalidLpmParameters_c = 0x011E,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnspecifiedError_c = 0x011F,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciLLResponseTimeout_c = 0x0122,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciLmpPduNotAllowed_c = 0x0124,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRequestedQosNotSupported_c = 0x0127,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciInstantPassed_c = 0x0128,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciDifferentTransactionCollision_c = 0x012A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciReserved_0x2B_c = 0x012B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciQosRejected_c = 0x012D,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciInsufficientSecurity_c = 0x012F,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciReserved_0x31_c = 0x0131,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRoleSwitchPending_c = 0x0132,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciReserved_0x33_c = 0x0133,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciReservedSlotViolation_c = 0x0134,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciRoleSwitchFailed_c = 0x0135,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciHostBusyPairing_c = 0x0138,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciControllerBusy_c = 0x013A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciMacConnectionFailed_c = 0x013F,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciType0SubmapNotDefined_c = 0x0141,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciLimitReached_c = 0x0143,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciOperationCancelledByHost_c = 0x0144,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciAlreadyInit_c = 0x01A0,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciInvalidParameter_c = 0x01A1,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciCallbackNotInstalled_c = 0x01A2,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciCommandNotSupported_c = 0x01A4,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciEventNotSupported_c = 0x01A5,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gHciTransportError_c = 0x01A6,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caAlreadyInit_c = 0x03F0,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caInsufficientResources_c = 0x03F1,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caLePsmInvalid_c = 0x03F4,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caChannelInvalid_c = 0x03F8,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caChannelClosed_c = 0x03F9,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caChannelBusy_c = 0x03FC,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caInvalidParameter_c = 0x03FE,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gL2caError_c = 0x03FF,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmNullCBFunction_c = 0x0401,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmCommandNotSupported_c = 0x0402,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnexpectedCommand_c = 0x0403,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidCommandCode_c = 0x0404,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidCommandLength_c = 0x0405,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidCommandParameter_c = 0x0406,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidDeviceId_c = 0x0407,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidInternalOperation_c = 0x0408,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidConnectionHandle_c = 0x0409,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInproperKeyDistributionField_c = 0x040A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnexpectedKeyType_c = 0x040B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnexpectedKeyset_c = 0x040D,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnknownSmpPacketType_c = 0x040F,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInvalidHciEventParameter_c = 0x0416,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmInsufficientResources_c = 0x0419,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmOobDataAddressMismatch_c = 0x041A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorTimeout_c = 0x0436,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingAlreadyStarted_c = 0x0438,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gSmTbInvalidDataSignature_c = 0x0461,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInvalidHandle_c = 0x0501,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttReadNotPermitted_c = 0x0502,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttWriteNotPermitted_c = 0x0503,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInvalidPdu_c = 0x0504,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInsufficientAuthentication_c = 0x0505,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttRequestNotSupported_c = 0x0506,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInvalidOffset_c = 0x0507,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInsufficientAuthorization_c = 0x0508,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttPrepareQueueFull_c = 0x0509,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttAttributeNotFound_c = 0x050A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttAttributeNotLong_c = 0x050B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttUnlikelyor_c = 0x050E,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInsufficientEncryption_c = 0x050F,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttUnsupportedGroupType_c = 0x0510,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttInsufficientResources_c = 0x0511,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gAttDatabaseOutOfSync_c = 0x0512,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattInvalidValueLength_c = 0x0605,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattServerTimeout_c = 0x0606,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattClientConfirmationTimeout_c = 0x0608,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattInvalidPduReceived_c = 0x0609,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattPeerDisconnected_c = 0x060A,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattMtuExchangeInProgress_c = 0x060B,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGapAdvDataTooLong_c = 0x0701,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGapScanRspDataTooLong_c = 0x0702,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGapDeviceNotBonded_c = 0x0703,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gDevDbCccdLimitReached_c = 0x0801,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gDevDbCccdNotFound_c = 0x0802,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattDbInvalidHandle_c = 0x0901,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattDbCccdNotFound_c = 0x0903,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattDbServiceNotFound_c = 0x0904,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattDbDescriptorNotFound_c = 0x0905,
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPConnectionEventChannelMapReadFailureIndication_FailReason_t;

typedef PACKED_STRUCT GAPConnectionEventChannelMapReadFailureIndication_tag {
	GAPConnectionEventChannelMapReadFailureIndication_FailReason_t FailReason;  // Reason for channel read failure
} GAPConnectionEventChannelMapReadFailureIndication_t;

typedef PACKED_STRUCT GAPGenericEventExtAdvertisingParamSetupCompleteIndication_tag {
	uint8_t SelectedTxPower;  // Indicates the transmit power selected by the Controller (in dBm)
} GAPGenericEventExtAdvertisingParamSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPGenericEventExtAdvertisingDataSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventExtAdvertisingDataSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvParamSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventPeriodicAdvParamSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvDataSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventPeriodicAdvDataSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvListUpdateCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventPeriodicAdvListUpdateCompleteIndication_t;

typedef PACKED_STRUCT GAPAdvertisingEventExtAdvertisingStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPAdvertisingEventExtAdvertisingStateChangedIndication_t;

/* The status of the request */
typedef enum GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_tag {
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleSuccess_c = 0x0000,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleOverflow_c = 0x0002,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleUnavailable_c = 0x0003,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleOsError_c = 0x0007,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleInvalidState_c = 0x0009,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gBleTimerError_c = 0x000A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciQosRejected_c = 0x012D,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciLimitReached_c = 0x0143,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gHciTransportError_c = 0x01A6,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gL2caError_c = 0x03FF,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_t;

typedef PACKED_STRUCT GAPAdvertisingEventAdvertisingSetTerminatedIndication_tag {
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_Status_t Status;  // The status of the request
	uint8_t Handle;  // Advertising set identifier
	uint8_t DeviceId;  // DeviceId if set terminated with a connection
	uint8_t NbExtAdvEvents;  // Number of extended advertising events the controller transmitted
} GAPAdvertisingEventAdvertisingSetTerminatedIndication_t;

typedef PACKED_STRUCT GAPAdvertisingEventExtAdvertisingSetRemoveCompletedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPAdvertisingEventExtAdvertisingSetRemoveCompletedIndication_t;

/* The type of the address of the scanner device */
typedef enum GAPAdvertisingEventExtScanReqReceivedIndication_ScannerAddressType_tag {
	GAPAdvertisingEventExtScanReqReceivedIndication_ScannerAddressType_gPublic_c = 0x00,
	GAPAdvertisingEventExtScanReqReceivedIndication_ScannerAddressType_gRandom_c = 0x01
} GAPAdvertisingEventExtScanReqReceivedIndication_ScannerAddressType_t;

typedef PACKED_STRUCT GAPAdvertisingEventExtScanReqReceivedIndication_tag {
	uint8_t Handle;  // Advertising set identifier
	GAPAdvertisingEventExtScanReqReceivedIndication_ScannerAddressType_t ScannerAddressType;  // The type of the address of the scanner device
	uint8_t ScannerAddress[6];  // The address of the scanner device
	bool_t ScannerAddressResolved;  // TRUE if the address contained in the ScannerAddressType and ScannerAddress fields corresponds to a Resolved Private Address
} GAPAdvertisingEventExtScanReqReceivedIndication_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvertisingStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventPeriodicAdvertisingStateChangedIndication_t;

/* Device's advertising address type */
typedef enum GAPScanningEventExtDeviceScannedIndication_AddressType_tag {
	GAPScanningEventExtDeviceScannedIndication_AddressType_gPublic_c = 0x00,
	GAPScanningEventExtDeviceScannedIndication_AddressType_gRandom_c = 0x01
} GAPScanningEventExtDeviceScannedIndication_AddressType_t;

/* Resolvable Private Address type */
typedef enum GAPScanningEventExtDeviceScannedIndication_DirectRpaType_tag {
	GAPScanningEventExtDeviceScannedIndication_DirectRpaType_gPublic_c = 0x00,
	GAPScanningEventExtDeviceScannedIndication_DirectRpaType_gRandom_c = 0x01
} GAPScanningEventExtDeviceScannedIndication_DirectRpaType_t;

typedef struct GAPScanningEventExtDeviceScannedIndication_tag {
	GAPScanningEventExtDeviceScannedIndication_AddressType_t AddressType;  // Device's advertising address type
	uint8_t Address[6];  // Device's advertising address
	uint8_t SID;  // Advertising set ID
	bool_t AdvertisingAddressResolved;  // TRUE if the address contained in the addressType and aAddress fields is the identity address of a resolved RPA from the Advertising Address field
	uint16_t AdvEventProperties;  // Advertising report properties, indicating what type of event generated this data (advertising, scan response)
	uint8_t Rssi;  // Value of the RSSI
	uint8_t TxPower;  // The Tx power level of the advertiser
	uint8_t PrimaryPHY;  // Advertiser PHY for primary channel
	uint8_t SecondaryPHY;  // Advertiser PHY for secondary channel
	uint16_t PeriodicAdvInterval;  // Interval of the periodic advertising. Zero if not periodic advertising
	bool_t DirectRpaUsed;  // TRUE if directed advertising with Resolvable Private Address as Direct Address was detected while Enhanced Privacy is enabled
	GAPScanningEventExtDeviceScannedIndication_DirectRpaType_t DirectRpaType;  // Resolvable Private Address type
	uint8_t DirectRpa[6];  // Resolvable Private Address set as Direct Address for directed advertising
	uint16_t DataLength;  // Length of the advertising or scan response data
	uint8_t *Data;  // Advertising or scan response data
} GAPScanningEventExtDeviceScannedIndication_t;

/* Periodic advertising sync status */
typedef enum GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_tag {
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleSuccess_c = 0x0000,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleOverflow_c = 0x0002,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleUnavailable_c = 0x0003,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleOsError_c = 0x0007,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleInvalidState_c = 0x0009,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gBleTimerError_c = 0x000A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciQosRejected_c = 0x012D,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciLimitReached_c = 0x0143,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gHciTransportError_c = 0x01A6,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gL2caError_c = 0x03FF,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_t;

typedef PACKED_STRUCT GAPScanningEventPeriodicAdvSyncEstablishedIndication_tag {
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_Status_t Status;  // Periodic advertising sync status
	uint16_t SyncHandle;  // Sync_Handle to be used to identify the periodic advertiser
} GAPScanningEventPeriodicAdvSyncEstablishedIndication_t;

typedef PACKED_STRUCT GAPScanningEventPeriodicAdvSyncTerminatedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPScanningEventPeriodicAdvSyncTerminatedIndication_t;

typedef PACKED_STRUCT GAPScanningEventPeriodicAdvSyncLostIndication_tag {
	uint16_t SyncHandle;  // Sync_Handle to be used to identify the periodic advertiser
} GAPScanningEventPeriodicAdvSyncLostIndication_t;

typedef struct GAPScanningEventPeriodicDeviceScannedIndication_tag {
	uint16_t SyncHandle;  // Sync Handle
	uint8_t Rssi;  // Value of the RSSI
	uint8_t TxPower;  // The Tx power level of the advertiser
	uint16_t DataLength;  // Length of the advertising or scan response data
	uint8_t *Data;  // Advertising or scan response data
} GAPScanningEventPeriodicDeviceScannedIndication_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvCreateSyncCancelledIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventPeriodicAdvCreateSyncCancelledIndication_t;

typedef PACKED_STRUCT GAPConnectionEventChannelSelectionAlgorithm2Indication_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
} GAPConnectionEventChannelSelectionAlgorithm2Indication_t;

typedef PACKED_STRUCT GAPGenericEventTxEntryAvailableIndication_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
} GAPGenericEventTxEntryAvailableIndication_t;

typedef PACKED_STRUCT GAPGenericEventControllerLocalRPAReadIndication_tag {
	uint8_t Address[6];  // Local Private Address
} GAPGenericEventControllerLocalRPAReadIndication_t;

typedef PACKED_STRUCT GAPCheckNvmIndexIndication_tag {
	bool_t IsFree;  // Free flag
} GAPCheckNvmIndexIndication_t;

typedef PACKED_STRUCT GAPGetDeviceIdFromConnHandleIndication_tag {
	uint8_t DeviceId;  // The device ID of the connected peer
} GAPGetDeviceIdFromConnHandleIndication_t;

typedef PACKED_STRUCT GAPGetConnectionHandleFromDeviceIdIndication_tag {
	uint16_t ConnectionHandle;  // The corresponding connection handle
} GAPGetConnectionHandleFromDeviceIdIndication_t;

typedef PACKED_STRUCT GAPPairingEventNoLTKIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
} GAPPairingEventNoLTKIndication_t;

typedef PACKED_STRUCT GAPPairingAlreadyStartedIndication_tag {
	uint8_t DeviceId;  // Device ID identifying the connection
} GAPPairingAlreadyStartedIndication_t;

typedef PACKED_STRUCT GAPGenericEventConnectionlessCteTransmitParamsSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventConnectionlessCteTransmitParamsSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPGenericEventConnectionlessCteTransmitStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPGenericEventConnectionlessCteTransmitStateChangedIndication_t;

typedef PACKED_STRUCT GAPGenericEventConnectionlessIqSamplingStateChangedIndication_tag {
	uint16_t SyncHandle;  // Sync handle for the advertising train
} GAPGenericEventConnectionlessIqSamplingStateChangedIndication_t;

typedef PACKED_STRUCT GAPGenericEventAntennaInformationReadIndication_tag {
	uint8_t SupportedSwitchingSamplingRates;  // 0x04|0x02|0x01  // Supported Switching Sampling Rates
	uint8_t NumAntennae;  // Number of antennae read
	uint8_t MaxSwitchingPatternLength;  // Max number of antennae
	uint8_t MaxCteLength;  // Max CTE length
} GAPGenericEventAntennaInformationReadIndication_t;

/* CTE Type */
typedef enum GAPScanningEventConnectionlessIqReportReceivedIndication_CteType_tag {
	GAPScanningEventConnectionlessIqReportReceivedIndication_CteType_gCteTypeAoA_c = 0x01,
	GAPScanningEventConnectionlessIqReportReceivedIndication_CteType_gCteTypeAoD1us_c = 0x02,
	GAPScanningEventConnectionlessIqReportReceivedIndication_CteType_gCteTypeAoD2us_c = 0x03,
	GAPScanningEventConnectionlessIqReportReceivedIndication_CteType_gCteTypeNoCte_c = 0xFF
} GAPScanningEventConnectionlessIqReportReceivedIndication_CteType_t;

/* Switching and sampling slot durations */
typedef enum GAPScanningEventConnectionlessIqReportReceivedIndication_SlotDurations_tag {
	GAPScanningEventConnectionlessIqReportReceivedIndication_SlotDurations_gSlotDurations1us_c = 0x01,
	GAPScanningEventConnectionlessIqReportReceivedIndication_SlotDurations_gSlotDurations2us_c = 0x02
} GAPScanningEventConnectionlessIqReportReceivedIndication_SlotDurations_t;

/* Status of a received packet containing IQ information */
typedef enum GAPScanningEventConnectionlessIqReportReceivedIndication_PacketStatus_tag {
	GAPScanningEventConnectionlessIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusCorrectCrc_c = 0x00,
	GAPScanningEventConnectionlessIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusCrcIncorrectUsedLength_c = 0x01,
	GAPScanningEventConnectionlessIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusCrcIncorrectUsedOther_c = 0x02,
	GAPScanningEventConnectionlessIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusInsufficientResources_c = 0xFF
} GAPScanningEventConnectionlessIqReportReceivedIndication_PacketStatus_t;

typedef struct GAPScanningEventConnectionlessIqReportReceivedIndication_tag {
	uint16_t SyncHandle;  // Sync_Handle to be used to identify the advertising set
	uint8_t ChannelIndex;  // Index of the channel on which the packet was received
	uint16_t Rssi;  // RSSI of the packet
	uint8_t RssiAntennaId;  // Antenna Id
	GAPScanningEventConnectionlessIqReportReceivedIndication_CteType_t CteType;  // CTE Type
	GAPScanningEventConnectionlessIqReportReceivedIndication_SlotDurations_t SlotDurations;  // Switching and sampling slot durations
	GAPScanningEventConnectionlessIqReportReceivedIndication_PacketStatus_t PacketStatus;  // Status of a received packet containing IQ information
	uint16_t PeriodicEventCounter;  // The value of paEventCounter for the reported AUX_SYNC_IND PDU
	uint8_t sampleCount;  // Total number of sample pairs
	uint8_t *I_samples;  // List of I_samples
	uint8_t *Q_samples;  // List of Q_samples
} GAPScanningEventConnectionlessIqReportReceivedIndication_t;

/* Receiver PHY for the connection */
typedef enum GAPConnectionEventIqReportReceivedIndication_RxPhy_tag {
	GAPConnectionEventIqReportReceivedIndication_RxPhy_gLePhy1M_c = 0x01,
	GAPConnectionEventIqReportReceivedIndication_RxPhy_gLePhy2M_c = 0x02,
	GAPConnectionEventIqReportReceivedIndication_RxPhy_gLePhyCoded_c = 0x03
} GAPConnectionEventIqReportReceivedIndication_RxPhy_t;

/* CTE Type */
typedef enum GAPConnectionEventIqReportReceivedIndication_CteType_tag {
	GAPConnectionEventIqReportReceivedIndication_CteType_gCteTypeAoA_c = 0x01,
	GAPConnectionEventIqReportReceivedIndication_CteType_gCteTypeAoD1us_c = 0x02,
	GAPConnectionEventIqReportReceivedIndication_CteType_gCteTypeAoD2us_c = 0x03,
	GAPConnectionEventIqReportReceivedIndication_CteType_gCteTypeNoCte_c = 0xFF
} GAPConnectionEventIqReportReceivedIndication_CteType_t;

/* Switching and sampling slot durations */
typedef enum GAPConnectionEventIqReportReceivedIndication_SlotDurations_tag {
	GAPConnectionEventIqReportReceivedIndication_SlotDurations_gSlotDurations1us_c = 0x01,
	GAPConnectionEventIqReportReceivedIndication_SlotDurations_gSlotDurations2us_c = 0x02
} GAPConnectionEventIqReportReceivedIndication_SlotDurations_t;

/* Status of a received packet containing IQ information */
typedef enum GAPConnectionEventIqReportReceivedIndication_PacketStatus_tag {
	GAPConnectionEventIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusCorrectCrc_c = 0x00,
	GAPConnectionEventIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusCrcIncorrectUsedLength_c = 0x01,
	GAPConnectionEventIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusCrcIncorrectUsedOther_c = 0x02,
	GAPConnectionEventIqReportReceivedIndication_PacketStatus_gIqReportPacketStatusInsufficientResources_c = 0xFF
} GAPConnectionEventIqReportReceivedIndication_PacketStatus_t;

typedef struct GAPConnectionEventIqReportReceivedIndication_tag {
	GAPConnectionEventIqReportReceivedIndication_RxPhy_t RxPhy;  // Receiver PHY for the connection
	uint8_t DataChannelIndex;  // Index of the data channel on which the Data Physical Channel PDU was received
	uint16_t Rssi;  // RSSI of the packet
	uint8_t RssiAntennaId;  // Antenna Id
	GAPConnectionEventIqReportReceivedIndication_CteType_t CteType;  // CTE Type
	GAPConnectionEventIqReportReceivedIndication_SlotDurations_t SlotDurations;  // Switching and sampling slot durations
	GAPConnectionEventIqReportReceivedIndication_PacketStatus_t PacketStatus;  // Status of a received packet containing IQ information
	uint16_t ConnEventCounter;  // The value of the connEventCounter for the reported PDU
	uint8_t sampleCount;  // Total number of sample pairs
	uint8_t *I_samples;  // List of I_samples
	uint8_t *Q_samples;  // List of Q_samples
} GAPConnectionEventIqReportReceivedIndication_t;

/* Status of the request fail */
typedef enum GAPConnectionEventCteRequestFailedIndication_Status_tag {
	GAPConnectionEventCteRequestFailedIndication_Status_gBleSuccess_c = 0x0000,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleOverflow_c = 0x0002,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleUnavailable_c = 0x0003,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleOsError_c = 0x0007,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleInvalidState_c = 0x0009,
	GAPConnectionEventCteRequestFailedIndication_Status_gBleTimerError_c = 0x000A,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciQosRejected_c = 0x012D,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciLimitReached_c = 0x0143,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPConnectionEventCteRequestFailedIndication_Status_gHciTransportError_c = 0x01A6,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPConnectionEventCteRequestFailedIndication_Status_gL2caError_c = 0x03FF,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPConnectionEventCteRequestFailedIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPConnectionEventCteRequestFailedIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPConnectionEventCteRequestFailedIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPConnectionEventCteRequestFailedIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPConnectionEventCteRequestFailedIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPConnectionEventCteRequestFailedIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPConnectionEventCteRequestFailedIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPConnectionEventCteRequestFailedIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPConnectionEventCteRequestFailedIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPConnectionEventCteRequestFailedIndication_Status_t;

typedef PACKED_STRUCT GAPConnectionEventCteRequestFailedIndication_tag {
	GAPConnectionEventCteRequestFailedIndication_Status_t Status;  // Status of the request fail
} GAPConnectionEventCteRequestFailedIndication_t;

typedef PACKED_STRUCT GAPConnectionEventCteReceiveParamsSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPConnectionEventCteReceiveParamsSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPConnectionEventCteTransmitParamsSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPConnectionEventCteTransmitParamsSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPConnectionEventCteReqStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPConnectionEventCteReqStateChangedIndication_t;

typedef PACKED_STRUCT GAPConnectionEventCteRspStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPConnectionEventCteRspStateChangedIndication_t;

/* Status of the operation */
typedef enum GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_tag {
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleSuccess_c = 0x0000,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleInvalidParameter_c = 0x0001,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleOverflow_c = 0x0002,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleUnavailable_c = 0x0003,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleFeatureNotSupported_c = 0x0004,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleOutOfMemory_c = 0x0005,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleAlreadyInitialized_c = 0x0006,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleOsError_c = 0x0007,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleUnexpectedError_c = 0x0008,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleInvalidState_c = 0x0009,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gBleTimerError_c = 0x000A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnknownHciCommand_c = 0x0101,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciHardwareFailure_c = 0x0103,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciPageTimeout_c = 0x0104,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciAuthenticationFailure_c = 0x0105,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciPinOrKeyMissing_c = 0x0106,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionTimeout_c = 0x0108,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionLimitExceeded_c = 0x0109,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciCommandDisallowed_c = 0x010C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRepeatedAttempts_c = 0x0117,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciPairingNotAllowed_c = 0x0118,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnknownLpmPdu_c = 0x0119,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciScoOffsetRejected_c = 0x011B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciScoIntervalRejected_c = 0x011C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciScoAirModeRejected_c = 0x011D,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciInvalidLpmParameters_c = 0x011E,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnspecifiedError_c = 0x011F,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciLLResponseTimeout_c = 0x0122,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciLmpPduNotAllowed_c = 0x0124,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRequestedQosNotSupported_c = 0x0127,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciInstantPassed_c = 0x0128,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciDifferentTransactionCollision_c = 0x012A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciReserved_0x2B_c = 0x012B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciQosRejected_c = 0x012D,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciInsufficientSecurity_c = 0x012F,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciReserved_0x31_c = 0x0131,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRoleSwitchPending_c = 0x0132,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciReserved_0x33_c = 0x0133,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciReservedSlotViolation_c = 0x0134,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciRoleSwitchFailed_c = 0x0135,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciHostBusyPairing_c = 0x0138,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciControllerBusy_c = 0x013A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciMacConnectionFailed_c = 0x013F,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciType0SubmapNotDefined_c = 0x0141,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciLimitReached_c = 0x0143,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciOperationCancelledByHost_c = 0x0144,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciAlreadyInit_c = 0x01A0,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciInvalidParameter_c = 0x01A1,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciCallbackNotInstalled_c = 0x01A2,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciCommandNotSupported_c = 0x01A4,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciEventNotSupported_c = 0x01A5,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gHciTransportError_c = 0x01A6,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caAlreadyInit_c = 0x03F0,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caInsufficientResources_c = 0x03F1,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caLePsmInvalid_c = 0x03F4,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caChannelInvalid_c = 0x03F8,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caChannelClosed_c = 0x03F9,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caChannelBusy_c = 0x03FC,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caInvalidParameter_c = 0x03FE,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gL2caError_c = 0x03FF,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmNullCBFunction_c = 0x0401,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmCommandNotSupported_c = 0x0402,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnexpectedCommand_c = 0x0403,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidCommandCode_c = 0x0404,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidCommandLength_c = 0x0405,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidCommandParameter_c = 0x0406,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidDeviceId_c = 0x0407,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidInternalOperation_c = 0x0408,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidConnectionHandle_c = 0x0409,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInproperKeyDistributionField_c = 0x040A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnexpectedKeyType_c = 0x040B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnexpectedKeyset_c = 0x040D,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnknownSmpPacketType_c = 0x040F,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInvalidHciEventParameter_c = 0x0416,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmInsufficientResources_c = 0x0419,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmOobDataAddressMismatch_c = 0x041A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorTimeout_c = 0x0436,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingAlreadyStarted_c = 0x0438,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gSmTbInvalidDataSignature_c = 0x0461,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInvalidHandle_c = 0x0501,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttReadNotPermitted_c = 0x0502,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttWriteNotPermitted_c = 0x0503,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInvalidPdu_c = 0x0504,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInsufficientAuthentication_c = 0x0505,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttRequestNotSupported_c = 0x0506,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInvalidOffset_c = 0x0507,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInsufficientAuthorization_c = 0x0508,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttPrepareQueueFull_c = 0x0509,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttAttributeNotFound_c = 0x050A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttAttributeNotLong_c = 0x050B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttUnlikelyor_c = 0x050E,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInsufficientEncryption_c = 0x050F,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttUnsupportedGroupType_c = 0x0510,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttInsufficientResources_c = 0x0511,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gAttDatabaseOutOfSync_c = 0x0512,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattInvalidValueLength_c = 0x0605,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattServerTimeout_c = 0x0606,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattClientConfirmationTimeout_c = 0x0608,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattInvalidPduReceived_c = 0x0609,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattPeerDisconnected_c = 0x060A,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattMtuExchangeInProgress_c = 0x060B,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGapAdvDataTooLong_c = 0x0701,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGapScanRspDataTooLong_c = 0x0702,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGapDeviceNotBonded_c = 0x0703,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gDevDbCccdLimitReached_c = 0x0801,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gDevDbCccdNotFound_c = 0x0802,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattDbInvalidHandle_c = 0x0901,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattDbCccdNotFound_c = 0x0903,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattDbServiceNotFound_c = 0x0904,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattDbDescriptorNotFound_c = 0x0905,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_tag {
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_PerAdvSyncTransferEnable_t PerAdvSyncTransferEnable;  // Status of the operation
} GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_t;

/* Status of the command complete event */
typedef enum GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_tag {
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleSuccess_c = 0x0000,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleOverflow_c = 0x0002,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleUnavailable_c = 0x0003,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleOsError_c = 0x0007,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleInvalidState_c = 0x0009,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gBleTimerError_c = 0x000A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciQosRejected_c = 0x012D,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciLimitReached_c = 0x0143,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gHciTransportError_c = 0x01A6,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gL2caError_c = 0x03FF,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_tag {
	uint8_t DeviceId;  // Identifier of the peer
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_Status_t Status;  // Status of the command complete event
} GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_t;

/* Status of the command complete event */
typedef enum GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_tag {
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleSuccess_c = 0x0000,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleOverflow_c = 0x0002,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleUnavailable_c = 0x0003,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleOsError_c = 0x0007,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleInvalidState_c = 0x0009,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gBleTimerError_c = 0x000A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciQosRejected_c = 0x012D,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciLimitReached_c = 0x0143,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gHciTransportError_c = 0x01A6,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gL2caError_c = 0x03FF,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_t;

typedef PACKED_STRUCT GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_tag {
	uint8_t DeviceId;  // Identifier of the peer
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_Status_t Status;  // Status of the command complete event
} GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_t;

/* Status of the command complete event */
typedef enum GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_tag {
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleSuccess_c = 0x0000,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleOverflow_c = 0x0002,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleUnavailable_c = 0x0003,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleOsError_c = 0x0007,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleInvalidState_c = 0x0009,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gBleTimerError_c = 0x000A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciQosRejected_c = 0x012D,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciLimitReached_c = 0x0143,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gHciTransportError_c = 0x01A6,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gL2caError_c = 0x03FF,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_t;

typedef PACKED_STRUCT GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_tag {
	uint8_t DeviceId;  // Identifier of the peer
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_Status_t Status;  // Status of the command complete event
} GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_t;

/* Status of the operation */
typedef enum GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_tag {
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleSuccess_c = 0x0000,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleInvalidParameter_c = 0x0001,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleOverflow_c = 0x0002,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleUnavailable_c = 0x0003,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleFeatureNotSupported_c = 0x0004,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleOutOfMemory_c = 0x0005,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleAlreadyInitialized_c = 0x0006,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleOsError_c = 0x0007,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleUnexpectedError_c = 0x0008,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleInvalidState_c = 0x0009,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gBleTimerError_c = 0x000A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnknownHciCommand_c = 0x0101,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciHardwareFailure_c = 0x0103,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciPageTimeout_c = 0x0104,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciAuthenticationFailure_c = 0x0105,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciPinOrKeyMissing_c = 0x0106,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionTimeout_c = 0x0108,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionLimitExceeded_c = 0x0109,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciCommandDisallowed_c = 0x010C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRepeatedAttempts_c = 0x0117,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciPairingNotAllowed_c = 0x0118,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnknownLpmPdu_c = 0x0119,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciScoOffsetRejected_c = 0x011B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciScoIntervalRejected_c = 0x011C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciScoAirModeRejected_c = 0x011D,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciInvalidLpmParameters_c = 0x011E,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnspecifiedError_c = 0x011F,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciLLResponseTimeout_c = 0x0122,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciLmpPduNotAllowed_c = 0x0124,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRequestedQosNotSupported_c = 0x0127,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciInstantPassed_c = 0x0128,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciDifferentTransactionCollision_c = 0x012A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciReserved_0x2B_c = 0x012B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciQosRejected_c = 0x012D,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciInsufficientSecurity_c = 0x012F,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciReserved_0x31_c = 0x0131,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRoleSwitchPending_c = 0x0132,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciReserved_0x33_c = 0x0133,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciReservedSlotViolation_c = 0x0134,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciRoleSwitchFailed_c = 0x0135,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciHostBusyPairing_c = 0x0138,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciControllerBusy_c = 0x013A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciMacConnectionFailed_c = 0x013F,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciType0SubmapNotDefined_c = 0x0141,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciLimitReached_c = 0x0143,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciOperationCancelledByHost_c = 0x0144,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciAlreadyInit_c = 0x01A0,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciInvalidParameter_c = 0x01A1,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciCallbackNotInstalled_c = 0x01A2,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciCommandNotSupported_c = 0x01A4,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciEventNotSupported_c = 0x01A5,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gHciTransportError_c = 0x01A6,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caAlreadyInit_c = 0x03F0,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caInsufficientResources_c = 0x03F1,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caLePsmInvalid_c = 0x03F4,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caChannelInvalid_c = 0x03F8,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caChannelClosed_c = 0x03F9,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caChannelBusy_c = 0x03FC,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caInvalidParameter_c = 0x03FE,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gL2caError_c = 0x03FF,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmNullCBFunction_c = 0x0401,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmCommandNotSupported_c = 0x0402,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnexpectedCommand_c = 0x0403,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidCommandCode_c = 0x0404,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidCommandLength_c = 0x0405,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidCommandParameter_c = 0x0406,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidDeviceId_c = 0x0407,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidInternalOperation_c = 0x0408,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidConnectionHandle_c = 0x0409,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInproperKeyDistributionField_c = 0x040A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnexpectedKeyType_c = 0x040B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnexpectedKeyset_c = 0x040D,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnknownSmpPacketType_c = 0x040F,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInvalidHciEventParameter_c = 0x0416,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmInsufficientResources_c = 0x0419,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmOobDataAddressMismatch_c = 0x041A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorTimeout_c = 0x0436,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingAlreadyStarted_c = 0x0438,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gSmTbInvalidDataSignature_c = 0x0461,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInvalidHandle_c = 0x0501,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttReadNotPermitted_c = 0x0502,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttWriteNotPermitted_c = 0x0503,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInvalidPdu_c = 0x0504,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInsufficientAuthentication_c = 0x0505,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttRequestNotSupported_c = 0x0506,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInvalidOffset_c = 0x0507,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInsufficientAuthorization_c = 0x0508,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttPrepareQueueFull_c = 0x0509,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttAttributeNotFound_c = 0x050A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttAttributeNotLong_c = 0x050B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttUnlikelyor_c = 0x050E,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInsufficientEncryption_c = 0x050F,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttUnsupportedGroupType_c = 0x0510,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttInsufficientResources_c = 0x0511,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gAttDatabaseOutOfSync_c = 0x0512,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattInvalidValueLength_c = 0x0605,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattServerTimeout_c = 0x0606,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattClientConfirmationTimeout_c = 0x0608,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattInvalidPduReceived_c = 0x0609,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattPeerDisconnected_c = 0x060A,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattMtuExchangeInProgress_c = 0x060B,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGapAdvDataTooLong_c = 0x0701,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGapScanRspDataTooLong_c = 0x0702,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGapDeviceNotBonded_c = 0x0703,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gDevDbCccdLimitReached_c = 0x0801,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gDevDbCccdNotFound_c = 0x0802,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattDbInvalidHandle_c = 0x0901,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattDbCccdNotFound_c = 0x0903,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattDbServiceNotFound_c = 0x0904,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattDbDescriptorNotFound_c = 0x0905,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_t;

typedef PACKED_STRUCT GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_tag {
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_PerAdvSetDefaultPerAdvSyncTransferParams_t PerAdvSetDefaultPerAdvSyncTransferParams;  // Status of the operation
} GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_t;

/* Status of the  syncronization operation */
typedef enum GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_tag {
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleSuccess_c = 0x0000,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleInvalidParameter_c = 0x0001,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleOverflow_c = 0x0002,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleUnavailable_c = 0x0003,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleFeatureNotSupported_c = 0x0004,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleOutOfMemory_c = 0x0005,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleAlreadyInitialized_c = 0x0006,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleOsError_c = 0x0007,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleUnexpectedError_c = 0x0008,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleInvalidState_c = 0x0009,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gBleTimerError_c = 0x000A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnknownHciCommand_c = 0x0101,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnknownConnectionIdentifier_c = 0x0102,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciHardwareFailure_c = 0x0103,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciPageTimeout_c = 0x0104,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciAuthenticationFailure_c = 0x0105,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciPinOrKeyMissing_c = 0x0106,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciMemoryCapacityExceeded_c = 0x0107,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionTimeout_c = 0x0108,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionLimitExceeded_c = 0x0109,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciSynchronousConnectionLimitToADeviceExceeded_c = 0x010A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciAclConnectionAlreadyExists_c = 0x010B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciCommandDisallowed_c = 0x010C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionRejectedDueToLimitedResources_c = 0x010D,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionRejectedDueToSecurityReasons_c = 0x010E,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionRejectedDueToUnacceptableBdAddr_c = 0x010F,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionAcceptTimeoutExceeded_c = 0x0110,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnsupportedFeatureOrParameterValue_c = 0x0111,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciInvalidHciCommandParameters_c = 0x0112,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRemoteUserTerminatedConnection_c = 0x0113,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRemoteDeviceTerminatedConnectionLowResources_c = 0x0114,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRemoteDeviceTerminatedConnectionPowerOff_c = 0x0115,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionTerminatedByLocalHost_c = 0x0116,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRepeatedAttempts_c = 0x0117,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciPairingNotAllowed_c = 0x0118,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnknownLpmPdu_c = 0x0119,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnsupportedRemoteFeature_c = 0x011A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciScoOffsetRejected_c = 0x011B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciScoIntervalRejected_c = 0x011C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciScoAirModeRejected_c = 0x011D,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciInvalidLpmParameters_c = 0x011E,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnspecifiedError_c = 0x011F,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnsupportedLpmParameterValue_c = 0x0120,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRoleChangeNotAllowed_c = 0x0121,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciLLResponseTimeout_c = 0x0122,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciLmpErrorTransactionCollision_c = 0x0123,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciLmpPduNotAllowed_c = 0x0124,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciEncryptionModeNotAcceptable_c = 0x0125,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciLinkKeyCannotBeChanged_c = 0x0126,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRequestedQosNotSupported_c = 0x0127,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciInstantPassed_c = 0x0128,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciPairingWithUnitKeyNotSupported_c = 0x0129,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciDifferentTransactionCollision_c = 0x012A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciReserved_0x2B_c = 0x012B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciQosNotAcceptableParameter_c = 0x012C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciQosRejected_c = 0x012D,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciChannelClassificationNotSupported_c = 0x012E,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciInsufficientSecurity_c = 0x012F,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciParameterOutOfMandatoryRange_c = 0x0130,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciReserved_0x31_c = 0x0131,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRoleSwitchPending_c = 0x0132,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciReserved_0x33_c = 0x0133,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciReservedSlotViolation_c = 0x0134,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciRoleSwitchFailed_c = 0x0135,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciExtendedInquiryResponseTooLarge_c = 0x0136,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciSecureSimplePairingNotSupportedByHost_c = 0x0137,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciHostBusyPairing_c = 0x0138,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionRejectedDueToNoSuitableChannelFound_c = 0x0139,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciControllerBusy_c = 0x013A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnacceptableConnectionParameters_c = 0x013B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciDirectedAdvertisingTimeout_c = 0x013C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionTerminatedDueToMicFailure_c = 0x013D,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciConnectionFailedToBeEstablishedOrSyncTimeout_c = 0x013E,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciMacConnectionFailed_c = 0x013F,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciCoarseClockAdjustmentRejected_c = 0x0140,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciType0SubmapNotDefined_c = 0x0141,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciUnknownAdvertisingIdentifier_c = 0x0142,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciLimitReached_c = 0x0143,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciOperationCancelledByHost_c = 0x0144,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciAlreadyInit_c = 0x01A0,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciInvalidParameter_c = 0x01A1,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciCallbackNotInstalled_c = 0x01A2,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciCallbackAlreadyInstalled_c = 0x01A3,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciCommandNotSupported_c = 0x01A4,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciEventNotSupported_c = 0x01A5,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gHciTransportError_c = 0x01A6,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caAlreadyInit_c = 0x03F0,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caInsufficientResources_c = 0x03F1,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caCallbackNotInstalled_c = 0x03F2,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caCallbackAlreadyInstalled_c = 0x03F3,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caLePsmInvalid_c = 0x03F4,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caLePsmAlreadyRegistered_c = 0x03F5,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caLePsmNotRegistered_c = 0x03F6,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caLePsmInsufficientResources_c = 0x03F7,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caChannelInvalid_c = 0x03F8,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caChannelClosed_c = 0x03F9,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caChannelAlreadyConnected_c = 0x03FA,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caConnectionParametersRejected_c = 0x03FB,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caChannelBusy_c = 0x03FC,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caInvalidParameter_c = 0x03FE,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gL2caError_c = 0x03FF,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmNullCBFunction_c = 0x0401,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmCommandNotSupported_c = 0x0402,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnexpectedCommand_c = 0x0403,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidCommandCode_c = 0x0404,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidCommandLength_c = 0x0405,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidCommandParameter_c = 0x0406,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidDeviceId_c = 0x0407,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidInternalOperation_c = 0x0408,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidConnectionHandle_c = 0x0409,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInproperKeyDistributionField_c = 0x040A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnexpectedKeyType_c = 0x040B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnexpectedPairingTerminationReason_c = 0x040C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnexpectedKeyset_c = 0x040D,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmSmpTimeoutOccurred_c = 0x040E,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnknownSmpPacketType_c = 0x040F,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidSmpPacketLength_c = 0x0410,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidSmpPacketParameter_c = 0x0411,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmReceivedUnexpectedSmpPacket_c = 0x0412,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmReceivedSmpPacketFromUnknownDevice_c = 0x0413,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmReceivedUnexpectedHciEvent_c = 0x0414,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmReceivedHciEventFromUnknownDevice_c = 0x0415,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInvalidHciEventParameter_c = 0x0416,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmLlConnectionEncryptionInProgress_c = 0x0417,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmLlConnectionEncryptionFailure_c = 0x0418,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmInsufficientResources_c = 0x0419,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmOobDataAddressMismatch_c = 0x041A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmSmpPacketReceivedAfterTimeoutOccurred_c = 0x041B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmReceivedTimerEventForUnknownDevice_c = 0x041C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnattainableLocalDeviceSecRequirements_c = 0x041D,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnattainableLocalDeviceMinKeySize_c = 0x041E,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmUnattainablePeripheralSecReqRequirements_c = 0x041F,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorPasskeyEntryFailed_c = 0x0431,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorConfirmValueFailed_c = 0x0432,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorCommandNotSupported_c = 0x0433,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorInvalidParameters_c = 0x0434,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorUnknownReason_c = 0x0435,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorTimeout_c = 0x0436,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorAuthenticationRequirements_c = 0x0437,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingAlreadyStarted_c = 0x0438,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmPairingErrorKeyRejected_c = 0x0439,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmTbResolvableAddressDoesNotMatchIrk_c = 0x0460,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gSmTbInvalidDataSignature_c = 0x0461,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInvalidHandle_c = 0x0501,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttReadNotPermitted_c = 0x0502,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttWriteNotPermitted_c = 0x0503,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInvalidPdu_c = 0x0504,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInsufficientAuthentication_c = 0x0505,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttRequestNotSupported_c = 0x0506,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInvalidOffset_c = 0x0507,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInsufficientAuthorization_c = 0x0508,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttPrepareQueueFull_c = 0x0509,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttAttributeNotFound_c = 0x050A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttAttributeNotLong_c = 0x050B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInsufficientEncryptionKeySize_c = 0x050C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInvalidAttributeValueLength_c = 0x050D,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttUnlikelyor_c = 0x050E,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInsufficientEncryption_c = 0x050F,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttUnsupportedGroupType_c = 0x0510,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttInsufficientResources_c = 0x0511,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gAttDatabaseOutOfSync_c = 0x0512,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattAnotherProcedureInProgress_c = 0x0601,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattLongAttributePacketsCorrupted_c = 0x0602,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattMultipleAttributesOverflow_c = 0x0603,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattUnexpectedReadMultipleResponseLength_c = 0x0604,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattInvalidValueLength_c = 0x0605,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattServerTimeout_c = 0x0606,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattIndicationAlreadyInProgress_c = 0x0607,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattClientConfirmationTimeout_c = 0x0608,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattInvalidPduReceived_c = 0x0609,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattPeerDisconnected_c = 0x060A,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattMtuExchangeInProgress_c = 0x060B,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattOutOfSyncProceduresOngoing_c = 0x060C,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGapAdvDataTooLong_c = 0x0701,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGapScanRspDataTooLong_c = 0x0702,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGapDeviceNotBonded_c = 0x0703,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGapAnotherProcedureInProgress_c = 0x0704,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gDevDbCccdLimitReached_c = 0x0801,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gDevDbCccdNotFound_c = 0x0802,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattDbInvalidHandle_c = 0x0901,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattDbCharacteristicNotFound_c = 0x0902,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattDbCccdNotFound_c = 0x0903,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattDbServiceNotFound_c = 0x0904,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattDbDescriptorNotFound_c = 0x0905,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_gGattDbServiceOrCharAlreadyDeclared_c = 0x0906
} GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_t;

/* Device's advertising address type */
typedef enum GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvAddressType_tag {
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvAddressType_gPublic_c = 0x00,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvAddressType_gRandom_c = 0x01
} GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvAddressType_t;

/* PHY used for advertising */
typedef enum GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvPhy_tag {
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvPhy_gLePhy1M_c = 0x01,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvPhy_gLePhy2M_c = 0x02,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvPhy_gLePhyCoded_c = 0x03
} GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvPhy_t;

/* Advertiser clock accuracy */
typedef enum GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_tag {
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc500ppm_c = 0x00,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc250ppm_c = 0x01,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc150ppm_c = 0x02,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc100ppm_c = 0x03,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc75ppm_c = 0x04,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc50ppm_c = 0x05,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc30ppm_c = 0x06,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_gCentralClkAcc20ppm_c = 0x07
} GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_t;

typedef PACKED_STRUCT GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_tag {
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_Status_t Status;  // Status of the  syncronization operation
	uint8_t DeviceId;  // Id of the connected device
	uint16_t ServiceData;  // Value provided by the peer device
	uint16_t SyncHandle;  // Sync_Handle to be used to identify periodic advertising train
	uint8_t AdvSID;  // Advertising SID
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvAddressType_t AdvAddressType;  // Device's advertising address type
	uint8_t AdvAddress[6];  // Device's advertising address
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvPhy_t AdvPhy;  // PHY used for advertising
	uint16_t PeriodicAdvInt;  // Periodic advertising interval
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_AdvClockAccuracy_t AdvClockAccuracy;  // Advertiser clock accuracy
} GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_t;

/* Type of entered zone */
typedef enum GAPConnectionEventPathLossThresholdIndication_ZoneEntered_tag {
	GAPConnectionEventPathLossThresholdIndication_ZoneEntered_gPathLossThresholdLowZone_c = 0x00,
	GAPConnectionEventPathLossThresholdIndication_ZoneEntered_gPathLossThresholdMiddleZone_c = 0x01,
	GAPConnectionEventPathLossThresholdIndication_ZoneEntered_gPathLossThresholdHighZone_c = 0x02
} GAPConnectionEventPathLossThresholdIndication_ZoneEntered_t;

typedef PACKED_STRUCT GAPConnectionEventPathLossThresholdIndication_tag {
	uint8_t CurrentPathLoss;  // Current path loss
	GAPConnectionEventPathLossThresholdIndication_ZoneEntered_t ZoneEntered;  // Type of entered zone
} GAPConnectionEventPathLossThresholdIndication_t;

/* Reason for generating the Transmit Power Reporting event */
typedef enum GAPConnectionEventTransmitPowerReportingIndication_Reason_tag {
	GAPConnectionEventTransmitPowerReportingIndication_Reason_gLocalTxPowerChanged_c = 0x00,
	GAPConnectionEventTransmitPowerReportingIndication_Reason_gRemoteTxPowerChanged_c = 0x01,
	GAPConnectionEventTransmitPowerReportingIndication_Reason_gReadRemoteTxPowerLevelCommandCompleted_c = 0x02
} GAPConnectionEventTransmitPowerReportingIndication_Reason_t;

/* The PHY for which the event is generated */
typedef enum GAPConnectionEventTransmitPowerReportingIndication_Phy_tag {
	GAPConnectionEventTransmitPowerReportingIndication_Phy_gPowerControlLePhy1M_c = 0x01,
	GAPConnectionEventTransmitPowerReportingIndication_Phy_gPowerControlLePhy2M_c = 0x02,
	GAPConnectionEventTransmitPowerReportingIndication_Phy_gPowerControlLePhyCodedS8_c = 0x03,
	GAPConnectionEventTransmitPowerReportingIndication_Phy_gPowerControlLePhyCodedS2_c = 0x04
} GAPConnectionEventTransmitPowerReportingIndication_Phy_t;

typedef PACKED_STRUCT GAPConnectionEventTransmitPowerReportingIndication_tag {
	GAPConnectionEventTransmitPowerReportingIndication_Reason_t Reason;  // Reason for generating the Transmit Power Reporting event
	GAPConnectionEventTransmitPowerReportingIndication_Phy_t Phy;  // The PHY for which the event is generated
	uint8_t TxPowerLevel;  // New transmit power level
	uint8_t Flags;  // 0x02|0x01  // Tx power level flags
	uint8_t Delta;  // Change in tx power level
} GAPConnectionEventTransmitPowerReportingIndication_t;

/* The PHY for which the event is generated */
typedef enum GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_Phy_tag {
	GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_Phy_gPowerControlLePhy1M_c = 0x01,
	GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_Phy_gPowerControlLePhy2M_c = 0x02,
	GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_Phy_gPowerControlLePhyCodedS8_c = 0x03,
	GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_Phy_gPowerControlLePhyCodedS2_c = 0x04
} GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_Phy_t;

typedef PACKED_STRUCT GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_tag {
	GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_Phy_t Phy;  // The PHY for which the event is generated
	uint8_t CurrTxPowerLevel;  // Current tx power level
	uint8_t MaxTxPowerLevel;  // Maximum tx power level
} GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_t;

typedef PACKED_STRUCT GAPConnectionEventPathLossReportingParamsSetupCompleteIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPConnectionEventPathLossReportingParamsSetupCompleteIndication_t;

typedef PACKED_STRUCT GAPConnectionEventPathLossReportingStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPConnectionEventPathLossReportingStateChangedIndication_t;

typedef PACKED_STRUCT GAPConnectionEventTransmitPowerReportingStateChangedIndication_tag {
	uint16_t PayloadLength;  // The FSCI length field
} GAPConnectionEventTransmitPowerReportingStateChangedIndication_t;

typedef PACKED_STRUCT GAPConnectionEventEattConnectionRequestIndication_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint16_t MTU;  // MTU
	uint8_t NoOfBearers;  // Number of bearers established
	uint16_t InitialCredits;  // Initial credits
} GAPConnectionEventEattConnectionRequestIndication_t;

typedef struct GAPConnectionEventEattConnectionCompleteIndication_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint8_t Status;  // Status
	uint16_t MTU;  // MTU
	uint8_t NoOfBearers;  // Number of bearers established
	uint8_t *BearerIDs;  // The ATT Bearers IDs for the established bearers
} GAPConnectionEventEattConnectionCompleteIndication_t;

typedef struct GAPConnectionEventEattReconfigureResponseIndication_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint8_t Status;  // Status
	uint16_t LocalMtu;  // Local MTU
	uint8_t NoOfBearers;  // Number of bearers for which the MTU was reconfigured
	uint8_t *BearerIDs;  // The ATT Bearers IDs for which the MTU was reconfigured
} GAPConnectionEventEattReconfigureResponseIndication_t;

/* Status */
typedef enum GAPConnectionEventEattBearerStatusNotificationIndication_Status_tag {
	GAPConnectionEventEattBearerStatusNotificationIndication_Status_gEnhancedBearerActive_c = 0x00,
	GAPConnectionEventEattBearerStatusNotificationIndication_Status_gEnhancedBearerSuspendedNoLocalCredits_c = 0x01,
	GAPConnectionEventEattBearerStatusNotificationIndication_Status_gEnhancedBearerNoPeerCredits_c = 0x02,
	GAPConnectionEventEattBearerStatusNotificationIndication_Status_gEnhancedBearerDisconnected_c = 0x03
} GAPConnectionEventEattBearerStatusNotificationIndication_Status_t;

typedef PACKED_STRUCT GAPConnectionEventEattBearerStatusNotificationIndication_tag {
	uint8_t DeviceId;  // The DeviceId for which the command is intended
	uint8_t BearerId;  // The BearerId of the ATT Bearer used
	GAPConnectionEventEattBearerStatusNotificationIndication_Status_t Status;  // Status
} GAPConnectionEventEattBearerStatusNotificationIndication_t;

typedef PACKED_STRUCT GAPGenericEventLeGenerateDhKeyCompleteIndication_tag {
	uint8_t DHKey[32];  // The newly generated key
} GAPGenericEventLeGenerateDhKeyCompleteIndication_t;

typedef PACKED_STRUCT GAPGetHostVersionIndication_tag {
	uint8_t BleHostVerMajor;  // Host Version Major
	uint8_t BleHostVerMinor;  // Host Version Minor
	uint8_t BleHostVerPatch;  // Host Version Patch
} GAPGetHostVersionIndication_t;

#endif  /* GAP_ENABLE */

typedef enum bleFsciIds_tag
{
	L2CAPInitRequest_FSCI_ID = 0x4101,
	L2CAPSendAttDataRequest_FSCI_ID = 0x4103,
	L2CAPSendSmpDataRequest_FSCI_ID = 0x4104,
	L2CAPRegisterAttCallbackRequest_FSCI_ID = 0x4106,
	L2CAPRegisterSmpCallbackRequest_FSCI_ID = 0x4107,
	L2CAPCBRegisterLeCbCallbacksRequest_FSCI_ID = 0x4201,
	L2CAPCBRegisterLePsmRequest_FSCI_ID = 0x4202,
	L2CAPCBDeregisterLePsmRequest_FSCI_ID = 0x4203,
	L2CAPCBConnectLePsmRequest_FSCI_ID = 0x4204,
	L2CAPCBDisconnectLeCbChannelRequest_FSCI_ID = 0x4205,
	L2CAPCBCancelConnectionRequest_FSCI_ID = 0x4206,
	L2CAPCBSendLeCbDataRequest_FSCI_ID = 0x4207,
	L2CAPCBSendLeCreditRequest_FSCI_ID = 0x4208,
	L2CAPCBEnhancedConnectLePsmRequest_FSCI_ID = 0x4209,
	L2CAPCBEnhancedChannelReconfigureRequest_FSCI_ID = 0x420A,
	L2CAPCBEnhancedCancelConnectionRequest_FSCI_ID = 0x420B,
	GATTInitRequest_FSCI_ID = 0x4501,
	GATTGetMtuRequest_FSCI_ID = 0x4502,
	GATTClientInitRequest_FSCI_ID = 0x4503,
	GATTClientResetProcedureRequest_FSCI_ID = 0x4504,
	GATTClientRegisterProcedureCallbackRequest_FSCI_ID = 0x4505,
	GATTClientRegisterNotificationCallbackRequest_FSCI_ID = 0x4506,
	GATTClientRegisterMultipleValueNotificationCallbackRequest_FSCI_ID = 0x4523,
	GATTClientRegisterIndicationCallbackRequest_FSCI_ID = 0x4507,
	GATTClientExchangeMtuRequest_FSCI_ID = 0x4508,
	GATTClientDiscoverAllPrimaryServicesRequest_FSCI_ID = 0x4509,
	GATTClientDiscoverPrimaryServicesByUuidRequest_FSCI_ID = 0x450A,
	GATTClientFindIncludedServicesRequest_FSCI_ID = 0x450B,
	GATTClientDiscoverAllCharacteristicsOfServiceRequest_FSCI_ID = 0x450C,
	GATTClientDiscoverCharacteristicOfServiceByUuidRequest_FSCI_ID = 0x450D,
	GATTClientDiscoverAllCharacteristicDescriptorsRequest_FSCI_ID = 0x450E,
	GATTClientReadCharacteristicValueRequest_FSCI_ID = 0x450F,
	GATTClientReadUsingCharacteristicUuidRequest_FSCI_ID = 0x4510,
	GATTClientReadMultipleCharacteristicValuesRequest_FSCI_ID = 0x4511,
	GATTClientWriteCharacteristicValueRequest_FSCI_ID = 0x4512,
	GATTClientReadCharacteristicDescriptorRequest_FSCI_ID = 0x4513,
	GATTClientWriteCharacteristicDescriptorRequest_FSCI_ID = 0x4514,
	GATTServerInitRequest_FSCI_ID = 0x4515,
	GATTServerRegisterCallbackRequest_FSCI_ID = 0x4516,
	GATTServerRegisterHandlesForWriteNotificationsRequest_FSCI_ID = 0x4517,
	GATTServerSendAttributeWrittenStatusRequest_FSCI_ID = 0x4518,
	GATTServerSendNotificationRequest_FSCI_ID = 0x4519,
	GATTServerSendIndicationRequest_FSCI_ID = 0x451A,
	GATTServerSendInstantValueNotificationRequest_FSCI_ID = 0x451B,
	GATTServerSendInstantValueIndicationRequest_FSCI_ID = 0x451C,
	GATTServerSendMultipleHandleValueNotificationRequest_FSCI_ID = 0x4522,
	GATTServerRegisterHandlesForReadNotificationsRequest_FSCI_ID = 0x451D,
	GATTServerSendAttributeReadStatusRequest_FSCI_ID = 0x451E,
	GATTServerRegisterUniqueHandlesForNotificationsRequest_FSCI_ID = 0x451F,
	GATTServerUnregisterHandlesForWriteNotificationsRequest_FSCI_ID = 0x4520,
	GATTServerUnregisterHandlesForReadNotificationsRequest_FSCI_ID = 0x4521,
	GATTClientReadMultipleVariableCharacteristicValuesRequest_FSCI_ID = 0x4524,
	GATTClientRegisterEnhancedProcedureCallbackRequest_FSCI_ID = 0x4525,
	GATTClientRegisterEnhancedNotificationCallbackRequest_FSCI_ID = 0x4526,
	GATTClientRegisterEnhancedIndicationCallbackRequest_FSCI_ID = 0x4527,
	GATTClientRegisterEnhancedMultipleValueNotificationCallbackRequest_FSCI_ID = 0x4528,
	GATTClientEnhancedDiscoverAllPrimaryServicesRequest_FSCI_ID = 0x4529,
	GATTClientEnhancedDiscoverPrimaryServicesByUuidRequest_FSCI_ID = 0x452A,
	GATTClientEnhancedFindIncludedServicesRequest_FSCI_ID = 0x452B,
	GATTClientEnhancedDiscoverAllCharacteristicsOfServiceRequest_FSCI_ID = 0x452C,
	GATTClientEnhancedDiscoverCharacteristicOfServiceByUuidRequest_FSCI_ID = 0x452D,
	GATTClientEnhancedDiscoverAllCharacteristicDescriptorsRequest_FSCI_ID = 0x452E,
	GATTClientEnhancedReadCharacteristicValueRequest_FSCI_ID = 0x452F,
	GATTClientEnhancedReadUsingCharacteristicUuidRequest_FSCI_ID = 0x4530,
	GATTClientEnhancedReadMultipleCharacteristicValuesRequest_FSCI_ID = 0x4531,
	GATTClientEnhancedWriteCharacteristicValueRequest_FSCI_ID = 0x4532,
	GATTClientEnhancedReadCharacteristicDescriptorRequest_FSCI_ID = 0x4533,
	GATTClientEnhancedWriteCharacteristicDescriptorRequest_FSCI_ID = 0x4534,
	GATTClientEnhancedReadMultipleVariableCharacteristicValuesRequest_FSCI_ID = 0x4535,
	GATTServerRegisterEnhancedCallbackRequest_FSCI_ID = 0x4536,
	GATTServerEnhancedSendAttributeWrittenStatusRequest_FSCI_ID = 0x4537,
	GATTServerEnhancedSendNotificationRequest_FSCI_ID = 0x4538,
	GATTServerEnhancedSendIndicationRequest_FSCI_ID = 0x4539,
	GATTServerEnhancedSendInstantValueNotificationRequest_FSCI_ID = 0x453A,
	GATTServerEnhancedSendInstantValueIndicationRequest_FSCI_ID = 0x453B,
	GATTServerEnhancedSendAttributeReadStatusRequest_FSCI_ID = 0x453C,
	GATTServerEnhancedSendMultipleHandleValueNotificationRequest_FSCI_ID = 0x453D,
	GATTClientGetDatabaseHashRequest_FSCI_ID = 0x453E,
	GATTDBWriteAttributeRequest_FSCI_ID = 0x4602,
	GATTDBReadAttributeRequest_FSCI_ID = 0x4603,
	GATTDBFindServiceHandleRequest_FSCI_ID = 0x4604,
	GATTDBFindCharValueHandleInServiceRequest_FSCI_ID = 0x4605,
	GATTDBFindCccdHandleForCharValueHandleRequest_FSCI_ID = 0x4606,
	GATTDBFindDescriptorHandleForCharValueHandleRequest_FSCI_ID = 0x4607,
	GATTDBDynamicInitRequest_FSCI_ID = 0x4608,
	GATTDBDynamicReleaseDatabaseRequest_FSCI_ID = 0x4609,
	GATTDBDynamicAddPrimaryServiceDeclarationRequest_FSCI_ID = 0x460A,
	GATTDBDynamicAddSecondaryServiceDeclarationRequest_FSCI_ID = 0x460B,
	GATTDBDynamicAddIncludeDeclarationRequest_FSCI_ID = 0x460C,
	GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_FSCI_ID = 0x460D,
	GATTDBDynamicAddCharacteristicDescriptorRequest_FSCI_ID = 0x460E,
	GATTDBDynamicAddCccdRequest_FSCI_ID = 0x460F,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_FSCI_ID = 0x4610,
	GATTDBDynamicRemoveServiceRequest_FSCI_ID = 0x4611,
	GATTDBDynamicRemoveCharacteristicRequest_FSCI_ID = 0x4612,
	GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_FSCI_ID = 0x4613,
	GATTDBDynamicEndDatabaseUpdateRequest_FSCI_ID = 0x4614,
	GATTDBAttFindInformationRequest_FSCI_ID = 0x4701,
	GATTDBAttFindByTypeValueRequest_FSCI_ID = 0x4702,
	GATTDBAttReadByTypeRequest_FSCI_ID = 0x4703,
	GATTDBAttReadRequest_FSCI_ID = 0x4704,
	GATTDBAttReadBlobRequest_FSCI_ID = 0x4705,
	GATTDBAttReadMultipleRequest_FSCI_ID = 0x4706,
	GATTDBAttReadByGroupTypeRequest_FSCI_ID = 0x4707,
	GATTDBAttWriteRequest_FSCI_ID = 0x4708,
	GATTDBAttWriteCommandRequest_FSCI_ID = 0x4709,
	GATTDBAttSignedWriteCommandRequest_FSCI_ID = 0x470A,
	GATTDBAttPrepareWriteRequest_FSCI_ID = 0x470B,
	GATTDBAttExecuteWriteRequest_FSCI_ID = 0x470C,
	GATTDBAttExecuteWriteFromQueueRequest_FSCI_ID = 0x470D,
	GATTDBAttPrepareNotificationIndicationRequest_FSCI_ID = 0x470E,
	BLEHostInitializeRequest_FSCI_ID = 0x4801,
	GAPRegisterDeviceSecurityRequirementsRequest_FSCI_ID = 0x4802,
	GAPSetAdvertisingParametersRequest_FSCI_ID = 0x4803,
	GAPSetAdvertisingDataRequest_FSCI_ID = 0x4804,
	GAPStartAdvertisingRequest_FSCI_ID = 0x4805,
	GAPStopAdvertisingRequest_FSCI_ID = 0x4806,
	GAPAuthorizeRequest_FSCI_ID = 0x4807,
	GAPSaveCccdRequest_FSCI_ID = 0x4808,
	GAPCheckNotificationStatusRequest_FSCI_ID = 0x4809,
	GAPCheckIndicationStatusRequest_FSCI_ID = 0x480A,
	GAPPairRequest_FSCI_ID = 0x480C,
	GAPSendPeripheralSecurityRequestRequest_FSCI_ID = 0x480D,
	GAPEncryptLinkRequest_FSCI_ID = 0x480E,
	GAPAcceptPairingRequestRequest_FSCI_ID = 0x480F,
	GAPRejectPairingRequest_FSCI_ID = 0x4810,
	GAPEnterPasskeyRequest_FSCI_ID = 0x4811,
	GAPProvideOobRequest_FSCI_ID = 0x4812,
	GAPRejectPasskeyRequestRequest_FSCI_ID = 0x4813,
	GAPSendSmpKeysRequest_FSCI_ID = 0x4814,
	GAPRejectKeyExchangeRequestRequest_FSCI_ID = 0x4815,
	GAPProvideLongTermKeyRequest_FSCI_ID = 0x4816,
	GAPDenyLongTermKeyRequest_FSCI_ID = 0x4817,
	GAPLoadEncryptionInformationRequest_FSCI_ID = 0x4818,
	GAPSetLocalPasskeyRequest_FSCI_ID = 0x4819,
	GAPStartScanningRequest_FSCI_ID = 0x481A,
	GAPStopScanningRequest_FSCI_ID = 0x481B,
	GAPConnectRequest_FSCI_ID = 0x481C,
	GAPDisconnectRequest_FSCI_ID = 0x481D,
	GAPSaveCustomPeerInformationRequest_FSCI_ID = 0x481E,
	GAPLoadCustomPeerInformationRequest_FSCI_ID = 0x481F,
	GAPCheckIfBondedRequest_FSCI_ID = 0x4820,
	GAPReadFilterAcceptListSizeRequest_FSCI_ID = 0x4821,
	GAPClearFilterAcceptListRequest_FSCI_ID = 0x4822,
	GAPAddDeviceToFilterAcceptListRequest_FSCI_ID = 0x4823,
	GAPRemoveDeviceFromFilterAcceptListRequest_FSCI_ID = 0x4824,
	GAPReadPublicDeviceAddressRequest_FSCI_ID = 0x4825,
	GAPCreateRandomDeviceAddressRequest_FSCI_ID = 0x4826,
	GAPSaveDeviceNameRequest_FSCI_ID = 0x4827,
	GAPGetBondedDevicesCountRequest_FSCI_ID = 0x4828,
	GAPGetBondedDeviceNameRequest_FSCI_ID = 0x4829,
	GAPRemoveBondRequest_FSCI_ID = 0x482A,
	GAPRemoveAllBondsRequest_FSCI_ID = 0x482B,
	GAPReadRadioPowerLevelRequest_FSCI_ID = 0x482C,
	GAPVerifyPrivateResolvableAddressRequest_FSCI_ID = 0x482D,
	GAPSetRandomAddressRequest_FSCI_ID = 0x482E,
	GAPSetScanModeRequest_FSCI_ID = 0x482F,
	GAPSetDefaultPairingParametersRequest_FSCI_ID = 0x4830,
	GAPUpdateConnectionParametersRequest_FSCI_ID = 0x4831,
	GAPEnableUpdateConnectionParametersRequest_FSCI_ID = 0x4832,
	GapUpdateLeDataLengthRequest_FSCI_ID = 0x4833,
	GAPEnableHostPrivacyRequest_FSCI_ID = 0x4835,
	GAPEnableControllerPrivacyRequest_FSCI_ID = 0x4836,
	GAPLeScRegeneratePublicKeyRequest_FSCI_ID = 0x4837,
	GAPLeScValidateNumericValueRequest_FSCI_ID = 0x4838,
	GAPLeScGetLocalOobDataRequest_FSCI_ID = 0x4839,
	GAPLeScSetPeerOobDataRequest_FSCI_ID = 0x483A,
	GAPLeScSendKeypressNotificationPrivacyRequest_FSCI_ID = 0x483B,
	GAPGetBondedDevicesIdentityInformationRequest_FSCI_ID = 0x483C,
	GAPSetTxPowerLevelRequest_FSCI_ID = 0x483D,
	GAPLeReadPhyRequest_FSCI_ID = 0x483E,
	GAPLeSetPhyRequest_FSCI_ID = 0x483F,
	GAPControllerEnhancedNotificationRequest_FSCI_ID = 0x4840,
	GAPLoadKeysRequest_FSCI_ID = 0x4841,
	GAPSaveKeysRequest_FSCI_ID = 0x4842,
	GAPSetChannelMapRequest_FSCI_ID = 0x4843,
	GAPReadChannelMapRequest_FSCI_ID = 0x4844,
	GAPSetPrivacyModeRequest_FSCI_ID = 0x4845,
	ControllerSetScanDupFiltModeRequest_FSCI_ID = 0x4846,
	GAPReadControllerLocalRPARequest_FSCI_ID = 0x4847,
	GAPCheckNvmIndexRequest_FSCI_ID = 0x4849,
	GAPGetDeviceIdFromConnHandleRequest_FSCI_ID = 0x484A,
	GAPGetConnectionHandleFromDeviceId_FSCI_ID = 0x484B,
	GAPAdvIndexChangeRequest_FSCI_ID = 0x484C,
	GAPGetHostVersion_FSCI_ID = 0x484D,
	GAPSetExtAdvertisingParametersRequest_FSCI_ID = 0x4850,
	GAPStartExtAdvertisingRequest_FSCI_ID = 0x4851,
	GAPRemoveAdvertisingSetRequest_FSCI_ID = 0x4852,
	GAPStopExtAdvertisingRequest_FSCI_ID = 0x4853,
	GAPUpdatePeriodicAdvListRequest_FSCI_ID = 0x4854,
	GAPSetPeriodicAdvParametersRequest_FSCI_ID = 0x4855,
	GAPStartPeriodicAdvertisingRequest_FSCI_ID = 0x4856,
	GAPStopPeriodicAdvertisingRequest_FSCI_ID = 0x4857,
	GAPSetExtAdvertisingDataRequest_FSCI_ID = 0x4858,
	GAPSetPeriodicAdvertisingDataRequest_FSCI_ID = 0x4859,
	GAPPeriodicAdvCreateSyncRequest_FSCI_ID = 0x485A,
	GAPPeriodicAdvTerminateSyncRequest_FSCI_ID = 0x485B,
	GAPGenerateDHKeyV2Request_FSCI_ID = 0x4865,
	GAPModifySleepClockAccuracyRequest_FSCI_ID = 0x4866,
	ControllerConfigureAdvCodingSchemeRequest_FSCI_ID = 0x485C,
	GAPSetConnectionlessCteTransmitParametersRequest_FSCI_ID = 0x485D,
	GAPEnableConnectionlessCteTransmitRequest_FSCI_ID = 0x485E,
	GAPEnableConnectionlessIqSamplingRequest_FSCI_ID = 0x485F,
	GAPSetConnectionCteReceiveParametersRequest_FSCI_ID = 0x4860,
	GAPSetConnectionCteTransmitParametersRequest_FSCI_ID = 0x4861,
	GAPEnableConnectionCteRequestRequest_FSCI_ID = 0x4862,
	GAPEnableConnectionCteResponseRequest_FSCI_ID = 0x4863,
	GAPReadAntennaInformationRequest_FSCI_ID = 0x4864,
	WritePublicDeviceAddressRequest_FSCI_ID = 0x4848,
	GAPPeriodicAdvReceiveEnableRequest_FSCI_ID = 0x4867,
	GAPPeriodicAdvReceiveDisableRequest_FSCI_ID = 0x4868,
	GAPPeriodicAdvSyncTransferRequest_FSCI_ID = 0x4869,
	GAPPeriodicAdvSetInfoTransferRequest_FSCI_ID = 0x486A,
	GAPSetPeriodicAdvSyncTransferParamsRequest_FSCI_ID = 0x486B,
	GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_FSCI_ID = 0x486C,
	GAPEnhancedReadTransmitPowerLevelRequest_FSCI_ID = 0x486D,
	GAPReadRemoteTransmitPowerLevelRequest_FSCI_ID = 0x486E,
	GAPSetPathLossReportingParametersRequest_FSCI_ID = 0x486F,
	GAPEnablePathLossReportingRequest_FSCI_ID = 0x4870,
	GAPEnableTransmitPowerReportingRequest_FSCI_ID = 0x4871,
	GAPEattConnectionRequest_FSCI_ID = 0x4872,
	GAPEattConnectionAccept_FSCI_ID = 0x4873,
	GAPEattReconfigureRequest_FSCI_ID = 0x4874,
	GAPEattSendCreditsRequest_FSCI_ID = 0x4875,
	FSCICPUResetRequest_FSCI_ID = 0xA308,
	FSCIGetNumberOfFreeBuffersRequest_FSCI_ID = 0xA309,
	FSCIAllowDeviceToSleepRequest_FSCI_ID = 0xA370,
	FSCIGetWakeupReasonRequest_FSCI_ID = 0xA372,
	FSCIGetNbuVersionRequest_FSCI_ID = 0xA3B3,
	FSCINVGetNvVPSizeRequest_FSCI_ID = 0xA7E7,
	FSCIGetNumberOfFreeBuffersResponse_FSCI_ID = 0xA409,
	FSCIAckIndication_FSCI_ID = 0xA4FD,
	FSCIErrorIndication_FSCI_ID = 0xA4FE,
	FSCIAllowDeviceToSleepConfirm_FSCI_ID = 0xA470,
	FSCIWakeUpIndication_FSCI_ID = 0xA471,
	FSCIGetWakeupReasonResponse_FSCI_ID = 0xA472,
	FSCIGetNbuVersionResponse_FSCI_ID = 0xA4B3,
	L2CAPCBConfirm_FSCI_ID = 0x4280,
	L2CAPCBLePsmConnectionRequestIndication_FSCI_ID = 0x4281,
	L2CAPCBLePsmConnectionCompleteIndication_FSCI_ID = 0x4282,
	L2CAPCBLePsmDisconnectNotificationIndication_FSCI_ID = 0x4283,
	L2CAPCBNoPeerCreditsIndication_FSCI_ID = 0x4284,
	L2CAPCBLocalCreditsNotificationIndication_FSCI_ID = 0x4285,
	L2CAPCBLeCbDataIndication_FSCI_ID = 0x4286,
	L2CAPCBErrorIndication_FSCI_ID = 0x4287,
	L2CAPCBChannelStatusNotificationIndication_FSCI_ID = 0x4288,
	L2CAPCBLePsmEnhancedConnectRequestIndication_FSCI_ID = 0x4289,
	L2CAPCBLePsmEnhancedConnectionCompleteIndication_FSCI_ID = 0x428A,
	L2CAPCBEnhancedReconfigureRequestIndication_FSCI_ID = 0x428B,
	L2CAPCBEnhancedReconfigureResponseIndication_FSCI_ID = 0x428C,
	GATTConfirm_FSCI_ID = 0x4580,
	GATTGetMtuIndication_FSCI_ID = 0x4581,
	GATTClientProcedureExchangeMtuIndication_FSCI_ID = 0x4582,
	GATTClientProcedureDiscoverAllPrimaryServicesIndication_FSCI_ID = 0x4583,
	GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_FSCI_ID = 0x4584,
	GATTClientProcedureFindIncludedServicesIndication_FSCI_ID = 0x4585,
	GATTClientProcedureDiscoverAllCharacteristicsIndication_FSCI_ID = 0x4586,
	GATTClientProcedureDiscoverCharacteristicByUuidIndication_FSCI_ID = 0x4587,
	GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_FSCI_ID = 0x4588,
	GATTClientProcedureReadCharacteristicValueIndication_FSCI_ID = 0x4589,
	GATTClientProcedureReadUsingCharacteristicUuidIndication_FSCI_ID = 0x458A,
	GATTClientProcedureReadMultipleCharacteristicValuesIndication_FSCI_ID = 0x458B,
	GATTClientProcedureWriteCharacteristicValueIndication_FSCI_ID = 0x458C,
	GATTClientProcedureReadCharacteristicDescriptorIndication_FSCI_ID = 0x458D,
	GATTClientProcedureWriteCharacteristicDescriptorIndication_FSCI_ID = 0x458E,
	GATTClientNotificationIndication_FSCI_ID = 0x458F,
	GATTClientMultipleHandleValueNotificationIndication_FSCI_ID = 0x4599,
	GATTClientProcedureReadMultipleVariableLenCharValuesIndication_FSCI_ID = 0x459A,
	GATTClientIndicationIndication_FSCI_ID = 0x4590,
	GATTServerMtuChangedIndication_FSCI_ID = 0x4591,
	GATTServerHandleValueConfirmationIndication_FSCI_ID = 0x4592,
	GATTServerAttributeWrittenIndication_FSCI_ID = 0x4593,
	GATTServerCharacteristicCccdWrittenIndication_FSCI_ID = 0x4594,
	GATTServerAttributeWrittenWithoutResponseIndication_FSCI_ID = 0x4595,
	GATTServerErrorIndication_FSCI_ID = 0x4596,
	GATTServerLongCharacteristicWrittenIndication_FSCI_ID = 0x4597,
	GATTServerAttributeReadIndication_FSCI_ID = 0x4598,
	GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_FSCI_ID = 0x459B,
	GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_FSCI_ID = 0x459C,
	GATTClientProcedureEnhancedFindIncludedServicesIndication_FSCI_ID = 0x459D,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_FSCI_ID = 0x459E,
	GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_FSCI_ID = 0x459F,
	GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_FSCI_ID = 0x45A0,
	GATTClientProcedureEnhancedReadCharacteristicValueIndication_FSCI_ID = 0x45A1,
	GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_FSCI_ID = 0x45A2,
	GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_FSCI_ID = 0x45A3,
	GATTClientProcedureEnhancedWriteCharacteristicValueIndication_FSCI_ID = 0x45A4,
	GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_FSCI_ID = 0x45A5,
	GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_FSCI_ID = 0x45A6,
	GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_FSCI_ID = 0x45A7,
	GATTClientEnhancedMultipleHandleValueNotificationIndication_FSCI_ID = 0x45A8,
	GATTClientEnhancedNotificationIndication_FSCI_ID = 0x45A9,
	GATTClientEnhancedIndicationIndication_FSCI_ID = 0x45AA,
	GATTServerEnhancedHandleValueConfirmationIndication_FSCI_ID = 0x45AB,
	GATTServerEnhancedAttributeWrittenIndication_FSCI_ID = 0x45AC,
	GATTServerEnhancedCharacteristicCccdWrittenIndication_FSCI_ID = 0x45AD,
	GATTServerEnhancedAttributeWrittenWithoutResponseIndication_FSCI_ID = 0x45AE,
	GATTServerEnhancedErrorIndication_FSCI_ID = 0x45AF,
	GATTServerEnhancedLongCharacteristicWrittenIndication_FSCI_ID = 0x45B0,
	GATTServerEnhancedAttributeReadIndication_FSCI_ID = 0x45B1,
	GATTDBConfirm_FSCI_ID = 0x4680,
	GATTDBReadAttributeIndication_FSCI_ID = 0x4681,
	GATTDBFindServiceHandleIndication_FSCI_ID = 0x4682,
	GATTDBFindCharValueHandleInServiceIndication_FSCI_ID = 0x4683,
	GATTDBFindCccdHandleForCharValueHandleIndication_FSCI_ID = 0x4684,
	GATTDBFindDescriptorHandleForCharValueHandleIndication_FSCI_ID = 0x4685,
	GATTDBDynamicAddPrimaryServiceDeclarationIndication_FSCI_ID = 0x4686,
	GATTDBDynamicAddSecondaryServiceDeclarationIndication_FSCI_ID = 0x4687,
	GATTDBDynamicAddIncludeDeclarationIndication_FSCI_ID = 0x4688,
	GATTDBDynamicAddCharacteristicDeclarationAndValueIndication_FSCI_ID = 0x4689,
	GATTDBDynamicAddCharacteristicDescriptorIndication_FSCI_ID = 0x468A,
	GATTDBDynamicAddCccdIndication_FSCI_ID = 0x468B,
	GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueIndication_FSCI_ID = 0x468C,
	GATTDBDynamicAddCharDescriptorWithUniqueValueIndication_FSCI_ID = 0x468D,
	GATTDBAttConfirm_FSCI_ID = 0x4780,
	GATTDBAttFindInformationIndication_FSCI_ID = 0x4781,
	GATTDBAttFindByTypeValueIndication_FSCI_ID = 0x4782,
	GATTDBAttReadByTypeIndication_FSCI_ID = 0x4783,
	GATTDBAttReadIndication_FSCI_ID = 0x4784,
	GATTDBAttReadBlobIndication_FSCI_ID = 0x4785,
	GATTDBAttReadMultipleIndication_FSCI_ID = 0x4786,
	GATTDBAttReadByGroupTypeIndication_FSCI_ID = 0x4787,
	GATTDBAttWriteIndication_FSCI_ID = 0x4788,
	GATTDBAttPrepareWriteIndication_FSCI_ID = 0x4789,
	GATTDBAttExecuteWriteIndication_FSCI_ID = 0x478A,
	GATTDBAttExecuteWriteFromQueueIndication_FSCI_ID = 0x478B,
	GATTDBAttPrepareNotificationIndicationIndication_FSCI_ID = 0x478C,
	GAPConfirm_FSCI_ID = 0x4880,
	GAPCheckNotificationStatusIndication_FSCI_ID = 0x4881,
	GAPCheckIndicationStatusIndication_FSCI_ID = 0x4882,
	GAPLoadKeysIndication_FSCI_ID = 0x4883,
	GAPLoadEncryptionInformationIndication_FSCI_ID = 0x4884,
	GAPLoadCustomPeerInformationIndication_FSCI_ID = 0x4885,
	GAPCheckIfBondedIndication_FSCI_ID = 0x4886,
	GAPGetBondedDevicesCountIndication_FSCI_ID = 0x4887,
	GAPGetBondedDeviceNameIndication_FSCI_ID = 0x4888,
	GAPGenericEventInitializationCompleteIndication_FSCI_ID = 0x4889,
	GAPGenericEventInternalErrorIndication_FSCI_ID = 0x488A,
	GAPGenericEventAdvertisingSetupFailedIndication_FSCI_ID = 0x488B,
	GAPGenericEventAdvertisingParametersSetupCompleteIndication_FSCI_ID = 0x488C,
	GAPGenericEventAdvertisingDataSetupCompleteIndication_FSCI_ID = 0x488D,
	GAPGenericEventFilterAcceptListSizeReadIndication_FSCI_ID = 0x488E,
	GAPGenericEventDeviceAddedToFilterAcceptListIndication_FSCI_ID = 0x488F,
	GAPGenericEventDeviceRemovedFromFilterAcceptListIndication_FSCI_ID = 0x4890,
	GAPGenericEventFilterAcceptListClearedIndication_FSCI_ID = 0x4891,
	GAPGenericEventRandomAddressReadyIndication_FSCI_ID = 0x4892,
	GAPGenericEventCreateConnectionCanceledIndication_FSCI_ID = 0x4893,
	GAPGenericEventPublicAddressReadIndication_FSCI_ID = 0x4894,
	GAPGenericEventAdvTxPowerLevelReadIndication_FSCI_ID = 0x4895,
	GAPGenericEventPrivateResolvableAddressVerifiedIndication_FSCI_ID = 0x4896,
	GAPGenericEventRandomAddressSetIndication_FSCI_ID = 0x4897,
	GAPAdvertisingEventStateChangedIndication_FSCI_ID = 0x4898,
	GAPAdvertisingEventCommandFailedIndication_FSCI_ID = 0x4899,
	GAPScanningEventStateChangedIndication_FSCI_ID = 0x489A,
	GAPScanningEventCommandFailedIndication_FSCI_ID = 0x489B,
	GAPScanningEventDeviceScannedIndication_FSCI_ID = 0x489C,
	GAPConnectionEventConnectedIndication_FSCI_ID = 0x489D,
	GAPConnectionEventPairingRequestIndication_FSCI_ID = 0x489E,
	GAPConnectionEventPeripheralSecurityRequestIndication_FSCI_ID = 0x489F,
	GAPConnectionEventPairingResponseIndication_FSCI_ID = 0x48A0,
	GAPConnectionEventAuthenticationRejectedIndication_FSCI_ID = 0x48A1,
	GAPConnectionEventPasskeyRequestIndication_FSCI_ID = 0x48A2,
	GAPConnectionEventOobRequestIndication_FSCI_ID = 0x48A3,
	GAPConnectionEventPasskeyDisplayIndication_FSCI_ID = 0x48A4,
	GAPConnectionEventKeyExchangeRequestIndication_FSCI_ID = 0x48A5,
	GAPConnectionEventKeysReceivedIndication_FSCI_ID = 0x48A6,
	GAPConnectionEventLongTermKeyRequestIndication_FSCI_ID = 0x48A7,
	GAPConnectionEventEncryptionChangedIndication_FSCI_ID = 0x48A8,
	GAPConnectionEventPairingCompleteIndication_FSCI_ID = 0x48A9,
	GAPConnectionEventDisconnectedIndication_FSCI_ID = 0x48AA,
	GAPConnectionEventRssiReadIndication_FSCI_ID = 0x48AB,
	GAPConnectionEventTxPowerLevelReadIndication_FSCI_ID = 0x48AC,
	GAPConnectionEventPowerReadFailureIndication_FSCI_ID = 0x48AD,
	GAPConnectionEventParameterUpdateRequestIndication_FSCI_ID = 0x48AE,
	GAPConnectionEventParameterUpdateCompleteIndication_FSCI_ID = 0x48AF,
	GAPConnectionEventLeDataLengthChangedIndication_FSCI_ID = 0x48B0,
	GAPConnectionEventLeScOobDataRequestIndication_FSCI_ID = 0x48B1,
	GAPConnectionEventLeScDisplayNumericValueIndication_FSCI_ID = 0x48B2,
	GAPConnectionEventLeScKeypressNotificationIndication_FSCI_ID = 0x48B3,
	GAPLeScPublicKeyRegeneratedIndication_FSCI_ID = 0x48B5,
	GAPGenericEventLeScLocalOobDataIndication_FSCI_ID = 0x48B6,
	GAPGenericEventHostPrivacyStateChangedIndication_FSCI_ID = 0x48B7,
	GAPGenericEventControllerPrivacyStateChangedIndication_FSCI_ID = 0x48B8,
	GAPGenericEventTxPowerLevelSetCompleteIndication_FSCI_ID = 0x48B9,
	GAPGenericEventLePhyEventIndication_FSCI_ID = 0x48BA,
	GAPGetBondedDevicesIdentityInformationIndication_FSCI_ID = 0x48BB,
	GAPControllerNotificationIndication_FSCI_ID = 0x48BC,
	GAPBondCreatedIndication_FSCI_ID = 0x48BD,
	GAPGenericEventChannelMapSetIndication_FSCI_ID = 0x48BE,
	GAPConnectionEventChannelMapReadIndication_FSCI_ID = 0x48BF,
	GAPConnectionEventChannelMapReadFailureIndication_FSCI_ID = 0x48C0,
	GAPGenericEventExtAdvertisingParamSetupCompleteIndication_FSCI_ID = 0x48C1,
	GAPGenericEventExtAdvertisingDataSetupCompleteIndication_FSCI_ID = 0x48C2,
	GAPGenericEventPeriodicAdvParamSetupCompleteIndication_FSCI_ID = 0x48C3,
	GAPGenericEventPeriodicAdvDataSetupCompleteIndication_FSCI_ID = 0x48C4,
	GAPGenericEventPeriodicAdvListUpdateCompleteIndication_FSCI_ID = 0x48C5,
	GAPAdvertisingEventExtAdvertisingStateChangedIndication_FSCI_ID = 0x48C6,
	GAPAdvertisingEventAdvertisingSetTerminatedIndication_FSCI_ID = 0x48C7,
	GAPAdvertisingEventExtAdvertisingSetRemoveCompletedIndication_FSCI_ID = 0x48C8,
	GAPAdvertisingEventExtScanReqReceivedIndication_FSCI_ID = 0x48C9,
	GAPGenericEventPeriodicAdvertisingStateChangedIndication_FSCI_ID = 0x48CA,
	GAPScanningEventExtDeviceScannedIndication_FSCI_ID = 0x48CB,
	GAPScanningEventPeriodicAdvSyncEstablishedIndication_FSCI_ID = 0x48CC,
	GAPScanningEventPeriodicAdvSyncTerminatedIndication_FSCI_ID = 0x48CD,
	GAPScanningEventPeriodicAdvSyncLostIndication_FSCI_ID = 0x48CE,
	GAPScanningEventPeriodicDeviceScannedIndication_FSCI_ID = 0x48CF,
	GAPGenericEventPeriodicAdvCreateSyncCancelledIndication_FSCI_ID = 0x48D0,
	GAPConnectionEventChannelSelectionAlgorithm2Indication_FSCI_ID = 0x48D1,
	GAPGenericEventTxEntryAvailableIndication_FSCI_ID = 0x48D2,
	GAPGenericEventControllerLocalRPAReadIndication_FSCI_ID = 0x48D3,
	GAPCheckNvmIndexIndication_FSCI_ID = 0x48D4,
	GAPGetDeviceIdFromConnHandleIndication_FSCI_ID = 0x48D5,
	GAPGetConnectionHandleFromDeviceIdIndication_FSCI_ID = 0x48D6,
	GAPPairingEventNoLTKIndication_FSCI_ID = 0x48D7,
	GAPPairingAlreadyStartedIndication_FSCI_ID = 0x48D8,
	GAPGenericEventConnectionlessCteTransmitParamsSetupCompleteIndication_FSCI_ID = 0x48D9,
	GAPGenericEventConnectionlessCteTransmitStateChangedIndication_FSCI_ID = 0x48DA,
	GAPGenericEventConnectionlessIqSamplingStateChangedIndication_FSCI_ID = 0x48DB,
	GAPGenericEventAntennaInformationReadIndication_FSCI_ID = 0x48DC,
	GAPScanningEventConnectionlessIqReportReceivedIndication_FSCI_ID = 0x48DD,
	GAPConnectionEventIqReportReceivedIndication_FSCI_ID = 0x48DE,
	GAPConnectionEventCteRequestFailedIndication_FSCI_ID = 0x48DF,
	GAPConnectionEventCteReceiveParamsSetupCompleteIndication_FSCI_ID = 0x48E0,
	GAPConnectionEventCteTransmitParamsSetupCompleteIndication_FSCI_ID = 0x48E1,
	GAPConnectionEventCteReqStateChangedIndication_FSCI_ID = 0x48E2,
	GAPConnectionEventCteRspStateChangedIndication_FSCI_ID = 0x48E3,
	GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_FSCI_ID = 0x48E4,
	GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_FSCI_ID = 0x48E5,
	GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_FSCI_ID = 0x48E6,
	GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_FSCI_ID = 0x48E7,
	GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_FSCI_ID = 0x48E8,
	GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_FSCI_ID = 0x48E9,
	GAPConnectionEventPathLossThresholdIndication_FSCI_ID = 0x48EA,
	GAPConnectionEventTransmitPowerReportingIndication_FSCI_ID = 0x48EB,
	GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_FSCI_ID = 0x48EC,
	GAPConnectionEventPathLossReportingParamsSetupCompleteIndication_FSCI_ID = 0x48ED,
	GAPConnectionEventPathLossReportingStateChangedIndication_FSCI_ID = 0x48EE,
	GAPConnectionEventTransmitPowerReportingStateChangedIndication_FSCI_ID = 0x48EF,
	GAPConnectionEventEattConnectionRequestIndication_FSCI_ID = 0x48F0,
	GAPConnectionEventEattConnectionCompleteIndication_FSCI_ID = 0x48F1,
	GAPConnectionEventEattReconfigureResponseIndication_FSCI_ID = 0x48F2,
	GAPConnectionEventEattBearerStatusNotificationIndication_FSCI_ID = 0x48F3,
	GAPGenericEventLeGenerateDhKeyCompleteIndication_FSCI_ID = 0x48F4,
	GAPGetHostVersionIndication_FSCI_ID = 0x48F5,
} bleFsciIds_t;

typedef struct bleEvtContainer_tag
{
	uint16_t id;
	union {
#if FSCI_ENABLE
		FSCIGetNumberOfFreeBuffersResponse_t FSCIGetNumberOfFreeBuffersResponse;
		FSCIAckIndication_t FSCIAckIndication;
		FSCIErrorIndication_t FSCIErrorIndication;
		FSCIAllowDeviceToSleepConfirm_t FSCIAllowDeviceToSleepConfirm;
		FSCIWakeUpIndication_t FSCIWakeUpIndication;
		FSCIGetWakeupReasonResponse_t FSCIGetWakeupReasonResponse;
		FSCIGetNbuVersionResponse_t FSCIGetNbuVersionResponse;
#endif  /* FSCI_ENABLE */

#if L2CAPCB_ENABLE
		L2CAPCBConfirm_t L2CAPCBConfirm;
		L2CAPCBLePsmConnectionRequestIndication_t L2CAPCBLePsmConnectionRequestIndication;
		L2CAPCBLePsmConnectionCompleteIndication_t L2CAPCBLePsmConnectionCompleteIndication;
		L2CAPCBLePsmDisconnectNotificationIndication_t L2CAPCBLePsmDisconnectNotificationIndication;
		L2CAPCBNoPeerCreditsIndication_t L2CAPCBNoPeerCreditsIndication;
		L2CAPCBLocalCreditsNotificationIndication_t L2CAPCBLocalCreditsNotificationIndication;
		L2CAPCBLeCbDataIndication_t L2CAPCBLeCbDataIndication;
		L2CAPCBErrorIndication_t L2CAPCBErrorIndication;
		L2CAPCBChannelStatusNotificationIndication_t L2CAPCBChannelStatusNotificationIndication;
		L2CAPCBLePsmEnhancedConnectRequestIndication_t L2CAPCBLePsmEnhancedConnectRequestIndication;
		L2CAPCBLePsmEnhancedConnectionCompleteIndication_t L2CAPCBLePsmEnhancedConnectionCompleteIndication;
		L2CAPCBEnhancedReconfigureRequestIndication_t L2CAPCBEnhancedReconfigureRequestIndication;
		L2CAPCBEnhancedReconfigureResponseIndication_t L2CAPCBEnhancedReconfigureResponseIndication;
#endif  /* L2CAPCB_ENABLE */

#if GATT_ENABLE
		GATTConfirm_t GATTConfirm;
		GATTGetMtuIndication_t GATTGetMtuIndication;
		GATTClientProcedureExchangeMtuIndication_t GATTClientProcedureExchangeMtuIndication;
		GATTClientProcedureDiscoverAllPrimaryServicesIndication_t GATTClientProcedureDiscoverAllPrimaryServicesIndication;
		GATTClientProcedureDiscoverPrimaryServicesByUuidIndication_t GATTClientProcedureDiscoverPrimaryServicesByUuidIndication;
		GATTClientProcedureFindIncludedServicesIndication_t GATTClientProcedureFindIncludedServicesIndication;
		GATTClientProcedureDiscoverAllCharacteristicsIndication_t GATTClientProcedureDiscoverAllCharacteristicsIndication;
		GATTClientProcedureDiscoverCharacteristicByUuidIndication_t GATTClientProcedureDiscoverCharacteristicByUuidIndication;
		GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication_t GATTClientProcedureDiscoverAllCharacteristicDescriptorsIndication;
		GATTClientProcedureReadCharacteristicValueIndication_t GATTClientProcedureReadCharacteristicValueIndication;
		GATTClientProcedureReadUsingCharacteristicUuidIndication_t GATTClientProcedureReadUsingCharacteristicUuidIndication;
		GATTClientProcedureReadMultipleCharacteristicValuesIndication_t GATTClientProcedureReadMultipleCharacteristicValuesIndication;
		GATTClientProcedureWriteCharacteristicValueIndication_t GATTClientProcedureWriteCharacteristicValueIndication;
		GATTClientProcedureReadCharacteristicDescriptorIndication_t GATTClientProcedureReadCharacteristicDescriptorIndication;
		GATTClientProcedureWriteCharacteristicDescriptorIndication_t GATTClientProcedureWriteCharacteristicDescriptorIndication;
		GATTClientNotificationIndication_t GATTClientNotificationIndication;
		GATTClientMultipleHandleValueNotificationIndication_t GATTClientMultipleHandleValueNotificationIndication;
		GATTClientProcedureReadMultipleVariableLenCharValuesIndication_t GATTClientProcedureReadMultipleVariableLenCharValuesIndication;
		GATTClientIndicationIndication_t GATTClientIndicationIndication;
		GATTServerMtuChangedIndication_t GATTServerMtuChangedIndication;
		GATTServerHandleValueConfirmationIndication_t GATTServerHandleValueConfirmationIndication;
		GATTServerAttributeWrittenIndication_t GATTServerAttributeWrittenIndication;
		GATTServerCharacteristicCccdWrittenIndication_t GATTServerCharacteristicCccdWrittenIndication;
		GATTServerAttributeWrittenWithoutResponseIndication_t GATTServerAttributeWrittenWithoutResponseIndication;
		GATTServerErrorIndication_t GATTServerErrorIndication;
		GATTServerLongCharacteristicWrittenIndication_t GATTServerLongCharacteristicWrittenIndication;
		GATTServerAttributeReadIndication_t GATTServerAttributeReadIndication;
		GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication_t GATTClientProcedureEnhancedDiscoverAllPrimaryServicesIndication;
		GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication_t GATTClientProcedureEnhancedDiscoverPrimaryServicesByUuidIndication;
		GATTClientProcedureEnhancedFindIncludedServicesIndication_t GATTClientProcedureEnhancedFindIncludedServicesIndication;
		GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication_t GATTClientProcedureEnhancedDiscoverAllCharacteristicsIndication;
		GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication_t GATTClientProcedureEnhancedDiscoverCharacteristicByUuidIndication;
		GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication_t GATTClientProcedureEnhancedDiscoverAllCharacteristicDescriptorsIndication;
		GATTClientProcedureEnhancedReadCharacteristicValueIndication_t GATTClientProcedureEnhancedReadCharacteristicValueIndication;
		GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication_t GATTClientProcedureEnhancedReadUsingCharacteristicUuidIndication;
		GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication_t GATTClientProcedureEnhancedReadMultipleCharacteristicValuesIndication;
		GATTClientProcedureEnhancedWriteCharacteristicValueIndication_t GATTClientProcedureEnhancedWriteCharacteristicValueIndication;
		GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication_t GATTClientProcedureEnhancedReadCharacteristicDescriptorIndication;
		GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication_t GATTClientProcedureEnhancedWriteCharacteristicDescriptorIndication;
		GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication_t GATTClientProcedureEnhancedReadMultipleVariableLenCharValuesIndication;
		GATTClientEnhancedMultipleHandleValueNotificationIndication_t GATTClientEnhancedMultipleHandleValueNotificationIndication;
		GATTClientEnhancedNotificationIndication_t GATTClientEnhancedNotificationIndication;
		GATTClientEnhancedIndicationIndication_t GATTClientEnhancedIndicationIndication;
		GATTServerEnhancedHandleValueConfirmationIndication_t GATTServerEnhancedHandleValueConfirmationIndication;
		GATTServerEnhancedAttributeWrittenIndication_t GATTServerEnhancedAttributeWrittenIndication;
		GATTServerEnhancedCharacteristicCccdWrittenIndication_t GATTServerEnhancedCharacteristicCccdWrittenIndication;
		GATTServerEnhancedAttributeWrittenWithoutResponseIndication_t GATTServerEnhancedAttributeWrittenWithoutResponseIndication;
		GATTServerEnhancedErrorIndication_t GATTServerEnhancedErrorIndication;
		GATTServerEnhancedLongCharacteristicWrittenIndication_t GATTServerEnhancedLongCharacteristicWrittenIndication;
		GATTServerEnhancedAttributeReadIndication_t GATTServerEnhancedAttributeReadIndication;
#endif  /* GATT_ENABLE */

#if GATTDB_APP_ENABLE
		GATTDBConfirm_t GATTDBConfirm;
		GATTDBReadAttributeIndication_t GATTDBReadAttributeIndication;
		GATTDBFindServiceHandleIndication_t GATTDBFindServiceHandleIndication;
		GATTDBFindCharValueHandleInServiceIndication_t GATTDBFindCharValueHandleInServiceIndication;
		GATTDBFindCccdHandleForCharValueHandleIndication_t GATTDBFindCccdHandleForCharValueHandleIndication;
		GATTDBFindDescriptorHandleForCharValueHandleIndication_t GATTDBFindDescriptorHandleForCharValueHandleIndication;
		GATTDBDynamicAddPrimaryServiceDeclarationIndication_t GATTDBDynamicAddPrimaryServiceDeclarationIndication;
		GATTDBDynamicAddSecondaryServiceDeclarationIndication_t GATTDBDynamicAddSecondaryServiceDeclarationIndication;
		GATTDBDynamicAddIncludeDeclarationIndication_t GATTDBDynamicAddIncludeDeclarationIndication;
		GATTDBDynamicAddCharacteristicDeclarationAndValueIndication_t GATTDBDynamicAddCharacteristicDeclarationAndValueIndication;
		GATTDBDynamicAddCharacteristicDescriptorIndication_t GATTDBDynamicAddCharacteristicDescriptorIndication;
		GATTDBDynamicAddCccdIndication_t GATTDBDynamicAddCccdIndication;
		GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueIndication_t GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueIndication;
		GATTDBDynamicAddCharDescriptorWithUniqueValueIndication_t GATTDBDynamicAddCharDescriptorWithUniqueValueIndication;
#endif  /* GATTDB_APP_ENABLE */

#if GATTDB_ATT_ENABLE
		GATTDBAttConfirm_t GATTDBAttConfirm;
		GATTDBAttFindInformationIndication_t GATTDBAttFindInformationIndication;
		GATTDBAttFindByTypeValueIndication_t GATTDBAttFindByTypeValueIndication;
		GATTDBAttReadByTypeIndication_t GATTDBAttReadByTypeIndication;
		GATTDBAttReadIndication_t GATTDBAttReadIndication;
		GATTDBAttReadBlobIndication_t GATTDBAttReadBlobIndication;
		GATTDBAttReadMultipleIndication_t GATTDBAttReadMultipleIndication;
		GATTDBAttReadByGroupTypeIndication_t GATTDBAttReadByGroupTypeIndication;
		GATTDBAttWriteIndication_t GATTDBAttWriteIndication;
		GATTDBAttPrepareWriteIndication_t GATTDBAttPrepareWriteIndication;
		GATTDBAttExecuteWriteIndication_t GATTDBAttExecuteWriteIndication;
		GATTDBAttExecuteWriteFromQueueIndication_t GATTDBAttExecuteWriteFromQueueIndication;
		GATTDBAttPrepareNotificationIndicationIndication_t GATTDBAttPrepareNotificationIndicationIndication;
#endif  /* GATTDB_ATT_ENABLE */

#if GAP_ENABLE
		GAPConfirm_t GAPConfirm;
		GAPCheckNotificationStatusIndication_t GAPCheckNotificationStatusIndication;
		GAPCheckIndicationStatusIndication_t GAPCheckIndicationStatusIndication;
		GAPLoadKeysIndication_t GAPLoadKeysIndication;
		GAPLoadEncryptionInformationIndication_t GAPLoadEncryptionInformationIndication;
		GAPLoadCustomPeerInformationIndication_t GAPLoadCustomPeerInformationIndication;
		GAPCheckIfBondedIndication_t GAPCheckIfBondedIndication;
		GAPGetBondedDevicesCountIndication_t GAPGetBondedDevicesCountIndication;
		GAPGetBondedDeviceNameIndication_t GAPGetBondedDeviceNameIndication;
		GAPGenericEventInitializationCompleteIndication_t GAPGenericEventInitializationCompleteIndication;
		GAPGenericEventInternalErrorIndication_t GAPGenericEventInternalErrorIndication;
		GAPGenericEventAdvertisingSetupFailedIndication_t GAPGenericEventAdvertisingSetupFailedIndication;
		GAPGenericEventAdvertisingParametersSetupCompleteIndication_t GAPGenericEventAdvertisingParametersSetupCompleteIndication;
		GAPGenericEventAdvertisingDataSetupCompleteIndication_t GAPGenericEventAdvertisingDataSetupCompleteIndication;
		GAPGenericEventFilterAcceptListSizeReadIndication_t GAPGenericEventFilterAcceptListSizeReadIndication;
		GAPGenericEventDeviceAddedToFilterAcceptListIndication_t GAPGenericEventDeviceAddedToFilterAcceptListIndication;
		GAPGenericEventDeviceRemovedFromFilterAcceptListIndication_t GAPGenericEventDeviceRemovedFromFilterAcceptListIndication;
		GAPGenericEventFilterAcceptListClearedIndication_t GAPGenericEventFilterAcceptListClearedIndication;
		GAPGenericEventRandomAddressReadyIndication_t GAPGenericEventRandomAddressReadyIndication;
		GAPGenericEventCreateConnectionCanceledIndication_t GAPGenericEventCreateConnectionCanceledIndication;
		GAPGenericEventPublicAddressReadIndication_t GAPGenericEventPublicAddressReadIndication;
		GAPGenericEventAdvTxPowerLevelReadIndication_t GAPGenericEventAdvTxPowerLevelReadIndication;
		GAPGenericEventPrivateResolvableAddressVerifiedIndication_t GAPGenericEventPrivateResolvableAddressVerifiedIndication;
		GAPGenericEventRandomAddressSetIndication_t GAPGenericEventRandomAddressSetIndication;
		GAPAdvertisingEventStateChangedIndication_t GAPAdvertisingEventStateChangedIndication;
		GAPAdvertisingEventCommandFailedIndication_t GAPAdvertisingEventCommandFailedIndication;
		GAPScanningEventStateChangedIndication_t GAPScanningEventStateChangedIndication;
		GAPScanningEventCommandFailedIndication_t GAPScanningEventCommandFailedIndication;
		GAPScanningEventDeviceScannedIndication_t GAPScanningEventDeviceScannedIndication;
		GAPConnectionEventConnectedIndication_t GAPConnectionEventConnectedIndication;
		GAPConnectionEventPairingRequestIndication_t GAPConnectionEventPairingRequestIndication;
		GAPConnectionEventPeripheralSecurityRequestIndication_t GAPConnectionEventPeripheralSecurityRequestIndication;
		GAPConnectionEventPairingResponseIndication_t GAPConnectionEventPairingResponseIndication;
		GAPConnectionEventAuthenticationRejectedIndication_t GAPConnectionEventAuthenticationRejectedIndication;
		GAPConnectionEventPasskeyRequestIndication_t GAPConnectionEventPasskeyRequestIndication;
		GAPConnectionEventOobRequestIndication_t GAPConnectionEventOobRequestIndication;
		GAPConnectionEventPasskeyDisplayIndication_t GAPConnectionEventPasskeyDisplayIndication;
		GAPConnectionEventKeyExchangeRequestIndication_t GAPConnectionEventKeyExchangeRequestIndication;
		GAPConnectionEventKeysReceivedIndication_t GAPConnectionEventKeysReceivedIndication;
		GAPConnectionEventLongTermKeyRequestIndication_t GAPConnectionEventLongTermKeyRequestIndication;
		GAPConnectionEventEncryptionChangedIndication_t GAPConnectionEventEncryptionChangedIndication;
		GAPConnectionEventPairingCompleteIndication_t GAPConnectionEventPairingCompleteIndication;
		GAPConnectionEventDisconnectedIndication_t GAPConnectionEventDisconnectedIndication;
		GAPConnectionEventRssiReadIndication_t GAPConnectionEventRssiReadIndication;
		GAPConnectionEventTxPowerLevelReadIndication_t GAPConnectionEventTxPowerLevelReadIndication;
		GAPConnectionEventPowerReadFailureIndication_t GAPConnectionEventPowerReadFailureIndication;
		GAPConnectionEventParameterUpdateRequestIndication_t GAPConnectionEventParameterUpdateRequestIndication;
		GAPConnectionEventParameterUpdateCompleteIndication_t GAPConnectionEventParameterUpdateCompleteIndication;
		GAPConnectionEventLeDataLengthChangedIndication_t GAPConnectionEventLeDataLengthChangedIndication;
		GAPConnectionEventLeScOobDataRequestIndication_t GAPConnectionEventLeScOobDataRequestIndication;
		GAPConnectionEventLeScDisplayNumericValueIndication_t GAPConnectionEventLeScDisplayNumericValueIndication;
		GAPConnectionEventLeScKeypressNotificationIndication_t GAPConnectionEventLeScKeypressNotificationIndication;
		GAPLeScPublicKeyRegeneratedIndication_t GAPLeScPublicKeyRegeneratedIndication;
		GAPGenericEventLeScLocalOobDataIndication_t GAPGenericEventLeScLocalOobDataIndication;
		GAPGenericEventHostPrivacyStateChangedIndication_t GAPGenericEventHostPrivacyStateChangedIndication;
		GAPGenericEventControllerPrivacyStateChangedIndication_t GAPGenericEventControllerPrivacyStateChangedIndication;
		GAPGenericEventTxPowerLevelSetCompleteIndication_t GAPGenericEventTxPowerLevelSetCompleteIndication;
		GAPGenericEventLePhyEventIndication_t GAPGenericEventLePhyEventIndication;
		GAPGetBondedDevicesIdentityInformationIndication_t GAPGetBondedDevicesIdentityInformationIndication;
		GAPControllerNotificationIndication_t GAPControllerNotificationIndication;
		GAPBondCreatedIndication_t GAPBondCreatedIndication;
		GAPGenericEventChannelMapSetIndication_t GAPGenericEventChannelMapSetIndication;
		GAPConnectionEventChannelMapReadIndication_t GAPConnectionEventChannelMapReadIndication;
		GAPConnectionEventChannelMapReadFailureIndication_t GAPConnectionEventChannelMapReadFailureIndication;
		GAPGenericEventExtAdvertisingParamSetupCompleteIndication_t GAPGenericEventExtAdvertisingParamSetupCompleteIndication;
		GAPGenericEventExtAdvertisingDataSetupCompleteIndication_t GAPGenericEventExtAdvertisingDataSetupCompleteIndication;
		GAPGenericEventPeriodicAdvParamSetupCompleteIndication_t GAPGenericEventPeriodicAdvParamSetupCompleteIndication;
		GAPGenericEventPeriodicAdvDataSetupCompleteIndication_t GAPGenericEventPeriodicAdvDataSetupCompleteIndication;
		GAPGenericEventPeriodicAdvListUpdateCompleteIndication_t GAPGenericEventPeriodicAdvListUpdateCompleteIndication;
		GAPAdvertisingEventExtAdvertisingStateChangedIndication_t GAPAdvertisingEventExtAdvertisingStateChangedIndication;
		GAPAdvertisingEventAdvertisingSetTerminatedIndication_t GAPAdvertisingEventAdvertisingSetTerminatedIndication;
		GAPAdvertisingEventExtAdvertisingSetRemoveCompletedIndication_t GAPAdvertisingEventExtAdvertisingSetRemoveCompletedIndication;
		GAPAdvertisingEventExtScanReqReceivedIndication_t GAPAdvertisingEventExtScanReqReceivedIndication;
		GAPGenericEventPeriodicAdvertisingStateChangedIndication_t GAPGenericEventPeriodicAdvertisingStateChangedIndication;
		GAPScanningEventExtDeviceScannedIndication_t GAPScanningEventExtDeviceScannedIndication;
		GAPScanningEventPeriodicAdvSyncEstablishedIndication_t GAPScanningEventPeriodicAdvSyncEstablishedIndication;
		GAPScanningEventPeriodicAdvSyncTerminatedIndication_t GAPScanningEventPeriodicAdvSyncTerminatedIndication;
		GAPScanningEventPeriodicAdvSyncLostIndication_t GAPScanningEventPeriodicAdvSyncLostIndication;
		GAPScanningEventPeriodicDeviceScannedIndication_t GAPScanningEventPeriodicDeviceScannedIndication;
		GAPGenericEventPeriodicAdvCreateSyncCancelledIndication_t GAPGenericEventPeriodicAdvCreateSyncCancelledIndication;
		GAPConnectionEventChannelSelectionAlgorithm2Indication_t GAPConnectionEventChannelSelectionAlgorithm2Indication;
		GAPGenericEventTxEntryAvailableIndication_t GAPGenericEventTxEntryAvailableIndication;
		GAPGenericEventControllerLocalRPAReadIndication_t GAPGenericEventControllerLocalRPAReadIndication;
		GAPCheckNvmIndexIndication_t GAPCheckNvmIndexIndication;
		GAPGetDeviceIdFromConnHandleIndication_t GAPGetDeviceIdFromConnHandleIndication;
		GAPGetConnectionHandleFromDeviceIdIndication_t GAPGetConnectionHandleFromDeviceIdIndication;
		GAPPairingEventNoLTKIndication_t GAPPairingEventNoLTKIndication;
		GAPPairingAlreadyStartedIndication_t GAPPairingAlreadyStartedIndication;
		GAPGenericEventConnectionlessCteTransmitParamsSetupCompleteIndication_t GAPGenericEventConnectionlessCteTransmitParamsSetupCompleteIndication;
		GAPGenericEventConnectionlessCteTransmitStateChangedIndication_t GAPGenericEventConnectionlessCteTransmitStateChangedIndication;
		GAPGenericEventConnectionlessIqSamplingStateChangedIndication_t GAPGenericEventConnectionlessIqSamplingStateChangedIndication;
		GAPGenericEventAntennaInformationReadIndication_t GAPGenericEventAntennaInformationReadIndication;
		GAPScanningEventConnectionlessIqReportReceivedIndication_t GAPScanningEventConnectionlessIqReportReceivedIndication;
		GAPConnectionEventIqReportReceivedIndication_t GAPConnectionEventIqReportReceivedIndication;
		GAPConnectionEventCteRequestFailedIndication_t GAPConnectionEventCteRequestFailedIndication;
		GAPConnectionEventCteReceiveParamsSetupCompleteIndication_t GAPConnectionEventCteReceiveParamsSetupCompleteIndication;
		GAPConnectionEventCteTransmitParamsSetupCompleteIndication_t GAPConnectionEventCteTransmitParamsSetupCompleteIndication;
		GAPConnectionEventCteReqStateChangedIndication_t GAPConnectionEventCteReqStateChangedIndication;
		GAPConnectionEventCteRspStateChangedIndication_t GAPConnectionEventCteRspStateChangedIndication;
		GAPGenericEventPeriodicAdvRecvEnableCompleteIndication_t GAPGenericEventPeriodicAdvRecvEnableCompleteIndication;
		GAPGenericEventPeriodicAdvSyncTransferCompleteIndication_t GAPGenericEventPeriodicAdvSyncTransferCompleteIndication;
		GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication_t GAPGenericEventPeriodicAdvSetInfoTransferCompleteIndication;
		GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication_t GAPGenericEventSetPeriodicAdvSyncTransferParamsCompleteIndication;
		GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication_t GAPGenericEventSetDefaultPeriodicAdvSyncTransferParamsCompleteIndication;
		GAPScanningEventPeriodicAdvSyncTransferReceivedIndication_t GAPScanningEventPeriodicAdvSyncTransferReceivedIndication;
		GAPConnectionEventPathLossThresholdIndication_t GAPConnectionEventPathLossThresholdIndication;
		GAPConnectionEventTransmitPowerReportingIndication_t GAPConnectionEventTransmitPowerReportingIndication;
		GAPConnectionEventEnhancedReadTransmitPowerLevelIndication_t GAPConnectionEventEnhancedReadTransmitPowerLevelIndication;
		GAPConnectionEventPathLossReportingParamsSetupCompleteIndication_t GAPConnectionEventPathLossReportingParamsSetupCompleteIndication;
		GAPConnectionEventPathLossReportingStateChangedIndication_t GAPConnectionEventPathLossReportingStateChangedIndication;
		GAPConnectionEventTransmitPowerReportingStateChangedIndication_t GAPConnectionEventTransmitPowerReportingStateChangedIndication;
		GAPConnectionEventEattConnectionRequestIndication_t GAPConnectionEventEattConnectionRequestIndication;
		GAPConnectionEventEattConnectionCompleteIndication_t GAPConnectionEventEattConnectionCompleteIndication;
		GAPConnectionEventEattReconfigureResponseIndication_t GAPConnectionEventEattReconfigureResponseIndication;
		GAPConnectionEventEattBearerStatusNotificationIndication_t GAPConnectionEventEattBearerStatusNotificationIndication;
		GAPGenericEventLeGenerateDhKeyCompleteIndication_t GAPGenericEventLeGenerateDhKeyCompleteIndication;
		GAPGetHostVersionIndication_t GAPGetHostVersionIndication;
#endif  /* GAP_ENABLE */
	} Data;
} bleEvtContainer_t;

typedef memStatus_t (*pfnBleEvtHandler)(bleEvtContainer_t *container, uint8_t *pPayload);

typedef struct bleEvtHandler_tag
{
	uint16_t id;
	pfnBleEvtHandler handlerFunc;
} bleEvtHandler_t;


typedef memStatus_t (*pfnBleUnEvtHandler)(bleEvtContainer_t *container);

typedef struct bleUnEvtHandler_tag
{
	uint16_t id;
	pfnBleUnEvtHandler handlerFunc;
} bleUnEvtHandler_t;


/*==================================================================================================
Public function prototypes
==================================================================================================*/
#if L2CAP_ENABLE
memStatus_t L2CAPInitRequest(void *arg, uint8_t fsciInterface);
memStatus_t L2CAPSendAttDataRequest(L2CAPSendAttDataRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPSendSmpDataRequest(L2CAPSendSmpDataRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPRegisterAttCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t L2CAPRegisterSmpCallbackRequest(void *arg, uint8_t fsciInterface);
#endif  /* L2CAP_ENABLE */

#if L2CAPCB_ENABLE
memStatus_t L2CAPCBRegisterLeCbCallbacksRequest(void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBRegisterLePsmRequest(L2CAPCBRegisterLePsmRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBDeregisterLePsmRequest(L2CAPCBDeregisterLePsmRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBConnectLePsmRequest(L2CAPCBConnectLePsmRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBDisconnectLeCbChannelRequest(L2CAPCBDisconnectLeCbChannelRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBCancelConnectionRequest(L2CAPCBCancelConnectionRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBSendLeCbDataRequest(L2CAPCBSendLeCbDataRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBSendLeCreditRequest(L2CAPCBSendLeCreditRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBEnhancedConnectLePsmRequest(L2CAPCBEnhancedConnectLePsmRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBEnhancedChannelReconfigureRequest(L2CAPCBEnhancedChannelReconfigureRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t L2CAPCBEnhancedCancelConnectionRequest(L2CAPCBEnhancedCancelConnectionRequest_t *req, void *arg, uint8_t fsciInterface);
#endif  /* L2CAPCB_ENABLE */

#if GATT_ENABLE
memStatus_t GATTInitRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTGetMtuRequest(GATTGetMtuRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientInitRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientResetProcedureRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterProcedureCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterNotificationCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterMultipleValueNotificationCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterIndicationCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientExchangeMtuRequest(GATTClientExchangeMtuRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientDiscoverAllPrimaryServicesRequest(GATTClientDiscoverAllPrimaryServicesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientDiscoverPrimaryServicesByUuidRequest(GATTClientDiscoverPrimaryServicesByUuidRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientFindIncludedServicesRequest(GATTClientFindIncludedServicesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientDiscoverAllCharacteristicsOfServiceRequest(GATTClientDiscoverAllCharacteristicsOfServiceRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientDiscoverCharacteristicOfServiceByUuidRequest(GATTClientDiscoverCharacteristicOfServiceByUuidRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientDiscoverAllCharacteristicDescriptorsRequest(GATTClientDiscoverAllCharacteristicDescriptorsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientReadCharacteristicValueRequest(GATTClientReadCharacteristicValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientReadUsingCharacteristicUuidRequest(GATTClientReadUsingCharacteristicUuidRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientReadMultipleCharacteristicValuesRequest(GATTClientReadMultipleCharacteristicValuesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientWriteCharacteristicValueRequest(GATTClientWriteCharacteristicValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientReadCharacteristicDescriptorRequest(GATTClientReadCharacteristicDescriptorRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientWriteCharacteristicDescriptorRequest(GATTClientWriteCharacteristicDescriptorRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerInitRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTServerRegisterCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTServerRegisterHandlesForWriteNotificationsRequest(GATTServerRegisterHandlesForWriteNotificationsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerSendAttributeWrittenStatusRequest(GATTServerSendAttributeWrittenStatusRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerSendNotificationRequest(GATTServerSendNotificationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerSendIndicationRequest(GATTServerSendIndicationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerSendInstantValueNotificationRequest(GATTServerSendInstantValueNotificationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerSendInstantValueIndicationRequest(GATTServerSendInstantValueIndicationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerSendMultipleHandleValueNotificationRequest(GATTServerSendMultipleHandleValueNotificationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerRegisterHandlesForReadNotificationsRequest(GATTServerRegisterHandlesForReadNotificationsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerSendAttributeReadStatusRequest(GATTServerSendAttributeReadStatusRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerRegisterUniqueHandlesForNotificationsRequest(GATTServerRegisterUniqueHandlesForNotificationsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerUnregisterHandlesForWriteNotificationsRequest(GATTServerUnregisterHandlesForWriteNotificationsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerUnregisterHandlesForReadNotificationsRequest(GATTServerUnregisterHandlesForReadNotificationsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientReadMultipleVariableCharacteristicValuesRequest(GATTClientReadMultipleVariableCharacteristicValuesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterEnhancedProcedureCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterEnhancedNotificationCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterEnhancedIndicationCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientRegisterEnhancedMultipleValueNotificationCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedDiscoverAllPrimaryServicesRequest(GATTClientEnhancedDiscoverAllPrimaryServicesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedDiscoverPrimaryServicesByUuidRequest(GATTClientEnhancedDiscoverPrimaryServicesByUuidRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedFindIncludedServicesRequest(GATTClientEnhancedFindIncludedServicesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedDiscoverAllCharacteristicsOfServiceRequest(GATTClientEnhancedDiscoverAllCharacteristicsOfServiceRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedDiscoverCharacteristicOfServiceByUuidRequest(GATTClientEnhancedDiscoverCharacteristicOfServiceByUuidRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedDiscoverAllCharacteristicDescriptorsRequest(GATTClientEnhancedDiscoverAllCharacteristicDescriptorsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedReadCharacteristicValueRequest(GATTClientEnhancedReadCharacteristicValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedReadUsingCharacteristicUuidRequest(GATTClientEnhancedReadUsingCharacteristicUuidRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedReadMultipleCharacteristicValuesRequest(GATTClientEnhancedReadMultipleCharacteristicValuesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedWriteCharacteristicValueRequest(GATTClientEnhancedWriteCharacteristicValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedReadCharacteristicDescriptorRequest(GATTClientEnhancedReadCharacteristicDescriptorRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedWriteCharacteristicDescriptorRequest(GATTClientEnhancedWriteCharacteristicDescriptorRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientEnhancedReadMultipleVariableCharacteristicValuesRequest(GATTClientEnhancedReadMultipleVariableCharacteristicValuesRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerRegisterEnhancedCallbackRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTServerEnhancedSendAttributeWrittenStatusRequest(GATTServerEnhancedSendAttributeWrittenStatusRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerEnhancedSendNotificationRequest(GATTServerEnhancedSendNotificationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerEnhancedSendIndicationRequest(GATTServerEnhancedSendIndicationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerEnhancedSendInstantValueNotificationRequest(GATTServerEnhancedSendInstantValueNotificationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerEnhancedSendInstantValueIndicationRequest(GATTServerEnhancedSendInstantValueIndicationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerEnhancedSendAttributeReadStatusRequest(GATTServerEnhancedSendAttributeReadStatusRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTServerEnhancedSendMultipleHandleValueNotificationRequest(GATTServerEnhancedSendMultipleHandleValueNotificationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTClientGetDatabaseHashRequest(GATTClientGetDatabaseHashRequest_t *req, void *arg, uint8_t fsciInterface);
#endif  /* GATT_ENABLE */

#if GATTDB_APP_ENABLE
memStatus_t GATTDBWriteAttributeRequest(GATTDBWriteAttributeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBReadAttributeRequest(GATTDBReadAttributeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBFindServiceHandleRequest(GATTDBFindServiceHandleRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBFindCharValueHandleInServiceRequest(GATTDBFindCharValueHandleInServiceRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBFindCccdHandleForCharValueHandleRequest(GATTDBFindCccdHandleForCharValueHandleRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBFindDescriptorHandleForCharValueHandleRequest(GATTDBFindDescriptorHandleForCharValueHandleRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicInitRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicReleaseDatabaseRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddPrimaryServiceDeclarationRequest(GATTDBDynamicAddPrimaryServiceDeclarationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddSecondaryServiceDeclarationRequest(GATTDBDynamicAddSecondaryServiceDeclarationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddIncludeDeclarationRequest(GATTDBDynamicAddIncludeDeclarationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddCharacteristicDeclarationAndValueRequest(GATTDBDynamicAddCharacteristicDeclarationAndValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddCharacteristicDescriptorRequest(GATTDBDynamicAddCharacteristicDescriptorRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddCccdRequest(void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest(GATTDBDynamicAddCharacteristicDeclarationWithUniqueValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicRemoveServiceRequest(GATTDBDynamicRemoveServiceRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicRemoveCharacteristicRequest(GATTDBDynamicRemoveCharacteristicRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicAddCharDescriptorWithUniqueValueRequest(GATTDBDynamicAddCharDescriptorWithUniqueValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBDynamicEndDatabaseUpdateRequest(void *arg, uint8_t fsciInterface);
#endif  /* GATTDB_APP_ENABLE */

#if GATTDB_ATT_ENABLE
memStatus_t GATTDBAttFindInformationRequest(GATTDBAttFindInformationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttFindByTypeValueRequest(GATTDBAttFindByTypeValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttReadByTypeRequest(GATTDBAttReadByTypeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttReadRequest(GATTDBAttReadRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttReadBlobRequest(GATTDBAttReadBlobRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttReadMultipleRequest(GATTDBAttReadMultipleRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttReadByGroupTypeRequest(GATTDBAttReadByGroupTypeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttWriteRequest(GATTDBAttWriteRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttWriteCommandRequest(GATTDBAttWriteCommandRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttSignedWriteCommandRequest(GATTDBAttSignedWriteCommandRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttPrepareWriteRequest(GATTDBAttPrepareWriteRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttExecuteWriteRequest(GATTDBAttExecuteWriteRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttExecuteWriteFromQueueRequest(GATTDBAttExecuteWriteFromQueueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GATTDBAttPrepareNotificationIndicationRequest(GATTDBAttPrepareNotificationIndicationRequest_t *req, void *arg, uint8_t fsciInterface);
#endif  /* GATTDB_ATT_ENABLE */

#if GAP_ENABLE
memStatus_t BLEHostInitializeRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPRegisterDeviceSecurityRequirementsRequest(GAPRegisterDeviceSecurityRequirementsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetAdvertisingParametersRequest(GAPSetAdvertisingParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetAdvertisingDataRequest(GAPSetAdvertisingDataRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPStartAdvertisingRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPStopAdvertisingRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPAuthorizeRequest(GAPAuthorizeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSaveCccdRequest(GAPSaveCccdRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPCheckNotificationStatusRequest(GAPCheckNotificationStatusRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPCheckIndicationStatusRequest(GAPCheckIndicationStatusRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPPairRequest(GAPPairRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSendPeripheralSecurityRequestRequest(GAPSendPeripheralSecurityRequestRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEncryptLinkRequest(GAPEncryptLinkRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPAcceptPairingRequestRequest(GAPAcceptPairingRequestRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPRejectPairingRequest(GAPRejectPairingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnterPasskeyRequest(GAPEnterPasskeyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPProvideOobRequest(GAPProvideOobRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPRejectPasskeyRequestRequest(GAPRejectPasskeyRequestRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSendSmpKeysRequest(GAPSendSmpKeysRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPRejectKeyExchangeRequestRequest(GAPRejectKeyExchangeRequestRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPProvideLongTermKeyRequest(GAPProvideLongTermKeyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPDenyLongTermKeyRequest(GAPDenyLongTermKeyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLoadEncryptionInformationRequest(GAPLoadEncryptionInformationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetLocalPasskeyRequest(GAPSetLocalPasskeyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPStartScanningRequest(GAPStartScanningRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPStopScanningRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPConnectRequest(GAPConnectRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPDisconnectRequest(GAPDisconnectRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSaveCustomPeerInformationRequest(GAPSaveCustomPeerInformationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLoadCustomPeerInformationRequest(GAPLoadCustomPeerInformationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPCheckIfBondedRequest(GAPCheckIfBondedRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPReadFilterAcceptListSizeRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPClearFilterAcceptListRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPAddDeviceToFilterAcceptListRequest(GAPAddDeviceToFilterAcceptListRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPRemoveDeviceFromFilterAcceptListRequest(GAPRemoveDeviceFromFilterAcceptListRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPReadPublicDeviceAddressRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPCreateRandomDeviceAddressRequest(GAPCreateRandomDeviceAddressRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSaveDeviceNameRequest(GAPSaveDeviceNameRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPGetBondedDevicesCountRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPGetBondedDeviceNameRequest(GAPGetBondedDeviceNameRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPRemoveBondRequest(GAPRemoveBondRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPRemoveAllBondsRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPReadRadioPowerLevelRequest(GAPReadRadioPowerLevelRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPVerifyPrivateResolvableAddressRequest(GAPVerifyPrivateResolvableAddressRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetRandomAddressRequest(GAPSetRandomAddressRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetScanModeRequest(GAPSetScanModeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetDefaultPairingParametersRequest(GAPSetDefaultPairingParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPUpdateConnectionParametersRequest(GAPUpdateConnectionParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableUpdateConnectionParametersRequest(GAPEnableUpdateConnectionParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GapUpdateLeDataLengthRequest(GapUpdateLeDataLengthRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableHostPrivacyRequest(GAPEnableHostPrivacyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableControllerPrivacyRequest(GAPEnableControllerPrivacyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLeScRegeneratePublicKeyRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPLeScValidateNumericValueRequest(GAPLeScValidateNumericValueRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLeScGetLocalOobDataRequest(void *arg, uint8_t fsciInterface);
memStatus_t GAPLeScSetPeerOobDataRequest(GAPLeScSetPeerOobDataRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLeScSendKeypressNotificationPrivacyRequest(GAPLeScSendKeypressNotificationPrivacyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPGetBondedDevicesIdentityInformationRequest(GAPGetBondedDevicesIdentityInformationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetTxPowerLevelRequest(GAPSetTxPowerLevelRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLeReadPhyRequest(GAPLeReadPhyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLeSetPhyRequest(GAPLeSetPhyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPControllerEnhancedNotificationRequest(GAPControllerEnhancedNotificationRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPLoadKeysRequest(GAPLoadKeysRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSaveKeysRequest(GAPSaveKeysRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetChannelMapRequest(GAPSetChannelMapRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPReadChannelMapRequest(GAPReadChannelMapRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetPrivacyModeRequest(GAPSetPrivacyModeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t ControllerSetScanDupFiltModeRequest(ControllerSetScanDupFiltModeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPReadControllerLocalRPARequest(GAPReadControllerLocalRPARequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPCheckNvmIndexRequest(GAPCheckNvmIndexRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPGetDeviceIdFromConnHandleRequest(GAPGetDeviceIdFromConnHandleRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPGetConnectionHandleFromDeviceId(GAPGetConnectionHandleFromDeviceId_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPAdvIndexChangeRequest(GAPAdvIndexChangeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPGetHostVersion(void *arg, uint8_t fsciInterface);
memStatus_t GAPSetExtAdvertisingParametersRequest(GAPSetExtAdvertisingParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPStartExtAdvertisingRequest(GAPStartExtAdvertisingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPRemoveAdvertisingSetRequest(GAPRemoveAdvertisingSetRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPStopExtAdvertisingRequest(GAPStopExtAdvertisingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPUpdatePeriodicAdvListRequest(GAPUpdatePeriodicAdvListRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetPeriodicAdvParametersRequest(GAPSetPeriodicAdvParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPStartPeriodicAdvertisingRequest(GAPStartPeriodicAdvertisingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPStopPeriodicAdvertisingRequest(GAPStopPeriodicAdvertisingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetExtAdvertisingDataRequest(GAPSetExtAdvertisingDataRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetPeriodicAdvertisingDataRequest(GAPSetPeriodicAdvertisingDataRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPPeriodicAdvCreateSyncRequest(GAPPeriodicAdvCreateSyncRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPPeriodicAdvTerminateSyncRequest(GAPPeriodicAdvTerminateSyncRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPGenerateDHKeyV2Request(GAPGenerateDHKeyV2Request_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPModifySleepClockAccuracyRequest(GAPModifySleepClockAccuracyRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t ControllerConfigureAdvCodingSchemeRequest(ControllerConfigureAdvCodingSchemeRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetConnectionlessCteTransmitParametersRequest(GAPSetConnectionlessCteTransmitParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableConnectionlessCteTransmitRequest(GAPEnableConnectionlessCteTransmitRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableConnectionlessIqSamplingRequest(GAPEnableConnectionlessIqSamplingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetConnectionCteReceiveParametersRequest(GAPSetConnectionCteReceiveParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetConnectionCteTransmitParametersRequest(GAPSetConnectionCteTransmitParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableConnectionCteRequestRequest(GAPEnableConnectionCteRequestRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableConnectionCteResponseRequest(GAPEnableConnectionCteResponseRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPReadAntennaInformationRequest(void *arg, uint8_t fsciInterface);
memStatus_t WritePublicDeviceAddressRequest(WritePublicDeviceAddressRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPPeriodicAdvReceiveEnableRequest(GAPPeriodicAdvReceiveEnableRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPPeriodicAdvReceiveDisableRequest(GAPPeriodicAdvReceiveDisableRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPPeriodicAdvSyncTransferRequest(GAPPeriodicAdvSyncTransferRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPPeriodicAdvSetInfoTransferRequest(GAPPeriodicAdvSetInfoTransferRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetPeriodicAdvSyncTransferParamsRequest(GAPSetPeriodicAdvSyncTransferParamsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetDefaultPeriodicAdvSyncTransferParamsRequest(GAPSetDefaultPeriodicAdvSyncTransferParamsRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnhancedReadTransmitPowerLevelRequest(GAPEnhancedReadTransmitPowerLevelRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPReadRemoteTransmitPowerLevelRequest(GAPReadRemoteTransmitPowerLevelRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPSetPathLossReportingParametersRequest(GAPSetPathLossReportingParametersRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnablePathLossReportingRequest(GAPEnablePathLossReportingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEnableTransmitPowerReportingRequest(GAPEnableTransmitPowerReportingRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEattConnectionRequest(GAPEattConnectionRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEattConnectionAccept(GAPEattConnectionAccept_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEattReconfigureRequest(GAPEattReconfigureRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t GAPEattSendCreditsRequest(GAPEattSendCreditsRequest_t *req, void *arg, uint8_t fsciInterface);
#endif  /* GAP_ENABLE */

#if FSCI_ENABLE
memStatus_t FSCICPUResetRequest(void *arg, uint8_t fsciInterface);
memStatus_t FSCIGetNumberOfFreeBuffersRequest(void *arg, uint8_t fsciInterface);
memStatus_t FSCIAllowDeviceToSleepRequest(FSCIAllowDeviceToSleepRequest_t *req, void *arg, uint8_t fsciInterface);
memStatus_t FSCIGetWakeupReasonRequest(void *arg, uint8_t fsciInterface);
memStatus_t FSCIGetNbuVersionRequest(void *arg, uint8_t fsciInterface);
#endif  /* FSCI_ENABLE */

#if NVM_ENABLE
memStatus_t FSCINVGetNvVPSizeRequest(void *arg, uint8_t fsciInterface);
#endif  /* NVM_ENABLE */

void KHC_BLE_RX_MsgHandler(void *pData, void *param, uint8_t fsciInterface);

void KHC_BLE_RX_UnMsgHandler(void *pData, void *param, uint8_t fsciInterface);

#endif  /* _BLE_CMD_H */
